<?php
$group_sort_y = array();
class CrossTableReport
{
	var $tableName;
	var $shortTableName;
	var $col_summary = array();
	var $group_header = array();
	var $rowinfo = array();
	var $total_summary;
	var $xml_array;
	var $is_value_empty;
	
	/*
	 *	@type Array
	 *	The list of field aliases from webreports
	 */
	var $arrDBFieldsList = array();
	
	/*
	 *	@type Boolean
	 *	Webreports - true if the report is database table based.
	 */
	var $wrdb = false;
	
	var $index_field_x;
	var $index_field_y;
	
	/**
	 * @type String
	 * The possible values are: "project", "db", "custom"
	 */
	protected $table_type = "project";
	
	/**
	 * An instance of the 'ViewControlsContainer' class 
	 * @type Object
	 */
	protected $viewControls;
	
	/**
	 * A flag indicating if the report is run from project pages (not a web-report)
	 * @type Boolean
	 */
	protected $fromWizard = false;
	
	/**
	 * An Instance of the 'ProjectSettings' class
	 * @type Object
	 */
	protected $pSet = null;
	
	/**
	 * The selected data field settings packed in an array.
	 * @type Array
	 */
	protected $dataFieldSettings = null;

	/**
	 * The report current data field name
	 */
	protected $dataField = "";
	
	/**
	 * The report current aggregate function
	 */
	protected $dataGroupFunction = "";
	
	/**
	 * An array of table keys
	 * @type Array	 
	 */
	protected $tableKeys = null;
	
	/**
	 * @type Connection
	 */
	protected $connection;
	
	protected $sessionPrefix;
	
	/**
	 * The flag indicating if the report
	 * is show on a dashboard
	 */
	protected $dashBased = false;
	
	/**
	 * The dashboard table name
	 */	
	protected $dashTName = "";
	
	
	/**
	 * @constructor
	 * @param Array rpt_array
	 * @param String strSQL
	 */
	function CrossTableReport($rpt_array, $strSQL)
	{
		$this->xml_array = $rpt_array;

		if( $rpt_array["table_type"] )
			$this->table_type = $rpt_array["table_type"];
		
		if( $rpt_array["fromWizard"] )
			$this->fromWizard = true;
			
		$this->wrdb = $rpt_array["wrdb"];
		$this->arrDBFieldsList = $rpt_array["arrDBFieldsList"];
		
		$this->tableName = $this->xml_array["tables"][0];
		$this->setDbConnection();
		
		$this->shortTableName = GetTableURL($this->tableName);
		if( strlen($this->shortTableName) == 0 )
			$this->shortTableName = $this->tableName;
			
		$this->pSet = new ProjectSettings($this->tableName, PAGE_REPORT);
		if( $this->fromWizard )
		{
			include_once getabspath("classes/controls/ViewControlsContainer.php");
			$this->viewControls = new ViewControlsContainer($this->pSet, PAGE_REPORT);
			$this->tableKeys = $this->pSet->getTableKeys();
			
			if( $rpt_array["dashBased"] )
			{
				$this->dashBased = true;
				$this->dashTName = $rpt_array["dashTName"];
			}
		}
		
		$this->setSessionPrefix( $rpt_array["sessionPrefix"] );
		$this->fillSessionVariables();


		$this->dataField = $this->getDataField( $_SESSION[$this->sessionPrefix."_field"] );
		if( !strlen($this->dataField) )
			$this->dataField = $_SESSION['webreports']['group_fields'][0]["name"];
		
		$this->initDataFieldSettings();
		$this->initDataGroupFunction($_SESSION[$this->sessionPrefix."_group_func"]);
		
				
		// assign index_field_x, index_field_y properties
		$this->setAxisFieldsIndices();

		if( $this->fromWizard )
			$fName = $this->CrossGoodFieldName( $this->dataField );		
		else
			$fName = $this->getDBFieldName($this->CrossGoodFieldName( $this->dataField ));

		if( $fName != " " )
			$ftype = $this->getFieldType($fName);

		$crtableSQL = $this->getstrSQL( $strSQL );
		$qResult = $this->connection->query( $crtableSQL );
		
		$group_y = array();
		$group_x = array();
		$sort_y = array();
		
		$arrdata = array();
		$arravgsum = array();
		$arravgcount = array();
		
		$avgsumx = array();
		$avgcountx = array();
		
		while( $data = $qResult->fetchNumeric() )
		{
			if( !in_array($data[1], $group_y) )
			{
				$group_y[] = $data[1];
				$sort_y[] = count($sort_y);
			}
			
			if( !in_array($data[2], $group_x) )
			{
				$group_x[] = $data[2];
				$this->col_summary["data"][ count($group_x) - 1 ]["col_summary"] = "&nbsp;";
				$this->col_summary["data"][ count($group_x) - 1 ]["id_col_summary"] = "total_x_".( count($group_x) - 1);
			}
			
			$key_y = array_search( $data[1], $group_y );
			
			$key_x = array_search( $data[2], $group_x );
			
			$avgsumx[ $key_x ] = 0;
			$avgcountx[ $key_x ] = 0;

			if( !$this->is_value_empty )
			{
				$arrdata[ $key_y ][ $key_x ] = $data[0];
				$arravgsum[ $key_y ][ $key_x ] = $data[3];
				$arravgcount[ $key_y ][ $key_x ] = $data[4];
			}
			else
				$arrdata[ $key_y ][ $key_x ] = "&nbsp;";
		}
		
		//	sort y groups
		global $group_sort_y;
		$group_sort_y = $group_y;
		SortForCrosstable($sort_y);
		
		
		foreach($sort_y as $key_y)
		{
			$value_y = $group_y[ $key_y ];
			$this->rowinfo[ $key_y ]["row_summary"] = "&nbsp;";
			$this->rowinfo[ $key_y ]["group_y"] = $this->getAxisDisplayValue($this->index_field_y, $value_y);
			
			foreach($group_x as $key_x => $value_x)
			{
				if( array_key_exists($key_y, $arrdata) )
				{
					$rowValue = "&nbsp;";
					
					if( array_key_exists($key_x, $arrdata[ $key_y ]) && !$this->is_value_empty && !is_null($arrdata[ $key_y ][ $key_x ]) )
					{
						$rowValue = $arrdata[ $key_y ][ $key_x ];
						if( $this->dataGroupFunction == "avg" && !IsTimeType($ftype))
							$rowValue = round($rowValue, 2);
					}
					
					$this->rowinfo[ $key_y ]["row_record"]["data"][ $key_x ]["row_value"] = $rowValue;	
					$this->rowinfo[ $key_y ]["row_record"]["data"][ $key_x ]["id_data"] = $key_y."_".$key_x;
				}
			}
			$this->rowinfo[ $key_y ]["id_row_summary"] = "total_y_".$key_y;
		}

		foreach($group_x as $key_x => $value_x)
		{
			if($value_x != "")
				$this->group_header["data"][ $key_x ]["gr_value"] = $this->getAxisDisplayValue($this->index_field_x, $value_x);
			else
				$this->group_header["data"][ $key_x ]["gr_value"]="&nbsp;";
		}

		$sum_x = $this->xml_array["group_fields"][ count($this->xml_array["group_fields"]) - 1 ]["sum_x"];
		$sum_y = $this->xml_array["group_fields"][ count($this->xml_array["group_fields"]) - 1 ]["sum_y"];
		$sum_total = $this->xml_array["group_fields"][ count($this->xml_array["group_fields"]) - 1 ]["sum_total"];
		
		$this->total_summary = "&nbsp;";
		foreach($this->rowinfo as $key_y => $obj_y)
		{
			$obj_x = $obj_y["row_record"]["data"];
			foreach($obj_x as $key_x => $value)
			{
				if($value["row_value"] !== "&nbsp;")
				{
					switch($this->dataGroupFunction)
					{
						case "sum":
							if(!is_null($value["row_value"]))
							{
								$this->rowinfo[$key_y]["row_summary"] += $value["row_value"];
								$this->col_summary["data"][$key_x]["col_summary"] += $value["row_value"];
								$this->total_summary += $value["row_value"];
							}
						break;
						case "min":
							if(($this->rowinfo[$key_y]["row_summary"] === "&nbsp;" || $value["row_value"]<$this->rowinfo[$key_y]["row_summary"]) && !is_null($value["row_value"]))
								$this->rowinfo[$key_y]["row_summary"] = $value["row_value"];
							if(($this->col_summary["data"][$key_x]["col_summary"] === "&nbsp;" || $this->col_summary["data"][$key_x]["col_summary"]>$value["row_value"]) && !is_null($value["row_value"]))
								$this->col_summary["data"][$key_x]["col_summary"] = $value["row_value"];
							if(($this->total_summary === "&nbsp;" || $this->total_summary>$value["row_value"]) && !is_null($value["row_value"]))
								$this->total_summary = $value["row_value"];
								
						break;
						case "max":
							if($this->rowinfo[$key_y]["row_summary"] === "&nbsp;" || $value["row_value"]>$this->rowinfo[$key_y]["row_summary"])
								$this->rowinfo[$key_y]["row_summary"] = $value["row_value"];
							if($this->col_summary["data"][$key_x]["col_summary"] === "&nbsp;" || $this->col_summary["data"][$key_x]["col_summary"]<$value["row_value"])								
								$this->col_summary["data"][$key_x]["col_summary"] = $value["row_value"];
							if($this->total_summary === "&nbsp;" || $this->total_summary<$value["row_value"])
								$this->total_summary = $value["row_value"];
						break;
						case "avg":
							$this->rowinfo[$key_y]["avgsumy"] += $arravgsum[$key_y][$key_x];
							$this->rowinfo[$key_y]["avgcounty"] += $arravgcount[$key_y][$key_x];
							$this->rowinfo[$key_y]["row_record"]["data"][$key_x]["avgsumx"] += $arravgsum[$key_y][$key_x];
							$this->rowinfo[$key_y]["row_record"]["data"][$key_x]["avgcountx"] += $arravgcount[$key_y][$key_x];
						break;
					}
					if($sum_x == true && !$this->is_value_empty && !is_null($this->col_summary["data"][$key_x]["col_summary"]))
					{
						if(is_numeric($this->col_summary["data"][$key_x]["col_summary"]))
							$this->col_summary["data"][$key_x]["col_summary"] = round($this->col_summary["data"][$key_x]["col_summary"],2);
					}
					else
						$this->col_summary["data"][$key_x]["col_summary"] = "&nbsp;";
				}
			}
			if($sum_y == true && !$this->is_value_empty && !is_null($this->rowinfo[$key_y]["row_summary"]))
			{
				if(is_numeric($this->rowinfo[$key_y]["row_summary"]))
					$this->rowinfo[$key_y]["row_summary"] = round($this->rowinfo[$key_y]["row_summary"],2);
			}
			else
				$this->rowinfo[$key_y]["row_summary"] = "&nbsp;";
		}
		
		if($this->dataGroupFunction == "avg")
		{
			$total_sum = 0;
			$total_count = 0;
			
			foreach($this->rowinfo as $key_y => $valuey)
			{
				if($valuey["avgcounty"])
				{
					$this->rowinfo[$key_y]["row_summary"] = round($valuey["avgsumy"]/$valuey["avgcounty"],2);
					$total_sum += $valuey["avgsumy"];
					$total_count += $valuey["avgcounty"];
				}
				foreach($valuey["row_record"]["data"] as $key_x => $valuex)
				{
					if($valuex["avgcountx"])
					{
						$avgsumx[$key_x] += $valuex["avgsumx"];
						$avgcountx[$key_x] += $valuex["avgcountx"];
						$total_sum += $valuex["avgsumx"];
						$total_count += $valuex["avgcountx"];
					}
				}
			}
			foreach($avgsumx as $key => $value)
			{
				if($avgcountx[$key])
					$this->col_summary["data"][$key]["col_summary"] = round($value/$avgcountx[$key],2);
			}
			if($total_count)
				$this->total_summary = $total_sum/$total_count;
		}
		
		if( $sum_total != true || $this->is_value_empty )
			$this->total_summary = "&nbsp;";
		elseif( is_numeric($this->total_summary) )
			$this->total_summary = round($this->total_summary,2);
		
		$this->updateRecordsDisplayedFields();
	}

	/**
	 * Get view value basing on 'view as'
	 */
	function getViewValue( $value, $useTimeFormat = true )
	{
		$strViewFormat = $this->pSet->getViewFormat( $this->dataField );
		if( $strViewFormat == FORMAT_TIME && is_numeric($value) )
		{
			$d = intval($value / 86400);
			$h = intval(($value % 86400) / 3600);
			$m = intval((($value % 86400) % 3600) / 60);
			$s = (($value % 86400) % 3600) % 60;

			$value = $d > 0 ? $d . 'd ' : '';
			if ( $useTimeFormat )
			{
				$value .= str_format_time(array(0, 0, 0, $h, $m, $s));
			}
			else
			{
				$value .= date("H:i:s", strtotime($h.":".$m.":".$s));
			}
		}
		else
		{
			$control = $this->viewControls->getControl( $this->dataField );
			$controlData = array( $this->dataField => $value );
			$value = $control->showDBValue( $controlData, "" );
		}

		return $value;
	}	

	/**
	 * Update the records and summaries data basing on 'view as' and 'total' settings 
	 */
	protected function updateRecordsDisplayedFields()
	{		
		if( !count($this->rowinfo) )
			return;
		
		if( !$this->fromWizard )
		{
			$this->updateWebReportRecordsDisplayedFields( );
			return;
		}
		
		foreach($this->rowinfo as $key_y => $data)
		{
			foreach($data["row_record"]["data"] as $key_x => $fieldData)
			{
				if( $fieldData["row_value"] == "&nbsp;" )
					continue;

				$this->rowinfo[ $key_y ]["row_record"]["data"][$key_x]["row_value"] = $this->getViewValue($fieldData["row_value"]);
			}

			if( $data["row_summary"] != "&nbsp;" ) 
			{
				$this->rowinfo[ $key_y ]["row_summary"] = $this->getViewValue( $data["row_summary"], false);
			}
		}

		if( $this->total_summary != "&nbsp;" )
		{
			$this->total_summary = $this->getViewValue($this->total_summary, false);
		}

		foreach($this->col_summary["data"] as $key => $summaryData)
		{
			if( $summaryData["col_summary"] == "&nbsp;" )
				continue;

			$this->col_summary["data"][$key]["col_summary"] = $this->getViewValue($summaryData["col_summary"], false);
		}		
	}
	
	/**
	 * Apply currency formatting to the data field values and totals (for Web reports only)
	 */
	protected function updateWebReportRecordsDisplayedFields()
	{
		if( $this->dataFieldSettings['curr'] != true )
			return;
			
		foreach($this->rowinfo as $arrkey => $arrfield)
		{
			foreach($arrfield["row_record"]["data"] as $fieldkey => $fieldvalue)
			{
				if( is_numeric($fieldvalue["row_value"]) )
					$this->rowinfo[$arrkey]["row_record"]["data"][$fieldkey]["row_value"] = str_format_currency($fieldvalue["row_value"]);
			}
			
			if( is_numeric($arrfield["row_summary"]) )
				$this->rowinfo[$arrkey]["row_summary"] = str_format_currency($arrfield["row_summary"]);
		}
		
		if( is_numeric($this->total_summary) )
			$this->total_summary = str_format_currency($this->total_summary);
		
		foreach( $this->col_summary["data"] as $arrkey => $arrvalue )
		{
			if(is_numeric($arrvalue["col_summary"]))
				$this->col_summary["data"][$arrkey]["col_summary"] = str_format_currency($arrvalue["col_summary"]);
		}	
	}
	
	/**
	 *
	 */
	protected function setSessionPrefix( $sessionPrefix = "")
	{
		if( $sessionPrefix )
			$this->sessionPrefix = $sessionPrefix;
		else
			$this->sessionPrefix = $this->shortTableName;
	}
	
	/**
	 * Fill a cross-table report's session variable
	 */
	protected function fillSessionVariables()
	{
		if( postvalue("group_func") != "" )
			$_SESSION[$this->sessionPrefix."_group_func"] = postvalue("group_func");
			
		if( postvalue("field") != "" )
			$_SESSION[$this->sessionPrefix."_field"] = postvalue("field");
			
		if( postvalue("axis_x") != "" )
			$_SESSION[$this->sessionPrefix."_gr_x"] = postvalue("axis_x");
			
		if( postvalue("axis_y") != "" )
			$_SESSION[$this->sessionPrefix."_gr_y"] = postvalue("axis_y");
			
		if( postvalue("rname") != "" )
			$_SESSION[$this->sessionPrefix."_rname"] = postvalue("rname");		
	}
	
	public function getCrossTableData()
	{
		return $this->rowinfo;
	}
	
	public function getCrossTableHeader()
	{
		return $this->group_header;
	}
	
	public function getCrossTableSummary()
	{
		return $this->col_summary;
	}
	
	public function getTotalSummary()
	{
		return $this->total_summary;
	}
	
	/**
	 * Assign index_field_x, index_field_y properties
	 */
	protected function setAxisFieldsIndices()
	{
		$gr_x = $_SESSION[$this->sessionPrefix."_gr_x"];
		$gr_y = $_SESSION[$this->sessionPrefix."_gr_y"];
		
		if( $gr_x == "" )
			$this->index_field_x = $this->getFirstGroupField("x");
		else 
			$this->index_field_x = $gr_x;
		
		if( $gr_y == "" )
			$this->index_field_y = $this->getFirstGroupField("y");
		else
			$this->index_field_y = $gr_y;		
	}	
	
	/**
	 * Assign 'connection' property 
	 */
	 
	protected function setDbConnection()
	{
		global $cman;
		if($this->isProjectDB())
			$this->connection = $cman->byTable( $this->tableName ); //#9875
		else
			$this->connection = $cman->getDefault();
	}	

	/**
	 * Get a report's SQL query string
	 * @param String strSQL		The report table's SQL query
	 * @return String	
	 */
	protected function getstrSQL( $strSQL )
	{
		global $strTableName;
			
		$group_x = $this->getIntervalType( $this->index_field_x );
		$group_y = $this->getIntervalType( $this->index_field_y );
		if( $this->fromWizard )
			$fName = $this->CrossGoodFieldName( $this->dataField );		
		else
			$fName = $this->getDBFieldName($this->CrossGoodFieldName( $this->dataField ));		
		
		$select_field = "' ', ";
		$avg_func = "";
		if( $fName != " " )
		{	
			$strViewFormat = $this->pSet->getViewFormat( $this->dataField );
			$ftype = $this->getFieldType($fName);
			$isTime = $strViewFormat == FORMAT_TIME || IsTimeType($ftype);
			
			if ( $isTime )
			{
				$select_field = $this->dataGroupFunction."(".$this->connection->timeToSecWrapper($fName)."), ";
			}
			else
			{
				$select_field = $this->dataGroupFunction."(".$this->connection->addFieldWrappers($fName)."), ";
			}
			
			$this->is_value_empty = false;

			if( $this->dataGroupFunction == "avg" && !IsDateFieldType($ftype) )
			{
				$sum_for_avg = !$isTime ? "sum(".$this->connection->addFieldWrappers($fName).")" : "sum(".$this->connection->timeToSecWrapper($fName).")";
				$avg_func = ", " . $sum_for_avg . " as ".$this->connection->addFieldWrappers("avg_sum")
					.", count(".$this->connection->addFieldWrappers($fName).") as ".$this->connection->addFieldWrappers("avg_count");
			}
			else
				$avg_func = ", 1 as ".$this->connection->addFieldWrappers("avg_sum").", 1 as ".$this->connection->addFieldWrappers("avg_count");	
		}
		
		$whereClause = "";
		if( tableEventExists("BeforeQueryReport", $strTableName) ) 
		{
			$eventObj = getEventObject($strTableName);
			$eventObj->BeforeQueryReport($whereClause);
			if( $whereClause )
				$whereClause = " where ".$whereClause;
		}
		if( $this->fromWizard ) {
			$gx0 = $group_x[0];
			$gx1 = $group_x[1];
			$gy0 = $group_y[0];
			$gy1 = $group_y[1];
		} else {
			$gx0 = $this->getDBFieldName($group_x[0]);
			$gx1 = $this->getDBFieldName($group_x[1]);
			$gy0 = $this->getDBFieldName($group_y[0]);
			$gy1 = $this->getDBFieldName($group_y[1]);
		}
		$selectClause = "select ".$select_field.$gy0.", ".$gx0. $avg_func;
		$groupByClause = "group by ".$gx1.", ".$gy1;
		$orderByClause = "order by ".$gx1.",".$gy1;
		
		if( $this->connection->dbType != nDATABASE_Oracle )
		{
			if( $this->connection->dbType == nDATABASE_MSSQLServer )
			{
				$pos = strrpos(strtoupper($strSQL), "ORDER BY");
				if( $pos )
					$strSQL = substr($strSQL, 0, $pos);
			}
			return $selectClause." from (".$strSQL.") as cross_table".$whereClause." ".$groupByClause." ".$orderByClause;
		}

		return $selectClause." from (".$strSQL.")".$whereClause." ".$groupByClause." ".$orderByClause;
	}
	
	/**
	 * @param Number index
	 * @return Array
	 */
	protected function getIntervalType($index)
	{
		$field = $this->xml_array["group_fields"][$index]["name"];
		$ftype = $this->getFieldType($field);
		
		$arr = $this->xml_array["group_fields"];
		for($i = 0; $i < count($arr) - 1; $i++)
		{
			if($field == $arr[$i]["name"] && $index == $i)
			{
				$int_type = $arr[$i]["int_type"];
				break;
			}
		}
		
		if( $int_type == 0 ) 
		{
			$wrappedGoodFieldName = $this->connection->addFieldWrappers($this->CrossGoodFieldName($field));
			return array( $wrappedGoodFieldName, $wrappedGoodFieldName );
		}
		
		if( IsNumberType($ftype) )
			return $this->getNumberTypeInterval($field, $int_type);
		
		if( IsCharType( $ftype ) )
			return $this->getCharTypeInterval($field, $int_type);
		
		if( IsDateFieldType( $ftype ) )
			return $this->getDateTypeInterval($field, $int_type);
	}
	
	protected function getDateTypeInterval($field, $int_type)
	{
		$field = $this->connection->addFieldWrappers( $this->CrossGoodFieldName($field) );
		switch( $this->connection->dbType )
		{
			case nDATABASE_MySQL:
				if($int_type == 1) // DATE_INTERVAL_YEAR
					return array("year(".$field.")*10000+0101","YEAR(".$field.")");
				elseif($int_type == 2) // DATE_INTERVAL_QUARTER
					return array("year(".$field.")*10000+QUARTER(".$field.")*100+1","year(".$field."),QUARTER(".$field.")");
				elseif($int_type == 3) // DATE_INTERVAL_MONTH
					return array("year(".$field.")*10000+month(".$field.")*100+1","year(".$field."),month(".$field.")");
				elseif($int_type == 4) // DATE_INTERVAL_WEEK
					return array("year(".$field.")*10000+week(".$field.")*100+01","year(".$field."),WEEK(".$field.")");
				elseif($int_type == 5) // DATE_INTERVAL_DAY
					return array("year(".$field.")*10000+month(".$field.")*100+day(".$field.")","year(".$field."),month(".$field."),day(".$field.")");
				elseif($int_type == 6) // DATE_INTERVAL_HOUR
					return array("year(".$field.")*1000000+month(".$field.")*10000+day(".$field.")*100+HOUR(".$field.")","year(".$field."),month(".$field."),day(".$field."),hour(".$field.")");
				elseif($int_type == 7) // DATE_INTERVAL_MINUTE
					return array("year(".$field.")*1000000+month(".$field.")*1000000+day(".$field.")*10000+HOUR(".$field.")*100+minute(".$field.")","year(".$field."),month(".$field."),day(".$field."),hour(".$field."),minute(".$field.")");
				break;

			case nDATABASE_Oracle:
				if($int_type == 1) // DATE_INTERVAL_YEAR
					return array("TO_CHAR(".$field.", 'YYYY')*10000+0101","TO_CHAR(".$field.", 'YYYY')");
				elseif($int_type == 2) // DATE_INTERVAL_QUARTER
					return array("TO_CHAR(".$field.", 'YYYY')*10000+TO_CHAR(".$field.",'Q')*100+1","TO_CHAR(".$field.", 'YYYY'),TO_CHAR(".$field.",'Q')");
				elseif($int_type == 3) // DATE_INTERVAL_MONTH
					return array("TO_CHAR(".$field.", 'YYYY')*10000+TO_CHAR(".$field.".'MM')*100+1","TO_CHAR(".$field.", 'YYYY'),TO_CHAR(".$field.".'MM')");
				elseif($int_type == 4) // DATE_INTERVAL_WEEK
					return array("TO_CHAR(".$field.", 'YYYY')*10000+TO_CHAR(".$field.",'W')*100+01","TO_CHAR(".$field.", 'YYYY'),TO_CHAR(".$field.",'W')");
				elseif($int_type == 5) // DATE_INTERVAL_DAY
					return array("TO_CHAR(".$field.", 'YYYY')*10000+TO_CHAR(".$field.",'MM')*100+TO_CHAR(".$field.",'DD')","TO_CHAR(".$field.", 'YYYY'),TO_CHAR(".$field.",'MM'),TO_CHAR(".$field.",'DD')");
				elseif($int_type == 6) // DATE_INTERVAL_HOUR
					return array("TO_CHAR(".$field.", 'YYYY')*1000000+TO_CHAR(".$field.",'MM')*10000+TO_CHAR(".$field.",'DD')*100+TO_CHAR(".$field.",'HH')","TO_CHAR(".$field.", 'YYYY'),TO_CHAR(".$field.",'MM'),TO_CHAR(".$field.",'DD'),TO_CHAR(".$field.",'HH')");
				elseif($int_type == 7) // DATE_INTERVAL_MINUTE
					return array("TO_CHAR(".$field.", 'YYYY')*1000000+TO_CHAR(".$field.",'MM')*1000000+TO_CHAR(".$field.",'DD')*10000+TO_CHAR(".$field.",'HH')*100+TO_CHAR(".$field.",'MI')","TO_CHAR(".$field.", 'YYYY'),TO_CHAR(".$field.",'MM'),TO_CHAR(".$field.",'DD'),TO_CHAR(".$field.",'HH'),TO_CHAR(".$field.",'MI')");
				break;

			case nDATABASE_MSSQLServer:
				if($int_type == 1) // DATE_INTERVAL_YEAR
					return array("datepart(yyyy,".$field.")*10000+0101","datepart(yyyy,".$field.")");
				elseif($int_type == 2) // DATE_INTERVAL_QUARTER
					return array("datepart(yyyy,".$field.")*10000+datepart(qq,".$field.")*100+1","datepart(yyyy,".$field."),datepart(qq,".$field.")");
				elseif($int_type == 3) // DATE_INTERVAL_MONTH
					return array("datepart(yyyy,".$field.")*10000+datepart(mm,".$field.")*100+1","datepart(yyyy,".$field."),datepart(mm,".$field.")");
				elseif($int_type == 4) // DATE_INTERVAL_WEEK
					return array("datepart(yyyy,".$field.")*10000+(datepart(ww,".$field.")-1)*100+01","datepart(yyyy,".$field."),datepart(ww,".$field.")");
				elseif($int_type == 5) // DATE_INTERVAL_DAY
					return array("datepart(yyyy,".$field.")*10000+datepart(mm,".$field.")*100+datepart(dd,".$field.")","datepart(yyyy,".$field."),datepart(mm,".$field."),datepart(dd,".$field.")");
				elseif($int_type == 6) // DATE_INTERVAL_HOUR
					return array("datepart(yyyy,".$field.")*1000000+datepart(mm,".$field.")*10000+datepart(dd,".$field.")*100+datepart(hh,".$field.")","datepart(yyyy,".$field."),datepart(mm,".$field."),datepart(dd,".$field."),datepart(hh,".$field.")");
				elseif($int_type == 7) // DATE_INTERVAL_MINUTE
					return array("datepart(yyyy,".$field.")*1000000+datepart(mm,".$field.")*1000000+datepart(dd,".$field.")*10000+datepart(hh,".$field.")*100+datepart(mi,".$field.")","datepart(yyyy,".$field."),datepart(mm,".$field."),datepart(dd,".$field."),datepart(hh,".$field."),datepart(mi,".$field.")");
				break;

			case nDATABASE_Access:
				if($int_type == 1) // DATE_INTERVAL_YEAR
					return array("datepart('yyyy',".$field.")*10000+0101","datepart('yyyy',".$field.")");
				elseif($int_type == 2) // DATE_INTERVAL_QUARTER
					return array("datepart('yyyy',".$field.")*10000+datepart('q',".$field.")*100+1","datepart('yyyy',".$field."),datepart('q',".$field.")");
				elseif($int_type == 3) // DATE_INTERVAL_MONTH
					return array("datepart('yyyy',".$field.")*10000+datepart('m',".$field.")*100+1","datepart('yyyy',".$field."),datepart('m',".$field.")");
				elseif($int_type == 4) // DATE_INTERVAL_WEEK
					return array("datepart('yyyy',".$field.")*10000+(datepart('ww',".$field.")-1)*100+01","datepart('yyyy',".$field."),datepart('ww',".$field.")");
				elseif($int_type == 5) // DATE_INTERVAL_DAY
					return array("datepart('yyyy',".$field.")*10000+datepart('m',".$field.")*100+datepart('d',".$field.")","datepart('yyyy',".$field."),datepart('m',".$field."),datepart('d',".$field.")");
				elseif($int_type == 6) // DATE_INTERVAL_HOUR
					return array("datepart('yyyy',".$field.")*1000000+datepart('m',".$field.")*10000+datepart('d',".$field.")*100+datepart('h',".$field.")","datepart('yyyy',".$field."),datepart('m',".$field."),datepart('d',".$field."),datepart('h',".$field.")");
				elseif($int_type == 7) // DATE_INTERVAL_MINUTE
					return array("datepart('yyyy',".$field.")*1000000+datepart('m',".$field.")*1000000+datepart('d',".$field.")*10000+datepart('h',".$field.")*100+datepart('n',".$field.")","datepart('yyyy',".$field."),datepart('m',".$field."),datepart('d',".$field."),datepart('h',".$field."),datepart('n',".$field.")");
				break;

			case nDATABASE_PostgreSQL: 
				if($int_type == 1) // DATE_INTERVAL_YEAR
					return array("date_part('year',".$field.")*10000+0101","date_part('year',".$field.")");
				elseif($int_type == 2) // DATE_INTERVAL_QUARTER
					return array("date_part('year',".$field.")*10000+date_part('quarter',".$field.")*100+1","date_part('year',".$field."),date_part('quarter',".$field.")");
				elseif($int_type == 3) // DATE_INTERVAL_MONTH
					return array("date_part('year',".$field.")*10000+date_part('month',".$field.")*100+1","date_part('year',".$field."),date_part('month',".$field.")");
				elseif($int_type == 4) // DATE_INTERVAL_WEEK
					return array("date_part('year',".$field.")*10000+(date_part('week',".$field.")-1)*100+01","date_part('year',".$field."),date_part('week',".$field.")");
				elseif($int_type == 5) // DATE_INTERVAL_DAY
					return array("date_part('year',".$field.")*10000+date_part('month',".$field.")*100+date_part('days',".$field.")","date_part('year',".$field."),date_part('month',".$field."),date_part('days',".$field.")");
				elseif($int_type == 6) // DATE_INTERVAL_HOUR
					return array("date_part('year',".$field.")*1000000+date_part('month',".$field.")*10000+date_part('days',".$field.")*100+date_part('hour',".$field.")","date_part('year',".$field."),date_part('month',".$field."),date_part('days',".$field."),date_part('hour',".$field.")");
				elseif($int_type == 7) // DATE_INTERVAL_MINUTE
					return array("date_part('year',".$field.")*1000000+date_part('month',".$field.")*1000000+date_part('days',".$field.")*10000+date_part('hour',".$field.")*100+date_part('minute',".$field.")","date_part('year',".$field."),date_part('month',".$field."),date_part('days',".$field."),date_part('hour',".$field."),date_part('minute',".$field.")");
				break;

			case nDATABASE_Informix:
				return "substring(".$field." from 1 for ".$int_type.")"; //fix it!

			case nDATABASE_SQLite3:
				return array($field, $field);
				
			case nDATABASE_DB2: 
				if($int_type == 1) // DATE_INTERVAL_YEAR
					return array("year(".$field.")*10000+0101","YEAR(".$field.")");
				elseif($int_type == 2) // DATE_INTERVAL_QUARTER
					return array("year(".$field.")*10000+QUARTER(".$field.")*100+1","year(".$field."),QUARTER(".$field.")");
				elseif($int_type == 3) // DATE_INTERVAL_MONTH
					return array("year(".$field.")*10000+month(".$field.")*100+1","year(".$field."),month(".$field.")");
				elseif($int_type == 4) // DATE_INTERVAL_WEEK
					return array("year(".$field.")*10000+week(".$field.")*100+01","year(".$field."),WEEK(".$field.")");
				elseif($int_type == 5) // DATE_INTERVAL_DAY
					return array("year(".$field.")*10000+month(".$field.")*100+day(".$field.")","year(".$field."),month(".$field."),day(".$field.")");
				elseif($int_type == 6) // DATE_INTERVAL_HOUR
					return array("year(".$field.")*1000000+month(".$field.")*10000+day(".$field.")*100+HOUR(".$field.")","year(".$field."),month(".$field."),day(".$field."),hour(".$field.")");
				elseif($int_type == 7) // DATE_INTERVAL_MINUTE
					return array("year(".$field.")*1000000+month(".$field.")*1000000+day(".$field.")*10000+HOUR(".$field.")*100+minute(".$field.")","year(".$field."),month(".$field."),day(".$field."),hour(".$field."),minute(".$field.")");
				break;
		}
	}
	
	protected function getNumberTypeInterval($field, $int_type)
	{
		return array("floor(".$this->connection->addFieldWrappers( $this->CrossGoodFieldName($field) )."/".$int_type.")*".$int_type,
			 "floor(".$this->connection->addFieldWrappers( $this->CrossGoodFieldName($field) )."/".$int_type.")*".$int_type);
	}
	
	/**
	 * @param String field
	 * @param Number int_type
	 * @return Array
	 */
	protected function getCharTypeInterval($field, $int_type)
	{
		$field = $this->connection->addFieldWrappers( $this->CrossGoodFieldName($field) );
		switch( $this->connection->dbType )
		{
			case nDATABASE_MySQL:
			case nDATABASE_MSSQLServer:
			case nDATABASE_Access:
				return array("left(".$field.",".$int_type.")","left(".$field.",".$int_type.")");

			case nDATABASE_PostgreSQL:
			case nDATABASE_Informix:
				return array("substring(".$field." from 1 for ".$int_type.")","substring(".$field." from 1 for ".$int_type.")");

			case nDATABASE_Oracle:
			case nDATABASE_SQLite3:
			case nDATABASE_DB2:
				return array("substr(".$field.",1,".$int_type.")","substr(".$field.",1,".$int_type.")");
		}
	}
	
	/**
	 * @return Array
	 */
	public function getSelectedValue()
	{
		$arr = array();
		$firstarr = array();
		foreach($this->xml_array["totals"] as $key => $value)
		{
			if(count($firstarr) == 0)
				$firstarr[] = $this->FullFieldName($value["name"],$value["table"]);
			if($value["min"] == true || $value["max"] == true || $value["sum"] == true || $value["avg"] == true)
			{
				$arr[] = $this->FullFieldName($value["name"],$value["table"]);
			}
		}
		if(count($arr) == 0)
			$arr = $firstarr;
		return $arr;
	}
	
	/**
	 * 
	 * @param {String} hostPageLocation	(optional)
	 * @param {String} hostPageId (optional)
	 */
	public function getRadioGroupFunctions( $hostPageLocation = "", $hostPageId = "" )
	{
		$arr = array();
		$arrDisplay = array();
		$res = "";
		if($this->dataFieldSettings["sum"] == true)
		{
			$arrDisplay[] = "Sum";
			$arr[] = "sum";
		}
		if($this->dataFieldSettings["max"] == true)
		{
			$arrDisplay[] = "Max";
			$arr[] = "max";
		}
		if($this->dataFieldSettings["min"] == true)
		{
			$arrDisplay[] = "Min";
			$arr[] = "min";
		}
		if($this->dataFieldSettings["avg"] == true)
		{
			$arrDisplay[] = "Average";
			$arr[] = "avg";
		}

		if(!count($arr))
		{
				$arr[] = "sum";
				$arrDisplay[] = "Sum";
		}
		
		$res = "";	
		$onclick = "onclick='refresh_crosstable(\"".$hostPageLocation."\", \"".$hostPageId."\", \"".$this->dashBased."\", \"".$this->dashTName."\");'";
		for($j = 0; $j < count($arr); $j++)
		{
			$s = "";
			if($res == "" || $this->dataGroupFunction == $arr[$j])
				$s = "checked";
				
			$res.= "<input type=radio value='".$arr[$j]."' name=\"group_func".$hostPageId."\" ".$s." ".$onclick."> "
				.$arrDisplay[$j]."&nbsp;&nbsp;";
		}
		return $res;
	}
	
	public function ajax_refresh_crosstable( $hostPageLocation = "", $hostPageId = "" )
	{
		$reportData = array(
			$this->rowinfo,
			$this->col_summary,
			$this->total_summary,
			$this->getTotalsName( $this->dataGroupFunction ),
			$this->getRadioGroupFunctions( $hostPageLocation, $hostPageId )
		);
	
		echo my_json_encode( $reportData );
	}
	
	public function getGroupFields($axis)
	{
		$res = "";
		$label = $this->xml_array["totals"];
		$arr = $this->xml_array["group_fields"];
		for($i = 0; $i < count($arr) - 1; $i++)
		{
			$s = "";
			if($axis == "x" && $arr[$i]["group_type"] == "x" || $axis == "y" && $arr[$i]["group_type"] == "y" || $arr[$i]["group_type"] == "all")
			{
				if($axis == "x" && $this->index_field_y != $i || $axis == "y" && $this->index_field_x != $i)
				{
					if($this->index_field_x == $i && $axis == "x" || $this->index_field_y == $i && $axis == "y")
						$s = "selected";
					$strlabel = "";
					foreach($label as $val)
					{
						if($arr[$i]["name"] == $this->FullFieldName($val["name"],$val["table"]))
						{
							$strlabel = $val["label"];
							break;
						}
					}
					$res.= "<option value='".$i."' ".$s.">".$strlabel."</option>";
				}
			}
		}
		return $res;
	}
	
	protected function getFirstGroupField($axis)
	{
		$arr = $this->xml_array["group_fields"];
		$arrX = array();
		$arrY = array();
		$arrAll = array();
		
		for($i = 0; $i < count($arr) - 1; $i++)
		{
			if($arr[$i]["group_type"] == "x")
				$arrX[] = $i;
			if($arr[$i]["group_type"] == "y")
				$arrY[] = $i;
			if($arr[$i]["group_type"] == "all")
				$arrAll[] = $i;
		}	
		if(count($arrX) > 0 && $axis == "x")
			return $arrX[0];
		if(count($arrY) > 0 && $axis == "y")
			return $arrY[0];
		if(count($arrX) == 0 && $axis == "x")
			return $arrAll[0];
		if(count($arrY) == 0 && $axis == "y")
		{
			if(count($arrX) == 0)
				return $arrAll[1];
			else
				return $arrAll[0];
		}
	}
	
	/**
	 * Get axes displyed values
	 * @param Number index
	 * @param String value
	 * @return String
	 */
	protected function getAxisDisplayValue($index, $value)
	{
		global $locale_info;
		
		if( $value == "" || is_null($value) )
			return "";
				
		$groupFieldsData = $this->xml_array["group_fields"];
		$field = $groupFieldsData[ $index ]["name"];
		$int_type = $groupFieldsData[ $index ]["int_type"];
		if( $this->fromWizard )
			$control = $this->viewControls->getControl( $field );
		
		if ( $int_type == 0 ) 
		{	
			// The 'Normal' interval is set
			if( $this->fromWizard )
			{
				$data = array( $field => $value );
				return $control->showDBValue( $data, "" );		
			}
			
			if ( $this->table_type != "db" )
				$fieldIdentifier = $this->xml_array["tables"][0]."_".$field;
			else
				$fieldIdentifier = $this->CrossGoodFieldName($field);
			
			if( $this->xml_array['totals'][ $fieldIdentifier ]['curr'] == true )
				return str_format_currency($value);
			
			return xmlencode($value);
		}
		
		$ftype = $this->getFieldType($field);		
		
		if ( IsNumberType( $ftype ) ) 
		{
			$start = $value - ($value % $int_type);
			$end = $start + $int_type;
			
			if( $this->fromWizard ) 
			{
				$dataStart = array( $field => $start );
				$dataEnd = array( $field => $end );
				return $control->showDBValue( $dataStart, "" )." - ".$control->showDBValue( $dataEnd, "");
			}

			if( $this->table_type != "db" )
				$fieldIdentifier = $this->xml_array["tables"][0]."_".$field;
			else
				$fieldIdentifier = $this->CrossGoodFieldName($field);
			
			if( $this->xml_array['totals'][ $fieldIdentifier ]['curr'] == true )
				return str_format_currency($start)." - ".str_format_currency($end);
			
			return $start." - ".$end;
		} 
		
		if ( IsCharType( $ftype ) ) 
		{
			return xmlencode(substr($value,0,$int_type));
		} 
		
		if ( IsDateFieldType( $ftype ) ) 
		{
			$dvalue = substr($value, 0, 4).'-'.substr($value, 4, 2).'-'.substr($value, 6, 2);
			
			if( strlen($value) == 10 )
				$dvalue.=" ".substr($value, 8, 2)."00:00";
			elseif( strlen($value) == 12 )
				$dvalue.=" ".substr($value, 8, 2).":".substr($value, 10, 2).":00";
			
			$tm = db2time($dvalue);
			if( !count($tm) )
				return "";
		
			switch( $int_type )
			{
				case 1: // DATE_INTERVAL_YEAR
					return $tm[0];
				case 2: // DATE_INTERVAL_QUARTER
					return $tm[0]."/Q".$tm[1];
				case 3: // DATE_INTERVAL_MONTH
					return @$locale_info[ "LOCALE_SABBREVMONTHNAME".$tm[1] ]." ".$tm[0];
				case 4: // DATE_INTERVAL_WEEK
					$dates = $this->getDatesByWeek($tm[1] + 1, $tm[0]);
					return format_shortdate( db2time($dates[0]) ) . ' - ' . format_shortdate( db2time($dates[1]) );				
				case 5: // DATE_INTERVAL_DAY
					return format_shortdate($tm);
				case 6: // DATE_INTERVAL_HOUR
					$tm[4] = 0;
					$tm[5] = 0;
					return str_format_datetime($tm);
				case 7: // DATE_INTERVAL_MINUTE
					$tm[5] = 0;
					return str_format_datetime($tm);
				default:
					return str_format_datetime($tm);
			}
		}
		
		return "";
	}
	
	protected function getDatesByWeek($week, $year) 
	{
		global $locale_info;
		$startweekday = 0;
		if($locale_info["LOCALE_IFIRSTDAYOFWEEK"]>0)
			$startweekday = 7 - $locale_info["LOCALE_IFIRSTDAYOFWEEK"];	

		$L = isleapyear($year) ? 1 : 0;
		$months = array(31, 28 + $L, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
		$total_days = ($week - 1) * 7; 
		$i = 0;
		$sum = 0;
		while($sum <= $total_days)
		{
			$sum += $months[$i++];
		}
		$sum -= $months[$i-1];
		$month = $i;
		$day = $total_days - $sum;
		$day_of_week = getdayofweek(array($year, $month, $day));
		if ($day_of_week == 0) 
			$day_of_week = 7;
		
		$day = $day - ($day_of_week - 1) - $startweekday;
		$dates = array();
		$dates[0] = getYMDdate(mktime(0,0,0, $month, $day, $year));
		$dates[1] = getYMDdate(mktime(1,1,1, $month, $day+6, $year));
		return $dates;
	}
	
	public function getValuesControl()
	{
		$arr_list = $this->getSelectedValue();
		$arr_label = $this->xml_array["totals"];
		$res = "";
		$first_field = 0;
		$i = 0;
		if(count($arr_list) != 0 )
		{
			foreach($arr_list as $value)
			{
				$s = "";
				if($i == 0 || $i == $_SESSION[$this->sessionPrefix."_field"])
				{
					$first_field = $i;
					$s = "selected";
				}
				$strlabel = "";
				foreach($arr_label as $val)
					if($value == $this->FullFieldName($val["name"],$val["table"]))
					{
						$strlabel = $val["label"];
						break;
					}
				$res.= "<option value=".$i." ".$s.">".runner_htmlspecialchars($strlabel)."</option>";
				$i++;
			}
		}
		return array($res,$first_field);
	}
	
	protected function FullFieldName($field, $table = "")
	{
		if(!$table)
			$table = $this->tableName;
		if($this->table_type == "db")
			if(strpos($field,".") === false)
				$res = $table.".".$field;
			else
				$res = $field;
		else
			$res = $field;
		return $res;
	}
	
	protected function CrossGoodFieldName($field)
	{
		if($this->table_type == "db")
			return GoodFieldName($field);
		else 
			return $field;
	}
	
	/**
	 * Get the report print header html-markup
	 * @return String
	 */
	public function getPrintCrossHeader()
	{					
		$fieldNameX = $this->xml_array["group_fields"][ $this->index_field_x ]["name"];
		$fieldNameY = $this->xml_array["group_fields"][ $this->index_field_y ]["name"];
		$fieldName = $this->dataField;
		
		if(!$this->fromWizard)
		{
			$prefix = "";
			if( $this->table_type != "db" )
				$prefix = $this->xml_array["tables"][0]."_";			
			
			$fieldNameX = $prefix.GoodFieldName($fieldNameX);
			$fieldNameY = $prefix.GoodFieldName($fieldNameY);
			$fieldName = $prefix.GoodFieldName($fieldName);
		}
		
		return "Group X"
			.":<b>".$this->xml_array["totals"][ $fieldNameX ]["label"]."</b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"."Group Y"
			.":<b>".$this->xml_array["totals"][ $fieldNameY ]["label"]."</b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"."Field"
			.":<b>".$this->xml_array["totals"][ $fieldName ]["label"]."</b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"."Group function"
			.":<b>".$this->dataGroupFunction."</b>";
	}
	
	public function getTotalsName($grfunc)
	{
		switch($grfunc)
		{
			case "sum":
				return "Sum";
				break;
			case "min":
				return "Min";
				break;
			case "max":
				return "Max";
				break;
			case "avg":
				return "Average";
				break;
		}
	}
	
	protected function getFieldType($field)
	{
		if($this->table_type == "db")
			$ftype = WRGetFieldType($this->FullFieldName($field));
		elseif($this->table_type == "project")
			$ftype = $this->pSet->getFieldType($field);
		else
		{ 
			$fields_type = WRGetAllCustomFieldType();
			$ftype = $fields_type[$field];
		}
		return $ftype;
	}
	
	protected function getDataField($idx)
	{
		$idx = 0+$idx;
		$arr_value = $this->getSelectedValue();	
		if($idx >= count($arr_value))
			return "";
		return $arr_value[$idx];
	}
	
	protected function initDataFieldSettings()
	{
		foreach($this->xml_array["totals"] as $key => $value)
		{
			if($this->FullFieldName($value["name"],$value["table"]) == $this->dataField)
			{
				$this->dataFieldSettings = $this->xml_array["totals"][$key];
				break; 
			}
		}
	}

	/**
	 * Checks whether passed function name is valid
	 * @param String func
	 */
	protected function initDataGroupFunction($func)
	{
		//	good
		if( $this->dataFieldSettings[$func] == true )
		{
			$this->dataGroupFunction = $func;
			return;
		}

		//	bad, set first possible
		$gfuncs = array("sum", "max", "min", "avg");
		foreach($gfuncs as $gf)
		{
			if( $this->dataFieldSettings[$gf] == true )
			{
				$this->dataGroupFunction = $gf;
				return;
			}
		}
		
		// default
		$this->dataGroupFunction = "sum";
	}
	
	/**
	 * Get the current group function name
	 * @return String
	 */
	public function getCurrentGroupFunction()
	{
		return $this->dataGroupFunction;
	}
	public function isProjectDB()
	{
		if($this->fromWizard)
			return true;
			
		$isDB = false;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("item" == $this->xml_array['tables'][0])
			$isDB = true;
		if("credit_terms" == $this->xml_array['tables'][0])
			$isDB = true;
		if("shipping_terms" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("general_journal" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("system_user" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fiscal_year" == $this->xml_array['tables'][0])
			$isDB = true;
		if("beg_bal" == $this->xml_array['tables'][0])
			$isDB = true;
		if("ewt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bank" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cost_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("accounting_period" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bank" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit" == $this->xml_array['tables'][0])
			$isDB = true;
		if("account_classification" == $this->xml_array['tables'][0])
			$isDB = true;
		if("chart_of_accounts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("general_journal_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("general_journal_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("vat_atc" == $this->xml_array['tables'][0])
			$isDB = true;
		if("rdo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("2550q" == $this->xml_array['tables'][0])
			$isDB = true;
		if("2550m" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("company_info" == $this->xml_array['tables'][0])
			$isDB = true;
		if("ppe_list" == $this->xml_array['tables'][0])
			$isDB = true;
		if("discounts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("item_group" == $this->xml_array['tables'][0])
			$isDB = true;
		if("location" == $this->xml_array['tables'][0])
			$isDB = true;
		if("meetings" == $this->xml_array['tables'][0])
			$isDB = true;
		if("calls" == $this->xml_array['tables'][0])
			$isDB = true;
		if("campaign" == $this->xml_array['tables'][0])
			$isDB = true;
		if("reimbursement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("reimbursement_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quotation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case" == $this->xml_array['tables'][0])
			$isDB = true;
		if("projects" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("backload" == $this->xml_array['tables'][0])
			$isDB = true;
		if("backload_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("vehicle" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_item" == $this->xml_array['tables'][0])
			$isDB = true;
		if("uom" == $this->xml_array['tables'][0])
			$isDB = true;
		if("trip_ticket" == $this->xml_array['tables'][0])
			$isDB = true;
		if("accident" == $this->xml_array['tables'][0])
			$isDB = true;
		if("approver_list" == $this->xml_array['tables'][0])
			$isDB = true;
		if("area" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance" == $this->xml_array['tables'][0])
			$isDB = true;
		if("currency" == $this->xml_array['tables'][0])
			$isDB = true;
		if("medical" == $this->xml_array['tables'][0])
			$isDB = true;
		if("position" == $this->xml_array['tables'][0])
			$isDB = true;
		if("service_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("seminar" == $this->xml_array['tables'][0])
			$isDB = true;
		if("service_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("service_report_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("tax_status" == $this->xml_array['tables'][0])
			$isDB = true;
		if("training" == $this->xml_array['tables'][0])
			$isDB = true;
		if("competitor" == $this->xml_array['tables'][0])
			$isDB = true;
		if("campaign_type" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_asset_return_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_assets_return" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("trip_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("beg_baldetail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursement_detail2" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case_solutions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt_detail2" == $this->xml_array['tables'][0])
			$isDB = true;
		if("general_journal_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("petty_cash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("petty_cash_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("center_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("center_replenishment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("credit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("credit_memo_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("debit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("debit_memo_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_replenishment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner_group" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_item" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_mode" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_account_type" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_item" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_item" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("company_info" == $this->xml_array['tables'][0])
			$isDB = true;
		if("item_bill_of_materials" == $this->xml_array['tables'][0])
			$isDB = true;
		if("item_uom_conversion" == $this->xml_array['tables'][0])
			$isDB = true;
		if("tranasactions_for_approval" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quotation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quotation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quotation_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quotation_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("reimbursement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("reimbursement_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("reimbursement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("reimbursement_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_replenishment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_replenishment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("center_replenishment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("center_replenishment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("return" == $this->xml_array['tables'][0])
			$isDB = true;
		if("return_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("return" == $this->xml_array['tables'][0])
			$isDB = true;
		if("return_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_asset_issue" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_asset_issue" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_asset_issue_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_asset_issue_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_assets_return" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_asset_return_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_assets_return" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_asset_return_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("petty_cash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("petty_cash_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("petty_cash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("petty_cash_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("credit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("credit_memo_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("credit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("credit_memo_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("debit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("debit_memo_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("debit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("debit_memo_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursement_detail2" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursement_detail2" == $this->xml_array['tables'][0])
			$isDB = true;
		if("document" == $this->xml_array['tables'][0])
			$isDB = true;
		if("general_journal" == $this->xml_array['tables'][0])
			$isDB = true;
		if("general_journal_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("general_journal" == $this->xml_array['tables'][0])
			$isDB = true;
		if("general_journal_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("for_submissions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("submission_details" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order_detail_billing_dates" == $this->xml_array['tables'][0])
			$isDB = true;
		if("for_submissions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("for_submissions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("years" == $this->xml_array['tables'][0])
			$isDB = true;
		if("document_type" == $this->xml_array['tables'][0])
			$isDB = true;
		if("estimate" == $this->xml_array['tables'][0])
			$isDB = true;
		if("estimate_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_adj_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_adj" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_tranfer" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_transfer_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner_contacts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner_contacts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner_contacts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner_contacts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("change_time" == $this->xml_array['tables'][0])
			$isDB = true;
		if("change_time" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_adj" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_adj_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_adj" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_adj_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("approval_alerts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("approval_alerts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("estimate" == $this->xml_array['tables'][0])
			$isDB = true;
		if("estimate" == $this->xml_array['tables'][0])
			$isDB = true;
		if("estimate_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("estimate_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("meetings" == $this->xml_array['tables'][0])
			$isDB = true;
		if("tasks" == $this->xml_array['tables'][0])
			$isDB = true;
		if("tasks" == $this->xml_array['tables'][0])
			$isDB = true;
		if("origin_fa" == $this->xml_array['tables'][0])
			$isDB = true;
		if("origin_serialized_supply" == $this->xml_array['tables'][0])
			$isDB = true;
		if("employees" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_aluminum" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_e_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_glass" == $this->xml_array['tables'][0])
			$isDB = true;
		if("recuring_benefits" == $this->xml_array['tables'][0])
			$isDB = true;
		if("recuring_deductions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("recurring_benefits_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("recurring_deductions_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_request_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance_summary" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_item" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_item" == $this->xml_array['tables'][0])
			$isDB = true;
		if("department" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_cash_disbursement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_center_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_credit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_debit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_depreciation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_general_journal" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_inventory_adjustment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_petty_cash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_project_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_purchases" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_service_revenue" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_report_per_item" == $this->xml_array['tables'][0])
			$isDB = true;
		if("leave" == $this->xml_array['tables'][0])
			$isDB = true;
		if("loan" == $this->xml_array['tables'][0])
			$isDB = true;
		if("loan__detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("over_time" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_request_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_tranfer" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_transfer_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("leave" == $this->xml_array['tables'][0])
			$isDB = true;
		if("loan" == $this->xml_array['tables'][0])
			$isDB = true;
		if("loan__detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("over_time" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_tranfer" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_transfer_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance_type" == $this->xml_array['tables'][0])
			$isDB = true;
		if("backload" == $this->xml_array['tables'][0])
			$isDB = true;
		if("backload_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("backload" == $this->xml_array['tables'][0])
			$isDB = true;
		if("backload_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("process_payroll" == $this->xml_array['tables'][0])
			$isDB = true;
		if("process_payroll" == $this->xml_array['tables'][0])
			$isDB = true;
		if("process_payroll_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("process_payroll_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("service_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("service_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("service_report_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("service_report_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("report_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_production" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_deposit" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_request_bom" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_bom" == $this->xml_array['tables'][0])
			$isDB = true;
		if("company_info" == $this->xml_array['tables'][0])
			$isDB = true;
		if("return" == $this->xml_array['tables'][0])
			$isDB = true;
		if("return_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_asset_issue" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fixed_asset_issue_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_request_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("trip_ticket" == $this->xml_array['tables'][0])
			$isDB = true;
		if("trip_ticket" == $this->xml_array['tables'][0])
			$isDB = true;
		if("trip_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("trip_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_cashreceipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_cashreceipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ar" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ar" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ap" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ap" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("pdc_registry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case" == $this->xml_array['tables'][0])
			$isDB = true;
		if("campaign" == $this->xml_array['tables'][0])
			$isDB = true;
		if("campaign" == $this->xml_array['tables'][0])
			$isDB = true;
		if("campaign" == $this->xml_array['tables'][0])
			$isDB = true;
		if("campaign" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("leave" == $this->xml_array['tables'][0])
			$isDB = true;
		if("leave" == $this->xml_array['tables'][0])
			$isDB = true;
		if("process_payroll" == $this->xml_array['tables'][0])
			$isDB = true;
		if("leave" == $this->xml_array['tables'][0])
			$isDB = true;
		if("employees" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quality_control" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quality_control_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("budget" == $this->xml_array['tables'][0])
			$isDB = true;
		if("budget_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("repairs_and_maintenance" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_form" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bluewave_employees" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bluewave_employees_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bluewave_employees_detail2" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bluewave_employees_detail3" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bluewave_employees_detail4" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bluewave_employees_detail5" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bluewave_employees_detail6" == $this->xml_array['tables'][0])
			$isDB = true;
		if("rfp" == $this->xml_array['tables'][0])
			$isDB = true;
		if("telex" == $this->xml_array['tables'][0])
			$isDB = true;
		if("repairs_and_maintenance_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("repair_list" == $this->xml_array['tables'][0])
			$isDB = true;
		if("marketing_colateral" == $this->xml_array['tables'][0])
			$isDB = true;
		if("email" == $this->xml_array['tables'][0])
			$isDB = true;
		if("accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order_materials_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order_progress_billing_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sub_contract" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sub_contract_billing_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sub_contract_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payroll_entry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("vat_relief_importation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("vat_relief_purchases" == $this->xml_array['tables'][0])
			$isDB = true;
		if("vat_relief_sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_ppe_disposal" == $this->xml_array['tables'][0])
			$isDB = true;
		if("credit_memo_detail_cgs" == $this->xml_array['tables'][0])
			$isDB = true;
		if("marketing_request_form" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quotation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("backload" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("debit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sub_contract" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sub_contract" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sub_contract_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sub_contract_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("budget" == $this->xml_array['tables'][0])
			$isDB = true;
		if("budget" == $this->xml_array['tables'][0])
			$isDB = true;
		if("budget_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("budget_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("undertime" == $this->xml_array['tables'][0])
			$isDB = true;
		if("undertime" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_payment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_payment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("repairs_and_maintenance_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_payment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_payment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("repairs_and_maintenance" == $this->xml_array['tables'][0])
			$isDB = true;
		if("repairs_and_maintenance" == $this->xml_array['tables'][0])
			$isDB = true;
		if("repairs_and_maintenance_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed" == $this->xml_array['tables'][0])
			$isDB = true;
		if("campaign" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_transfer_cost_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transaction" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transaction" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("packing_list" == $this->xml_array['tables'][0])
			$isDB = true;
		if("trip_ticket" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_tranfer" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_disbursment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("meetings" == $this->xml_array['tables'][0])
			$isDB = true;
		if("meetings" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("calls" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("process_payroll" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_report_per_item" == $this->xml_array['tables'][0])
			$isDB = true;
		if("basic_system_audit" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sale_cost_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("currency_exchange_rate" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt_cost_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_cost_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quotation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("reimbursement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("loan" == $this->xml_array['tables'][0])
			$isDB = true;
		if("leave" == $this->xml_array['tables'][0])
			$isDB = true;
		if("over_time" == $this->xml_array['tables'][0])
			$isDB = true;
		if("undertime" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fieldwork" == $this->xml_array['tables'][0])
			$isDB = true;
		if("change_time" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_aluminum" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_e_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_glass" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sum_dr_qty" == $this->xml_array['tables'][0])
			$isDB = true;
		if("check_book_registry" == $this->xml_array['tables'][0])
			$isDB = true;
		if("check_book_registry_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("vat_type" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quotation_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_purchase_order_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_sales_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("pending_items_reminders" == $this->xml_array['tables'][0])
			$isDB = true;
		if("petty_cash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("payment_method" == $this->xml_array['tables'][0])
			$isDB = true;
		if("account_type" == $this->xml_array['tables'][0])
			$isDB = true;
		if("phase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("item_cost_recalculation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue_components" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue_components_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("unbuild_assemblies" == $this->xml_array['tables'][0])
			$isDB = true;
		if("unbuild_assemblies_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("demand_plan" == $this->xml_array['tables'][0])
			$isDB = true;
		if("input_vat_allocation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case_type" == $this->xml_array['tables'][0])
			$isDB = true;
		if("qc_inspection" == $this->xml_array['tables'][0])
			$isDB = true;
		if("packing_list" == $this->xml_array['tables'][0])
			$isDB = true;
		if("packing_list" == $this->xml_array['tables'][0])
			$isDB = true;
		if("packing_list_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("packing_list_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed_inv_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed_inv" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed_inv" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed_inv" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed_inv_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("disposed_inv_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("approval_alerts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("approval_alerts" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_payment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("credit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("debit_memo" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("packing_list" == $this->xml_array['tables'][0])
			$isDB = true;
		if("packing_list_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_quote" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_estimates" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_po" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_pr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_production_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_qc" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_request_for_payment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_so" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_so_for_invoice" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_so_for_project_management" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_backload" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_backload" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_estimates" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_po" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_pr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_production_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_qc" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_so_for_project_management" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_so_for_invoice" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_so" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_cost_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue_components" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue_components" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue_components_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("issue_components_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fieldwork" == $this->xml_array['tables'][0])
			$isDB = true;
		if("fieldwork" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_order_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_tranfer" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_inv_purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("center_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("center_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_inv_purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_inv_production_fg" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_production_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("unbuild_assemblies" == $this->xml_array['tables'][0])
			$isDB = true;
		if("unbuild_assemblies" == $this->xml_array['tables'][0])
			$isDB = true;
		if("unbuild_assemblies_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quality_control" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quality_control" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quality_control_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_cashreceipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_cashpayment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("opportunity" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_lyl_acc" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_lyl_glass" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_lyl_eacc" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_lyl_allum" == $this->xml_array['tables'][0])
			$isDB = true;
		if("opportunity" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("marketing_request_form" == $this->xml_array['tables'][0])
			$isDB = true;
		if("marketing_request_form" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("employees" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ar" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ar" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ar" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_payment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ar" == $this->xml_array['tables'][0])
			$isDB = true;
		if("loan" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ap" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production" == $this->xml_array['tables'][0])
			$isDB = true;
		if("aging_ap" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("trip_ticket" == $this->xml_array['tables'][0])
			$isDB = true;
		if("dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("campaign" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case" == $this->xml_array['tables'][0])
			$isDB = true;
		if("service_center" == $this->xml_array['tables'][0])
			$isDB = true;
		if("meetings" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("attendance_summary" == $this->xml_array['tables'][0])
			$isDB = true;
		if("employees" == $this->xml_array['tables'][0])
			$isDB = true;
		if("holiday" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transactions" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_quote" == $this->xml_array['tables'][0])
			$isDB = true;
		if("case" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("open_receiving" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production" == $this->xml_array['tables'][0])
			$isDB = true;
		if("" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inv_ledger" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inventory_tranfer" == $this->xml_array['tables'][0])
			$isDB = true;
		if("business_partner" == $this->xml_array['tables'][0])
			$isDB = true;
		if("opportunity" == $this->xml_array['tables'][0])
			$isDB = true;
		if("opportunity" == $this->xml_array['tables'][0])
			$isDB = true;
		if("intercompany_sales_order_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("intercompany_sales_order" == $this->xml_array['tables'][0])
			$isDB = true;
		if("production_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quality_control_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_aluminum" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_e_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_glass" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_aluminum" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_e_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_glass" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_aluminum" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_e_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_detail_glass" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_aluminum" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_e_accessories" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lyl_estimate_dr_detail_glass" == $this->xml_array['tables'][0])
			$isDB = true;
		if("cash_receipt" == $this->xml_array['tables'][0])
			$isDB = true;
		if("1601e" == $this->xml_array['tables'][0])
			$isDB = true;
		if("1601eform" == $this->xml_array['tables'][0])
			$isDB = true;
		if("1601fform" == $this->xml_array['tables'][0])
			$isDB = true;
		if("tax_person_status" == $this->xml_array['tables'][0])
			$isDB = true;
		if("journal_entry_inv_transfer" == $this->xml_array['tables'][0])
			$isDB = true;
		if("deposit" == $this->xml_array['tables'][0])
			$isDB = true;
		if("lot" == $this->xml_array['tables'][0])
			$isDB = true;
		if("serial_no" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_quote" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_quoter_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_quote" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_quote" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_quoter_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_quoter_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("estimate_cost" == $this->xml_array['tables'][0])
			$isDB = true;
		if("airlines" == $this->xml_array['tables'][0])
			$isDB = true;
		if("airport" == $this->xml_array['tables'][0])
			$isDB = true;
		if("inco_terms" == $this->xml_array['tables'][0])
			$isDB = true;
		if("seaports" == $this->xml_array['tables'][0])
			$isDB = true;
		if("shipment_classification" == $this->xml_array['tables'][0])
			$isDB = true;
		if("shipping_lines" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_sales_order_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_cont_seal" == $this->xml_array['tables'][0])
			$isDB = true;
		if("projects" == $this->xml_array['tables'][0])
			$isDB = true;
		if("billable_liquidation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("billable_pettycash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("billable_checkvoucher" == $this->xml_array['tables'][0])
			$isDB = true;
		if("projects" == $this->xml_array['tables'][0])
			$isDB = true;
		if("employees" == $this->xml_array['tables'][0])
			$isDB = true;
		if("8boxerpugrights" == $this->xml_array['tables'][0])
			$isDB = true;
		if("system_user" == $this->xml_array['tables'][0])
			$isDB = true;
		if("system_user" == $this->xml_array['tables'][0])
			$isDB = true;
		if("company" == $this->xml_array['tables'][0])
			$isDB = true;
		if("quotation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_all_transaction" == $this->xml_array['tables'][0])
			$isDB = true;
		if("petty_cash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("petty_cash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("reimbursement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("reimbursement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("billable_cost" == $this->xml_array['tables'][0])
			$isDB = true;
		if("centerrep_pettycash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("centerrep_empadvance" == $this->xml_array['tables'][0])
			$isDB = true;
		if("centerrep_liquidation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("replenishment_pettycash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("replenishment_liquidation" == $this->xml_array['tables'][0])
			$isDB = true;
		if("replenishment_empadv" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bank_statement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("other_charges" == $this->xml_array['tables'][0])
			$isDB = true;
		if("system_user" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_liquidation_detail2" == $this->xml_array['tables'][0])
			$isDB = true;
		if("balance_advance" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("bal_center_replenishment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_and_pettycash_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_and_empadv_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_and_liquidation_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_and_sales_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("project_and_voucher_report" == $this->xml_array['tables'][0])
			$isDB = true;
		if("projects" == $this->xml_array['tables'][0])
			$isDB = true;
		if("projects" == $this->xml_array['tables'][0])
			$isDB = true;
		if("shipping_date" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_detail_sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_detail_pettycash" == $this->xml_array['tables'][0])
			$isDB = true;
		if("entry_detail_cash_disbursement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("representative" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales_sub_invoice" == $this->xml_array['tables'][0])
			$isDB = true;
		if("search_cash_disbursement" == $this->xml_array['tables'][0])
			$isDB = true;
		if("sales" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request" == $this->xml_array['tables'][0])
			$isDB = true;
		if("purchase_request_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_payment" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_payment_detail" == $this->xml_array['tables'][0])
			$isDB = true;
		if("po_netpr" == $this->xml_array['tables'][0])
			$isDB = true;
		if("request_for_payment_imp" == $this->xml_array['tables'][0])
			$isDB = true;
		if("birsalesrelief" == $this->xml_array['tables'][0])
			$isDB = true;
		return $isDB;
	}
	
	protected function getDBFieldName($name)
	{
		if( !$this->wrdb )
			return $name;
		else
			return $this->arrDBFieldsList[ GoodFieldName($name) ];
	}
	
}
?>