<?php
class ProjectSettings
{
	var $_table;
	
	var $_pageMode;
	
	var $_viewPage = PAGE_VIEW;

	var $_defaultViewPage = PAGE_VIEW;
	
	var $_editPage = PAGE_EDIT;
	
	var $_defaultEditPage = PAGE_EDIT;
	
	var $_tableData = array();
	
	var $_mastersTableData = array();
	
	var $_detailsTableData = array();
	
	var $_dashboardElemPSet = array();
		
	function ProjectSettings($table = "", $page = "")
	{
		if($table && $table != NOT_TABLE_BASED_TNAME)
			$this->setTable($table);
		if($page)
			$this->setPage($page);
	}
	
	
	function setTable($table)
	{
		$this->_table = $table;
		global $tables_data, $field_labels, $fieldToolTips, $page_titles, $detailsTablesData, $masterTablesData, $tableEvents, $bSubqueriesSupported;
		if(GetTableURL($table) != "")
			include_once(getabspath("include/".GetTableURL($table)."_settings.php"));
		
		if(isset($tables_data[$this->_table]))
			$this->_tableData = &$tables_data[$this->_table];
		
		$this->_mastersTableData = &$masterTablesData[$this->_table];
		$this->_detailsTableData = &$detailsTablesData[$this->_table];
		
		$tableType = $this->getTableType();
		
		$this->_editPage = $this->getDefaultEditPageType($tableType);	
		$this->_viewPage = $this->getDefaultViewPageType($tableType);	
		$this->_defaultEditPage = $this->_editPage;
		$this->_defaultViewPage = $this->_viewPage;
	}

	function getDefaultViewPageType($tableType)
	{
		if($tableType == PAGE_CHART || $tableType == PAGE_REPORT)
			return $tableType;

		return PAGE_VIEW;
	}

	function getDefaultEditPageType($tableType)
	{
		if($tableType == PAGE_CHART || $tableType == PAGE_REPORT)
			return PAGE_SEARCH;
			
		return PAGE_EDIT;
	}
	
	function setPage($page)
	{
		//	a deeper checking for table and page types compatibility might be added here
		if($this->isPageTypeForView($page))
		{
			$tableType = $this->getTableType();
			if($tableType != "report" && $tableType != "chart"
				&& ($page == PAGE_CHART || $page == PAGE_REPORT))
				$this->_viewPage = PAGE_LIST;
			else 
				$this->_viewPage = $page;
			$this->_defaultViewPage = $this->getDefaultViewPageType($tableType);
		}
		if($this->isPageTypeForEdit($page))
		{
			$this->_editPage = $page;
			$this->_defaultEditPage = $this->getDefaultEditPageType($this->getTableType());
		}
	}

	function getEditPageType()
	{
		return $this->_editPage;
	}
	
	function isPageTypeForView($ptype)
	{
		global $pageTypesForView;
		return in_array(strtolower($ptype), $pageTypesForView);
	}

	function isPageTypeForEdit($ptype)
	{
		global $pageTypesForEdit;
		return in_array(strtolower($ptype), $pageTypesForEdit);	
	}

	function getTable($table, $page = "")
	{
		return new ProjectSettings($table, $page);
	}
	
	function getPageTypeByFieldEditFormat($field, $editFormat)
	{
		if(isset($this->_tableData[$field]) && isset($this->_tableData[$field][FORMAT_EDIT]))
		{
			foreach($this->_tableData[$field][FORMAT_EDIT] as $pageType => $editSettings)
			{
				if(isset($editSettings["EditFormat"]) && $editSettings["EditFormat"] == $editFormat)
					return $pageType;
			}
		}
		return "";
	}
	
	function getTableData($key)
	{
		if(!$this->isExistsTableKey($key))
			return $this->getDefaultValueByKey(substr($key,1));
		return $this->_tableData[$key];
	}
	
	private function getEffectiveEditPage( $field )
	{
		if( $this->isSeparate($field) )
		{
			return $this->_editPage;
		}
		return $this->_defaultEditPage;
	}

	private function getEffectiveViewPage( $field )
	{
		if( $this->isSeparate($field) )
		{
			if( $this->_pageMode == EDIT_INLINE )
				return PAGE_LIST;
			return $this->_viewPage;
		}
		return $this->_defaultViewPage;
	}
	function getFieldData( $field, $key )
	{
		global $g_settingsType;
		if( $this->getEntityType() == titDASHBOARD )
		{
			return $this->getDashFieldData( $field, $key );
		}
		
		if(!isset($this->_tableData[$field]))
			return $this->getDefaultValueByKey($key);
		
		$settingType = $g_settingsType[$key];
		if($settingType == null)
			$settingType = "";
		
		switch($settingType)
		{
			case SETTING_TYPE_VIEW:
				$viewPage = $this->getEffectiveViewPage( $field );
				
				if(isset($this->_tableData[$field][FORMAT_VIEW][$viewPage][$key]))
					return $this->_tableData[$field][FORMAT_VIEW][$viewPage][$key];
				break;
			case SETTING_TYPE_EDIT:
				$editPage = $this->getEffectiveEditPage( $field );
		
				if(isset($this->_tableData[$field][FORMAT_EDIT][$editPage][$key]))
					return $this->_tableData[$field][FORMAT_EDIT][$editPage][$key];
				break;
			default:
				if (isset($this->_tableData[$field][$key]))
					return $this->_tableData[$field][$key];
				break;
		}
		return $this->getDefaultValueByKey($key);
	}

	function setFieldData( $field, $key, $value )
	{
		$oldValue = $this->getFieldData( $field, $key );
		global $g_settingsType;
		
		$settingType = $g_settingsType[$key];
		if($settingType == null)
			$settingType = "";
		
		switch($settingType)
		{
			case SETTING_TYPE_VIEW:
				$viewPage = $this->getEffectiveViewPage( $field );
				$this->_tableData[$field][FORMAT_VIEW][$viewPage][$key] = $value;
				break;
			case SETTING_TYPE_EDIT:
				$editPage = $this->getEffectiveEditPage( $field );
				$this->_tableData[$field][FORMAT_EDIT][$editPage][$key] = $value;
				break;
			default:
				$this->_tableData[$field][$key] = $value;
				break;
		}
		return $oldValue;
	}

	/**
	 * getTableName
	 * Returns table name the class is created for
	 */
	function getTableName()
	{
		return $this->_table;
	}
	
	/**
	 * addCustomExpressionIndex
	 * Add new index to list, for determination of custom expressions position in SQL query 
	 * @param {string} table wich contain a lookup field
	 * @param {string} name of lookup field
	 * @param {int} index
	 */
	function addCustomExpressionIndex($mainTable, $mainField, $index)
	{
		if(!$this->isExistsTableKey(".customExpressionIndexes"))
			$this->_tableData[".customExpressionIndexes"] = array();
		if(!isset($this->_tableData[".customExpressionIndexes"][$mainTable]))
			$this->_tableData[".customExpressionIndexes"][$mainTable] = array();
		$this->_tableData[".customExpressionIndexes"][$mainTable][$mainField] = $index;
	}
	
	/**
	 * getCustomExpressionIndex
	 * Get index of custom expression in SQL field
	 * @param {string} table wich contain a lookup field
	 * @param {string} name of lookup field
	 */
	function getCustomExpressionIndex($mainTable, $mainField)
	{
		if(!$this->isExistsTableKey(".customExpressionIndexes"))
			$this->_tableData[".customExpressionIndexes"] = array();
		if(isset($this->_tableData[".customExpressionIndexes"][$mainTable]) 
			&& isset($this->_tableData[".customExpressionIndexes"][$mainTable][$mainField]))
			return $this->_tableData[".customExpressionIndexes"][$mainTable][$mainField];
			
		return FALSE;
	}
	
	function isExistsTableKey($key)
	{
		if(!isset($this->_tableData[$key]))
			return false;
		return true;
	}

	function isExistsFieldKey($field, $key)
	{
		if(!$this->isExistsTableKey($field))
			return false;
		if(!isset($this->_tableData[$field][$key]))
			return false;
		return true;
	}
	
	function getDefaultValueByKey($key)
	{
		global $g_defaultOptionValues;
		if(isset($g_defaultOptionValues[$key]))
			return $g_defaultOptionValues[$key];
		return false;
	}
	
	/**
	 * Returns array of master tables , which are master for current detail table
	 * @param string $tName - it's data source detail table name
	 * @return array if success otherwise false
	 */
	function getMasterTablesArr($tName) 
	{
		return $this->_mastersTableData;
	}
	
	/**
	 * Returns array of master keys for passed detailTable
	 *
	 * @param string $dTableName - it's detail data sourse table name, 
	 * @return array if success otherwise false
	 */
	function getMasterKeysByDetailTable($dTableName, $default = array())
	{
		if(!$dTableName)
			return $default;
		foreach ($this->_detailsTableData as $dTableDataArr)
		{
			if ($dTableDataArr['dDataSourceTable'] == $dTableName)
				return $dTableDataArr['masterKeys'];
		}
		return $default;
	}

	/**
	 * Returns array of detail tables , which are detail for current master table
	 * @param string $tName - it's data source master table name
	 * @return array if success otherwise false
	 */
	function getDetailTablesArr() 
	{
		return $this->_detailsTableData;
	}
	
	/**
	 * Returns array of detail keys for passed masterTable
	 *
	 * @param string $mTableName - it's master table name, 
	 * @param string $tName - it's current (detail) table
	 * @return array if success otherwise default value
	 */
	function getDetailKeysByMasterTable($mTableName = "", $default = array())
	{
		if(!$mTableName)
			return $default;
		foreach($this->_mastersTableData as $mTableDataArr)
		{
			if ($mTableDataArr['mDataSourceTable'] == $mTableName)
				return $mTableDataArr['detailKeys'];
		}
		return $default;
	}
	
	/**
	 * Returns array of detail keys for passed detailTable
	 *
	 * @param string $dTableName - It's detail data sourse table name
	 * @param string $tName - current(master) table name
	 * @return array if success otherwise false
	 */
	function getDetailKeysByDetailTable($dTableName, $default = array())
	{
		foreach ($this->_detailsTableData as $dTableDataArr)
		{
			if ($dTableDataArr['dDataSourceTable'] == $dTableName)
				return $dTableDataArr['detailKeys'];
		}
		return $default;
	}
	
	/**
	 * Returns details preview Type 
	 *
	 * @param string $dTableName - it's detail data sourse table name, 
	 * @param string $tName - current(master) table name
	 * @return array if success otherwise false
	 */
	function getDPType($dTableName) 
	{
		if(!$dTableName)
			return false;
		foreach ($this->_detailsTableData as $dTableDataArr)
		{
			if ($dTableDataArr['dDataSourceTable'] == $dTableName)
				return $dTableDataArr['previewOnList'];
		}
		return false;
	}

	function GetFieldByIndex($index)
	{
		foreach($this->_tableData as $key => $value)
		{
			if(!is_array($value))
				continue;
			elseif(!isset($value["Index"]))
				continue;
			if($value["Index"] == $index && $this->getFieldIndex($key))
				return $key;
		}
		return null;
	}
	
	//	Is field has separate type for editing and viewing 
	function isSeparate($field)
	{
		if (isset($this->_tableData[$field]["isSeparate"]))
			return $this->_tableData[$field]["isSeparate"];
		return false;
	}
	
	// return field label
	function label($field)
	{
		$result = GetFieldLabel( GoodFieldName($this->_table), GoodFieldName($field) ); 
		return $result != "" ? $result : $field;
	}

	//	return filename field if any
	//	viewFormat setting
	function getFilenameField($field)
	{
		return $this->getFieldData($field, "Filename");
	}

	//	return hyperlink prefix
	//	viewFormat setting
	function getLinkPrefix($field)
	{
		return $this->getFieldData($field, "LinkPrefix");
	}
	
	/**
	 * Get hyperlink type
	 */
	function getLinkType($field)
	{
		return $this->getFieldData($field, "hlType");
	}
	
	/**
	 * Get hyperlink display field for title
	 * @return string field name
	 */
	function getLinkDisplayField($field)
	{
		return $this->getFieldData($field, "hlTitleField");
	}
	
	function openLinkInNewWindow($field)
	{
		return $this->getFieldData($field, "hlNewWindow");
	}
	
	function getLinkWordNameType($field)
	{
		return $this->getFieldData($field, "hlLinkWordNameType");
	}
	
	function getLinkWordText($field)
	{
		return $this->getFieldData($field, "hlLinkWordText");
	}

	//	return database field type
	//	using ADO DataTypeEnum constants
	//	the full list available at:
	//	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/ado270/htm/mdcstdatatypeenum.asp
	function getFieldType($field)
	{
		return $this->getFieldData($field, "FieldType");
	}

	function isAutoincField($field)
	{
		return $this->getFieldData($field, "AutoInc");
	}
	
	function getOraSequenceName($field)
	{
		return $this->getFieldData($field, "OraSequenceName");
	}
	
	function getDefaultValue($field, $table = "")
	{
		$tableName = $table ? $table : $this->_table;
		$editPage = $this->_editPage;
		if(!$this->isSeparate($field))
			$editPage = $this->getDefaultEditPageType($this->getTableType());;
		return GetDefaultValue($field, $editPage, $tableName);
	}
	
	function isAutoUpdatable($field)
	{
		return $this->getFieldData($field, "autoUpdatable");
	}
	
	function getAutoUpdateValue($field)
	{
		$editPage = $this->_editPage;
		if(!$this->isSeparate($field))
			$editPage = $this->getDefaultEditPageType($this->getTableType());;
		return GetAutoUpdateValue($field, $editPage, $this->_table);
	}	

	//	return Edit format
	//	editFormats
	function getEditFormat($field)
	{
		return $this->getFieldData($field, "EditFormat");
	}
	
	function isReadonly($field)
	{
		if($this->getEditFormat($field) == EDIT_FORMAT_READONLY)
			return true;
		return false;
	}

	//	return View format
	//	viewFormat setting
	function getViewFormat($field)
	{
		return $this->getFieldData($field, "ViewFormat");
	}

	//	show time in datepicker or not
	function dateEditShowTime($field)
	{
		return $this->getFieldData($field, "ShowTime");
	}

	function lookupControlType($field)
	{
		return $this->getFieldData($field, "LCType");
	}
	
	function isDeleteAssociatedFile($field)
	{
		return $this->getFieldData($field, "DeleteAssociatedFile");
	}

	//	is Lookup wizard dependent or not
	function useCategory($field)
	{
		return $this->getFieldData($field, "UseCategory");
	}
	
	//	is Lookup wizard with multiple selection
	function multiSelect($field)
	{
		return $this->getFieldData($field, "Multiselect");
	}

	// Lookup wizard select size
	function selectSize($field)
	{
		return $this->getFieldData($field, "SelectSize");
	}

	function showThumbnail($field)
	{
		return $this->getFieldData($field, "ShowThumbnail");
	}
	
	function showCustomExpr($field)
	{
		return $this->getFieldData($field, "ShowCustomExpr");
	}

	function showFileSize($field)
	{
		return $this->getFieldData($field, "ShowFileSize");
	}
	
	function showIcon($field)
	{
		return $this->getFieldData($field, "ShowIcon");
	}
	
	function getImageWidth($field)
	{
		return $this->getFieldData($field, "ImageWidth");
	}

	function getImageHeight($field)
	{
		return $this->getFieldData($field, "ImageHeight");
	}
	
	function getThumbnailWidth($field)
	{
		return $this->getFieldData($field, "ThumbWidth");
	}

	function getThumbnailHeight($field)
	{
		return $this->getFieldData($field, "ThumbHeight");
	}	
	
	// Get nLookupType for current field
	function getLookupType($field)
	{
		return $this->getFieldData($field, "LookupType");
	}

	//Get lookup table name
	function getLookupTable($field)
	{
		return $this->getFieldData($field, "LookupTable");
	}
	
	function getNotProjectLookupTableConnId($field)
	{
		return $this->getFieldData($field, "LookupConnId");
	}
	
	function getLinkField($field)
	{
		return $this->getFieldData($field, "LinkField");
	}
		
	function getLWLinkFieldType($field)
	{
		return $this->getFieldData($field, "LinkFieldType");
	}
	
	function getDisplayField($field)
	{
		return $this->getFieldData($field, "DisplayField");
	}
	
	function getCustomDisplay($field)
	{
		return $this->getFieldData($field, 'CustomDisplay');
	}
	
	//	viewFormats
	function NeedEncode($field)
	{
		return $this->getFieldData($field, "NeedEncode");
	}
	
	/**
	 * Get array of validation for control
	 * return array - of validations
	 */
	function getValidation($field)
	{
		return $this->getFieldData($field, "validateAs");
	}

	/** 
	 * Check is appear current field on list page
	 * return boolean - true or false	
	 */
	function appearOnListPage($field)
	{	
		return $this->getFieldData($field, "bListPage");
	}

	/** 
	 * Check is appear current field on add page
	 * return boolean - true or false	
	 */
	function appearOnAddPage($field)
	{
		return $this->getFieldData($field,"bAddPage");
	}

	/** 
	 * Check is appear current field on inline add
	 * return boolean	
	 */
	function appearOnInlineAdd($field)
	{
		return $this->getFieldData($field,"bInlineAdd");
	}

	/** 
	 * Check is appear current field on edit page
	 * return boolean - true or false	
	 */
	function appearOnEditPage($field)
	{
		return $this->getFieldData($field, "bEditPage");
	}

	/** 
	 * Check is appear current field on edit page
	 * return boolean - true or false	
	 */
	function appearOnInlineEdit($field)
	{
		return $this->getFieldData($field, "bInlineEdit");
	}

	/** 
	 * Check is appear current field on view page
	 * return boolean - true or false	
	 */
	function appearOnViewPage($field)
	{
		return $this->getFieldData($field, "bViewPage");
	}

	/** 
	 * Check is appear current field on print page
	 * return boolean - true or false	
	 */
	function appearOnPrinterPage($field)
	{
		return $this->getFieldData($field, "bPrinterPage");
	}
	
	function isVideoUrlField($field)
	{
		return $this->getFieldData($field, "fieldIsVideoUrl");
	}
	
	function isAbsolute($field)
	{
		return $this->getFieldData($field, "Absolute");
	}
	
	function getAudioTitleField($field)
	{
		return $this->getFieldData($field, "audioTitleField");
	}
	
	function getVideoWidth($field)
	{
		return $this->getFieldData($field, "videoWidth");
	}	

	function getVideoHeight($field)
	{
		return $this->getFieldData($field, "videoHeight");
	}
	
	function isRewindEnabled($field)
	{
		return $this->getFieldData($field, "RewindEnabled");
	}

	/**
	 * @param String field
	 * @return Array
	 */
	function getParentFieldsData( $field )
	{
		return $this->getFieldData($field, "categoryFields");
	}
	
	/**
	 * @param String field	
	 * @return Array
	 */	
	function getLookupParentFNames( $field )
	{
		$fNames = array();
		foreach( $this->getParentFieldsData( $field ) as $data ) 
		{
			$fNames[] = $data["main"];
		}
		return $fNames;
	}

	function isLookupUnique($field)
	{
		return $this->getFieldData($field, "LookupUnique");
	}

	function getLookupOrderBy($field)
	{
		return $this->getFieldData($field, "LookupOrderBy");
	}

	function isLookupDesc($field)
	{
		return $this->getFieldData($field, "LookupDesc");
	}

	function getOwnerTable($field)
	{
		return $this->getFieldData($field, "ownerTable");
	}

	function isFieldEncrypted($field)
	{
		return $this->getFieldData($field, "bIsEncrypted");
	}

	function isAllowToAdd($field)
	{
		return $this->getFieldData($field, "AllowToAdd");
	}

	function isSimpleAdd($field)
	{
		return $this->getFieldData($field, "SimpleAdd");
	}

	/**
	 * Get the array containing the autocomplete fields data
	 * basing on page's type and lookup wizard settings
	 * @param String field
	 * @return Array
	 */
	function getAutoCompleteFields($field)
	{
		$editPageType = $this->getEditPageType();
		
		if( $editPageType == PAGE_REGISTER || $editPageType == PAGE_ADD || $editPageType == PAGE_EDIT && ( $this->isSeparate($field) || $this->isAutoCompleteFieldsOnEdit($field) ) )
		{
			return $this->getFieldData($field, "autoCompleteFields");
		}
	
		return $this->getDefaultValueByKey("autoCompleteFields");
	}

	function isAutoCompleteFieldsOnEdit($field)
	{
		return $this->getFieldData($field, "autoCompleteFieldsOnEdit");
	}

	function isFreeInput($field)
	{
		return $this->getFieldData($field, "freeInput");
	}

	function getMapData($field)
	{
		return $this->getFieldData($field, "mapData");
	}

	function getFormatTimeAttrs($field)
	{
		return $this->getFieldData($field, "FormatTimeAttrs");
	}
		
	/** 
	 * Check is appear current field on export page
	 * return boolean - true or false	
	 */
	function appearOnExportPage($field)
	{
		return $this->getFieldData($field, "bExportPage");
	}
	
	/** 
	 * Check is appear current field on register page
	 * return boolean
	 */
	function appearOnRegisterOrSearchPage($field, $pageType)
	{
		if( $pageType != PAGE_REGISTER && $pageType != PAGE_SEARCH )
			return false;
		
		$arrFields = array();
		if( $pageType == PAGE_REGISTER )
			$arrFields = $this->getFieldsForRegister();
		elseif( $pageType == PAGE_SEARCH )
			$arrFields = $this->getAllSearchFields();
		
		if( in_array($field, $arrFields) )
			return true;
		
		return false;		
	} 

	/**
	 * Return original table name for report or chart
	 */
	function getStrOriginalTableName()
	{
		return $this->getTableData(".strOriginalTableName");
	}
	
	function getFieldsForRegister()
	{
		return $this->getTableData(".fieldsForRegister");
	}
	
	function getAllSearchFields()
	{
		return $this->getTableData('.allSearchFields');
	}
	
	function getAdvSearchFields()
	{
		return $this->getEntityType() == titDASHBOARD ? $this->getAllSearchFields() :  $this->getTableData('.advSearchFields');
	}
	
	function isUseTimeForSearch()
	{
		return $this->getTableData(".isUseTimeForSearch");
	}
	
	function isUseToolTips()
	{
		return $this->getTableData(".isUseToolTips");
	}

	function isUseVideo()
	{
		return $this->getTableData(".isUseVideo");
	}

	function isUseAudio()
	{
		return $this->getTableData(".isUseAudio");
	}
	
	function isUseAudioOnDetails()
	{
		for($i = 0; $i < count($this->_detailsTableData); $i++)
		{
			if($this->_detailsTableData[$i]["isUseAudio"])
				return true;
		}
		return false;
	}

	function getTableType()
	{
		return $this->getTableData(".tableType");
	}	

	function getShortTableName()
	{
		return $this->getTableData(".shortTableName");
	}

	function isShowAddInPopup()
	{
		return $this->getTableData(".showAddInPopup");
	}
	
	function isShowEditInPopup()
	{
		return $this->getTableData(".showEditInPopup");
	}
	
	function isShowViewInPopup()
	{
		return $this->getTableData(".showViewInPopup");
	}
	
	function getPopupPagesLayoutNames()
	{
		return $this->getTableData(".popupPagesLayoutNames");
	}
	
	function isResizeColumns()
	{
		return $this->getTableData(".isResizeColumns");
	}
	
	function isUseAjaxSuggest()
	{
		return $this->getTableData(".isUseAjaxSuggest");
	}
	
	function getDetailsLinksOnList()
	{
		return $this->getTableData(".detailsLinksOnList");
	}
	
	function getPanelSearchFields()
	{
		return $this->getTableData(".panelSearchFields");
	}

	function getSearchPanelOptions()
	{
		return $this->GetTableData(".searchPanelOptions");
	}
	
	function getGoogleLikeFields()
	{
		return $this->getTableData(".googleLikeFields");
	}

	function getInlineEditFields()
	{
		return $this->getTableData(".inlineEditFields");
	}
	
	function getExportFields()
	{
		return $this->getTableData(".exportFields");
	}
	
	function getImportFields()
	{
		return $this->getTableData(".importFields");
	}
	
	function getEditFields()
	{
		return $this->getTableData(".editFields");
	}

	function getInlineAddFields()
	{
		return $this->getTableData(".inlineAddFields");
	}

	function getAddFields()
	{
		return $this->getTableData(".addFields");
	}
	
	function getMasterListFields()
	{
		return $this->getTableData(".masterListFields");
	}
	
	function getViewFields()
	{
		return $this->getTableData(".viewFields");
	}
	
	function getPrinterFields()
	{
		return $this->getTableData(".printFields");
	}

	function getListFields()
	{
		return $this->getTableData(".listFields");
	}

	function isAddPageEvents()
	{
		return $this->getTableData(".addPageEvents");
	}

	function isUsebuttonHandlers()
	{
		return $this->getTableData(".isUsebuttonHandlers");
	}

	function isUseMainMaps()
	{
		return $this->getTableData(".isUseMainMaps");
	}

	function isUseFieldsMaps()
	{
		return $this->getTableData(".isUseFieldsMaps");
	}

	function getOrderIndexes()
	{
		return $this->getTableData(".orderindexes");
	}

	function getStrOrderBy()
	{
		return $this->getTableData(".strOrderBy");
	}

	function getSQLQuery()
	{
		$query = $this->getTableData(".sqlquery");
		if($query != null)
		{
			return $query;
		}
		return null;
	}
	/* Dashboard-safe version of getSQLQuery 
	*/
	function getSQLQueryByField( $field )
	{
		if( $this->getTableType() == PAGE_DASHBOARD )
		{
			$query = $this->getDashTableData($field, ".sqlquery");
			if($query != null)
			{
				return $query;
			}
			return null;
		
		}
		else
		{
			$query = $this->getTableData(".sqlquery");
			if($query != null)
			{
				return $query;
			}
			return null;
		}
			
	}

	/**
	 * Create Thumbnail or not
	 */
	function getCreateThumbnail($field)
	{
		return $this->getFieldData($field, "CreateThumbnail");
	}

	/**
	 * Return Thumbnail prefix
	 */
	function getStrThumbnail($field)
	{
		return $this->getFieldData($field, "StrThumbnail");
	}
	
	/**
	 * Return Thumbnail prefix
	 */
	function getThumbnailSize($field)
	{
		return $this->getFieldData($field, "ThumbnailSize");
	}

	/**
	 * Resize on upload
	 */
	function getResizeOnUpload($field)
	{
		return $this->getFieldData($field, "ResizeImage");
	}
	
	/**
	 * True if FileField must work in old single-file mode
	 */
	function isBasicUploadUsed($field)
	{
		return $this->getFieldData($field, "CompatibilityMode");
	}
	
	/**
	 * True if file in FileField must be uploaded immediately after choosing or dropping 
	 */
	function isAutoUpload($field)
	{
		return $this->getFieldData($field, "autoUpload");
	}
	
	/**
	 * Get size to reduce image after upload
	 */
	function getNewImageSize($field)
	{
		return $this->getFieldData($field, "NewSize");
	}
	
	function getAcceptFileTypes($field)
	{
		return $this->getFieldData($field, "acceptFileTypes");
	}
	
	/**
	 * Get maximum allowed size for uploaded files
	 */
	function getMaxFileSize($field)
	{
		return $this->getFieldData($field, "maxFileSize");
	}
	
	/**
	 * Get maximum allowed size for all uploaded files per field
	 */
	function getMaxTotalFilesSize($field)
	{
		return $this->getFieldData($field, "maxTotalFilesSize");
	}
		
	/**
	 * Get maximum allowed number of uploaded files
	 */
	function getMaxNumberOfFiles($field)
	{
		return $this->getFieldData($field, "maxNumberOfFiles");
	}
	
	/**
	 * Get maximum allowed width of uploaded image
	 */
	function getMaxImageWidth($field)
	{
		return $this->getFieldData($field, "maxImageWidth");
	}
	
	/**
	 * Get maximum allowed heiht of uploaded image
	 */
	function getMaxImageHeight($field)
	{
		return $this->getFieldData($field, "maxImageHeight");
	}	

	/**
	 * Get size to reduce image after upload
	 */
	function getStrFilename($field)
	{
		return $this->getFieldData($field, "strFilename");
	}	
	
	/**
	 * Return height of text area
	 * EditFormat setting
	 */
	function getNRows($field)
	{
		return $this->getFieldData($field, "nRows");
	}

	/**
	 * Return width of text area
	 * EditFormat setting
	 */
	function getNCols($field)
	{
		return $this->getFieldData($field, "nCols");
	}

	/**
	 * Return original table name
	 */
	function getOriginalTableName()
	{
		$result = $this->getTableData(".OriginalTable");
		return $result != "" ? $result : $this->_table;
	}

	/**
	 * Return list of key fields
	 */
	function getTableKeys()
	{
		return $this->getTableData(".Keys");
	}

	function isLargeTextTruncationSet() 
	{
		return $this->getTableData(".truncateText");
	}
	
	/**
	 * Return number of chars to show before "More..." link
	 */
	function getNumberOfChars()
	{
		return $this->getTableData(".NumberOfChars");
	}
	
	/**
	 * Check if the field is an sql expression
	 */
	function isSQLExpression($field)
	{
		return $this->getFieldData($field, "isSQLExpression");
	}
	
	/**
	 * Get full field name
	 */
	function getFullFieldName($field)
	{
		return $this->getFieldData($field, "FullName");
	}

	/**
	 * set full field name
	 */
	function setFullFieldName($field, $value)
	{
		return $this->setFieldData($field, "FullName", $value);
	}

	
	/**
	 * Return table Owner ID field
	 */
	function getTableOwnerID()
	{
		return $this->getTableData(".OwnerID");
	}

	/**
	 * Is field marked as required
	 * EditFormat setting 
	 */
	function isRequired($field)
	{
		return $this->getFieldData($field, "IsRequired");
	}

	/**
	 * IS use Rich Text Editor or not
	 */
	function isUseRTE($field)
	{
		return $this->getFieldData($field, "UseRTE");
	}

	/**
	 * Is use Rich Text Editor BASIC or not
	 */
	function isUseRTEBasic($field)
	{
		global $isUseRTEBasic;
		return $this->isUseRTE($field) && $isUseRTEBasic;
	}

	/**
	 * Is use Rich Text Editor FCK or not
	 */
	function isUseRTEFCK($field)
	{
		global $isUseRTECK;
		return $this->isUseRTE($field) && $isUseRTECK;
	}

	/**
	 * Is use Rich Text Editor INNOVA or not
	 */
	function isUseRTEInnova($field)
	{
		global $isUseRTEInnova;
		return $this->isUseRTE($field) && $isUseRTEInnova;
	}

	/**
	 * Add timestamp to filename when files uploading or not
	 * EditFormat setting
	 */
	function isUseTimestamp($field)
	{
		return $this->getFieldData($field, "UseTimestamp");
	}
	
	function getFieldIndex($field)
	{
		return $this->getFieldData($field, "Index");
	}
	
	function getEntityType()
	{
		return $this->getTableData(".entityType");
	}
	
	
	/**
	 * Return Date field edit type
	 */
	function getDateEditType($field)
	{
		return $this->getFieldData($field, "DateEditType");                  
	}

	function getHTML5InputType($field)
	{
		return $this->getFieldData($field, "HTML5InuptType");
	}
	
	/**
	 * Return text edit parameters
	 */
	function getEditParams($field)
	{
		return $this->getFieldData($field, "EditParams");
	}
	
	
	function getControlWidth($field)
	{
		return $this->getFieldData($field, "controlWidth");
	}
	
	/**
	 * Check whether field is viewable
	 */
	function checkFieldPermissions($field)
	{
		return $this->getFieldData($field,"FieldPermissions");
	}
	
	function getTableOwnerIdField()
	{
		return $this->getTableData(".mainTableOwnerID");
	}
	
	function getDependentLookups($field)
	{
		return $this->getFieldData($field, "DependentLookups");
	}

	function isHorizontalLookup($field)
	{
		return $this->getFieldData($field, "HorizontalLookup");
	}

	function isDecimalDigits($field)
	{
		return $this->getFieldData($field, "DecimalDigits");
	}
	
	function getLookupValues($field)
	{
		return $this->getFieldData($field, "LookupValues");
	}
	
	function hasEditPage()
	{
		return $this->getTableData(".edit");
	}

	function hasAddPage()
	{
		return $this->getTableData(".add");
	}
	
	function hasListPage()
	{
		return $this->getTableData(".list");
	}	
	
	function hasImportPage()
	{
		return $this->getTableData(".import");		
	}
	
	function hasInlineEdit()
	{
		return $this->getTableData(".inlineEdit");
	}
	function hasCopyPage()
	{
		return $this->getTableData(".copy");
	}
	function hasViewPage()
	{
		return $this->getTableData(".view");
	}
	function hasExportPage()
	{
		return $this->getTableData(".exportTo");
	}
	function hasPrintPage()
	{
		return $this->getTableData(".printFriendly");
	}
	function hasDelete()
	{
		return $this->getTableData(".delete");
	}
	function getTotalsFields()
	{
		return $this->getTableData(".totalsFields");
	}
	function getAdvancedSecurityType()
	{
		return $this->getTableData(".nSecOptions");
	}
	function displayLoading()
	{
		return $this->getTableData(".isDisplayLoading");
	}
	function getRecordsPerPageArray()
	{
		return $this->getTableData(".arrRecsPerPage");
	}
	function getGroupsPerPageArray()
	{
		return $this->getTableData(".arrGroupsPerPage");
	}
	
	function isReportWithGroups()
	{
		return $this->getTableData(".reportGroupFields");
	}

	function isCrossTabReport()
	{
		return $this->getTableData(".crossTabReport");
	}
	
	function getReportGroupFieldsData()
	{
		return $this->getTableData(".reportGroupFieldsData");
	}
	
	function reportHasHorizontalSummary()
	{
		return $this->getTableData(".reportHorizontalSummary");
	}
	
	function reportHasVerticalSummary()
	{
		return $this->getTableData(".reportVerticalSummary");
	}
	
	function reportHasPageSummary()
	{
		return $this->getTableData(".reportPageSummary");
	}
	
	function reportHasGlobalSummary()
	{
		return $this->getTableData(".reportGlobalSummary");
	}
	
	function getReportLayout()
	{
		return  $this->getTableData(".reportLayout");
	}
	
	function isGroupSummaryCountShown()
	{
		return  $this->getTableData(".showGroupSummaryCount");
	}
	
	function reportDetailsShown()
	{
		return $this->getTableData(".repShowDet");
	}
	
	function reportTotalFieldsExist()
	{
		return $this->getTableData(".isExistTotalFields");
	}
	
	function noRecordsOnFirstPage()
	{
		return $this->getTableData(".noRecordsFirstPage");
	}
	
	function isViewPagePDF()
	{
		return $this->getTableData(".isViewPagePDF");
	}
	
	function isLandscapeViewPDFOrientation()
	{
		return $this->getTableData(".isLandscapeViewPDFOrientation");
	}
	
	function isViewPagePDFFitToPage()
	{
		return $this->getTableData(".isViewPagePDFFitToPage");
	}
	
	function getViewPagePDFScale()
	{
		return $this->getTableData(".nViewPagePDFScale");
	}
	
	function isLandscapePrinterPagePDFOrientation()
	{
		return $this->getTableData(".isLandscapePrinterPagePDFOrientation");
	}
	
	function isPrinterPagePDFFitToPage()
	{
		return $this->getTableData(".isPrinterPagePDFFitToPage");
	}
	
	function getPrinterPagePDFScale()
	{
		return $this->getTableData(".nPrinterPagePDFScale");
	}

	function isPrinterPageFitToPage()
	{
		return $this->getTableData(".isPrinterPageFitToPage");
	}
	
	function getPrinterPageScale()
	{
		return $this->getTableData(".nPrinterPageScale");
	}

	function getPrinterSplitRecords()
	{
		return $this->getTableData(".nPrinterSplitRecords");
	}

	function getPrinterPDFSplitRecords()
	{
		return $this->getTableData(".nPrinterPDFSplitRecords");
	}
	
	function isPrinterPagePDF()
	{
		return $this->getTableData(".isPrinterPagePDF");
	}

	function isCaptchaEnabledOnEdit()
	{
		return $this->getTableData(".isCaptchaEnabledOnEdit");
	}

	function isCaptchaEnabledOnAdd()
	{
		return $this->getTableData(".isCaptchaEnabledOnAdd");
	}

	function isSearchRequiredForFiltering() 
	{
		return $this->getTableData(".searchIsRequiredForFilters");
	}
	function warnLeavingPages()
	{
		return $this->getTableData(".warnLeavingPages");	
	}
	
	function hideEmptyViewFields()
	{
		return $this->getTableData(".hideEmptyFieldsOnView");
	}
	
	function getInitialPageSize()
	{
		return $this->getTableData(".pageSize");
	}
	
	function getRecordsPerRowList()
	{
		return $this->getTableData(".recsPerRowList");
	}

	function getRecordsPerRowPrint()
	{
		return $this->getTableData(".recsPerRowPrint");
	}
	
	function useMoveNext()
	{
		return $this->getTableData(".moveNext");
	}
	
	function highlightRows()
	{
		return $this->getTableData(".rowHighlite");
	}
	
	function hasInlineAdd()
	{
		return $this->getTableData(".inlineAdd");
	}
	
	function getListGridLayout()
	{
		return $this->getTableData(".listGridLayout");
	}
	
	function getPrintGridLayout()
	{
		return $this->getTableData(".printGridLayout");
	}
	
	function getReportPrintLayout()
	{
		return $this->getTableData(".printReportLayout");
	}
	
	function getPrinterPageOrientation()
	{
		return $this->getTableData(".printerPageOrientation");
	}
	
	function getReportPrintPartitionType()
	{
		return $this->getTableData(".reportPrintPartitionType");
	}
	
	function getReportPrintGroupsPerPage()
	{
		return $this->getTableData(".reportPrintGroupsPerPage");
	}
	
	function getReportPrintPDFGroupsPerPage()
	{
		return $this->getTableData(".reportPrintPDFGroupsPerPage");
	}
	
	
	function getLowGroup()
	{
		return $this->getTableData(".lowGroup");
	}

	function ajaxBasedListPage()
	{
		return $this->getTableData(".listAjax");
	}
	
	
	/**
	 * Returns true when the Add page has multistep layout
	 * @return boolean
	 */
	function isAddMultistep()
	{	
		return $this->getTableData(".addMultistep");
	}

	/**
	 * Returns true when the Edit page has multistep layout
	 * @return boolean
	 */
	function isEditMultistep()
	{	
		return $this->getTableData(".editMultistep");
	}

	/**
	 * Returns true when the View page has multistep layout
	 * @return boolean
	 */
	function isViewMultistep()
	{	
		return $this->getTableData(".viewMultistep");
	}
	
	
	/**
	 * Returns array of tabs and sections, which use on add page
	 * @param string $table - current data source table name
	 * @return array
	 */
	function getAddTabs()
	{	
		return $this->getTableData(".arrAddTabs");
	}
	
	/**
	 * Check use tabs and sections on add page or not
	 * @param string $table - current data source table name
	 * @return boolean result - true or false
	 */
	function useTabsOnAdd()
	{
		if(count($this->getAddTabs()))
			return true;
		return false;
	}
	
	/**
	 * Returns array of tabs and sections, which use on edit page
	 * @param string $table - current data source table name
	 * @return array
	 */
	function getEditTabs()
	{	
		return $this->getTableData(".arrEditTabs");
	}
	
	/**
	 * Check use tabs and sections on edit page or not
	 * @param string $table - current data source table name
	 * @return boolean result - true or false
	 */
	function useTabsOnEdit()
	{
		if(count($this->getEditTabs()))
			return true;
		return false;
	}
	
	/**
	 * Returns array of tabs and sections, which use on view page
	 * @param string $table - current data source table name
	 * @return array
	 */
	function getViewTabs()
	{	
		return $this->getTableData(".arrViewTabs");
	}

	/**
	 * Check use tabs and sections on view page or not
	 * @param string $table - current data source table name
	 * @return boolean result - true or false
	 */
	function useTabsOnView()
	{
		if(count($this->getViewTabs()))
			return true;
		return false;
	}
	
	function highlightSearchResults()
	{
		return $this->getTableData(".highlightSearchResults");
	}
	
	
	function getFieldsList()
	{
		if(is_null($this->_tableData))
			return array();
		$t = array_keys($this->_tableData);
		$arr = array();
		foreach($t as $f)
			if(substr($f,0,1)!=".")
				$arr[] = $f;
		return $arr;
	}

	function getBinaryFieldsIndices()
	{
		$fields = $this->getFieldsList();
		$out = array();
		foreach($fields as $idx => $f)
		{
			if(IsBinaryType($this->getFieldType($f)))
				$out[] = $idx + 1;
		}
		return $out;
	}
	
	function getNBFieldsList()
	{
		$t = $this->getFieldsList();
		$arr = array();
		foreach($t as $f)
			if(!IsBinaryType($this->getFieldType($f)))
				$arr[] = $f;
		return $arr;
	}

	/**
	 * Get field by good field name 
	 * @param string	$field The good field name 
	 * @return string	
	 */
	function getFieldByGoodFieldName($field)
	{
		foreach($this->_tableData as $key => $value)
		{
			if(count($value) > 1)
			{
				if($value["GoodName"] == $field)
					return $key;
			}
		}
		return "";
	}
	
	/**
	 * getUploadFolder
	 * Return inputed value or calculated path for upload folder 
	 * @param {string} field name 
	 * @param {array} file info (name, type, size)
	 */
	function getUploadFolder($field, $fileData = array())
	{
		if($this->isUploadCodeExpression($field))
			$path = GetUploadFolderExpression($field, $fileData);
		else
			$path = $this->getFieldData($field, "UploadFolder");
				if(strlen($path) && substr($path,strlen($path)-1) != "/")
			$path.="/";
		return $path;
	}
	
	function isMakeDirectoryNeeded($field)
	{
		return $this->isUploadCodeExpression($field) || !$this->isAbsolute($field);
	}
	
	function getFinalUploadFolder($field, $fileData = array())
	{
		if($this->isAbsolute($field))
			$path = $this->getUploadFolder($field, $fileData);
		else
			$path = getabspath($this->getUploadFolder($field, $fileData));
				if(strlen($path) && substr($path,strlen($path)-1) != "/")
			$path.="/";
		return $path;
	}
	
	function isUploadCodeExpression($field)
	{
		return $this->getFieldData($field, "UploadCodeExpression");
	}

	function &getQueryObject()
	{
		$queryObj = $this->getSQLQuery();
		return $queryObj;
	}

	function getListOfFieldsByExprType($needaggregate)
	{
		$query = &$this->getSQLQuery();
		$fields = $this->getFieldsList();
		$out = array();
		foreach($fields as $idx=>$f)
		{
			$aggr = $query->IsAggrFuncField($idx);
			if($needaggregate && $aggr || !$needaggregate && !$aggr)
				$out[] = $f;
		}
		return $out;
	}
	
	function isAggregateField($field)
	{
		$query = &$this->getSQLQuery();
		$idx = $this->getFieldIndex($field) - 1;
		return $query->IsAggrFuncField($idx);
	}
	
	/**
	 * Check if searching is case insensitive for the table
	 * @return Boolean
	 */ 
	function getNCSearch()
	{
		return $this->getTableData(".NCSearch");
	}
	
	function getChartType()
	{
		return $this->getTableData(".chartType");
	}
	
	function getChartRefreshTime()
	{
		return $this->getTableData(".ChartRefreshTime");
	}
	
	function getChartXml()
	{
		return $this->getTableData(".chartXml");
	}
	
	function auditEnabled()
	{
		return $this->getTableData(".audit");
	}
	
	function isSearchSavingEnabled()
	{
		return $this->getTableData(".searchSaving");
	}
	
	function lockingEnabled()
	{
		return $this->getTableData(".locking");
	}
	
	function hasEncryptedFields()
	{
		return $this->getTableData(".hasEncryptedFields");
	}
	
	function showSearchPanel()
	{
		return $this->getTableData(".showSearchPanel");
	}
	
	function isFlexibleSearch()
	{
		return $this->getTableData(".flexibleSearch");
	}
	
	function getSearchRequiredFields()
	{
		return $this->getTableData(".requiredSearchFields");
	}
	
	function showSimpleSearchOptions()
	{
		return $this->getTableData(".showSimpleSearchOptions");
	}
	
	function getFilterFields()
	{
		return $this->getTableData(".filterFields");
	}
	
	function getFilterFieldFormat($field)
	{
		return $this->getFieldData($field, "filterFormat");
	}
	
	function getFilterFieldTotal($field)
	{
		return $this->getFieldData($field, "filterTotals");
	}
	
	function showWithNoRecords($field)
	{
		return $this->getFieldData($field, "showWithNoRecords");
	}
	
	function getFilterSortValueType($field)
	{
		return $this->getFieldData($field, "sortValueType");
	}
	
	function isFilterSortOrderDescending($field)
	{
		return $this->getFieldData($field, "descendingOrder");
	}	
	
	function getNumberOfVisibleFilterItems($field)
	{
		return $this->getFieldData($field, "numberOfVisibleItems");
	}
		
	function getParentFilterName($field)
	{
		return $this->getFieldData($field, "parentFilterField");
	}	
	
	function getParentFiltersNames($field)
	{
		return $this->getFieldData($field, "parentFilters");
	}
	
	function hasDependentFilter($field)
	{
		return $this->getDependentFilterName($field) != "";
	}
	
	function getDependentFilterName($field)
	{
		return $this->getFieldData($field, "dependentFilterName");
	}
	
	function getDependentFiltersNames($field)
	{
		return $this->getFieldData($field, "dependentFilters");
	}
	
	function getFilterIntervals($field)
	{
		return $this->getFieldData($field, "filterIntervals");
	}
	
	function showCollapsed($field)
	{
		return $this->getFieldData($field, "showCollapsed");
	}
	
	function getFilterIntervalDatabyIndex($field, $idx)
	{
		$intervalData = array();
		
		$filterIntervalsData = $this->getFilterIntervals($field);
		foreach($filterIntervalsData as $interval)
		{
			if($interval["index"] == $idx)
			{
				$intervalData = $interval;
				break;
			}
		}
		
		return $intervalData;
	}
	
	function getFilterTotalsField($field)
	{
		return $this->getFieldData($field, "filterTotalFields");
	}
	
	function getFilterFiledMultiSelect($field)
	{
		return $this->getFieldData($field, "filterMultiSelect");
	}
	
	function getFilterCheckedMessage($field)
	{
		return $this->getFieldData($field, "filterCheckedMessageText");
	}
	
	function getFilterCheckedMessageType($field)
	{
		return $this->getFieldData($field, "filterCheckedMessageType");
	}
	
	function getFilterUncheckedMessage($field)
	{
		return $this->getFieldData($field, "filterUncheckedMessageText");
	}
	
	function getFilterUncheckedMessageType($field)
	{
		return $this->getFieldData($field, "filterUncheckedMessageType");
	}
	
	function getFilterStepType($field)
	{
		return $this->getFieldData($field, "filterSliderStepType");
	}
	
	function getFilterStepValue($field)
	{
		return $this->getFieldData($field, "filterSliderStepValue");
	}
	
	function getFilterKnobsType($field)
	{
		return $this->getFieldData($field, "filterKnobsType");
	}
	
	function isFilterApplyBtnSet($field)
	{
		return $this->getFieldData($field, "filterApplyBtn");
	}
	
	function isCaseInsensitiveUsername()
	{
		global $caseInsensitiveUsername;
		return $caseInsensitiveUsername;
	}
	
	function getCaseSensitiveUsername($value)
	{
		if (!$this->isCaseInsensitiveUsername())
			return $value;
		return strtoupper($value);
	}
	
	function getStrField($field)
	{
		return $this->getFieldData($field, "strField");
	}
	
	function getScrollGridBody()
	{
		return $this->getTableData(".scrollGridBody");
	}
	
	/**
	 * Is 'UpdateLatLng' ticked for the table
	 */
	function isUpdateLatLng()
	{
		return $this->getTableData(".geocodingEnabled");
	}

	/**
	 * get geocoding data for the table
	 */
	function getGeocodingData()
	{
		return $this->getTableData(".geocodingData");
	}
	
	function allowDuplicateValues($field)
	{
		return !$this->getFieldData($field, "denyDuplicates");		
	}
	
	function getDashFieldData($field, $key)
	{
		$dashSearchFields = $this->getDashboardSearchFields();
		$dfield = $dashSearchFields[ $field ];
		if( $dfield )
			$table = $dfield[0]["table"];
		if( !$dfield || !$table)
			return $this->getDefaultValueByKey( $key );

		if( !$this->_dashboardElemPSet[ $table ] )
			$this->_dashboardElemPSet[ $table ] = new ProjectSettings( $table, $this->_editPage );
		
		return $this->_dashboardElemPSet[ $table ]->getFieldData( $dfield[0]["field"], $key );
	}

	function getDashTableData($field, $key)
	{
		$dashSearchFields = $this->getDashboardSearchFields();
		$tableSettings = new ProjectSettings($dashSearchFields[$field][0]["table"], $this->_editPage);
		return $tableSettings->getTableData( $key );
	}


	/**
	 * It returns an empty array if the 'Select all' (search options) is ticked
	 */
	function getSearchOptionsList($field)
	{
		return $this->getFieldData($field, "searchOptionsList");
	}
	
	
	function getDefaultSearchOption($field)
	{
		$defaultOpt = $this->getFieldData($field, "defaultSearchOption");
		
		if( !$defaultOpt )
		{
			$searchOptionsList = $this->getSearchOptionsList($field);
			if( count($searchOptionsList) )
				$defaultOpt = $searchOptionsList[0];
		}
		return $defaultOpt;
	}
	
	/**
	 * Get status show list of thumbnails
	 * @param string $field field name
	 * @return boolean
	 */
	function showListOfThumbnails($field)
	{
		return $this->getFieldData($field, "ShowListOfThumbnails");
	}
	/**
	 * Returns menu name for given menuId and page
	 * @param string $page page name
	 * @param string $menuId menu id
	 * @return string
	 */
	function getMenuName($page, $id, $horizontal)
	{
		global $menuAssignments;
		$menuId = $id;
		$isho = strlen($horizontal) > 0 ? "1" : "0";
		foreach($menuAssignments as $m)
		{
			if($m["page"] == $page && $m["id"] == $menuId && $isho == $m["horizontal"])
				return $m["name"];
		}
		return "main";
	}

	function getMenuStyle($page, $id, $horizontal)
	{
		global $menuStyles;
		$menuId = $id;
		$isho = (bool)strlen($horizontal);
		foreach($menuStyles as $m)
		{
			if($m["page"] == $page && $m["id"] == $menuId && $isho == (bool)$m["horizontal"])
				return $m["style"];
		}
	// return default style
		if($id == "main")
			return 0;
		return 1;
	}
	
	function setPageMode($pageMode)
	{
		$this->_pageMode = $pageMode;
	}

	function editPageHasDenyDuplicatesFields()
	{
		foreach($this->getEditFields() as $fieldName) 
		{
			if( !$this->allowDuplicateValues($fieldName) ) 
			{
				return true;
			}
		}	
		return false;
	}
	
	/**
	 * Get a field's RTE type for RTE fields 
	 * and the empty string for others 
	 * @param {String} $field 	A field's name
	 * @return {String}
	 */
	function getRTEType($field)
	{
		if($this->isUseRTEBasic($field))
		{
			return "RTE";
		}
		if($this->isUseRTEFCK($field))
		{
			return "RTECK";		
		}	
		if($this->isUseRTEInnova($field))
		{
			return "RTEINNOVA";
		}
		return "";
	}

	/**
	 * Get the list of the fields that must be hidden on the List page on a particular device
	 * @param {Number} Device code : 1-4 
	 * @return {Array} array in the form of: $arr[$field]=true, where the $field must be hidden.
	 */
	function getHiddenFields($device)
	{
		$list = $this->getTableData( ".hideMobileList" );
		if( isset( $list[$device] ) )
			return $list[$device];
		return array();
	}

	/**
	 * Build CSS media clause for given device code. 
	 * Thanks to mobile device vendors, there are plenty of weird heuristics here, take it easy.
	 */	
	static function getDeviceMediaClause($device)
	{
		if( $device == DESKTOP )
		{
			//	Desktop. We don't care about prehistoric VGA displays
			return "@media (min-device-width: 1280px) and (min-device-height: 1024px), (min-device-width: 1360px)";
		}
		else if( $device == TABLET_10_IN )	//	10" tablets
		{
			// All iPads including mini go here, because there is no way to tell full-sized iPad from mini.
			return "@media (device-width: 768px) and (device-height: 1024px)".

			// The rest of contemporary 10" devices supposedly go here
				" , (min-device-width: 1025px) and (max-device-width: 1280px) and (max-device-height: 1023px) , (min-device-height: 1025px) and (max-device-height: 1280px) and (max-device-width: 1023px)";
		}
		else if( $device == TABLET_7_IN )
		{
			return "@media (max-device-width: 1024px) and (max-device-height: 800px) , (max-device-height: 1024px) and (max-device-width: 800px)";
		}

		else if( $device == SMARTPHONE_LANDSCAPE )
		{
			return "@media (orientation: landscape) and (max-device-height: 400px), (orientation: landscape) and (max-device-width: 400px)";
		}		

		else if( $device == SMARTPHONE_PORTRAIT )
		{
			return "@media (orientation: portrait) and (max-device-height: 400px), (orientation: portrait) and (max-device-width: 400px)";
		}		
	}
	 
	/**
	 * @return Mixed
	 */
	public static function getForLogin()
	{
		return new ProjectSettings("system_user", PAGE_LIST);
	}
	
	/** 
	 * Returns the list of the dashboard search fields. 
	 * Each field is an array of "table", "field" pairs.
	 */
	function getDashboardSearchFields() 
	{
		return $this->getTableData(".searchFields");
	}
	/** 
	 * Returns the list of the dashboard elements
	 * Each element is represented by the array of "name", "table" and "type"
	 */
	function getDashboardElements() 
	{
		return $this->getTableData(".dashElements");
	}
	
	/**
	 * @param String dashElementName
	 * @return Array
	 */
	function getDashboardElementData( $dashElementName )
	{
		$dElements = $this->getTableData(".dashElements");
		foreach( $dElements as $dElemData )
		{
			if( $dElemData["elementName"] == $dashElementName ) 
				return $dElemData;
		}
		return array();
	}
	
	/**
	 * @return Number
	 */
	function getAfterAddAction()
	{
		return $this->getTableData(".afterAddAction");
	}
	
	/**
	 * @return String
	 */	
	function getAADetailTable()
	{
		return $this->getTableData(".afterAddActionDetTable");
	}
	
	/**
	 * @return Boolean
	 */
	function checkClosePopupAfterAdd()
	{
		return $this->getTableData(".closePopupAfterAdd");
	}
	
	/**
	 * @return Number
	 */
	function getAfterEditAction()
	{
		return $this->getTableData(".afterEditAction");
	}
	
	/**
	 * @return String
	 */
	function getAEDetailTable()
	{
		return $this->getTableData(".afterEditActionDetTable");
	}	
	
	/**
	 * @return Boolean
	 */
	function checkClosePopupAfterEdit()
	{
		return $this->getTableData(".closePopupAfterEdit");
	}	
	
	function getMapIcon($field)
	{
		if( !$this->isMapIconCustom($field) ) {
			$mapData = $this->getMapData($field);
			return $mapData["mapIcon"];
		}
		else {
			return getCustomMapIcon($field);
		}
	}
	
	function isMapIconCustom($field)
	{
		$mapData = $this->getMapData($field);
		return $mapData["isMapIconCustom"];
	}
}

$pageTypesForView = array();
$pageTypesForView[] = "list";
$pageTypesForView[] = "view";
$pageTypesForView[] = "export";
$pageTypesForView[] = "print";
$pageTypesForView[] = "report";
$pageTypesForView[] = "rprint";
$pageTypesForView[] = "chart";

$pageTypesForEdit = array();
$pageTypesForEdit[] = "add";
$pageTypesForEdit[] = "edit";
$pageTypesForEdit[] = "search";
$pageTypesForEdit[] = "register";

//	return table short name
function GetTableURL($table = "")
{
	global $strTableName, $projectTables;
	if(!$table)
		$table=$strTableName;
	if("business_partner" == $table) 
		return "business_partner";
	if("item" == $table) 
		return "item";
	if("credit_terms" == $table) 
		return "credit_terms";
	if("shipping_terms" == $table) 
		return "shipping_terms";
	if("cash_receipt" == $table) 
		return "cash_receipt";
	if("purchase" == $table) 
		return "purchase";
	if("cash_disbursment" == $table) 
		return "cash_disbursment";
	if("general_journal" == $table) 
		return "general_journal";
	if("report_center" == $table) 
		return "report_center";
	if("Sales Report" == $table) 
		return "Sales_Report";
	if("Disbursement Reports" == $table) 
		return "Disbursement_Reports";
	if("Financial Reports" == $table) 
		return "Financial_Reports";
	if("Purchase Reports" == $table) 
		return "Purchase_Reports";
	if("Receivable Reports" == $table) 
		return "Receivable_Reports";
	if("Taxation Reports" == $table) 
		return "Taxation_Reports";
	if("system_user" == $table) 
		return "system_user";
	if("fiscal_year" == $table) 
		return "fiscal_year";
	if("beg_bal" == $table) 
		return "beg_bal";
	if("ewt" == $table) 
		return "ewt";
	if("bank" == $table) 
		return "bank";
	if("cost_center" == $table) 
		return "cost_center";
	if("General Journal Reports" == $table) 
		return "General_Journal_Reports";
	if("Inventory Reports" == $table) 
		return "Inventory_Reports";
	if("accounting_period" == $table) 
		return "accounting_period";
	if("bank_recon" == $table) 
		return "bank_recon";
	if("deposit" == $table) 
		return "deposit";
	if("account_classification" == $table) 
		return "account_classification";
	if("chart_of_accounts" == $table) 
		return "chart_of_accounts";
	if("recon_CR" == $table) 
		return "recon_CR";
	if("Recon_deposit" == $table) 
		return "Recon_deposit";
	if("Recon_CV" == $table) 
		return "Recon_CV";
	if("Recon_GV_dep" == $table) 
		return "Recon_GV_dep";
	if("Recon_GV_CV" == $table) 
		return "Recon_GV_CV";
	if("vat_atc" == $table) 
		return "vat_atc";
	if("rdo" == $table) 
		return "rdo";
	if("2550q" == $table) 
		return "2550q";
	if("2550m" == $table) 
		return "2550m";
	if("dr" == $table) 
		return "dr";
	if("purchase_order" == $table) 
		return "purchase_order";
	if("purchase_order_receiving" == $table) 
		return "purchase_order_receiving";
	if("System_preference" == $table) 
		return "System_preference";
	if("ppe_list" == $table) 
		return "ppe_list";
	if("discounts" == $table) 
		return "discounts";
	if("item_group" == $table) 
		return "item_group";
	if("location" == $table) 
		return "location";
	if("meetings" == $table) 
		return "meetings";
	if("calls" == $table) 
		return "calls";
	if("campaign" == $table) 
		return "campaign";
	if("reimbursement" == $table) 
		return "reimbursement";
	if("reimbursement_detail" == $table) 
		return "reimbursement_detail";
	if("Target_customers" == $table) 
		return "Target_customers";
	if("quotation" == $table) 
		return "quotation";
	if("case" == $table) 
		return "case";
	if("projects" == $table) 
		return "projects";
	if("production" == $table) 
		return "production";
	if("issue" == $table) 
		return "issue";
	if("issue_detail" == $table) 
		return "issue_detail";
	if("disposed" == $table) 
		return "disposed";
	if("disposed_detail" == $table) 
		return "disposed_detail";
	if("backload" == $table) 
		return "backload";
	if("backload_detail" == $table) 
		return "backload_detail";
	if("vehicle" == $table) 
		return "vehicle";
	if("payroll_item" == $table) 
		return "payroll_item";
	if("uom" == $table) 
		return "uom";
	if("trip_ticket" == $table) 
		return "trip_ticket";
	if("accident" == $table) 
		return "accident";
	if("approver_list" == $table) 
		return "approver_list";
	if("area" == $table) 
		return "area";
	if("attendance" == $table) 
		return "attendance";
	if("currency" == $table) 
		return "currency";
	if("medical" == $table) 
		return "medical";
	if("position" == $table) 
		return "position";
	if("service_center" == $table) 
		return "service_center";
	if("seminar" == $table) 
		return "seminar";
	if("service_report" == $table) 
		return "service_report";
	if("service_report_detail" == $table) 
		return "service_report_detail";
	if("tax_status" == $table) 
		return "tax_status";
	if("training" == $table) 
		return "training";
	if("competitor" == $table) 
		return "competitor";
	if("campaign_type" == $table) 
		return "campaign_type";
	if("fixed_asset_return_detail" == $table) 
		return "fixed_asset_return_detail";
	if("fixed_assets_return" == $table) 
		return "fixed_assets_return";
	if("Prospect_customers" == $table) 
		return "Prospect_customers";
	if("trip_detail" == $table) 
		return "trip_detail";
	if("beg_baldetail" == $table) 
		return "beg_baldetail";
	if("cash_disbursement_detail2" == $table) 
		return "cash_disbursement_detail2";
	if("case_solutions" == $table) 
		return "case_solutions";
	if("cash_disbursment_detail" == $table) 
		return "cash_disbursment_detail";
	if("cash_receipt_detail" == $table) 
		return "cash_receipt_detail";
	if("cash_receipt_detail2" == $table) 
		return "cash_receipt_detail2";
	if("general_journal_detail" == $table) 
		return "general_journal_detail";
	if("employee_list" == $table) 
		return "employee_list";
	if("petty_cash" == $table) 
		return "petty_cash";
	if("petty_cash_detail" == $table) 
		return "petty_cash_detail";
	if("center_replenishment" == $table) 
		return "center_replenishment";
	if("center_replenishment_detail" == $table) 
		return "center_replenishment_detail";
	if("credit_memo" == $table) 
		return "credit_memo";
	if("credit_memo_detail" == $table) 
		return "credit_memo_detail";
	if("debit_memo" == $table) 
		return "debit_memo";
	if("debit_memo_detail" == $table) 
		return "debit_memo_detail";
	if("project_replenishment" == $table) 
		return "project_replenishment";
	if("project_replenishment_detail" == $table) 
		return "project_replenishment_detail";
	if("business_partner_group" == $table) 
		return "business_partner_group";
	if("Sales_Representatives" == $table) 
		return "Sales_Representatives";
	if("Types_of_leaves" == $table) 
		return "Types_of_leaves";
	if("payroll_mode" == $table) 
		return "payroll_mode";
	if("payroll_account_type" == $table) 
		return "payroll_account_type";
	if("Types_of_Overtime" == $table) 
		return "Types_of_Overtime";
	if("Types_of_Loan" == $table) 
		return "Types_of_Loan";
	if("sales" == $table) 
		return "sales";
	if("sales_detail" == $table) 
		return "sales_detail";
	if("Payroll_Company_Setup" == $table) 
		return "Payroll_Company_Setup";
	if("item_bill_of_materials" == $table) 
		return "item_bill_of_materials";
	if("item_uom_conversion" == $table) 
		return "item_uom_conversion";
	if("tranasactions_for_approval" == $table) 
		return "tranasactions_for_approval";
	if("deposit_detail" == $table) 
		return "deposit_detail";
	if("purchase_detail" == $table) 
		return "purchase_detail";
	if("dr_detail" == $table) 
		return "dr_detail";
	if("purchase_order_detail" == $table) 
		return "purchase_order_detail";
	if("Sales_Order_for_checking" == $table) 
		return "Sales_Order_for_checking";
	if("sales_order_detail_for_checking" == $table) 
		return "sales_order_detail_for_checking";
	if("Sales_Order_for_approval" == $table) 
		return "Sales_Order_for_approval";
	if("sales_order_detail_for_approval" == $table) 
		return "sales_order_detail_for_approval";
	if("Quotation_for_approval" == $table) 
		return "Quotation_for_approval";
	if("Quotation_for_checking" == $table) 
		return "Quotation_for_checking";
	if("quotation_detail_for_checking" == $table) 
		return "quotation_detail_for_checking";
	if("quotation_detail_for_approval" == $table) 
		return "quotation_detail_for_approval";
	if("Reimbursement_for_checking" == $table) 
		return "Reimbursement_for_checking";
	if("reimbursement_detail_for_checking" == $table) 
		return "reimbursement_detail_for_checking";
	if("Reimbursement_for_approval" == $table) 
		return "Reimbursement_for_approval";
	if("reimbursement_detail_for_approval" == $table) 
		return "reimbursement_detail_for_approval";
	if("Project_Replenishment_for_checking" == $table) 
		return "Project_Replenishment_for_checking";
	if("project_replenishment_detail_for_checking" == $table) 
		return "project_replenishment_detail_for_checking";
	if("Project_Replenishment_for_approval" == $table) 
		return "Project_Replenishment_for_approval";
	if("project_replenishment_detail_for_approval" == $table) 
		return "project_replenishment_detail_for_approval";
	if("center_replenishment_detail_for_checking" == $table) 
		return "center_replenishment_detail_for_checking";
	if("center_replenishment_detail_for_approval" == $table) 
		return "center_replenishment_detail_for_approval";
	if("Purchases_Order_for_checking2" == $table) 
		return "Purchases_Order_for_checking2";
	if("purchase_order_detail_for_checking" == $table) 
		return "purchase_order_detail_for_checking";
	if("Purchases_Order_for_approval2" == $table) 
		return "Purchases_Order_for_approval2";
	if("purchase_order_detail_for_approval" == $table) 
		return "purchase_order_detail_for_approval";
	if("Receiving_for_checking2" == $table) 
		return "Receiving_for_checking2";
	if("purchase_order_receiving_detail_for_checking" == $table) 
		return "purchase_order_receiving_detail_for_checking";
	if("Receiving_for_approval2" == $table) 
		return "Receiving_for_approval2";
	if("purchase_order_receiving_detail_for_approval" == $table) 
		return "purchase_order_receiving_detail_for_approval";
	if("Delivery_Receipt_for_checking" == $table) 
		return "Delivery_Receipt_for_checking";
	if("dr_detail_for_checking" == $table) 
		return "dr_detail_for_checking";
	if("Delivery_Receipt_for_approval" == $table) 
		return "Delivery_Receipt_for_approval";
	if("dr_detail_for_approval" == $table) 
		return "dr_detail_for_approval";
	if("Production_for_checking" == $table) 
		return "Production_for_checking";
	if("production_detail_for_checking" == $table) 
		return "production_detail_for_checking";
	if("Production_for_approval" == $table) 
		return "Production_for_approval";
	if("production_detail_for_approval" == $table) 
		return "production_detail_for_approval";
	if("Issue_for_checking" == $table) 
		return "Issue_for_checking";
	if("supply_issue_detail_for_checking" == $table) 
		return "supply_issue_detail_for_checking";
	if("Issue_for_approval" == $table) 
		return "Issue_for_approval";
	if("supply_issue_detail_for_approval" == $table) 
		return "supply_issue_detail_for_approval";
	if("Return_for_checking" == $table) 
		return "Return_for_checking";
	if("supply_return_detail_for_checking" == $table) 
		return "supply_return_detail_for_checking";
	if("Return_for_approval" == $table) 
		return "Return_for_approval";
	if("supply_return_detail_approval" == $table) 
		return "supply_return_detail_approval";
	if("supply_disposed_detail_for_checking" == $table) 
		return "supply_disposed_detail_for_checking";
	if("Disposal_for_approval" == $table) 
		return "Disposal_for_approval";
	if("supply_disposed_detail_for_approval" == $table) 
		return "supply_disposed_detail_for_approval";
	if("Fixed_Asset_Issue_for_checking" == $table) 
		return "Fixed_Asset_Issue_for_checking";
	if("Fixed_Asset_Issue_for_approval" == $table) 
		return "Fixed_Asset_Issue_for_approval";
	if("fixed_asset_issue_detail_for_approval" == $table) 
		return "fixed_asset_issue_detail_for_approval";
	if("fixed_asset_issue_detail_for_checking" == $table) 
		return "fixed_asset_issue_detail_for_checking";
	if("Fixed_Asset_Return_for_checking" == $table) 
		return "Fixed_Asset_Return_for_checking";
	if("fixed_asset_return_detail_for_checking" == $table) 
		return "fixed_asset_return_detail_for_checking";
	if("Fixed_Asset_Return_for_approval" == $table) 
		return "Fixed_Asset_Return_for_approval1";
	if("fixed_asset_return_detail_for_approval" == $table) 
		return "fixed_asset_return_detail_for_approval";
	if("Petty_Cash_for_checking" == $table) 
		return "Petty_Cash_for_checking";
	if("petty_cash_detail_for_checking" == $table) 
		return "petty_cash_detail_for_checking";
	if("Petty_Cash_for_approval" == $table) 
		return "Petty_Cash_for_approval";
	if("petty_cash_detail_for_approval" == $table) 
		return "petty_cash_detail_for_approval";
	if("Deposit_for_checking" == $table) 
		return "Deposit_for_checking";
	if("deposit_detail_for_checking" == $table) 
		return "deposit_detail_for_checking";
	if("Deposit_for_approval" == $table) 
		return "Deposit_for_approval";
	if("deposit_detail_for_approval" == $table) 
		return "deposit_detail_for_approval";
	if("Credit_Memo_for_checking" == $table) 
		return "Credit_Memo_for_checking";
	if("credit_memo_detail_for_checking" == $table) 
		return "credit_memo_detail_for_checking";
	if("Credit_Memo_for_approval" == $table) 
		return "Credit_Memo_for_approval";
	if("credit_memo_detail_for_approval" == $table) 
		return "credit_memo_detail_for_approval";
	if("Debit_Memo_for_checking" == $table) 
		return "Debit_Memo_for_checking";
	if("debit_memo_detail_for_checking" == $table) 
		return "debit_memo_detail_for_checking";
	if("debit_memo_for_approval" == $table) 
		return "debit_memo_for_approval";
	if("debit_memo_detail_for_approval" == $table) 
		return "debit_memo_detail_for_approval";
	if("Purchases_for_checking" == $table) 
		return "Purchases_for_checking";
	if("credit_purchase_detail_for_checking" == $table) 
		return "credit_purchase_detail_for_checking";
	if("Purchases_for_approval" == $table) 
		return "Purchases_for_approval";
	if("credit_purchase_detail_for_approval" == $table) 
		return "credit_purchase_detail_for_approval";
	if("Sales_for_checking" == $table) 
		return "Sales_for_checking";
	if("Sales_for_approval" == $table) 
		return "Sales_for_approval";
	if("credit_sales_detail_for_checking" == $table) 
		return "credit_sales_detail_for_checking";
	if("credit_sales_detail_for_approval" == $table) 
		return "credit_sales_detail_for_approval";
	if("Cash_Disbursement_for_checking" == $table) 
		return "Cash_Disbursement_for_checking";
	if("check_voucher_detail_for_checking" == $table) 
		return "check_voucher_detail_for_checking";
	if("check_voucher_detail2_for_checking" == $table) 
		return "check_voucher_detail2_for_checking";
	if("Cash_Disbursement_for_approval" == $table) 
		return "Cash_Disbursement_for_approval";
	if("check_voucher_detail_for_approval" == $table) 
		return "check_voucher_detail_for_approval";
	if("check_voucher_detail2_for_approval" == $table) 
		return "check_voucher_detail2_for_approval";
	if("document" == $table) 
		return "document";
	if("General_Journal_for_checking" == $table) 
		return "General_Journal_for_checking";
	if("general_journal_detail_for_checking" == $table) 
		return "general_journal_detail_for_checking";
	if("General_Journal_for_approval" == $table) 
		return "General_Journal_for_approval";
	if("general_journal_detail_for_approval" == $table) 
		return "general_journal_detail_for_approval";
	if("for_submissions" == $table) 
		return "for_submissions";
	if("submission_details" == $table) 
		return "submission_details";
	if("sales_order_detail_billing_dates" == $table) 
		return "sales_order_detail_billing_dates";
	if("Submission_Search" == $table) 
		return "Submission_Search";
	if("Submission_Reminders" == $table) 
		return "Submission_Reminders";
	if("years" == $table) 
		return "years";
	if("document_type" == $table) 
		return "document_type";
	if("estimate" == $table) 
		return "estimate";
	if("estimate_detail" == $table) 
		return "estimate_detail";
	if("purchase_request" == $table) 
		return "purchase_request";
	if("purchase_request_detail" == $table) 
		return "purchase_request_detail";
	if("inv_adj_detail" == $table) 
		return "inv_adj_detail";
	if("inventory_adj" == $table) 
		return "inventory_adj";
	if("inventory_tranfer" == $table) 
		return "inventory_tranfer";
	if("inventory_transfer_detail" == $table) 
		return "inventory_transfer_detail";
	if("Customers" == $table) 
		return "Customers";
	if("Suppliers" == $table) 
		return "Suppliers";
	if("Target_contacts" == $table) 
		return "Target_contacts";
	if("Prospect_contacts" == $table) 
		return "Prospect_contacts";
	if("Customer_Contacts" == $table) 
		return "Customer_Contacts";
	if("Supplier_Contacts" == $table) 
		return "Supplier_Contacts";
	if("Statement_of_Account" == $table) 
		return "Statement_of_Account";
	if("Change_Time_Schedule_for_approval" == $table) 
		return "Change_Time_Schedule_for_approval";
	if("Change_Time_Schedule_for_checking" == $table) 
		return "Change_Time_Schedule_for_checking";
	if("Inventory_Adjustment_for_checking" == $table) 
		return "Inventory_Adjustment_for_checking";
	if("inv_adj_detail_for_checking" == $table) 
		return "inv_adj_detail_for_checking";
	if("Inventory_Adjustment_for_approval" == $table) 
		return "Inventory_Adjustment_for_approval";
	if("inv_adj_detail_for_approval" == $table) 
		return "inv_adj_detail_for_approval";
	if("for_checking" == $table) 
		return "for_checking";
	if("for_approval" == $table) 
		return "for_approval";
	if("Estimate_for_approval" == $table) 
		return "Estimate_for_approval";
	if("Estimate_for_checking" == $table) 
		return "Estimate_for_checking";
	if("estimate_detail_for_checking" == $table) 
		return "estimate_detail_for_checking";
	if("estimate_detail_for_approval" == $table) 
		return "estimate_detail_for_approval";
	if("Follow_up" == $table) 
		return "Follow_up";
	if("coming_meetings" == $table) 
		return "coming_meetings";
	if("tasks" == $table) 
		return "tasks";
	if("Project_phases" == $table) 
		return "Project_phases";
	if("origin_fa" == $table) 
		return "origin_fa";
	if("origin_serialized_supply" == $table) 
		return "origin_serialized_supply";
	if("2301" == $table) 
		return "2301";
	if("Prospect_customers_to_Customer" == $table) 
		return "Prospect_customers_to_Customer";
	if("lyl_estimate" == $table) 
		return "lyl_estimate";
	if("lyl_estimate_detail" == $table) 
		return "lyl_estimate_detail";
	if("lyl_estimate_detail_accessories" == $table) 
		return "lyl_estimate_detail_accessories";
	if("lyl_estimate_detail_aluminum" == $table) 
		return "lyl_estimate_detail_aluminum";
	if("lyl_estimate_detail_e_accessories" == $table) 
		return "lyl_estimate_detail_e_accessories";
	if("lyl_estimate_detail_glass" == $table) 
		return "lyl_estimate_detail_glass";
	if("recuring_benefits" == $table) 
		return "recuring_benefits";
	if("recuring_deductions" == $table) 
		return "recuring_deductions";
	if("recurring_benefits_detail" == $table) 
		return "recurring_benefits_detail";
	if("recurring_deductions_detail" == $table) 
		return "recurring_deductions_detail";
	if("production_request" == $table) 
		return "production_request";
	if("production_request_detail" == $table) 
		return "production_request_detail";
	if("attendance_summary" == $table) 
		return "attendance_summary";
	if("Other_Incentives" == $table) 
		return "Other_Incentives";
	if("Other_Deductions" == $table) 
		return "Other_Deductions";
	if("department" == $table) 
		return "department";
	if("journal_entry_cash_disbursement" == $table) 
		return "journal_entry_cash_disbursement";
	if("journal_entry_cash_receipt" == $table) 
		return "journal_entry_cash_receipt";
	if("journal_entry_center_replenishment" == $table) 
		return "journal_entry_center_replenishment";
	if("journal_entry_credit_memo" == $table) 
		return "journal_entry_credit_memo";
	if("journal_entry_debit_memo" == $table) 
		return "journal_entry_debit_memo";
	if("journal_entry_depreciation" == $table) 
		return "journal_entry_depreciation";
	if("journal_entry_general_journal" == $table) 
		return "journal_entry_general_journal";
	if("journal_entry_inventory_adjustment" == $table) 
		return "journal_entry_inventory_adjustment";
	if("journal_entry_petty_cash" == $table) 
		return "journal_entry_petty_cash";
	if("journal_entry_project_replenishment" == $table) 
		return "journal_entry_project_replenishment";
	if("journal_entry_purchases" == $table) 
		return "journal_entry_purchases";
	if("journal_entry_replenishment" == $table) 
		return "journal_entry_replenishment";
	if("journal_entry_service_revenue" == $table) 
		return "journal_entry_service_revenue";
	if("Credit_Card_monitoring" == $table) 
		return "Credit_Card_monitoring";
	if("sales_report_per_item Chart" == $table) 
		return "sales_report_per_item_Chart";
	if("Leave_for_approval" == $table) 
		return "Leave_for_approval";
	if("loan_for_approval" == $table) 
		return "loan_for_approval";
	if("loan_detail_for_approval" == $table) 
		return "loan_detail_for_approval";
	if("Overtime_for_approval" == $table) 
		return "Overtime_for_approval";
	if("Purchase_Request_for_approval" == $table) 
		return "Purchase_Request_for_approval";
	if("purchase_request_detail_approval" == $table) 
		return "purchase_request_detail_approval";
	if("Production_Request_for_approval" == $table) 
		return "Production_Request_for_approval";
	if("production_request_detail_for_approval" == $table) 
		return "production_request_detail_for_approval";
	if("Inventory_Transfer_for_approval" == $table) 
		return "Inventory_Transfer_for_approval";
	if("inventory_transfer_detail_for_approval" == $table) 
		return "inventory_transfer_detail_for_approval";
	if("Leave_for_checking" == $table) 
		return "Leave_for_checking";
	if("Loan_for_checking" == $table) 
		return "Loan_for_checking";
	if("loan_detail_for_checking" == $table) 
		return "loan_detail_for_checking";
	if("Overtime_for_checking" == $table) 
		return "Overtime_for_checking";
	if("Purchase_Request_for_checking" == $table) 
		return "Purchase_Request_for_checking";
	if("Production_Request_for_checking" == $table) 
		return "Production_Request_for_checking";
	if("Inventory_Transfer_for_checking" == $table) 
		return "Inventory_Transfer_for_checking";
	if("inventory_transfer_detail_for_checking" == $table) 
		return "inventory_transfer_detail_for_checking";
	if("attendance_type" == $table) 
		return "attendance_type";
	if("Backload_for_approval" == $table) 
		return "Backload_for_approval";
	if("backload_detail_for_approval" == $table) 
		return "backload_detail_for_approval";
	if("Backload_for_checking" == $table) 
		return "Backload_for_checking";
	if("backload_detail_for_checking" == $table) 
		return "backload_detail_for_checking";
	if("Cash_Receipt_for_approval" == $table) 
		return "Cash_Receipt_for_approval";
	if("Cash_Receipt_for_checking" == $table) 
		return "Cash_Receipt_for_checking";
	if("cash_receipt_detail_for_checking" == $table) 
		return "cash_receipt_detail_for_checking";
	if("cash_receipt_detail_for_approval" == $table) 
		return "cash_receipt_detail_for_approval";
	if("Payroll_Entry_for_approval" == $table) 
		return "Payroll_Entry_for_approval";
	if("Payroll_Entry_for_checking" == $table) 
		return "Payroll_Entry_for_checking";
	if("payroll_entry_detail_for_approval" == $table) 
		return "payroll_entry_detail_for_approval";
	if("payroll_entry_detail_for_checking" == $table) 
		return "payroll_entry_detail_for_checking";
	if("Process_Payroll_for_approval" == $table) 
		return "Process_Payroll_for_approval";
	if("Process_Payroll_for_checking" == $table) 
		return "Process_Payroll_for_checking";
	if("process_payroll_detail_for_approval" == $table) 
		return "process_payroll_detail_for_approval";
	if("process_payroll_detail_for_checking" == $table) 
		return "process_payroll_detail_for_checking";
	if("Service_Report_for_approval" == $table) 
		return "Service_Report_for_approval";
	if("Service_Report_for_checking" == $table) 
		return "Service_Report_for_checking";
	if("service_report_detail_for_approval" == $table) 
		return "service_report_detail_for_approval";
	if("service_report_detail_for_checking" == $table) 
		return "service_report_detail_for_checking";
	if("Banch Reports" == $table) 
		return "Banch_Reports";
	if("journal_entry_production" == $table) 
		return "journal_entry_production";
	if("journal_entry_deposit" == $table) 
		return "journal_entry_deposit";
	if("production_request_bom" == $table) 
		return "production_request_bom";
	if("production_bom" == $table) 
		return "production_bom";
	if("company_info" == $table) 
		return "company_info";
	if("return" == $table) 
		return "return";
	if("return_detail" == $table) 
		return "return_detail";
	if("fixed_asset_issue" == $table) 
		return "fixed_asset_issue";
	if("fixed_asset_issue_detail" == $table) 
		return "fixed_asset_issue_detail";
	if("production_request_detail_for_checking" == $table) 
		return "production_request_detail_for_checking";
	if("cash_disbursment_detail_for_approval" == $table) 
		return "cash_disbursment_detail_for_approval";
	if("cash_disbursment_detail_for_checking" == $table) 
		return "cash_disbursment_detail_for_checking";
	if("purchase_request_detail_for_checking" == $table) 
		return "purchase_request_detail_for_checking";
	if("trip_ticket_for_approval" == $table) 
		return "trip_ticket_for_approval";
	if("trip_ticket_for_checking" == $table) 
		return "trip_ticket_for_checking";
	if("trip_detail_for_approval" == $table) 
		return "trip_detail_for_approval";
	if("trip_detail_for_checking" == $table) 
		return "trip_detail_for_checking";
	if("sales_report" == $table) 
		return "sales_report1";
	if("Customer_Balance_Report" == $table) 
		return "Customer_Balance_Report";
	if("Customer_Balance_Report_detail" == $table) 
		return "Customer_Balance_Report_detail";
	if("Collection_Report_per_Customer" == $table) 
		return "Collection_Report_per_Customer";
	if("Aging_of_AR_Report" == $table) 
		return "Aging_of_AR_Report";
	if("Aging_of_AR_Report_detail" == $table) 
		return "Aging_of_AR_Report_detail";
	if("Payment_per_Supplier_Report" == $table) 
		return "Payment_per_Supplier_Report";
	if("Aging of_AP_Report" == $table) 
		return "Aging_of_AP_Report";
	if("Aging of_AP_Report_detail" == $table) 
		return "Aging_of_AP_Report_detail";
	if("Inventory_Balance_Report" == $table) 
		return "Inventory_Balance_Report";
	if("Inventory_Valuation_Report" == $table) 
		return "Inventory_Valuation_Report";
	if("Inventory_Below_Reorder_Point_Report" == $table) 
		return "Inventory_Below_Reorder_Point_Report";
	if("Sales_Report_per_BP" == $table) 
		return "Sales_Report_per_BP";
	if("Sales_Report_per_Item" == $table) 
		return "Sales_Report_per_Item";
	if("Sales_Report_per_Cost_Center" == $table) 
		return "Sales_Report_per_Cost_Center";
	if("Sales_Report_per_BP_detail" == $table) 
		return "Sales_Report_per_BP_detail";
	if("Sales_Report_per_Item_detail" == $table) 
		return "Sales_Report_per_Item_detail";
	if("Sales_Report_per_Cost_Center_detail" == $table) 
		return "Sales_Report_per_Cost_Center_detail";
	if("Purchase_Report" == $table) 
		return "Purchase_Report";
	if("Purchase_Report_per_BP" == $table) 
		return "Purchase_Report_per_BP";
	if("Purchase_Report_per_Item" == $table) 
		return "Purchase_Report_per_Item";
	if("Purchase_Report_per_Cost_Center" == $table) 
		return "Purchase_Report_per_Cost_Center";
	if("Purchase_Report_per_BP_detail" == $table) 
		return "Purchase_Report_per_BP_detail";
	if("Purchase_Report_per_Item_detail" == $table) 
		return "Purchase_Report_per_Item_detail";
	if("Purchase_Report_per_Cost_Center_detail" == $table) 
		return "Purchase_Report_per_Cost_Center_detail";
	if("Transaction Registry" == $table) 
		return "Transaction_Registry";
	if("Sales_Report_per_sales_rep" == $table) 
		return "Sales_Report_per_sales_rep";
	if("PDC_Registry_Report" == $table) 
		return "PDC_Registry_Report";
	if("Available_Vehicles" == $table) 
		return "Available_Vehicles1";
	if("Trip_Ticket_Report" == $table) 
		return "Trip_Ticket_Report";
	if("Purchase_Request_Report" == $table) 
		return "Purchase_Request_Report";
	if("Purchase_Import_Costing" == $table) 
		return "Purchase_Import_Costing";
	if("Production_Costing_Report" == $table) 
		return "Production_Costing_Report";
	if("Production_Report" == $table) 
		return "Production_Report";
	if("Closed_Opportunities" == $table) 
		return "Closed_Opportunities";
	if("Cases_with_Sotutions" == $table) 
		return "Cases_with_Sotutions";
	if("Totals_Cases_Created" == $table) 
		return "Totals_Cases_Created";
	if("Totals_Open_Cases_by_Agent" == $table) 
		return "Totals_Open_Cases_by_Agent";
	if("Campaign_Contacts" == $table) 
		return "Campaign_Contacts";
	if("Campaign_leads" == $table) 
		return "Campaign_leads";
	if("Campaign_ROI_Analysis" == $table) 
		return "Campaign_ROI_Analysis";
	if("Campaign_Revenue" == $table) 
		return "Campaign_Revenue";
	if("Opportunity_Schedule" == $table) 
		return "Opportunity_Schedule";
	if("Sales_by_Account" == $table) 
		return "Sales_by_Account";
	if("Sales_by_Lead_Source" == $table) 
		return "Sales_by_Lead_Source";
	if("Sales_by_Partner" == $table) 
		return "Sales_by_Partner";
	if("Sales_by_Rep" == $table) 
		return "Sales_by_Rep";
	if("Leave_Ledger_Report" == $table) 
		return "Leave_Ledger_Report";
	if("Leave_Summary" == $table) 
		return "Leave_Summary";
	if("Payroll_Register" == $table) 
		return "Payroll_Register";
	if("Employee_Leave_History" == $table) 
		return "Employee_Leave_History";
	if("Employee_Register" == $table) 
		return "Employee_Register";
	if("DR_Reports" == $table) 
		return "DR_Reports";
	if("Sales_Order_Report" == $table) 
		return "Sales_Order_Report";
	if("Case_History" == $table) 
		return "Case_History";
	if("quality_control" == $table) 
		return "quality_control";
	if("quality_control_detail" == $table) 
		return "quality_control_detail";
	if("budget" == $table) 
		return "budget";
	if("budget_detail" == $table) 
		return "budget_detail";
	if("repairs_and_maintenance" == $table) 
		return "repairs_and_maintenance";
	if("production_form" == $table) 
		return "production_form";
	if("bluewave_employees" == $table) 
		return "bluewave_employees";
	if("bluewave_employees_detail" == $table) 
		return "bluewave_employees_detail";
	if("bluewave_employees_detail2" == $table) 
		return "bluewave_employees_detail2";
	if("bluewave_employees_detail3" == $table) 
		return "bluewave_employees_detail3";
	if("bluewave_employees_detail4" == $table) 
		return "bluewave_employees_detail4";
	if("bluewave_employees_detail5" == $table) 
		return "bluewave_employees_detail5";
	if("bluewave_employees_detail6" == $table) 
		return "bluewave_employees_detail6";
	if("rfp" == $table) 
		return "rfp";
	if("telex" == $table) 
		return "telex";
	if("repairs_and_maintenance_detail" == $table) 
		return "repairs_and_maintenance_detail";
	if("repair_list" == $table) 
		return "repair_list";
	if("marketing_colateral" == $table) 
		return "marketing_colateral";
	if("email" == $table) 
		return "email";
	if("accessories" == $table) 
		return "accessories";
	if("sales_order_materials_detail" == $table) 
		return "sales_order_materials_detail";
	if("sales_order_progress_billing_detail" == $table) 
		return "sales_order_progress_billing_detail";
	if("sub_contract" == $table) 
		return "sub_contract";
	if("sub_contract_billing_detail" == $table) 
		return "sub_contract_billing_detail";
	if("sub_contract_detail" == $table) 
		return "sub_contract_detail";
	if("Monthly_Pag-ibig_Summary_Report" == $table) 
		return "Monthly_Pag_ibig_Summary_Report";
	if("Monthly_Philhealth_Report" == $table) 
		return "Monthly_Philhealth_Report";
	if("Monthly_Philhealth_Summary_Report" == $table) 
		return "Monthly_Philhealth_Summary_Report";
	if("Monthly_SSS_Report" == $table) 
		return "Monthly_SSS_Report";
	if("Monthly_SSS_Report_by_Department_Report" == $table) 
		return "Monthly_SSS_Report_by_Department_Report";
	if("Monthly_Withholding_Tax_Prooflist_Report" == $table) 
		return "Monthly_Withholding_Tax_Prooflist_Report";
	if("Monthly_Witholding_Tax_Collection_List_by_Department_Report" == $table) 
		return "Monthly_Witholding_Tax_Collection_List_by_Department_Report";
	if("Overtime_Summary_Report" == $table) 
		return "Overtime_Summary_Report";
	if("Monthly_Pag-ibig_Report" == $table) 
		return "Monthly_Pag_ibig_Report";
	if("sales_order" == $table) 
		return "sales_order";
	if("sales_order_detail" == $table) 
		return "sales_order_detail";
	if("vat_relief_importation" == $table) 
		return "vat_relief_importation";
	if("vat_relief_purchases" == $table) 
		return "vat_relief_purchases";
	if("vat_relief_sales" == $table) 
		return "vat_relief_sales";
	if("journal_ppe_disposal" == $table) 
		return "journal_ppe_disposal";
	if("credit_memo_detail_cgs" == $table) 
		return "credit_memo_detail_cgs";
	if("marketing_request_form" == $table) 
		return "marketing_request_form";
	if("Quotation_Dashboard_Customer" == $table) 
		return "Quotation_Dashboard_Customer";
	if("Sales_Order_Dashboard_Customer" == $table) 
		return "Sales_Order_Dashboard_Customer";
	if("Delivery_Receipt_Dashboard_Customer" == $table) 
		return "Delivery_Receipt_Dashboard_Customer";
	if("Sales_Dashboard_Customer" == $table) 
		return "Sales_Dashboard_Customer";
	if("Backload_Dashboard_Customer" == $table) 
		return "Backload_Dashboard_Customer";
	if("Purchase_Request_Dashboard_Supplier" == $table) 
		return "Purchase_Request_Dashboard_Supplier";
	if("Purchase_Order_Dashboard_Supplier" == $table) 
		return "Purchase_Order_Dashboard_Supplier";
	if("Purchase_Dashboard_Supplier" == $table) 
		return "Purchase_Dashboard_Supplier";
	if("Cash_Disbursement_Dashboard_Supplier" == $table) 
		return "Cash_Disbursement_Dashboard_Supplier";
	if("Debit_Memo_Dashboard_Supplier" == $table) 
		return "Debit_Memo_Dashboard_Supplier";
	if("Purchase_Request_Dashboard_Item" == $table) 
		return "Purchase_Request_Dashboard_Item1";
	if("Purchase_Order_Dashboard_Item" == $table) 
		return "Purchase_Order_Dashboard_Item";
	if("Purchase_Order_Receiving_Dashboard_Item" == $table) 
		return "Purchase_Order_Receiving_Dashboard_Item";
	if("Production_Dashboard_Item" == $table) 
		return "Production_Dashboard_Item";
	if("sub_contract_for_approval" == $table) 
		return "sub_contract_for_approval";
	if("sub_contract_for_checking" == $table) 
		return "sub_contract_for_checking";
	if("sub_contract_detail_for_approval" == $table) 
		return "sub_contract_detail_for_approval";
	if("sub_contract_detail_for_checking" == $table) 
		return "sub_contract_detail_for_checking";
	if("budget_for_approval" == $table) 
		return "budget_for_approval";
	if("budget_for_checking" == $table) 
		return "budget_for_checking";
	if("budget_detail_for_checking" == $table) 
		return "budget_detail_for_checking";
	if("budget_detail_for_approval" == $table) 
		return "budget_detail_for_approval";
	if("undertime_for_approval" == $table) 
		return "undertime_for_approval";
	if("undertime_for_checking" == $table) 
		return "undertime_for_checking";
	if("request_for_payment_for_approval" == $table) 
		return "request_for_payment_for_approval";
	if("request_for_payment_for_checking" == $table) 
		return "request_for_payment_for_checking";
	if("repairs_and_maintenance_detail_for_approval" == $table) 
		return "repairs_and_maintenance_detail_for_approval";
	if("request_for_payment_detail_for_approval" == $table) 
		return "request_for_payment_detail_for_approval";
	if("request_for_payment_detail_for_checking" == $table) 
		return "request_for_payment_detail_for_checking";
	if("repairs_and_maintenance_for_approval" == $table) 
		return "repairs_and_maintenance_for_approval";
	if("repairs_and_maintenance_for_checking" == $table) 
		return "repairs_and_maintenance_for_checking";
	if("repairs_and_maintenance_detail_for_checking" == $table) 
		return "repairs_and_maintenance_detail_for_checking";
	if("Disposal_for_checking" == $table) 
		return "Disposal_for_checking";
	if("Active_campaign_report" == $table) 
		return "Active_campaign_report";
	if("For_Billing_report" == $table) 
		return "For_Billing_report";
	if("sales_order_for_project_mngmt" == $table) 
		return "sales_order_for_project_mngmt";
	if("inventory_transfer_cost_detail" == $table) 
		return "inventory_transfer_cost_detail";
	if("Supplier_Balance_Report_Details" == $table) 
		return "Supplier_Balance_Report_Details";
	if("Supplier_Balance_Report" == $table) 
		return "Supplier_Balance_Report";
	if("Inventory_per_Cost_center_Report" == $table) 
		return "Inventory_per_Cost_center_Report";
	if("Deposit_Report" == $table) 
		return "Deposit_Report";
	if("SO_for_Billing_Report" == $table) 
		return "SO_for_Billing_Report";
	if("Billing_due_dates_Report" == $table) 
		return "Billing_due_dates_Report";
	if("Packing_List_Report" == $table) 
		return "Packing_List_Report";
	if("Open_Trip_Ticket_Report" == $table) 
		return "Open_Trip_Ticket_Report";
	if("Transfer_Report" == $table) 
		return "Transfer_Report";
	if("Production_Report_invw" == $table) 
		return "Production_Report_invw";
	if("Supply_per_Department__Report_invw" == $table) 
		return "Supply_per_Department__Report_invw";
	if("Item_Balances_Report_invw" == $table) 
		return "Item_Balances_Report_invw";
	if("Open_Case_Report_s" == $table) 
		return "Open_Case_Report_s";
	if("Sales_Order_per_Campaign_Report_marketing" == $table) 
		return "Sales_Order_per_Campaign_Report_marketing";
	if("Sales_per_campaign_Report_marketing" == $table) 
		return "Sales_per_campaign_Report_marketing";
	if("Asset_per_Accountability_Report_invw" == $table) 
		return "Asset_per_Accountability_Report_invw";
	if("Deposit_in_transit_Report" == $table) 
		return "Deposit_in_transit_Report";
	if("Outstanding_Checks_Report" == $table) 
		return "Outstanding_Checks_Report";
	if("Absences_Detail_Report" == $table) 
		return "Absences_Detail_Report";
	if("Absences_Report" == $table) 
		return "Absences_Report";
	if("Attendance_Report" == $table) 
		return "Attendance_Report";
	if("DTR_Report" == $table) 
		return "DTR_Report";
	if("Summary_of_Tardiness_Report" == $table) 
		return "Summary_of_Tardiness_Report";
	if("Followup_Report" == $table) 
		return "Followup_Report";
	if("Coming_Meeting_Report" == $table) 
		return "Coming_Meeting_Report";
	if("Meeting_Report" == $table) 
		return "Meeting_Report";
	if("Next_Action_Report" == $table) 
		return "Next_Action_Report";
	if("Calls_Report" == $table) 
		return "Calls_Report";
	if("sales_per_sales_rep_Report" == $table) 
		return "sales_per_sales_rep_Report";
	if("Expense_per_Project_Report" == $table) 
		return "Expense_per_Project_Report";
	if("Tardiness_Report_Summary" == $table) 
		return "Tardiness_Report_Summary";
	if("Timekeeping_(Hours_Worked)_Report_Summary" == $table) 
		return "Timekeeping__Hours_Worked__Report_Summary";
	if("Withholding_Tax_Collection_List_per_Month_Report" == $table) 
		return "Withholding_Tax_Collection_List_per_Month_Report";
	if("Business_Owner_Dashboard" == $table) 
		return "Business_Owner_Dashboard";
	if("Finance_Dashboard" == $table) 
		return "Finance_Dashboard";
	if("Purchasing_Dashboard" == $table) 
		return "Purchasing_Dashboard";
	if("Production_Dashboard" == $table) 
		return "Production_Dashboard1";
	if("Marketing_Dashboard" == $table) 
		return "Marketing_Dashboard1";
	if("Support_Dashboard" == $table) 
		return "Support_Dashboard";
	if("Sales_Dashboard" == $table) 
		return "Sales_Dashboard";
	if("Human_Resource_Dashboard" == $table) 
		return "Human_Resource_Dashboard";
	if("Project_Manager_Dashboard" == $table) 
		return "Project_Manager_Dashboard";
	if("System_Admistration_Dashboard" == $table) 
		return "System_Admistration_Dashboard";
	if("purchase_report_per_item Chart" == $table) 
		return "purchase_report_per_item_Chart";
	if("System_Audit_Report" == $table) 
		return "System_Audit_Report";
	if("sale_cost_detail" == $table) 
		return "sale_cost_detail";
	if("currency_exchange_rate" == $table) 
		return "currency_exchange_rate";
	if("cash_receipt_cost_detail" == $table) 
		return "cash_receipt_cost_detail";
	if("production_cost_detail" == $table) 
		return "production_cost_detail";
	if("Employee" == $table) 
		return "Employee";
	if("Quotation_Dashboard" == $table) 
		return "Quotation_Dashboard";
	if("Sales_Order_Dashboard" == $table) 
		return "Sales_Order_Dashboard";
	if("Reimbursement_Dashboard" == $table) 
		return "Reimbursement_Dashboard";
	if("Transaction_Registry_Dashboard_SaleRep" == $table) 
		return "Transaction_Registry_Dashboard_SaleRep";
	if("Loan_Dashboard" == $table) 
		return "Loan_Dashboard";
	if("Leave_Dashboard" == $table) 
		return "Leave_Dashboard";
	if("Over_Time_Dashboard" == $table) 
		return "Over_Time_Dashboard";
	if("Undertime_Dashboard" == $table) 
		return "Undertime_Dashboard";
	if("Fieldwork_Dashboard" == $table) 
		return "Fieldwork_Dashboard";
	if("Change_Time_Dashboard" == $table) 
		return "Change_Time_Dashboard";
	if("Transaction_Registry_Dashboard_Employee" == $table) 
		return "Transaction_Registry_Dashboard_Employee";
	if("Transaction_Registry_Dashboard_Supplier" == $table) 
		return "Transaction_Registry_Dashboard_Supplier";
	if("Transaction_Registry_Dashboard_Customer" == $table) 
		return "Transaction_Registry_Dashboard_Customer";
	if("lyl_estimate_dr" == $table) 
		return "lyl_estimate_dr";
	if("lyl_estimate_dr_detail" == $table) 
		return "lyl_estimate_dr_detail";
	if("lyl_estimate_dr_detail_accessories" == $table) 
		return "lyl_estimate_dr_detail_accessories";
	if("lyl_estimate_dr_detail_aluminum" == $table) 
		return "lyl_estimate_dr_detail_aluminum";
	if("lyl_estimate_dr_detail_e_accessories" == $table) 
		return "lyl_estimate_dr_detail_e_accessories";
	if("lyl_estimate_dr_detail_glass" == $table) 
		return "lyl_estimate_dr_detail_glass";
	if("sum_dr_qty" == $table) 
		return "sum_dr_qty";
	if("check_book_registry" == $table) 
		return "check_book_registry";
	if("check_book_registry_detail" == $table) 
		return "check_book_registry_detail";
	if("vat_type" == $table) 
		return "vat_type";
	if("quotation_detail" == $table) 
		return "quotation_detail";
	if("bal_purchase_order_detail" == $table) 
		return "bal_purchase_order_detail";
	if("bal_sales_detail" == $table) 
		return "bal_sales_detail";
	if("pending_items_reminders" == $table) 
		return "pending_items_reminders";
	if("petty_cash_for_liquidation" == $table) 
		return "petty_cash_for_liquidation";
	if("payment_method" == $table) 
		return "payment_method";
	if("account_type" == $table) 
		return "account_type";
	if("phase" == $table) 
		return "phase";
	if("item_cost_recalculation" == $table) 
		return "item_cost_recalculation";
	if("issue_components" == $table) 
		return "issue_components";
	if("issue_components_detail" == $table) 
		return "issue_components_detail";
	if("unbuild_assemblies" == $table) 
		return "unbuild_assemblies";
	if("unbuild_assemblies_detail" == $table) 
		return "unbuild_assemblies_detail";
	if("demand_plan" == $table) 
		return "demand_plan";
	if("input_vat_allocation" == $table) 
		return "input_vat_allocation";
	if("case_type" == $table) 
		return "case_type";
	if("qc_inspection" == $table) 
		return "qc_inspection";
	if("packing_list_for_checking" == $table) 
		return "packing_list_for_checking";
	if("packing_list_for_approval" == $table) 
		return "packing_list_for_approval";
	if("packing_list_detail_for_checking" == $table) 
		return "packing_list_detail_for_checking";
	if("packing_list_detail_for_approval" == $table) 
		return "packing_list_detail_for_approval";
	if("disposed_inv_detail" == $table) 
		return "disposed_inv_detail";
	if("disposed_inv" == $table) 
		return "disposed_inv";
	if("disposed_inv_for_checking" == $table) 
		return "disposed_inv_for_checking";
	if("disposed_inv_for_approval" == $table) 
		return "disposed_inv_for_approval";
	if("disposed_inv_detail_for_checking" == $table) 
		return "disposed_inv_detail_for_checking";
	if("disposed_inv_detail_for_approval" == $table) 
		return "disposed_inv_detail_for_approval";
	if("For_Approval_Dashboard" == $table) 
		return "For_Approval_Dashboard";
	if("For_Checking_Dashboard" == $table) 
		return "For_Checking_Dashboard";
	if("Customer_For_Approval" == $table) 
		return "Customer_For_Approval";
	if("request_for_payment_report" == $table) 
		return "request_for_payment_report";
	if("credit_memo_report" == $table) 
		return "credit_memo_report";
	if("debit_memo_report" == $table) 
		return "debit_memo_report";
	if("production_request_report" == $table) 
		return "production_request_report";
	if("Supplies_Balance_report" == $table) 
		return "Supplies_Balance_report";
	if("approved_sales_order" == $table) 
		return "approved_sales_order";
	if("purchase_order_for_receiving" == $table) 
		return "purchase_order_for_receiving";
	if("Dashboard" == $table) 
		return "Dashboard";
	if("packing_list" == $table) 
		return "packing_list";
	if("packing_list_detail" == $table) 
		return "packing_list_detail";
	if("open_quote" == $table) 
		return "open_quote";
	if("open_estimates" == $table) 
		return "open_estimates";
	if("open_po" == $table) 
		return "open_po";
	if("open_pr" == $table) 
		return "open_pr";
	if("open_production_request" == $table) 
		return "open_production_request";
	if("open_qc" == $table) 
		return "open_qc";
	if("open_receiving" == $table) 
		return "open_receiving";
	if("open_request_for_payment" == $table) 
		return "open_request_for_payment";
	if("open_so" == $table) 
		return "open_so";
	if("open_so_for_invoice" == $table) 
		return "open_so_for_invoice";
	if("open_so_for_project_management" == $table) 
		return "open_so_for_project_management";
	if("open_backload" == $table) 
		return "open_backload";
	if("purchase_order_receiving_detail" == $table) 
		return "purchase_order_receiving_detail";
	if("Purchase_Order_Receiving_Dashboard_Supplier" == $table) 
		return "Purchase_Order_Receiving_Dashboard_Supplier1";
	if("avg_inv" == $table) 
		return "avg_inv";
	if("open_backload_Report" == $table) 
		return "open_backload_Report";
	if("open_estimates_Report" == $table) 
		return "open_estimates_Report";
	if("open_po_Report" == $table) 
		return "open_po_Report";
	if("open_pr_Report" == $table) 
		return "open_pr_Report";
	if("open_production_request_Report" == $table) 
		return "open_production_request_Report";
	if("open_qc_Report" == $table) 
		return "open_qc_Report";
	if("open_so_for_project_management_Report" == $table) 
		return "open_so_for_project_management_Report";
	if("open_so_for_invoice_Report" == $table) 
		return "open_so_for_invoice_Report";
	if("open_so_Report" == $table) 
		return "open_so_Report";
	if("open_receiving_Report" == $table) 
		return "open_receiving_Report";
	if("monthly_consuption_for_production_Report" == $table) 
		return "monthly_consuption_for_production_Report";
	if("issue_components_for_approval" == $table) 
		return "issue_components_for_approval";
	if("issue_components_for_checking" == $table) 
		return "issue_components_for_checking";
	if("issue_components_detail_for_checking" == $table) 
		return "issue_components_detail_for_checking";
	if("issue_components_detail_for_approval" == $table) 
		return "issue_components_detail_for_approval";
	if("fieldwork_for_approval" == $table) 
		return "fieldwork_for_approval";
	if("fieldwork_for_checking" == $table) 
		return "fieldwork_for_checking";
	if("Purchases_Order_for_checking" == $table) 
		return "Purchases_Order_for_checking";
	if("Purchases_Order_for_approval" == $table) 
		return "Purchases_Order_for_approval";
	if("Receiving_for_checking" == $table) 
		return "Receiving_for_checking";
	if("Receiving_for_approval" == $table) 
		return "Receiving_for_approval";
	if("inventory_tranfer_Report" == $table) 
		return "inventory_tranfer_Report";
	if("Purchase_vs_Production_Report" == $table) 
		return "Purchase_vs_Production_Report";
	if("cash_sales_Report" == $table) 
		return "cash_sales_Report";
	if("Center_Replenishment_for_approval" == $table) 
		return "Center_Replenishment_for_approval";
	if("Center_Replenishment_for_checking" == $table) 
		return "Center_Replenishment_for_checking";
	if("Purchase_vs_Production" == $table) 
		return "Purchase_vs_Production";
	if("Production_Summary" == $table) 
		return "Production_Summary";
	if("pending_production_request" == $table) 
		return "pending_production_request";
	if("unbuild_assemblies_for_checking" == $table) 
		return "unbuild_assemblies_for_checking";
	if("unbuild_assemblies_for_approval" == $table) 
		return "unbuild_assemblies_for_approval";
	if("unbuild_assemblies_detail_for_checking" == $table) 
		return "unbuild_assemblies_detail_for_checking";
	if("quality_control_for_approval" == $table) 
		return "quality_control_for_approval";
	if("quality_control_for_checking" == $table) 
		return "quality_control_for_checking";
	if("quality_control_detail_for_approval" == $table) 
		return "quality_control_detail_for_approval";
	if("raw_cake_balance" == $table) 
		return "raw_cake_balance";
	if("bal_cashreceipt" == $table) 
		return "bal_cashreceipt";
	if("bal_cashpayment" == $table) 
		return "bal_cashpayment";
	if("opportunity" == $table) 
		return "opportunity";
	if("Current_Accounts" == $table) 
		return "Current_Accounts";
	if("bal_lyl_acc" == $table) 
		return "bal_lyl_acc";
	if("bal_lyl_glass" == $table) 
		return "bal_lyl_glass";
	if("bal_lyl_eacc" == $table) 
		return "bal_lyl_eacc";
	if("bal_lyl_allum" == $table) 
		return "bal_lyl_allum";
	if("opportunity_details" == $table) 
		return "opportunity_details";
	if("Collection_Report_per_Sales_Rep" == $table) 
		return "Collection_Report_per_Sales_Rep";
	if("Inventory_per_lot_no" == $table) 
		return "Inventory_per_lot_no";
	if("marketing_request_form_for_checking" == $table) 
		return "marketing_request_form_for_checking";
	if("marketing_request_form_for_approval" == $table) 
		return "marketing_request_form_for_approval";
	if("Accounts_Payable_Dashboard" == $table) 
		return "Accounts_Payable_Dashboard";
	if("201" == $table) 
		return "201";
	if("Cashflow_forcast" == $table) 
		return "Cashflow_forcast";
	if("Business_Owner_Sales_Report" == $table) 
		return "Business_Owner_Sales_Report";
	if("Business_Owner_Purchase_Report" == $table) 
		return "Business_Owner_Purchase_Report";
	if("Business_Owner_Cash_Balance" == $table) 
		return "Business_Owner_Cash_Balance";
	if("Business_Owner_Top_Customer_Income" == $table) 
		return "Business_Owner_Top_Customer_Income";
	if("AR_Billing_due_dates_Report" == $table) 
		return "AR_Billing_due_dates_Report";
	if("AR_Aging_of_AR_Report" == $table) 
		return "AR_Aging_of_AR_Report";
	if("AR_Top_5_Customer_balances" == $table) 
		return "AR_Top_5_Customer_balances";
	if("AR_Progress_Billing_Alert" == $table) 
		return "AR_Progress_Billing_Alert";
	if("Account_Receivable_Dashboard" == $table) 
		return "Account_Receivable_Dashboard";
	if("Finance_Cash_Balance" == $table) 
		return "Finance_Cash_Balance";
	if("Finance_approved_request_for_payment" == $table) 
		return "Finance_approved_request_for_payment";
	if("Finance_Cash_flow_forcast" == $table) 
		return "Finance_Cash_flow_forcast";
	if("Finance_Approved_Employee_Loan" == $table) 
		return "Finance_Approved_Employee_Loan";
	if("Finance_Cash_Balances" == $table) 
		return "Finance_Cash_Balances";
	if("AP_Aging_of_Payables" == $table) 
		return "AP_Aging_of_Payables";
	if("Purchasing_Item_below_reorder" == $table) 
		return "Purchasing_Item_below_reorder";
	if("production_per_item" == $table) 
		return "production_per_item";
	if("AP_Top_5_Supplier_balances" == $table) 
		return "AP_Top_5_Supplier_balances";
	if("Warehouse_Item_balances" == $table) 
		return "Warehouse_Item_balances";
	if("DR_per_Item" == $table) 
		return "DR_per_Item";
	if("Logistic_Open_Trip_Ticket" == $table) 
		return "Logistic_Open_Trip_Ticket";
	if("Logistic_Available_Vehicles" == $table) 
		return "Logistic_Available_Vehicles";
	if("Marketing_Open_Campaign" == $table) 
		return "Marketing_Open_Campaign";
	if("Marketing_Sales_per_campaign" == $table) 
		return "Marketing_Sales_per_campaign";
	if("Support_Open_Case_Report" == $table) 
		return "Support_Open_Case_Report";
	if("Support_Service_center" == $table) 
		return "Support_Service_center";
	if("Sale_Meeting_Report" == $table) 
		return "Sale_Meeting_Report";
	if("Sales_Next_Action_Report" == $table) 
		return "Sales_Next_Action_Report";
	if("HR_attendance_summary" == $table) 
		return "HR_attendance_summary";
	if("HR_Birthday_Alert" == $table) 
		return "HR_Birthday_Alert";
	if("HR_holiday" == $table) 
		return "HR_holiday";
	if("Project_Expense_per_Project" == $table) 
		return "Project_Expense_per_Project";
	if("Project_Expense_per_Project_graph" == $table) 
		return "Project_Expense_per_Project_graph";
	if("Petty_Cash_Balance" == $table) 
		return "Petty_Cash_Balance";
	if("Petty_Cash_Unliquidated" == $table) 
		return "Petty_Cash_Unliquidated";
	if("Petty_Cash_Purchases_per_Item" == $table) 
		return "Petty_Cash_Purchases_per_Item";
	if("Petty_Cash_Dashboard" == $table) 
		return "Petty_Cash_Dashboard";
	if("Accounting_Income_per_Center" == $table) 
		return "Accounting_Income_per_Center";
	if("open_quote_Report" == $table) 
		return "open_quote_Report";
	if("Open_cases_per_agent" == $table) 
		return "Open_cases_per_agent";
	if("Marketing_sales_per_campaing_chart" == $table) 
		return "Marketing_sales_per_campaing_chart";
	if("Logistics_Dashboard" == $table) 
		return "Logistics_Dashboard";
	if("Accounting_Dashboard" == $table) 
		return "Accounting_Dashboard";
	if("AP_open_po_receiving" == $table) 
		return "AP_open_po_receiving";
	if("Production_Inventory_Balance_Report" == $table) 
		return "Production_Inventory_Balance_Report";
	if("Production_WIP" == $table) 
		return "Production_WIP";
	if("Warehouse_Dashboard" == $table) 
		return "Warehouse_Dashboard";
	if("Warehouse_Inventory_Balance_Report" == $table) 
		return "Warehouse_Inventory_Balance_Report";
	if("logistic_open_inventory_tranfer" == $table) 
		return "logistic_open_inventory_tranfer";
	if("Sales_Leads" == $table) 
		return "Sales_Leads";
	if("Sales_opportunity_Next_Action" == $table) 
		return "Sales_opportunity_Next_Action";
	if("Sales_Active_opportunity" == $table) 
		return "Sales_Active_opportunity";
	if("intercompany_sales_order_detail" == $table) 
		return "intercompany_sales_order_detail";
	if("intercompany_sales_order" == $table) 
		return "intercompany_sales_order";
	if("production_detail" == $table) 
		return "production_detail";
	if("quality_control_detail_for_checking" == $table) 
		return "quality_control_detail_for_checking";
	if("lyl_estimate_for_approval" == $table) 
		return "lyl_estimate_for_approval";
	if("lyl_estimate_dr_for_approval" == $table) 
		return "lyl_estimate_dr_for_approval";
	if("lyl_estimate_detail_for_approval" == $table) 
		return "lyl_estimate_detail_for_approval";
	if("lyl_estimate_detail_accessories_for_approval" == $table) 
		return "lyl_estimate_detail_accessories_for_approval";
	if("lyl_estimate_detail_aluminum_for_approval" == $table) 
		return "lyl_estimate_detail_aluminum_for_approval";
	if("lyl_estimate_detail_e_accessories_for_approval" == $table) 
		return "lyl_estimate_detail_e_accessories_for_approval";
	if("lyl_estimate_detail_glass_for_approval" == $table) 
		return "lyl_estimate_detail_glass_for_approval";
	if("lyl_estimate_dr_detail_for_approval" == $table) 
		return "lyl_estimate_dr_detail_for_approval";
	if("lyl_estimate_dr_detail_accessories_for_approval" == $table) 
		return "lyl_estimate_dr_detail_accessories_for_approval";
	if("lyl_estimate_dr_detail_aluminum_for_approval" == $table) 
		return "lyl_estimate_dr_detail_aluminum_for_approval";
	if("lyl_estimate_dr_detail_e_accessories_for_approval" == $table) 
		return "lyl_estimate_dr_detail_e_accessories_for_approval";
	if("lyl_estimate_dr_detail_glass_for_approval" == $table) 
		return "lyl_estimate_dr_detail_glass_for_approval";
	if("lyl_estimate_for_checking" == $table) 
		return "lyl_estimate_for_checking";
	if("lyl_estimate_detail_for_checking" == $table) 
		return "lyl_estimate_detail_for_checking";
	if("lyl_estimate_detail_accessories_for_checking" == $table) 
		return "lyl_estimate_detail_accessories_for_checking";
	if("lyl_estimate_detail_aluminum_for_checking" == $table) 
		return "lyl_estimate_detail_aluminum_for_checking";
	if("lyl_estimate_detail_e_accessories_for_checking" == $table) 
		return "lyl_estimate_detail_e_accessories_for_checking";
	if("lyl_estimate_detail_glass_for_checking" == $table) 
		return "lyl_estimate_detail_glass_for_checking";
	if("lyl_estimate_dr_for_checking" == $table) 
		return "lyl_estimate_dr_for_checking";
	if("lyl_estimate_dr_detail_for_checking" == $table) 
		return "lyl_estimate_dr_detail_for_checking";
	if("lyl_estimate_dr_detail_accessories_for_checking" == $table) 
		return "lyl_estimate_dr_detail_accessories_for_checking";
	if("lyl_estimate_dr_detail_aluminum_for_checking" == $table) 
		return "lyl_estimate_dr_detail_aluminum_for_checking";
	if("lyl_estimate_dr_detail_e_accessories_for_checking" == $table) 
		return "lyl_estimate_dr_detail_e_accessories_for_checking";
	if("lyl_estimate_dr_detail_glass_for_checking" == $table) 
		return "lyl_estimate_dr_detail_glass_for_checking";
	if("EWT_Certificates_summay" == $table) 
		return "EWT_Certificates_summay";
	if("1601e" == $table) 
		return "1601e";
	if("1601eform" == $table) 
		return "1601eform";
	if("1601fform" == $table) 
		return "1601fform";
	if("tax_person_status" == $table) 
		return "tax_person_status";
	if("journal_entry_inv_transfer" == $table) 
		return "journal_entry_inv_transfer";
	if("Recon_withdraw" == $table) 
		return "Recon_withdraw";
	if("lot" == $table) 
		return "lot";
	if("serial_no" == $table) 
		return "serial_no";
	if("request_for_quote" == $table) 
		return "request_for_quote";
	if("request_for_quoter_detail" == $table) 
		return "request_for_quoter_detail";
	if("request_for_quote_for_checking" == $table) 
		return "request_for_quote_for_checking";
	if("request_for_quote_for_approval" == $table) 
		return "request_for_quote_for_approval";
	if("request_for_quoter_detail_for_checking" == $table) 
		return "request_for_quoter_detail_for_checking";
	if("request_for_quoter_detail_for_approval" == $table) 
		return "request_for_quoter_detail_for_approval";
	if("estimate_cost" == $table) 
		return "estimate_cost";
	if("airlines" == $table) 
		return "airlines";
	if("airport" == $table) 
		return "airport";
	if("inco_terms" == $table) 
		return "inco_terms";
	if("seaports" == $table) 
		return "seaports";
	if("shipment_classification" == $table) 
		return "shipment_classification";
	if("shipping_lines" == $table) 
		return "shipping_lines";
	if("bal_sales_order_detail" == $table) 
		return "bal_sales_order_detail";
	if("project_cont_seal" == $table) 
		return "project_cont_seal";
	if("project_house" == $table) 
		return "project_house";
	if("billable_liquidation" == $table) 
		return "billable_liquidation";
	if("billable_pettycash" == $table) 
		return "billable_pettycash";
	if("billable_checkvoucher" == $table) 
		return "billable_checkvoucher";
	if("projects_dashboard" == $table) 
		return "projects_dashboard";
	if("employees" == $table) 
		return "employees";
	if("admin_rights" == $table) 
		return "admin_rights";
	if("admin_members" == $table) 
		return "admin_members";
	if("admin_users" == $table) 
		return "admin_users";
	if("company" == $table) 
		return "company";
	if("notifiy quotation Report" == $table) 
		return "notifiy_quotation_Report";
	if("expenses Report" == $table) 
		return "expenses_Report";
	if("petty cash release" == $table) 
		return "petty_cash_release";
	if("petty cash replenishment" == $table) 
		return "petty_cash_replenishment";
	if("Employee Advance Release" == $table) 
		return "Employee_Advance_Release";
	if("Employee Advance Replenishment" == $table) 
		return "Employee_Advance_Replenishment";
	if("Liquidation Release" == $table) 
		return "Liquidation_Release";
	if("Liquidation Replenishment" == $table) 
		return "Liquidation_Replenishment";
	if("Billable Cost Report" == $table) 
		return "Billable_Cost_Report";
	if("centerrep_pettycash" == $table) 
		return "centerrep_pettycash";
	if("centerrep_empadvance" == $table) 
		return "centerrep_empadvance";
	if("centerrep_liquidation" == $table) 
		return "centerrep_liquidation";
	if("replenishment_pettycash" == $table) 
		return "replenishment_pettycash";
	if("replenishment_liquidation" == $table) 
		return "replenishment_liquidation";
	if("replenishment_empadv" == $table) 
		return "replenishment_empadv";
	if("bank_statement" == $table) 
		return "bank_statement";
	if("other_charges" == $table) 
		return "other_charges";
	if("change_password" == $table) 
		return "change_password";
	if("project_liquidation_detail2" == $table) 
		return "project_liquidation_detail2";
	if("balance_advance" == $table) 
		return "balance_advance";
	if("project_replenishment_deposited" == $table) 
		return "project_replenishment_deposited";
	if("bal_center_replenishment" == $table) 
		return "bal_center_replenishment";
	if("Project and Pettycash Report" == $table) 
		return "Project_and_Pettycash_Report";
	if("Project and EmpAdv Report" == $table) 
		return "Project_and_EmpAdv_Report";
	if("Project and Liquidation Report" == $table) 
		return "Project_and_Liquidation_Report";
	if("Project and Sales Report" == $table) 
		return "Project_and_Sales_Report";
	if("Project and Voucher Report" == $table) 
		return "Project_and_Voucher_Report";
	if("projects_view" == $table) 
		return "projects_view";
	if("projects_lite" == $table) 
		return "projects_lite";
	if("shipping_date" == $table) 
		return "shipping_date";
	if("entry_detail_sales" == $table) 
		return "entry_detail_sales";
	if("entry_detail_pettycash" == $table) 
		return "entry_detail_pettycash";
	if("entry_detail_cash_disbursement" == $table) 
		return "entry_detail_cash_disbursement";
	if("representative" == $table) 
		return "representative";
	if("sales_sub_invoice" == $table) 
		return "sales_sub_invoice";
	if("search_cash_disbursement" == $table) 
		return "search_cash_disbursement";
	if("sales_business_partner" == $table) 
		return "sales_business_partner";
	if("canvas" == $table) 
		return "canvas";
	if("canvas_detail" == $table) 
		return "canvas_detail";
	if("request_for_payment" == $table) 
		return "request_for_payment";
	if("request_for_payment_detail" == $table) 
		return "request_for_payment_detail";
	if("po_netpr" == $table) 
		return "po_netpr";
	if("request_for_payment_imp" == $table) 
		return "request_for_payment_imp";
	if("birsalesrelief" == $table) 
		return "birsalesrelief";

	return "";
}

//	return strTableName by short table name
function GetTableByShort($shortTName = "")
{	
	global $projectTables;
	if(!$shortTName)
		return false;
	if("business_partner" == $shortTName) 
		return "business_partner";
	if("item" == $shortTName) 
		return "item";
	if("credit_terms" == $shortTName) 
		return "credit_terms";
	if("shipping_terms" == $shortTName) 
		return "shipping_terms";
	if("cash_receipt" == $shortTName) 
		return "cash_receipt";
	if("purchase" == $shortTName) 
		return "purchase";
	if("cash_disbursment" == $shortTName) 
		return "cash_disbursment";
	if("general_journal" == $shortTName) 
		return "general_journal";
	if("report_center" == $shortTName) 
		return "report_center";
	if("Sales_Report" == $shortTName) 
		return "Sales Report";
	if("Disbursement_Reports" == $shortTName) 
		return "Disbursement Reports";
	if("Financial_Reports" == $shortTName) 
		return "Financial Reports";
	if("Purchase_Reports" == $shortTName) 
		return "Purchase Reports";
	if("Receivable_Reports" == $shortTName) 
		return "Receivable Reports";
	if("Taxation_Reports" == $shortTName) 
		return "Taxation Reports";
	if("system_user" == $shortTName) 
		return "system_user";
	if("fiscal_year" == $shortTName) 
		return "fiscal_year";
	if("beg_bal" == $shortTName) 
		return "beg_bal";
	if("ewt" == $shortTName) 
		return "ewt";
	if("bank" == $shortTName) 
		return "bank";
	if("cost_center" == $shortTName) 
		return "cost_center";
	if("General_Journal_Reports" == $shortTName) 
		return "General Journal Reports";
	if("Inventory_Reports" == $shortTName) 
		return "Inventory Reports";
	if("accounting_period" == $shortTName) 
		return "accounting_period";
	if("bank_recon" == $shortTName) 
		return "bank_recon";
	if("deposit" == $shortTName) 
		return "deposit";
	if("account_classification" == $shortTName) 
		return "account_classification";
	if("chart_of_accounts" == $shortTName) 
		return "chart_of_accounts";
	if("recon_CR" == $shortTName) 
		return "recon_CR";
	if("Recon_deposit" == $shortTName) 
		return "Recon_deposit";
	if("Recon_CV" == $shortTName) 
		return "Recon_CV";
	if("Recon_GV_dep" == $shortTName) 
		return "Recon_GV_dep";
	if("Recon_GV_CV" == $shortTName) 
		return "Recon_GV_CV";
	if("vat_atc" == $shortTName) 
		return "vat_atc";
	if("rdo" == $shortTName) 
		return "rdo";
	if("2550q" == $shortTName) 
		return "2550q";
	if("2550m" == $shortTName) 
		return "2550m";
	if("dr" == $shortTName) 
		return "dr";
	if("purchase_order" == $shortTName) 
		return "purchase_order";
	if("purchase_order_receiving" == $shortTName) 
		return "purchase_order_receiving";
	if("System_preference" == $shortTName) 
		return "System_preference";
	if("ppe_list" == $shortTName) 
		return "ppe_list";
	if("discounts" == $shortTName) 
		return "discounts";
	if("item_group" == $shortTName) 
		return "item_group";
	if("location" == $shortTName) 
		return "location";
	if("meetings" == $shortTName) 
		return "meetings";
	if("calls" == $shortTName) 
		return "calls";
	if("campaign" == $shortTName) 
		return "campaign";
	if("reimbursement" == $shortTName) 
		return "reimbursement";
	if("reimbursement_detail" == $shortTName) 
		return "reimbursement_detail";
	if("Target_customers" == $shortTName) 
		return "Target_customers";
	if("quotation" == $shortTName) 
		return "quotation";
	if("case" == $shortTName) 
		return "case";
	if("projects" == $shortTName) 
		return "projects";
	if("production" == $shortTName) 
		return "production";
	if("issue" == $shortTName) 
		return "issue";
	if("issue_detail" == $shortTName) 
		return "issue_detail";
	if("disposed" == $shortTName) 
		return "disposed";
	if("disposed_detail" == $shortTName) 
		return "disposed_detail";
	if("backload" == $shortTName) 
		return "backload";
	if("backload_detail" == $shortTName) 
		return "backload_detail";
	if("vehicle" == $shortTName) 
		return "vehicle";
	if("payroll_item" == $shortTName) 
		return "payroll_item";
	if("uom" == $shortTName) 
		return "uom";
	if("trip_ticket" == $shortTName) 
		return "trip_ticket";
	if("accident" == $shortTName) 
		return "accident";
	if("approver_list" == $shortTName) 
		return "approver_list";
	if("area" == $shortTName) 
		return "area";
	if("attendance" == $shortTName) 
		return "attendance";
	if("currency" == $shortTName) 
		return "currency";
	if("medical" == $shortTName) 
		return "medical";
	if("position" == $shortTName) 
		return "position";
	if("service_center" == $shortTName) 
		return "service_center";
	if("seminar" == $shortTName) 
		return "seminar";
	if("service_report" == $shortTName) 
		return "service_report";
	if("service_report_detail" == $shortTName) 
		return "service_report_detail";
	if("tax_status" == $shortTName) 
		return "tax_status";
	if("training" == $shortTName) 
		return "training";
	if("competitor" == $shortTName) 
		return "competitor";
	if("campaign_type" == $shortTName) 
		return "campaign_type";
	if("fixed_asset_return_detail" == $shortTName) 
		return "fixed_asset_return_detail";
	if("fixed_assets_return" == $shortTName) 
		return "fixed_assets_return";
	if("Prospect_customers" == $shortTName) 
		return "Prospect_customers";
	if("trip_detail" == $shortTName) 
		return "trip_detail";
	if("beg_baldetail" == $shortTName) 
		return "beg_baldetail";
	if("cash_disbursement_detail2" == $shortTName) 
		return "cash_disbursement_detail2";
	if("case_solutions" == $shortTName) 
		return "case_solutions";
	if("cash_disbursment_detail" == $shortTName) 
		return "cash_disbursment_detail";
	if("cash_receipt_detail" == $shortTName) 
		return "cash_receipt_detail";
	if("cash_receipt_detail2" == $shortTName) 
		return "cash_receipt_detail2";
	if("general_journal_detail" == $shortTName) 
		return "general_journal_detail";
	if("employee_list" == $shortTName) 
		return "employee_list";
	if("petty_cash" == $shortTName) 
		return "petty_cash";
	if("petty_cash_detail" == $shortTName) 
		return "petty_cash_detail";
	if("center_replenishment" == $shortTName) 
		return "center_replenishment";
	if("center_replenishment_detail" == $shortTName) 
		return "center_replenishment_detail";
	if("credit_memo" == $shortTName) 
		return "credit_memo";
	if("credit_memo_detail" == $shortTName) 
		return "credit_memo_detail";
	if("debit_memo" == $shortTName) 
		return "debit_memo";
	if("debit_memo_detail" == $shortTName) 
		return "debit_memo_detail";
	if("project_replenishment" == $shortTName) 
		return "project_replenishment";
	if("project_replenishment_detail" == $shortTName) 
		return "project_replenishment_detail";
	if("business_partner_group" == $shortTName) 
		return "business_partner_group";
	if("Sales_Representatives" == $shortTName) 
		return "Sales_Representatives";
	if("Types_of_leaves" == $shortTName) 
		return "Types_of_leaves";
	if("payroll_mode" == $shortTName) 
		return "payroll_mode";
	if("payroll_account_type" == $shortTName) 
		return "payroll_account_type";
	if("Types_of_Overtime" == $shortTName) 
		return "Types_of_Overtime";
	if("Types_of_Loan" == $shortTName) 
		return "Types_of_Loan";
	if("sales" == $shortTName) 
		return "sales";
	if("sales_detail" == $shortTName) 
		return "sales_detail";
	if("Payroll_Company_Setup" == $shortTName) 
		return "Payroll_Company_Setup";
	if("item_bill_of_materials" == $shortTName) 
		return "item_bill_of_materials";
	if("item_uom_conversion" == $shortTName) 
		return "item_uom_conversion";
	if("tranasactions_for_approval" == $shortTName) 
		return "tranasactions_for_approval";
	if("deposit_detail" == $shortTName) 
		return "deposit_detail";
	if("purchase_detail" == $shortTName) 
		return "purchase_detail";
	if("dr_detail" == $shortTName) 
		return "dr_detail";
	if("purchase_order_detail" == $shortTName) 
		return "purchase_order_detail";
	if("Sales_Order_for_checking" == $shortTName) 
		return "Sales_Order_for_checking";
	if("sales_order_detail_for_checking" == $shortTName) 
		return "sales_order_detail_for_checking";
	if("Sales_Order_for_approval" == $shortTName) 
		return "Sales_Order_for_approval";
	if("sales_order_detail_for_approval" == $shortTName) 
		return "sales_order_detail_for_approval";
	if("Quotation_for_approval" == $shortTName) 
		return "Quotation_for_approval";
	if("Quotation_for_checking" == $shortTName) 
		return "Quotation_for_checking";
	if("quotation_detail_for_checking" == $shortTName) 
		return "quotation_detail_for_checking";
	if("quotation_detail_for_approval" == $shortTName) 
		return "quotation_detail_for_approval";
	if("Reimbursement_for_checking" == $shortTName) 
		return "Reimbursement_for_checking";
	if("reimbursement_detail_for_checking" == $shortTName) 
		return "reimbursement_detail_for_checking";
	if("Reimbursement_for_approval" == $shortTName) 
		return "Reimbursement_for_approval";
	if("reimbursement_detail_for_approval" == $shortTName) 
		return "reimbursement_detail_for_approval";
	if("Project_Replenishment_for_checking" == $shortTName) 
		return "Project_Replenishment_for_checking";
	if("project_replenishment_detail_for_checking" == $shortTName) 
		return "project_replenishment_detail_for_checking";
	if("Project_Replenishment_for_approval" == $shortTName) 
		return "Project_Replenishment_for_approval";
	if("project_replenishment_detail_for_approval" == $shortTName) 
		return "project_replenishment_detail_for_approval";
	if("center_replenishment_detail_for_checking" == $shortTName) 
		return "center_replenishment_detail_for_checking";
	if("center_replenishment_detail_for_approval" == $shortTName) 
		return "center_replenishment_detail_for_approval";
	if("Purchases_Order_for_checking2" == $shortTName) 
		return "Purchases_Order_for_checking2";
	if("purchase_order_detail_for_checking" == $shortTName) 
		return "purchase_order_detail_for_checking";
	if("Purchases_Order_for_approval2" == $shortTName) 
		return "Purchases_Order_for_approval2";
	if("purchase_order_detail_for_approval" == $shortTName) 
		return "purchase_order_detail_for_approval";
	if("Receiving_for_checking2" == $shortTName) 
		return "Receiving_for_checking2";
	if("purchase_order_receiving_detail_for_checking" == $shortTName) 
		return "purchase_order_receiving_detail_for_checking";
	if("Receiving_for_approval2" == $shortTName) 
		return "Receiving_for_approval2";
	if("purchase_order_receiving_detail_for_approval" == $shortTName) 
		return "purchase_order_receiving_detail_for_approval";
	if("Delivery_Receipt_for_checking" == $shortTName) 
		return "Delivery_Receipt_for_checking";
	if("dr_detail_for_checking" == $shortTName) 
		return "dr_detail_for_checking";
	if("Delivery_Receipt_for_approval" == $shortTName) 
		return "Delivery_Receipt_for_approval";
	if("dr_detail_for_approval" == $shortTName) 
		return "dr_detail_for_approval";
	if("Production_for_checking" == $shortTName) 
		return "Production_for_checking";
	if("production_detail_for_checking" == $shortTName) 
		return "production_detail_for_checking";
	if("Production_for_approval" == $shortTName) 
		return "Production_for_approval";
	if("production_detail_for_approval" == $shortTName) 
		return "production_detail_for_approval";
	if("Issue_for_checking" == $shortTName) 
		return "Issue_for_checking";
	if("supply_issue_detail_for_checking" == $shortTName) 
		return "supply_issue_detail_for_checking";
	if("Issue_for_approval" == $shortTName) 
		return "Issue_for_approval";
	if("supply_issue_detail_for_approval" == $shortTName) 
		return "supply_issue_detail_for_approval";
	if("Return_for_checking" == $shortTName) 
		return "Return_for_checking";
	if("supply_return_detail_for_checking" == $shortTName) 
		return "supply_return_detail_for_checking";
	if("Return_for_approval" == $shortTName) 
		return "Return_for_approval";
	if("supply_return_detail_approval" == $shortTName) 
		return "supply_return_detail_approval";
	if("supply_disposed_detail_for_checking" == $shortTName) 
		return "supply_disposed_detail_for_checking";
	if("Disposal_for_approval" == $shortTName) 
		return "Disposal_for_approval";
	if("supply_disposed_detail_for_approval" == $shortTName) 
		return "supply_disposed_detail_for_approval";
	if("Fixed_Asset_Issue_for_checking" == $shortTName) 
		return "Fixed_Asset_Issue_for_checking";
	if("Fixed_Asset_Issue_for_approval" == $shortTName) 
		return "Fixed_Asset_Issue_for_approval";
	if("fixed_asset_issue_detail_for_approval" == $shortTName) 
		return "fixed_asset_issue_detail_for_approval";
	if("fixed_asset_issue_detail_for_checking" == $shortTName) 
		return "fixed_asset_issue_detail_for_checking";
	if("Fixed_Asset_Return_for_checking" == $shortTName) 
		return "Fixed_Asset_Return_for_checking";
	if("fixed_asset_return_detail_for_checking" == $shortTName) 
		return "fixed_asset_return_detail_for_checking";
	if("Fixed_Asset_Return_for_approval1" == $shortTName) 
		return "Fixed_Asset_Return_for_approval";
	if("fixed_asset_return_detail_for_approval" == $shortTName) 
		return "fixed_asset_return_detail_for_approval";
	if("Petty_Cash_for_checking" == $shortTName) 
		return "Petty_Cash_for_checking";
	if("petty_cash_detail_for_checking" == $shortTName) 
		return "petty_cash_detail_for_checking";
	if("Petty_Cash_for_approval" == $shortTName) 
		return "Petty_Cash_for_approval";
	if("petty_cash_detail_for_approval" == $shortTName) 
		return "petty_cash_detail_for_approval";
	if("Deposit_for_checking" == $shortTName) 
		return "Deposit_for_checking";
	if("deposit_detail_for_checking" == $shortTName) 
		return "deposit_detail_for_checking";
	if("Deposit_for_approval" == $shortTName) 
		return "Deposit_for_approval";
	if("deposit_detail_for_approval" == $shortTName) 
		return "deposit_detail_for_approval";
	if("Credit_Memo_for_checking" == $shortTName) 
		return "Credit_Memo_for_checking";
	if("credit_memo_detail_for_checking" == $shortTName) 
		return "credit_memo_detail_for_checking";
	if("Credit_Memo_for_approval" == $shortTName) 
		return "Credit_Memo_for_approval";
	if("credit_memo_detail_for_approval" == $shortTName) 
		return "credit_memo_detail_for_approval";
	if("Debit_Memo_for_checking" == $shortTName) 
		return "Debit_Memo_for_checking";
	if("debit_memo_detail_for_checking" == $shortTName) 
		return "debit_memo_detail_for_checking";
	if("debit_memo_for_approval" == $shortTName) 
		return "debit_memo_for_approval";
	if("debit_memo_detail_for_approval" == $shortTName) 
		return "debit_memo_detail_for_approval";
	if("Purchases_for_checking" == $shortTName) 
		return "Purchases_for_checking";
	if("credit_purchase_detail_for_checking" == $shortTName) 
		return "credit_purchase_detail_for_checking";
	if("Purchases_for_approval" == $shortTName) 
		return "Purchases_for_approval";
	if("credit_purchase_detail_for_approval" == $shortTName) 
		return "credit_purchase_detail_for_approval";
	if("Sales_for_checking" == $shortTName) 
		return "Sales_for_checking";
	if("Sales_for_approval" == $shortTName) 
		return "Sales_for_approval";
	if("credit_sales_detail_for_checking" == $shortTName) 
		return "credit_sales_detail_for_checking";
	if("credit_sales_detail_for_approval" == $shortTName) 
		return "credit_sales_detail_for_approval";
	if("Cash_Disbursement_for_checking" == $shortTName) 
		return "Cash_Disbursement_for_checking";
	if("check_voucher_detail_for_checking" == $shortTName) 
		return "check_voucher_detail_for_checking";
	if("check_voucher_detail2_for_checking" == $shortTName) 
		return "check_voucher_detail2_for_checking";
	if("Cash_Disbursement_for_approval" == $shortTName) 
		return "Cash_Disbursement_for_approval";
	if("check_voucher_detail_for_approval" == $shortTName) 
		return "check_voucher_detail_for_approval";
	if("check_voucher_detail2_for_approval" == $shortTName) 
		return "check_voucher_detail2_for_approval";
	if("document" == $shortTName) 
		return "document";
	if("General_Journal_for_checking" == $shortTName) 
		return "General_Journal_for_checking";
	if("general_journal_detail_for_checking" == $shortTName) 
		return "general_journal_detail_for_checking";
	if("General_Journal_for_approval" == $shortTName) 
		return "General_Journal_for_approval";
	if("general_journal_detail_for_approval" == $shortTName) 
		return "general_journal_detail_for_approval";
	if("for_submissions" == $shortTName) 
		return "for_submissions";
	if("submission_details" == $shortTName) 
		return "submission_details";
	if("sales_order_detail_billing_dates" == $shortTName) 
		return "sales_order_detail_billing_dates";
	if("Submission_Search" == $shortTName) 
		return "Submission_Search";
	if("Submission_Reminders" == $shortTName) 
		return "Submission_Reminders";
	if("years" == $shortTName) 
		return "years";
	if("document_type" == $shortTName) 
		return "document_type";
	if("estimate" == $shortTName) 
		return "estimate";
	if("estimate_detail" == $shortTName) 
		return "estimate_detail";
	if("purchase_request" == $shortTName) 
		return "purchase_request";
	if("purchase_request_detail" == $shortTName) 
		return "purchase_request_detail";
	if("inv_adj_detail" == $shortTName) 
		return "inv_adj_detail";
	if("inventory_adj" == $shortTName) 
		return "inventory_adj";
	if("inventory_tranfer" == $shortTName) 
		return "inventory_tranfer";
	if("inventory_transfer_detail" == $shortTName) 
		return "inventory_transfer_detail";
	if("Customers" == $shortTName) 
		return "Customers";
	if("Suppliers" == $shortTName) 
		return "Suppliers";
	if("Target_contacts" == $shortTName) 
		return "Target_contacts";
	if("Prospect_contacts" == $shortTName) 
		return "Prospect_contacts";
	if("Customer_Contacts" == $shortTName) 
		return "Customer_Contacts";
	if("Supplier_Contacts" == $shortTName) 
		return "Supplier_Contacts";
	if("Statement_of_Account" == $shortTName) 
		return "Statement_of_Account";
	if("Change_Time_Schedule_for_approval" == $shortTName) 
		return "Change_Time_Schedule_for_approval";
	if("Change_Time_Schedule_for_checking" == $shortTName) 
		return "Change_Time_Schedule_for_checking";
	if("Inventory_Adjustment_for_checking" == $shortTName) 
		return "Inventory_Adjustment_for_checking";
	if("inv_adj_detail_for_checking" == $shortTName) 
		return "inv_adj_detail_for_checking";
	if("Inventory_Adjustment_for_approval" == $shortTName) 
		return "Inventory_Adjustment_for_approval";
	if("inv_adj_detail_for_approval" == $shortTName) 
		return "inv_adj_detail_for_approval";
	if("for_checking" == $shortTName) 
		return "for_checking";
	if("for_approval" == $shortTName) 
		return "for_approval";
	if("Estimate_for_approval" == $shortTName) 
		return "Estimate_for_approval";
	if("Estimate_for_checking" == $shortTName) 
		return "Estimate_for_checking";
	if("estimate_detail_for_checking" == $shortTName) 
		return "estimate_detail_for_checking";
	if("estimate_detail_for_approval" == $shortTName) 
		return "estimate_detail_for_approval";
	if("Follow_up" == $shortTName) 
		return "Follow_up";
	if("coming_meetings" == $shortTName) 
		return "coming_meetings";
	if("tasks" == $shortTName) 
		return "tasks";
	if("Project_phases" == $shortTName) 
		return "Project_phases";
	if("origin_fa" == $shortTName) 
		return "origin_fa";
	if("origin_serialized_supply" == $shortTName) 
		return "origin_serialized_supply";
	if("2301" == $shortTName) 
		return "2301";
	if("Prospect_customers_to_Customer" == $shortTName) 
		return "Prospect_customers_to_Customer";
	if("lyl_estimate" == $shortTName) 
		return "lyl_estimate";
	if("lyl_estimate_detail" == $shortTName) 
		return "lyl_estimate_detail";
	if("lyl_estimate_detail_accessories" == $shortTName) 
		return "lyl_estimate_detail_accessories";
	if("lyl_estimate_detail_aluminum" == $shortTName) 
		return "lyl_estimate_detail_aluminum";
	if("lyl_estimate_detail_e_accessories" == $shortTName) 
		return "lyl_estimate_detail_e_accessories";
	if("lyl_estimate_detail_glass" == $shortTName) 
		return "lyl_estimate_detail_glass";
	if("recuring_benefits" == $shortTName) 
		return "recuring_benefits";
	if("recuring_deductions" == $shortTName) 
		return "recuring_deductions";
	if("recurring_benefits_detail" == $shortTName) 
		return "recurring_benefits_detail";
	if("recurring_deductions_detail" == $shortTName) 
		return "recurring_deductions_detail";
	if("production_request" == $shortTName) 
		return "production_request";
	if("production_request_detail" == $shortTName) 
		return "production_request_detail";
	if("attendance_summary" == $shortTName) 
		return "attendance_summary";
	if("Other_Incentives" == $shortTName) 
		return "Other_Incentives";
	if("Other_Deductions" == $shortTName) 
		return "Other_Deductions";
	if("department" == $shortTName) 
		return "department";
	if("journal_entry_cash_disbursement" == $shortTName) 
		return "journal_entry_cash_disbursement";
	if("journal_entry_cash_receipt" == $shortTName) 
		return "journal_entry_cash_receipt";
	if("journal_entry_center_replenishment" == $shortTName) 
		return "journal_entry_center_replenishment";
	if("journal_entry_credit_memo" == $shortTName) 
		return "journal_entry_credit_memo";
	if("journal_entry_debit_memo" == $shortTName) 
		return "journal_entry_debit_memo";
	if("journal_entry_depreciation" == $shortTName) 
		return "journal_entry_depreciation";
	if("journal_entry_general_journal" == $shortTName) 
		return "journal_entry_general_journal";
	if("journal_entry_inventory_adjustment" == $shortTName) 
		return "journal_entry_inventory_adjustment";
	if("journal_entry_petty_cash" == $shortTName) 
		return "journal_entry_petty_cash";
	if("journal_entry_project_replenishment" == $shortTName) 
		return "journal_entry_project_replenishment";
	if("journal_entry_purchases" == $shortTName) 
		return "journal_entry_purchases";
	if("journal_entry_replenishment" == $shortTName) 
		return "journal_entry_replenishment";
	if("journal_entry_service_revenue" == $shortTName) 
		return "journal_entry_service_revenue";
	if("Credit_Card_monitoring" == $shortTName) 
		return "Credit_Card_monitoring";
	if("sales_report_per_item_Chart" == $shortTName) 
		return "sales_report_per_item Chart";
	if("Leave_for_approval" == $shortTName) 
		return "Leave_for_approval";
	if("loan_for_approval" == $shortTName) 
		return "loan_for_approval";
	if("loan_detail_for_approval" == $shortTName) 
		return "loan_detail_for_approval";
	if("Overtime_for_approval" == $shortTName) 
		return "Overtime_for_approval";
	if("Purchase_Request_for_approval" == $shortTName) 
		return "Purchase_Request_for_approval";
	if("purchase_request_detail_approval" == $shortTName) 
		return "purchase_request_detail_approval";
	if("Production_Request_for_approval" == $shortTName) 
		return "Production_Request_for_approval";
	if("production_request_detail_for_approval" == $shortTName) 
		return "production_request_detail_for_approval";
	if("Inventory_Transfer_for_approval" == $shortTName) 
		return "Inventory_Transfer_for_approval";
	if("inventory_transfer_detail_for_approval" == $shortTName) 
		return "inventory_transfer_detail_for_approval";
	if("Leave_for_checking" == $shortTName) 
		return "Leave_for_checking";
	if("Loan_for_checking" == $shortTName) 
		return "Loan_for_checking";
	if("loan_detail_for_checking" == $shortTName) 
		return "loan_detail_for_checking";
	if("Overtime_for_checking" == $shortTName) 
		return "Overtime_for_checking";
	if("Purchase_Request_for_checking" == $shortTName) 
		return "Purchase_Request_for_checking";
	if("Production_Request_for_checking" == $shortTName) 
		return "Production_Request_for_checking";
	if("Inventory_Transfer_for_checking" == $shortTName) 
		return "Inventory_Transfer_for_checking";
	if("inventory_transfer_detail_for_checking" == $shortTName) 
		return "inventory_transfer_detail_for_checking";
	if("attendance_type" == $shortTName) 
		return "attendance_type";
	if("Backload_for_approval" == $shortTName) 
		return "Backload_for_approval";
	if("backload_detail_for_approval" == $shortTName) 
		return "backload_detail_for_approval";
	if("Backload_for_checking" == $shortTName) 
		return "Backload_for_checking";
	if("backload_detail_for_checking" == $shortTName) 
		return "backload_detail_for_checking";
	if("Cash_Receipt_for_approval" == $shortTName) 
		return "Cash_Receipt_for_approval";
	if("Cash_Receipt_for_checking" == $shortTName) 
		return "Cash_Receipt_for_checking";
	if("cash_receipt_detail_for_checking" == $shortTName) 
		return "cash_receipt_detail_for_checking";
	if("cash_receipt_detail_for_approval" == $shortTName) 
		return "cash_receipt_detail_for_approval";
	if("Payroll_Entry_for_approval" == $shortTName) 
		return "Payroll_Entry_for_approval";
	if("Payroll_Entry_for_checking" == $shortTName) 
		return "Payroll_Entry_for_checking";
	if("payroll_entry_detail_for_approval" == $shortTName) 
		return "payroll_entry_detail_for_approval";
	if("payroll_entry_detail_for_checking" == $shortTName) 
		return "payroll_entry_detail_for_checking";
	if("Process_Payroll_for_approval" == $shortTName) 
		return "Process_Payroll_for_approval";
	if("Process_Payroll_for_checking" == $shortTName) 
		return "Process_Payroll_for_checking";
	if("process_payroll_detail_for_approval" == $shortTName) 
		return "process_payroll_detail_for_approval";
	if("process_payroll_detail_for_checking" == $shortTName) 
		return "process_payroll_detail_for_checking";
	if("Service_Report_for_approval" == $shortTName) 
		return "Service_Report_for_approval";
	if("Service_Report_for_checking" == $shortTName) 
		return "Service_Report_for_checking";
	if("service_report_detail_for_approval" == $shortTName) 
		return "service_report_detail_for_approval";
	if("service_report_detail_for_checking" == $shortTName) 
		return "service_report_detail_for_checking";
	if("Banch_Reports" == $shortTName) 
		return "Banch Reports";
	if("journal_entry_production" == $shortTName) 
		return "journal_entry_production";
	if("journal_entry_deposit" == $shortTName) 
		return "journal_entry_deposit";
	if("production_request_bom" == $shortTName) 
		return "production_request_bom";
	if("production_bom" == $shortTName) 
		return "production_bom";
	if("company_info" == $shortTName) 
		return "company_info";
	if("return" == $shortTName) 
		return "return";
	if("return_detail" == $shortTName) 
		return "return_detail";
	if("fixed_asset_issue" == $shortTName) 
		return "fixed_asset_issue";
	if("fixed_asset_issue_detail" == $shortTName) 
		return "fixed_asset_issue_detail";
	if("production_request_detail_for_checking" == $shortTName) 
		return "production_request_detail_for_checking";
	if("cash_disbursment_detail_for_approval" == $shortTName) 
		return "cash_disbursment_detail_for_approval";
	if("cash_disbursment_detail_for_checking" == $shortTName) 
		return "cash_disbursment_detail_for_checking";
	if("purchase_request_detail_for_checking" == $shortTName) 
		return "purchase_request_detail_for_checking";
	if("trip_ticket_for_approval" == $shortTName) 
		return "trip_ticket_for_approval";
	if("trip_ticket_for_checking" == $shortTName) 
		return "trip_ticket_for_checking";
	if("trip_detail_for_approval" == $shortTName) 
		return "trip_detail_for_approval";
	if("trip_detail_for_checking" == $shortTName) 
		return "trip_detail_for_checking";
	if("sales_report1" == $shortTName) 
		return "sales_report";
	if("Customer_Balance_Report" == $shortTName) 
		return "Customer_Balance_Report";
	if("Customer_Balance_Report_detail" == $shortTName) 
		return "Customer_Balance_Report_detail";
	if("Collection_Report_per_Customer" == $shortTName) 
		return "Collection_Report_per_Customer";
	if("Aging_of_AR_Report" == $shortTName) 
		return "Aging_of_AR_Report";
	if("Aging_of_AR_Report_detail" == $shortTName) 
		return "Aging_of_AR_Report_detail";
	if("Payment_per_Supplier_Report" == $shortTName) 
		return "Payment_per_Supplier_Report";
	if("Aging_of_AP_Report" == $shortTName) 
		return "Aging of_AP_Report";
	if("Aging_of_AP_Report_detail" == $shortTName) 
		return "Aging of_AP_Report_detail";
	if("Inventory_Balance_Report" == $shortTName) 
		return "Inventory_Balance_Report";
	if("Inventory_Valuation_Report" == $shortTName) 
		return "Inventory_Valuation_Report";
	if("Inventory_Below_Reorder_Point_Report" == $shortTName) 
		return "Inventory_Below_Reorder_Point_Report";
	if("Sales_Report_per_BP" == $shortTName) 
		return "Sales_Report_per_BP";
	if("Sales_Report_per_Item" == $shortTName) 
		return "Sales_Report_per_Item";
	if("Sales_Report_per_Cost_Center" == $shortTName) 
		return "Sales_Report_per_Cost_Center";
	if("Sales_Report_per_BP_detail" == $shortTName) 
		return "Sales_Report_per_BP_detail";
	if("Sales_Report_per_Item_detail" == $shortTName) 
		return "Sales_Report_per_Item_detail";
	if("Sales_Report_per_Cost_Center_detail" == $shortTName) 
		return "Sales_Report_per_Cost_Center_detail";
	if("Purchase_Report" == $shortTName) 
		return "Purchase_Report";
	if("Purchase_Report_per_BP" == $shortTName) 
		return "Purchase_Report_per_BP";
	if("Purchase_Report_per_Item" == $shortTName) 
		return "Purchase_Report_per_Item";
	if("Purchase_Report_per_Cost_Center" == $shortTName) 
		return "Purchase_Report_per_Cost_Center";
	if("Purchase_Report_per_BP_detail" == $shortTName) 
		return "Purchase_Report_per_BP_detail";
	if("Purchase_Report_per_Item_detail" == $shortTName) 
		return "Purchase_Report_per_Item_detail";
	if("Purchase_Report_per_Cost_Center_detail" == $shortTName) 
		return "Purchase_Report_per_Cost_Center_detail";
	if("Transaction_Registry" == $shortTName) 
		return "Transaction Registry";
	if("Sales_Report_per_sales_rep" == $shortTName) 
		return "Sales_Report_per_sales_rep";
	if("PDC_Registry_Report" == $shortTName) 
		return "PDC_Registry_Report";
	if("Available_Vehicles1" == $shortTName) 
		return "Available_Vehicles";
	if("Trip_Ticket_Report" == $shortTName) 
		return "Trip_Ticket_Report";
	if("Purchase_Request_Report" == $shortTName) 
		return "Purchase_Request_Report";
	if("Purchase_Import_Costing" == $shortTName) 
		return "Purchase_Import_Costing";
	if("Production_Costing_Report" == $shortTName) 
		return "Production_Costing_Report";
	if("Production_Report" == $shortTName) 
		return "Production_Report";
	if("Closed_Opportunities" == $shortTName) 
		return "Closed_Opportunities";
	if("Cases_with_Sotutions" == $shortTName) 
		return "Cases_with_Sotutions";
	if("Totals_Cases_Created" == $shortTName) 
		return "Totals_Cases_Created";
	if("Totals_Open_Cases_by_Agent" == $shortTName) 
		return "Totals_Open_Cases_by_Agent";
	if("Campaign_Contacts" == $shortTName) 
		return "Campaign_Contacts";
	if("Campaign_leads" == $shortTName) 
		return "Campaign_leads";
	if("Campaign_ROI_Analysis" == $shortTName) 
		return "Campaign_ROI_Analysis";
	if("Campaign_Revenue" == $shortTName) 
		return "Campaign_Revenue";
	if("Opportunity_Schedule" == $shortTName) 
		return "Opportunity_Schedule";
	if("Sales_by_Account" == $shortTName) 
		return "Sales_by_Account";
	if("Sales_by_Lead_Source" == $shortTName) 
		return "Sales_by_Lead_Source";
	if("Sales_by_Partner" == $shortTName) 
		return "Sales_by_Partner";
	if("Sales_by_Rep" == $shortTName) 
		return "Sales_by_Rep";
	if("Leave_Ledger_Report" == $shortTName) 
		return "Leave_Ledger_Report";
	if("Leave_Summary" == $shortTName) 
		return "Leave_Summary";
	if("Payroll_Register" == $shortTName) 
		return "Payroll_Register";
	if("Employee_Leave_History" == $shortTName) 
		return "Employee_Leave_History";
	if("Employee_Register" == $shortTName) 
		return "Employee_Register";
	if("DR_Reports" == $shortTName) 
		return "DR_Reports";
	if("Sales_Order_Report" == $shortTName) 
		return "Sales_Order_Report";
	if("Case_History" == $shortTName) 
		return "Case_History";
	if("quality_control" == $shortTName) 
		return "quality_control";
	if("quality_control_detail" == $shortTName) 
		return "quality_control_detail";
	if("budget" == $shortTName) 
		return "budget";
	if("budget_detail" == $shortTName) 
		return "budget_detail";
	if("repairs_and_maintenance" == $shortTName) 
		return "repairs_and_maintenance";
	if("production_form" == $shortTName) 
		return "production_form";
	if("bluewave_employees" == $shortTName) 
		return "bluewave_employees";
	if("bluewave_employees_detail" == $shortTName) 
		return "bluewave_employees_detail";
	if("bluewave_employees_detail2" == $shortTName) 
		return "bluewave_employees_detail2";
	if("bluewave_employees_detail3" == $shortTName) 
		return "bluewave_employees_detail3";
	if("bluewave_employees_detail4" == $shortTName) 
		return "bluewave_employees_detail4";
	if("bluewave_employees_detail5" == $shortTName) 
		return "bluewave_employees_detail5";
	if("bluewave_employees_detail6" == $shortTName) 
		return "bluewave_employees_detail6";
	if("rfp" == $shortTName) 
		return "rfp";
	if("telex" == $shortTName) 
		return "telex";
	if("repairs_and_maintenance_detail" == $shortTName) 
		return "repairs_and_maintenance_detail";
	if("repair_list" == $shortTName) 
		return "repair_list";
	if("marketing_colateral" == $shortTName) 
		return "marketing_colateral";
	if("email" == $shortTName) 
		return "email";
	if("accessories" == $shortTName) 
		return "accessories";
	if("sales_order_materials_detail" == $shortTName) 
		return "sales_order_materials_detail";
	if("sales_order_progress_billing_detail" == $shortTName) 
		return "sales_order_progress_billing_detail";
	if("sub_contract" == $shortTName) 
		return "sub_contract";
	if("sub_contract_billing_detail" == $shortTName) 
		return "sub_contract_billing_detail";
	if("sub_contract_detail" == $shortTName) 
		return "sub_contract_detail";
	if("Monthly_Pag_ibig_Summary_Report" == $shortTName) 
		return "Monthly_Pag-ibig_Summary_Report";
	if("Monthly_Philhealth_Report" == $shortTName) 
		return "Monthly_Philhealth_Report";
	if("Monthly_Philhealth_Summary_Report" == $shortTName) 
		return "Monthly_Philhealth_Summary_Report";
	if("Monthly_SSS_Report" == $shortTName) 
		return "Monthly_SSS_Report";
	if("Monthly_SSS_Report_by_Department_Report" == $shortTName) 
		return "Monthly_SSS_Report_by_Department_Report";
	if("Monthly_Withholding_Tax_Prooflist_Report" == $shortTName) 
		return "Monthly_Withholding_Tax_Prooflist_Report";
	if("Monthly_Witholding_Tax_Collection_List_by_Department_Report" == $shortTName) 
		return "Monthly_Witholding_Tax_Collection_List_by_Department_Report";
	if("Overtime_Summary_Report" == $shortTName) 
		return "Overtime_Summary_Report";
	if("Monthly_Pag_ibig_Report" == $shortTName) 
		return "Monthly_Pag-ibig_Report";
	if("sales_order" == $shortTName) 
		return "sales_order";
	if("sales_order_detail" == $shortTName) 
		return "sales_order_detail";
	if("vat_relief_importation" == $shortTName) 
		return "vat_relief_importation";
	if("vat_relief_purchases" == $shortTName) 
		return "vat_relief_purchases";
	if("vat_relief_sales" == $shortTName) 
		return "vat_relief_sales";
	if("journal_ppe_disposal" == $shortTName) 
		return "journal_ppe_disposal";
	if("credit_memo_detail_cgs" == $shortTName) 
		return "credit_memo_detail_cgs";
	if("marketing_request_form" == $shortTName) 
		return "marketing_request_form";
	if("Quotation_Dashboard_Customer" == $shortTName) 
		return "Quotation_Dashboard_Customer";
	if("Sales_Order_Dashboard_Customer" == $shortTName) 
		return "Sales_Order_Dashboard_Customer";
	if("Delivery_Receipt_Dashboard_Customer" == $shortTName) 
		return "Delivery_Receipt_Dashboard_Customer";
	if("Sales_Dashboard_Customer" == $shortTName) 
		return "Sales_Dashboard_Customer";
	if("Backload_Dashboard_Customer" == $shortTName) 
		return "Backload_Dashboard_Customer";
	if("Purchase_Request_Dashboard_Supplier" == $shortTName) 
		return "Purchase_Request_Dashboard_Supplier";
	if("Purchase_Order_Dashboard_Supplier" == $shortTName) 
		return "Purchase_Order_Dashboard_Supplier";
	if("Purchase_Dashboard_Supplier" == $shortTName) 
		return "Purchase_Dashboard_Supplier";
	if("Cash_Disbursement_Dashboard_Supplier" == $shortTName) 
		return "Cash_Disbursement_Dashboard_Supplier";
	if("Debit_Memo_Dashboard_Supplier" == $shortTName) 
		return "Debit_Memo_Dashboard_Supplier";
	if("Purchase_Request_Dashboard_Item1" == $shortTName) 
		return "Purchase_Request_Dashboard_Item";
	if("Purchase_Order_Dashboard_Item" == $shortTName) 
		return "Purchase_Order_Dashboard_Item";
	if("Purchase_Order_Receiving_Dashboard_Item" == $shortTName) 
		return "Purchase_Order_Receiving_Dashboard_Item";
	if("Production_Dashboard_Item" == $shortTName) 
		return "Production_Dashboard_Item";
	if("sub_contract_for_approval" == $shortTName) 
		return "sub_contract_for_approval";
	if("sub_contract_for_checking" == $shortTName) 
		return "sub_contract_for_checking";
	if("sub_contract_detail_for_approval" == $shortTName) 
		return "sub_contract_detail_for_approval";
	if("sub_contract_detail_for_checking" == $shortTName) 
		return "sub_contract_detail_for_checking";
	if("budget_for_approval" == $shortTName) 
		return "budget_for_approval";
	if("budget_for_checking" == $shortTName) 
		return "budget_for_checking";
	if("budget_detail_for_checking" == $shortTName) 
		return "budget_detail_for_checking";
	if("budget_detail_for_approval" == $shortTName) 
		return "budget_detail_for_approval";
	if("undertime_for_approval" == $shortTName) 
		return "undertime_for_approval";
	if("undertime_for_checking" == $shortTName) 
		return "undertime_for_checking";
	if("request_for_payment_for_approval" == $shortTName) 
		return "request_for_payment_for_approval";
	if("request_for_payment_for_checking" == $shortTName) 
		return "request_for_payment_for_checking";
	if("repairs_and_maintenance_detail_for_approval" == $shortTName) 
		return "repairs_and_maintenance_detail_for_approval";
	if("request_for_payment_detail_for_approval" == $shortTName) 
		return "request_for_payment_detail_for_approval";
	if("request_for_payment_detail_for_checking" == $shortTName) 
		return "request_for_payment_detail_for_checking";
	if("repairs_and_maintenance_for_approval" == $shortTName) 
		return "repairs_and_maintenance_for_approval";
	if("repairs_and_maintenance_for_checking" == $shortTName) 
		return "repairs_and_maintenance_for_checking";
	if("repairs_and_maintenance_detail_for_checking" == $shortTName) 
		return "repairs_and_maintenance_detail_for_checking";
	if("Disposal_for_checking" == $shortTName) 
		return "Disposal_for_checking";
	if("Active_campaign_report" == $shortTName) 
		return "Active_campaign_report";
	if("For_Billing_report" == $shortTName) 
		return "For_Billing_report";
	if("sales_order_for_project_mngmt" == $shortTName) 
		return "sales_order_for_project_mngmt";
	if("inventory_transfer_cost_detail" == $shortTName) 
		return "inventory_transfer_cost_detail";
	if("Supplier_Balance_Report_Details" == $shortTName) 
		return "Supplier_Balance_Report_Details";
	if("Supplier_Balance_Report" == $shortTName) 
		return "Supplier_Balance_Report";
	if("Inventory_per_Cost_center_Report" == $shortTName) 
		return "Inventory_per_Cost_center_Report";
	if("Deposit_Report" == $shortTName) 
		return "Deposit_Report";
	if("SO_for_Billing_Report" == $shortTName) 
		return "SO_for_Billing_Report";
	if("Billing_due_dates_Report" == $shortTName) 
		return "Billing_due_dates_Report";
	if("Packing_List_Report" == $shortTName) 
		return "Packing_List_Report";
	if("Open_Trip_Ticket_Report" == $shortTName) 
		return "Open_Trip_Ticket_Report";
	if("Transfer_Report" == $shortTName) 
		return "Transfer_Report";
	if("Production_Report_invw" == $shortTName) 
		return "Production_Report_invw";
	if("Supply_per_Department__Report_invw" == $shortTName) 
		return "Supply_per_Department__Report_invw";
	if("Item_Balances_Report_invw" == $shortTName) 
		return "Item_Balances_Report_invw";
	if("Open_Case_Report_s" == $shortTName) 
		return "Open_Case_Report_s";
	if("Sales_Order_per_Campaign_Report_marketing" == $shortTName) 
		return "Sales_Order_per_Campaign_Report_marketing";
	if("Sales_per_campaign_Report_marketing" == $shortTName) 
		return "Sales_per_campaign_Report_marketing";
	if("Asset_per_Accountability_Report_invw" == $shortTName) 
		return "Asset_per_Accountability_Report_invw";
	if("Deposit_in_transit_Report" == $shortTName) 
		return "Deposit_in_transit_Report";
	if("Outstanding_Checks_Report" == $shortTName) 
		return "Outstanding_Checks_Report";
	if("Absences_Detail_Report" == $shortTName) 
		return "Absences_Detail_Report";
	if("Absences_Report" == $shortTName) 
		return "Absences_Report";
	if("Attendance_Report" == $shortTName) 
		return "Attendance_Report";
	if("DTR_Report" == $shortTName) 
		return "DTR_Report";
	if("Summary_of_Tardiness_Report" == $shortTName) 
		return "Summary_of_Tardiness_Report";
	if("Followup_Report" == $shortTName) 
		return "Followup_Report";
	if("Coming_Meeting_Report" == $shortTName) 
		return "Coming_Meeting_Report";
	if("Meeting_Report" == $shortTName) 
		return "Meeting_Report";
	if("Next_Action_Report" == $shortTName) 
		return "Next_Action_Report";
	if("Calls_Report" == $shortTName) 
		return "Calls_Report";
	if("sales_per_sales_rep_Report" == $shortTName) 
		return "sales_per_sales_rep_Report";
	if("Expense_per_Project_Report" == $shortTName) 
		return "Expense_per_Project_Report";
	if("Tardiness_Report_Summary" == $shortTName) 
		return "Tardiness_Report_Summary";
	if("Timekeeping__Hours_Worked__Report_Summary" == $shortTName) 
		return "Timekeeping_(Hours_Worked)_Report_Summary";
	if("Withholding_Tax_Collection_List_per_Month_Report" == $shortTName) 
		return "Withholding_Tax_Collection_List_per_Month_Report";
	if("Business_Owner_Dashboard" == $shortTName) 
		return "Business_Owner_Dashboard";
	if("Finance_Dashboard" == $shortTName) 
		return "Finance_Dashboard";
	if("Purchasing_Dashboard" == $shortTName) 
		return "Purchasing_Dashboard";
	if("Production_Dashboard1" == $shortTName) 
		return "Production_Dashboard";
	if("Marketing_Dashboard1" == $shortTName) 
		return "Marketing_Dashboard";
	if("Support_Dashboard" == $shortTName) 
		return "Support_Dashboard";
	if("Sales_Dashboard" == $shortTName) 
		return "Sales_Dashboard";
	if("Human_Resource_Dashboard" == $shortTName) 
		return "Human_Resource_Dashboard";
	if("Project_Manager_Dashboard" == $shortTName) 
		return "Project_Manager_Dashboard";
	if("System_Admistration_Dashboard" == $shortTName) 
		return "System_Admistration_Dashboard";
	if("purchase_report_per_item_Chart" == $shortTName) 
		return "purchase_report_per_item Chart";
	if("System_Audit_Report" == $shortTName) 
		return "System_Audit_Report";
	if("sale_cost_detail" == $shortTName) 
		return "sale_cost_detail";
	if("currency_exchange_rate" == $shortTName) 
		return "currency_exchange_rate";
	if("cash_receipt_cost_detail" == $shortTName) 
		return "cash_receipt_cost_detail";
	if("production_cost_detail" == $shortTName) 
		return "production_cost_detail";
	if("Employee" == $shortTName) 
		return "Employee";
	if("Quotation_Dashboard" == $shortTName) 
		return "Quotation_Dashboard";
	if("Sales_Order_Dashboard" == $shortTName) 
		return "Sales_Order_Dashboard";
	if("Reimbursement_Dashboard" == $shortTName) 
		return "Reimbursement_Dashboard";
	if("Transaction_Registry_Dashboard_SaleRep" == $shortTName) 
		return "Transaction_Registry_Dashboard_SaleRep";
	if("Loan_Dashboard" == $shortTName) 
		return "Loan_Dashboard";
	if("Leave_Dashboard" == $shortTName) 
		return "Leave_Dashboard";
	if("Over_Time_Dashboard" == $shortTName) 
		return "Over_Time_Dashboard";
	if("Undertime_Dashboard" == $shortTName) 
		return "Undertime_Dashboard";
	if("Fieldwork_Dashboard" == $shortTName) 
		return "Fieldwork_Dashboard";
	if("Change_Time_Dashboard" == $shortTName) 
		return "Change_Time_Dashboard";
	if("Transaction_Registry_Dashboard_Employee" == $shortTName) 
		return "Transaction_Registry_Dashboard_Employee";
	if("Transaction_Registry_Dashboard_Supplier" == $shortTName) 
		return "Transaction_Registry_Dashboard_Supplier";
	if("Transaction_Registry_Dashboard_Customer" == $shortTName) 
		return "Transaction_Registry_Dashboard_Customer";
	if("lyl_estimate_dr" == $shortTName) 
		return "lyl_estimate_dr";
	if("lyl_estimate_dr_detail" == $shortTName) 
		return "lyl_estimate_dr_detail";
	if("lyl_estimate_dr_detail_accessories" == $shortTName) 
		return "lyl_estimate_dr_detail_accessories";
	if("lyl_estimate_dr_detail_aluminum" == $shortTName) 
		return "lyl_estimate_dr_detail_aluminum";
	if("lyl_estimate_dr_detail_e_accessories" == $shortTName) 
		return "lyl_estimate_dr_detail_e_accessories";
	if("lyl_estimate_dr_detail_glass" == $shortTName) 
		return "lyl_estimate_dr_detail_glass";
	if("sum_dr_qty" == $shortTName) 
		return "sum_dr_qty";
	if("check_book_registry" == $shortTName) 
		return "check_book_registry";
	if("check_book_registry_detail" == $shortTName) 
		return "check_book_registry_detail";
	if("vat_type" == $shortTName) 
		return "vat_type";
	if("quotation_detail" == $shortTName) 
		return "quotation_detail";
	if("bal_purchase_order_detail" == $shortTName) 
		return "bal_purchase_order_detail";
	if("bal_sales_detail" == $shortTName) 
		return "bal_sales_detail";
	if("pending_items_reminders" == $shortTName) 
		return "pending_items_reminders";
	if("petty_cash_for_liquidation" == $shortTName) 
		return "petty_cash_for_liquidation";
	if("payment_method" == $shortTName) 
		return "payment_method";
	if("account_type" == $shortTName) 
		return "account_type";
	if("phase" == $shortTName) 
		return "phase";
	if("item_cost_recalculation" == $shortTName) 
		return "item_cost_recalculation";
	if("issue_components" == $shortTName) 
		return "issue_components";
	if("issue_components_detail" == $shortTName) 
		return "issue_components_detail";
	if("unbuild_assemblies" == $shortTName) 
		return "unbuild_assemblies";
	if("unbuild_assemblies_detail" == $shortTName) 
		return "unbuild_assemblies_detail";
	if("demand_plan" == $shortTName) 
		return "demand_plan";
	if("input_vat_allocation" == $shortTName) 
		return "input_vat_allocation";
	if("case_type" == $shortTName) 
		return "case_type";
	if("qc_inspection" == $shortTName) 
		return "qc_inspection";
	if("packing_list_for_checking" == $shortTName) 
		return "packing_list_for_checking";
	if("packing_list_for_approval" == $shortTName) 
		return "packing_list_for_approval";
	if("packing_list_detail_for_checking" == $shortTName) 
		return "packing_list_detail_for_checking";
	if("packing_list_detail_for_approval" == $shortTName) 
		return "packing_list_detail_for_approval";
	if("disposed_inv_detail" == $shortTName) 
		return "disposed_inv_detail";
	if("disposed_inv" == $shortTName) 
		return "disposed_inv";
	if("disposed_inv_for_checking" == $shortTName) 
		return "disposed_inv_for_checking";
	if("disposed_inv_for_approval" == $shortTName) 
		return "disposed_inv_for_approval";
	if("disposed_inv_detail_for_checking" == $shortTName) 
		return "disposed_inv_detail_for_checking";
	if("disposed_inv_detail_for_approval" == $shortTName) 
		return "disposed_inv_detail_for_approval";
	if("For_Approval_Dashboard" == $shortTName) 
		return "For_Approval_Dashboard";
	if("For_Checking_Dashboard" == $shortTName) 
		return "For_Checking_Dashboard";
	if("Customer_For_Approval" == $shortTName) 
		return "Customer_For_Approval";
	if("request_for_payment_report" == $shortTName) 
		return "request_for_payment_report";
	if("credit_memo_report" == $shortTName) 
		return "credit_memo_report";
	if("debit_memo_report" == $shortTName) 
		return "debit_memo_report";
	if("production_request_report" == $shortTName) 
		return "production_request_report";
	if("Supplies_Balance_report" == $shortTName) 
		return "Supplies_Balance_report";
	if("approved_sales_order" == $shortTName) 
		return "approved_sales_order";
	if("purchase_order_for_receiving" == $shortTName) 
		return "purchase_order_for_receiving";
	if("Dashboard" == $shortTName) 
		return "Dashboard";
	if("packing_list" == $shortTName) 
		return "packing_list";
	if("packing_list_detail" == $shortTName) 
		return "packing_list_detail";
	if("open_quote" == $shortTName) 
		return "open_quote";
	if("open_estimates" == $shortTName) 
		return "open_estimates";
	if("open_po" == $shortTName) 
		return "open_po";
	if("open_pr" == $shortTName) 
		return "open_pr";
	if("open_production_request" == $shortTName) 
		return "open_production_request";
	if("open_qc" == $shortTName) 
		return "open_qc";
	if("open_receiving" == $shortTName) 
		return "open_receiving";
	if("open_request_for_payment" == $shortTName) 
		return "open_request_for_payment";
	if("open_so" == $shortTName) 
		return "open_so";
	if("open_so_for_invoice" == $shortTName) 
		return "open_so_for_invoice";
	if("open_so_for_project_management" == $shortTName) 
		return "open_so_for_project_management";
	if("open_backload" == $shortTName) 
		return "open_backload";
	if("purchase_order_receiving_detail" == $shortTName) 
		return "purchase_order_receiving_detail";
	if("Purchase_Order_Receiving_Dashboard_Supplier1" == $shortTName) 
		return "Purchase_Order_Receiving_Dashboard_Supplier";
	if("avg_inv" == $shortTName) 
		return "avg_inv";
	if("open_backload_Report" == $shortTName) 
		return "open_backload_Report";
	if("open_estimates_Report" == $shortTName) 
		return "open_estimates_Report";
	if("open_po_Report" == $shortTName) 
		return "open_po_Report";
	if("open_pr_Report" == $shortTName) 
		return "open_pr_Report";
	if("open_production_request_Report" == $shortTName) 
		return "open_production_request_Report";
	if("open_qc_Report" == $shortTName) 
		return "open_qc_Report";
	if("open_so_for_project_management_Report" == $shortTName) 
		return "open_so_for_project_management_Report";
	if("open_so_for_invoice_Report" == $shortTName) 
		return "open_so_for_invoice_Report";
	if("open_so_Report" == $shortTName) 
		return "open_so_Report";
	if("open_receiving_Report" == $shortTName) 
		return "open_receiving_Report";
	if("monthly_consuption_for_production_Report" == $shortTName) 
		return "monthly_consuption_for_production_Report";
	if("issue_components_for_approval" == $shortTName) 
		return "issue_components_for_approval";
	if("issue_components_for_checking" == $shortTName) 
		return "issue_components_for_checking";
	if("issue_components_detail_for_checking" == $shortTName) 
		return "issue_components_detail_for_checking";
	if("issue_components_detail_for_approval" == $shortTName) 
		return "issue_components_detail_for_approval";
	if("fieldwork_for_approval" == $shortTName) 
		return "fieldwork_for_approval";
	if("fieldwork_for_checking" == $shortTName) 
		return "fieldwork_for_checking";
	if("Purchases_Order_for_checking" == $shortTName) 
		return "Purchases_Order_for_checking";
	if("Purchases_Order_for_approval" == $shortTName) 
		return "Purchases_Order_for_approval";
	if("Receiving_for_checking" == $shortTName) 
		return "Receiving_for_checking";
	if("Receiving_for_approval" == $shortTName) 
		return "Receiving_for_approval";
	if("inventory_tranfer_Report" == $shortTName) 
		return "inventory_tranfer_Report";
	if("Purchase_vs_Production_Report" == $shortTName) 
		return "Purchase_vs_Production_Report";
	if("cash_sales_Report" == $shortTName) 
		return "cash_sales_Report";
	if("Center_Replenishment_for_approval" == $shortTName) 
		return "Center_Replenishment_for_approval";
	if("Center_Replenishment_for_checking" == $shortTName) 
		return "Center_Replenishment_for_checking";
	if("Purchase_vs_Production" == $shortTName) 
		return "Purchase_vs_Production";
	if("Production_Summary" == $shortTName) 
		return "Production_Summary";
	if("pending_production_request" == $shortTName) 
		return "pending_production_request";
	if("unbuild_assemblies_for_checking" == $shortTName) 
		return "unbuild_assemblies_for_checking";
	if("unbuild_assemblies_for_approval" == $shortTName) 
		return "unbuild_assemblies_for_approval";
	if("unbuild_assemblies_detail_for_checking" == $shortTName) 
		return "unbuild_assemblies_detail_for_checking";
	if("quality_control_for_approval" == $shortTName) 
		return "quality_control_for_approval";
	if("quality_control_for_checking" == $shortTName) 
		return "quality_control_for_checking";
	if("quality_control_detail_for_approval" == $shortTName) 
		return "quality_control_detail_for_approval";
	if("raw_cake_balance" == $shortTName) 
		return "raw_cake_balance";
	if("bal_cashreceipt" == $shortTName) 
		return "bal_cashreceipt";
	if("bal_cashpayment" == $shortTName) 
		return "bal_cashpayment";
	if("opportunity" == $shortTName) 
		return "opportunity";
	if("Current_Accounts" == $shortTName) 
		return "Current_Accounts";
	if("bal_lyl_acc" == $shortTName) 
		return "bal_lyl_acc";
	if("bal_lyl_glass" == $shortTName) 
		return "bal_lyl_glass";
	if("bal_lyl_eacc" == $shortTName) 
		return "bal_lyl_eacc";
	if("bal_lyl_allum" == $shortTName) 
		return "bal_lyl_allum";
	if("opportunity_details" == $shortTName) 
		return "opportunity_details";
	if("Collection_Report_per_Sales_Rep" == $shortTName) 
		return "Collection_Report_per_Sales_Rep";
	if("Inventory_per_lot_no" == $shortTName) 
		return "Inventory_per_lot_no";
	if("marketing_request_form_for_checking" == $shortTName) 
		return "marketing_request_form_for_checking";
	if("marketing_request_form_for_approval" == $shortTName) 
		return "marketing_request_form_for_approval";
	if("Accounts_Payable_Dashboard" == $shortTName) 
		return "Accounts_Payable_Dashboard";
	if("201" == $shortTName) 
		return "201";
	if("Cashflow_forcast" == $shortTName) 
		return "Cashflow_forcast";
	if("Business_Owner_Sales_Report" == $shortTName) 
		return "Business_Owner_Sales_Report";
	if("Business_Owner_Purchase_Report" == $shortTName) 
		return "Business_Owner_Purchase_Report";
	if("Business_Owner_Cash_Balance" == $shortTName) 
		return "Business_Owner_Cash_Balance";
	if("Business_Owner_Top_Customer_Income" == $shortTName) 
		return "Business_Owner_Top_Customer_Income";
	if("AR_Billing_due_dates_Report" == $shortTName) 
		return "AR_Billing_due_dates_Report";
	if("AR_Aging_of_AR_Report" == $shortTName) 
		return "AR_Aging_of_AR_Report";
	if("AR_Top_5_Customer_balances" == $shortTName) 
		return "AR_Top_5_Customer_balances";
	if("AR_Progress_Billing_Alert" == $shortTName) 
		return "AR_Progress_Billing_Alert";
	if("Account_Receivable_Dashboard" == $shortTName) 
		return "Account_Receivable_Dashboard";
	if("Finance_Cash_Balance" == $shortTName) 
		return "Finance_Cash_Balance";
	if("Finance_approved_request_for_payment" == $shortTName) 
		return "Finance_approved_request_for_payment";
	if("Finance_Cash_flow_forcast" == $shortTName) 
		return "Finance_Cash_flow_forcast";
	if("Finance_Approved_Employee_Loan" == $shortTName) 
		return "Finance_Approved_Employee_Loan";
	if("Finance_Cash_Balances" == $shortTName) 
		return "Finance_Cash_Balances";
	if("AP_Aging_of_Payables" == $shortTName) 
		return "AP_Aging_of_Payables";
	if("Purchasing_Item_below_reorder" == $shortTName) 
		return "Purchasing_Item_below_reorder";
	if("production_per_item" == $shortTName) 
		return "production_per_item";
	if("AP_Top_5_Supplier_balances" == $shortTName) 
		return "AP_Top_5_Supplier_balances";
	if("Warehouse_Item_balances" == $shortTName) 
		return "Warehouse_Item_balances";
	if("DR_per_Item" == $shortTName) 
		return "DR_per_Item";
	if("Logistic_Open_Trip_Ticket" == $shortTName) 
		return "Logistic_Open_Trip_Ticket";
	if("Logistic_Available_Vehicles" == $shortTName) 
		return "Logistic_Available_Vehicles";
	if("Marketing_Open_Campaign" == $shortTName) 
		return "Marketing_Open_Campaign";
	if("Marketing_Sales_per_campaign" == $shortTName) 
		return "Marketing_Sales_per_campaign";
	if("Support_Open_Case_Report" == $shortTName) 
		return "Support_Open_Case_Report";
	if("Support_Service_center" == $shortTName) 
		return "Support_Service_center";
	if("Sale_Meeting_Report" == $shortTName) 
		return "Sale_Meeting_Report";
	if("Sales_Next_Action_Report" == $shortTName) 
		return "Sales_Next_Action_Report";
	if("HR_attendance_summary" == $shortTName) 
		return "HR_attendance_summary";
	if("HR_Birthday_Alert" == $shortTName) 
		return "HR_Birthday_Alert";
	if("HR_holiday" == $shortTName) 
		return "HR_holiday";
	if("Project_Expense_per_Project" == $shortTName) 
		return "Project_Expense_per_Project";
	if("Project_Expense_per_Project_graph" == $shortTName) 
		return "Project_Expense_per_Project_graph";
	if("Petty_Cash_Balance" == $shortTName) 
		return "Petty_Cash_Balance";
	if("Petty_Cash_Unliquidated" == $shortTName) 
		return "Petty_Cash_Unliquidated";
	if("Petty_Cash_Purchases_per_Item" == $shortTName) 
		return "Petty_Cash_Purchases_per_Item";
	if("Petty_Cash_Dashboard" == $shortTName) 
		return "Petty_Cash_Dashboard";
	if("Accounting_Income_per_Center" == $shortTName) 
		return "Accounting_Income_per_Center";
	if("open_quote_Report" == $shortTName) 
		return "open_quote_Report";
	if("Open_cases_per_agent" == $shortTName) 
		return "Open_cases_per_agent";
	if("Marketing_sales_per_campaing_chart" == $shortTName) 
		return "Marketing_sales_per_campaing_chart";
	if("Logistics_Dashboard" == $shortTName) 
		return "Logistics_Dashboard";
	if("Accounting_Dashboard" == $shortTName) 
		return "Accounting_Dashboard";
	if("AP_open_po_receiving" == $shortTName) 
		return "AP_open_po_receiving";
	if("Production_Inventory_Balance_Report" == $shortTName) 
		return "Production_Inventory_Balance_Report";
	if("Production_WIP" == $shortTName) 
		return "Production_WIP";
	if("Warehouse_Dashboard" == $shortTName) 
		return "Warehouse_Dashboard";
	if("Warehouse_Inventory_Balance_Report" == $shortTName) 
		return "Warehouse_Inventory_Balance_Report";
	if("logistic_open_inventory_tranfer" == $shortTName) 
		return "logistic_open_inventory_tranfer";
	if("Sales_Leads" == $shortTName) 
		return "Sales_Leads";
	if("Sales_opportunity_Next_Action" == $shortTName) 
		return "Sales_opportunity_Next_Action";
	if("Sales_Active_opportunity" == $shortTName) 
		return "Sales_Active_opportunity";
	if("intercompany_sales_order_detail" == $shortTName) 
		return "intercompany_sales_order_detail";
	if("intercompany_sales_order" == $shortTName) 
		return "intercompany_sales_order";
	if("production_detail" == $shortTName) 
		return "production_detail";
	if("quality_control_detail_for_checking" == $shortTName) 
		return "quality_control_detail_for_checking";
	if("lyl_estimate_for_approval" == $shortTName) 
		return "lyl_estimate_for_approval";
	if("lyl_estimate_dr_for_approval" == $shortTName) 
		return "lyl_estimate_dr_for_approval";
	if("lyl_estimate_detail_for_approval" == $shortTName) 
		return "lyl_estimate_detail_for_approval";
	if("lyl_estimate_detail_accessories_for_approval" == $shortTName) 
		return "lyl_estimate_detail_accessories_for_approval";
	if("lyl_estimate_detail_aluminum_for_approval" == $shortTName) 
		return "lyl_estimate_detail_aluminum_for_approval";
	if("lyl_estimate_detail_e_accessories_for_approval" == $shortTName) 
		return "lyl_estimate_detail_e_accessories_for_approval";
	if("lyl_estimate_detail_glass_for_approval" == $shortTName) 
		return "lyl_estimate_detail_glass_for_approval";
	if("lyl_estimate_dr_detail_for_approval" == $shortTName) 
		return "lyl_estimate_dr_detail_for_approval";
	if("lyl_estimate_dr_detail_accessories_for_approval" == $shortTName) 
		return "lyl_estimate_dr_detail_accessories_for_approval";
	if("lyl_estimate_dr_detail_aluminum_for_approval" == $shortTName) 
		return "lyl_estimate_dr_detail_aluminum_for_approval";
	if("lyl_estimate_dr_detail_e_accessories_for_approval" == $shortTName) 
		return "lyl_estimate_dr_detail_e_accessories_for_approval";
	if("lyl_estimate_dr_detail_glass_for_approval" == $shortTName) 
		return "lyl_estimate_dr_detail_glass_for_approval";
	if("lyl_estimate_for_checking" == $shortTName) 
		return "lyl_estimate_for_checking";
	if("lyl_estimate_detail_for_checking" == $shortTName) 
		return "lyl_estimate_detail_for_checking";
	if("lyl_estimate_detail_accessories_for_checking" == $shortTName) 
		return "lyl_estimate_detail_accessories_for_checking";
	if("lyl_estimate_detail_aluminum_for_checking" == $shortTName) 
		return "lyl_estimate_detail_aluminum_for_checking";
	if("lyl_estimate_detail_e_accessories_for_checking" == $shortTName) 
		return "lyl_estimate_detail_e_accessories_for_checking";
	if("lyl_estimate_detail_glass_for_checking" == $shortTName) 
		return "lyl_estimate_detail_glass_for_checking";
	if("lyl_estimate_dr_for_checking" == $shortTName) 
		return "lyl_estimate_dr_for_checking";
	if("lyl_estimate_dr_detail_for_checking" == $shortTName) 
		return "lyl_estimate_dr_detail_for_checking";
	if("lyl_estimate_dr_detail_accessories_for_checking" == $shortTName) 
		return "lyl_estimate_dr_detail_accessories_for_checking";
	if("lyl_estimate_dr_detail_aluminum_for_checking" == $shortTName) 
		return "lyl_estimate_dr_detail_aluminum_for_checking";
	if("lyl_estimate_dr_detail_e_accessories_for_checking" == $shortTName) 
		return "lyl_estimate_dr_detail_e_accessories_for_checking";
	if("lyl_estimate_dr_detail_glass_for_checking" == $shortTName) 
		return "lyl_estimate_dr_detail_glass_for_checking";
	if("EWT_Certificates_summay" == $shortTName) 
		return "EWT_Certificates_summay";
	if("1601e" == $shortTName) 
		return "1601e";
	if("1601eform" == $shortTName) 
		return "1601eform";
	if("1601fform" == $shortTName) 
		return "1601fform";
	if("tax_person_status" == $shortTName) 
		return "tax_person_status";
	if("journal_entry_inv_transfer" == $shortTName) 
		return "journal_entry_inv_transfer";
	if("Recon_withdraw" == $shortTName) 
		return "Recon_withdraw";
	if("lot" == $shortTName) 
		return "lot";
	if("serial_no" == $shortTName) 
		return "serial_no";
	if("request_for_quote" == $shortTName) 
		return "request_for_quote";
	if("request_for_quoter_detail" == $shortTName) 
		return "request_for_quoter_detail";
	if("request_for_quote_for_checking" == $shortTName) 
		return "request_for_quote_for_checking";
	if("request_for_quote_for_approval" == $shortTName) 
		return "request_for_quote_for_approval";
	if("request_for_quoter_detail_for_checking" == $shortTName) 
		return "request_for_quoter_detail_for_checking";
	if("request_for_quoter_detail_for_approval" == $shortTName) 
		return "request_for_quoter_detail_for_approval";
	if("estimate_cost" == $shortTName) 
		return "estimate_cost";
	if("airlines" == $shortTName) 
		return "airlines";
	if("airport" == $shortTName) 
		return "airport";
	if("inco_terms" == $shortTName) 
		return "inco_terms";
	if("seaports" == $shortTName) 
		return "seaports";
	if("shipment_classification" == $shortTName) 
		return "shipment_classification";
	if("shipping_lines" == $shortTName) 
		return "shipping_lines";
	if("bal_sales_order_detail" == $shortTName) 
		return "bal_sales_order_detail";
	if("project_cont_seal" == $shortTName) 
		return "project_cont_seal";
	if("project_house" == $shortTName) 
		return "project_house";
	if("billable_liquidation" == $shortTName) 
		return "billable_liquidation";
	if("billable_pettycash" == $shortTName) 
		return "billable_pettycash";
	if("billable_checkvoucher" == $shortTName) 
		return "billable_checkvoucher";
	if("projects_dashboard" == $shortTName) 
		return "projects_dashboard";
	if("employees" == $shortTName) 
		return "employees";
	if("admin_rights" == $shortTName) 
		return "admin_rights";
	if("admin_members" == $shortTName) 
		return "admin_members";
	if("admin_users" == $shortTName) 
		return "admin_users";
	if("company" == $shortTName) 
		return "company";
	if("notifiy_quotation_Report" == $shortTName) 
		return "notifiy quotation Report";
	if("expenses_Report" == $shortTName) 
		return "expenses Report";
	if("petty_cash_release" == $shortTName) 
		return "petty cash release";
	if("petty_cash_replenishment" == $shortTName) 
		return "petty cash replenishment";
	if("Employee_Advance_Release" == $shortTName) 
		return "Employee Advance Release";
	if("Employee_Advance_Replenishment" == $shortTName) 
		return "Employee Advance Replenishment";
	if("Liquidation_Release" == $shortTName) 
		return "Liquidation Release";
	if("Liquidation_Replenishment" == $shortTName) 
		return "Liquidation Replenishment";
	if("Billable_Cost_Report" == $shortTName) 
		return "Billable Cost Report";
	if("centerrep_pettycash" == $shortTName) 
		return "centerrep_pettycash";
	if("centerrep_empadvance" == $shortTName) 
		return "centerrep_empadvance";
	if("centerrep_liquidation" == $shortTName) 
		return "centerrep_liquidation";
	if("replenishment_pettycash" == $shortTName) 
		return "replenishment_pettycash";
	if("replenishment_liquidation" == $shortTName) 
		return "replenishment_liquidation";
	if("replenishment_empadv" == $shortTName) 
		return "replenishment_empadv";
	if("bank_statement" == $shortTName) 
		return "bank_statement";
	if("other_charges" == $shortTName) 
		return "other_charges";
	if("change_password" == $shortTName) 
		return "change_password";
	if("project_liquidation_detail2" == $shortTName) 
		return "project_liquidation_detail2";
	if("balance_advance" == $shortTName) 
		return "balance_advance";
	if("project_replenishment_deposited" == $shortTName) 
		return "project_replenishment_deposited";
	if("bal_center_replenishment" == $shortTName) 
		return "bal_center_replenishment";
	if("Project_and_Pettycash_Report" == $shortTName) 
		return "Project and Pettycash Report";
	if("Project_and_EmpAdv_Report" == $shortTName) 
		return "Project and EmpAdv Report";
	if("Project_and_Liquidation_Report" == $shortTName) 
		return "Project and Liquidation Report";
	if("Project_and_Sales_Report" == $shortTName) 
		return "Project and Sales Report";
	if("Project_and_Voucher_Report" == $shortTName) 
		return "Project and Voucher Report";
	if("projects_view" == $shortTName) 
		return "projects_view";
	if("projects_lite" == $shortTName) 
		return "projects_lite";
	if("shipping_date" == $shortTName) 
		return "shipping_date";
	if("entry_detail_sales" == $shortTName) 
		return "entry_detail_sales";
	if("entry_detail_pettycash" == $shortTName) 
		return "entry_detail_pettycash";
	if("entry_detail_cash_disbursement" == $shortTName) 
		return "entry_detail_cash_disbursement";
	if("representative" == $shortTName) 
		return "representative";
	if("sales_sub_invoice" == $shortTName) 
		return "sales_sub_invoice";
	if("search_cash_disbursement" == $shortTName) 
		return "search_cash_disbursement";
	if("sales_business_partner" == $shortTName) 
		return "sales_business_partner";
	if("canvas" == $shortTName) 
		return "canvas";
	if("canvas_detail" == $shortTName) 
		return "canvas_detail";
	if("request_for_payment" == $shortTName) 
		return "request_for_payment";
	if("request_for_payment_detail" == $shortTName) 
		return "request_for_payment_detail";
	if("po_netpr" == $shortTName) 
		return "po_netpr";
	if("request_for_payment_imp" == $shortTName) 
		return "request_for_payment_imp";
	if("birsalesrelief" == $shortTName) 
		return "birsalesrelief";
}

//	A
	$g_defaultOptionValues["add"] = false;
	$g_defaultOptionValues["addMultistep"] = false;
	$g_defaultOptionValues["Absolute"] = false;
	$g_defaultOptionValues["acceptFileTypes"] = ".+$";
	$g_defaultOptionValues["addFields"] = array();
	$g_defaultOptionValues["addPageEvents"] = false;
	$g_defaultOptionValues["advSearchFields"] = array();
	$g_defaultOptionValues["afterAddAction"] = 1;
	$g_defaultOptionValues["afterAddActionDetTable"] = "";
	$g_defaultOptionValues["afterEditAction"] = 1;
	$g_defaultOptionValues["afterEditActionDetTable"] = "";
	$g_defaultOptionValues["AllowToAdd"] = false;
	$g_defaultOptionValues["allSearchFields"] = array();
	$g_defaultOptionValues["arrGroupsPerPage"] = array();
	$g_defaultOptionValues["arrKeyFields"] = array();
	$g_defaultOptionValues["arrAddTabs"] = array();
	$g_defaultOptionValues["arrEditTabs"] = array();
	$g_defaultOptionValues["arrRecsPerPage"] = array();
	$g_defaultOptionValues["arrViewTabs"] = array();
	$g_defaultOptionValues["audioTitleField"] = "";
	$g_defaultOptionValues["audit"] = false;
	$g_defaultOptionValues["autoCompleteFields"] = array();
	$g_defaultOptionValues["autoCompleteFieldsAdd"] = array();
	$g_defaultOptionValues["autoCompleteFieldsEdit"] = array();
	$g_defaultOptionValues["autoCompleteFieldsOnEdit"] = false;
	$g_defaultOptionValues["AutoInc"] = false;
	$g_defaultOptionValues["autoUpload"] = false;
	$g_defaultOptionValues["autoUpdatable"] = false;
//	B
	$g_defaultOptionValues["bAddPage"] = false;
	$g_defaultOptionValues["bAdvancedSearch"] = false;
	$g_defaultOptionValues["bEditPage"] = false;
	$g_defaultOptionValues["bExportPage"] = false;
	$g_defaultOptionValues["bInlineAdd"] = false;
	$g_defaultOptionValues["bInlineEdit"] = false;
	$g_defaultOptionValues["bIsEncrypted"] = false;
	$g_defaultOptionValues["bListPage"] = false;
	$g_defaultOptionValues["bPrinterPage"] = false;
	$g_defaultOptionValues["bViewPage"] = false;
	$g_defaultOptionValues["isCaptchaEnabledOnAdd"] = false;
	$g_defaultOptionValues["isCaptchaEnabledOnEdit"] = false;
//	C
	$g_defaultOptionValues["CompatibilityMode"] = false;
	$g_defaultOptionValues["categoryFields"] = array();
	$g_defaultOptionValues["ChartRefreshTime"] = 60;
	$g_defaultOptionValues["chartXml"] = "";
	$g_defaultOptionValues["chartType"] = "";
	$g_defaultOptionValues["closePopupAfterAdd"] = false;
	$g_defaultOptionValues["closePopupAfterEdit"] = false;
	$g_defaultOptionValues["controlWidth"] = 160;
	$g_defaultOptionValues["copy"] = false;
	$g_defaultOptionValues["CreateThumbnail"] = false;
	$g_defaultOptionValues["crossTabReport"] = false;
	$g_defaultOptionValues["CustomDisplay"] = false;
//	D
	$g_defaultOptionValues["detailsLinksOnList"] = DL_SINGLE;
	$g_defaultOptionValues["dashElements"] = array();
	$g_defaultOptionValues["dashCells"] = array();
	$g_defaultOptionValues["DateEditType"] = 0;
	$g_defaultOptionValues["DecimalDigits"] = "";
	$g_defaultOptionValues["defaultSearchOption"] = "";
	$g_defaultOptionValues["DefaultValue"] = "";
	$g_defaultOptionValues["delete"] = false;
	$g_defaultOptionValues["DeleteAssociatedFile"] = false;
	$g_defaultOptionValues["denyDuplicates"] = false;
	$g_defaultOptionValues["DependentLookups"] = array();
	$g_defaultOptionValues["dependentFilterName"] = "";
	$g_defaultOptionValues["dependentFilters"] = array();
	$g_defaultOptionValues["descendingOrder"] = false;
	$g_defaultOptionValues["DisplayField"] = "";
//	E
	$g_defaultOptionValues["edit"] = false;
	$g_defaultOptionValues["editMultistep"] = false;
	$g_defaultOptionValues["editFields"] = array();
	$g_defaultOptionValues["EditFormat"] = "";
	$g_defaultOptionValues["EditParams"] = "";
	$g_defaultOptionValues["exportTo"] = false;
//	F
	$g_defaultOptionValues["fieldIsVideoUrl"] = false;
	$g_defaultOptionValues["FieldType"] = "";
	$g_defaultOptionValues["FieldPermissions"] = false;
	$g_defaultOptionValues["fieldsForRegister"] = array();
	$g_defaultOptionValues["Filename"] = "";
	$g_defaultOptionValues["filterFields"] = array();
	$g_defaultOptionValues["filterFormat"] = FF_VALUE_LIST;
	$g_defaultOptionValues["filterIntervals"] = array();
	$g_defaultOptionValues["filterTotals"] = FT_NONE;
	$g_defaultOptionValues["filterMultiSelect"] = FM_NONE;
	$g_defaultOptionValues["filterCheckedMessageText"] = "";
	$g_defaultOptionValues["filterCheckedMessageType"] = "";
	$g_defaultOptionValues["filterUncheckedMessageText"] = "";
	$g_defaultOptionValues["filterUncheckedMessageType"] = "";
	$g_defaultOptionValues["filterSliderStepType"] = 1;
	$g_defaultOptionValues["filterSliderStepValue"] = 1;
	$g_defaultOptionValues["filterKnobsType"] = 0;
	$g_defaultOptionValues["filterApplyBtn"] = false;
	$g_defaultOptionValues["flexibleSearch"] = false;
	$g_defaultOptionValues["FormatTimeAttrs"] = array();
	$g_defaultOptionValues["freeInput"] = false;
	$g_defaultOptionValues["FullName"] = "";
//	G
	$g_defaultOptionValues["googleLikeFields"] = array();
	$g_defaultOptionValues["GoodName"] = "";
//	H
	$g_defaultOptionValues["hasDependentFilter"] = false;	
	$g_defaultOptionValues["hasEncryptedFields"] = false;
	$g_defaultOptionValues["hideEmptyFieldsOnView"] = false;
	$g_defaultOptionValues["HorizontalLookup"] = false;
	$g_defaultOptionValues["hlNewWindow"] = false;
	$g_defaultOptionValues["hlType"] = 0;
	$g_defaultOptionValues["hlLinkWordNameType"] = "";
	$g_defaultOptionValues["hlLinkWordText"] = "";
	$g_defaultOptionValues["hlTitleField"] = "";
	$g_defaultOptionValues["highlightSearchResults"] = false;
	$g_defaultOptionValues["HTML5InuptType"] = "text";
//	I
	$g_defaultOptionValues["Index"] = 0;
	$g_defaultOptionValues["InitialYearFactor"] = 100;
	$g_defaultOptionValues["ImageHeight"] = 0;
	$g_defaultOptionValues["ImageWidth"] = 0;
	$g_defaultOptionValues["import"] = false;
	$g_defaultOptionValues["inlineAdd"] = false;
	$g_defaultOptionValues["inlineAddFields"] = array();
	$g_defaultOptionValues["inlineEdit"] = false;
	$g_defaultOptionValues["inlineEditFields"] = array();
	$g_defaultOptionValues["isDisplayLoading"] = false;
	$g_defaultOptionValues["tableType"] = "";
	$g_defaultOptionValues["IsRequired"] = false;
	$g_defaultOptionValues["isResizeColumns"] = false;
	$g_defaultOptionValues["isSeparate"] = false;
	$g_defaultOptionValues["UpdateLatLng"] = false;
	$g_defaultOptionValues["isUseAjaxSuggest"] = false;
	$g_defaultOptionValues["isUseAudio"] = false;
	$g_defaultOptionValues["isUsebuttonHandlers"] = false;
	$g_defaultOptionValues["isUseFieldsMaps"] = false;
	$g_defaultOptionValues["isUseMainMaps"] = false;
	$g_defaultOptionValues["isUseTimeForSearch"] = false;
	$g_defaultOptionValues["isUseToolTips"] = false;
	$g_defaultOptionValues["isUseVideo"] = false;
	$g_defaultOptionValues["listGridLayout"] = 0;
	$g_defaultOptionValues["isExistTotalFields"] = false;
	$g_defaultOptionValues["isTotalMin"] = false;
	$g_defaultOptionValues["isTotalAvg"] = false;
	$g_defaultOptionValues["isTotalMax"] = false;
	$g_defaultOptionValues["isTotalSum"] = false;
	$g_defaultOptionValues["isViewPagePDFFitToPage"] = 1;
	$g_defaultOptionValues["isPrinterPagePDFFitToPage"] = 1;
	$g_defaultOptionValues["isLandscapeViewPDFOrientation"] = 0;
	$g_defaultOptionValues["isLandscapePrinterPagePDFOrientation"] = 0;
	$g_defaultOptionValues["isPrinterPagePDF"] = false;
	$g_defaultOptionValues["isViewPagePDF"] = false;
	$g_defaultOptionValues["isSQLExpression"] = false;
	$g_defaultOptionValues["isPrinterPageFitToPage"] = 1;
//	J
//	K
	$g_defaultOptionValues["Keys"] = array();
//	L
	$g_defaultOptionValues["Label"] = "";
	$g_defaultOptionValues["LookupConnId"] = "";
	$g_defaultOptionValues["LastYearFactor"] = 10;
	$g_defaultOptionValues["LCType"] = LCT_DROPDOWN;
	$g_defaultOptionValues["LinkField"] = "";
	$g_defaultOptionValues["LinkFieldType"] = 0;
	$g_defaultOptionValues["LinkPrefix"] = "";
	$g_defaultOptionValues["list"] = false;
	$g_defaultOptionValues["listAjax"] = false;
	$g_defaultOptionValues["listFields"] = array();
	$g_defaultOptionValues["locking"] = false;
	$g_defaultOptionValues["LookupDesc"] = false;
	$g_defaultOptionValues["LookupOrderBy"] = "";
	$g_defaultOptionValues["LookupTable"] = "";
	$g_defaultOptionValues["LookupType"] = 0;
	$g_defaultOptionValues["LookupUnique"] = false;
	$g_defaultOptionValues["LookupValues"] = array();
	$g_defaultOptionValues["LookupWhere"] = "";
//	M
	$g_defaultOptionValues["mainTableOwnerID"] = "";
	$g_defaultOptionValues["mapData"] = array();
	$g_defaultOptionValues["masterListFields"] = array();
	$g_defaultOptionValues["maxFileSize"] = null;
	$g_defaultOptionValues["maxImageHeight"] = null;
	$g_defaultOptionValues["maxImageWidth"] = null;
	$g_defaultOptionValues["maxNumberOfFiles"] = 1;
	$g_defaultOptionValues["maxTotalFilesSize"] = null;
	$g_defaultOptionValues["moveNext"] = "";
	$g_defaultOptionValues["Multiselect"] = false;
//	N
	$g_defaultOptionValues["NCSearch"] = false;
	$g_defaultOptionValues["NeedEncode"] = false;
	$g_defaultOptionValues["NewSize"] = 0;
	$g_defaultOptionValues["noRecordsFirstPage"] = false;
	$g_defaultOptionValues["nSecOptions"] = ADVSECURITY_NONE;
	$g_defaultOptionValues["NumberOfChars"] = 0;
	$g_defaultOptionValues["numberOfVisibleItems"] = 10;
	$g_defaultOptionValues["nViewPagePDFScale"] = 100;
	$g_defaultOptionValues["nPrinterPagePDFScale"] = 100;
	$g_defaultOptionValues["nPrinterPageScale"] = 100;
	$g_defaultOptionValues["nPrinterPDFSplitRecords"] = 0;
	$g_defaultOptionValues["nPrinterSplitRecords"] = 0;
	

//	O
	$g_defaultOptionValues["OriginalTable"] = "";
	$g_defaultOptionValues["orderindexes"] = array();
	$g_defaultOptionValues["OwnerID"] = 0;
	$g_defaultOptionValues["ownerTable"] = "";
	$g_defaultOptionValues["OraSequenceName"] = "";
//	P
	$g_defaultOptionValues["pageSize"] = 0;
	$g_defaultOptionValues["panelSearchFields"] = array();
	$g_defaultOptionValues["parentFilterField"] = "";
	$g_defaultOptionValues["parentFilters"] = array();
	$g_defaultOptionValues["printFriendly"] = false;
	$g_defaultOptionValues["printFields"] = array();
	$g_defaultOptionValues["popupPagesLayoutNames"] = array();
	$g_defaultOptionValues["printGridLayout"] = 0;
	$g_defaultOptionValues["printReportLayout"] = 0;
	$g_defaultOptionValues["printerPageOrientation"] = 0;
//	Q
//	R
	$g_defaultOptionValues["recsPerRowList"] = 1;
	$g_defaultOptionValues["recsPerRowPrint"] = 1;
	$g_defaultOptionValues["reportGroupFields"] = false;
	$g_defaultOptionValues["reportGroupFieldsData"] = array();
	$g_defaultOptionValues["reportHorizontalSummary"] = false;
	$g_defaultOptionValues["reportVerticalSummary"] = false;
	$g_defaultOptionValues["reportPageSummary"] = false;
	$g_defaultOptionValues["reportGlobalSummary"] = false;
	$g_defaultOptionValues["repShowDet"] = false;
	$g_defaultOptionValues["ResizeImage"] = false;
	$g_defaultOptionValues["RewindEnabled"] = false;
	$g_defaultOptionValues["rowHighlite"] = false;
	$g_defaultOptionValues["requiredSearchFields"] = array();
	$g_defaultOptionValues["reportPrintPartitionType"] = 0;
	$g_defaultOptionValues["reportPrintGroupsPerPage"] = 3;
	$g_defaultOptionValues["reportPrintPDFGroupsPerPage"] = 0;
	
//	S
	$g_defaultOptionValues["searchFields"] = false;
	$g_defaultOptionValues["searchSaving"] = false;
	$g_defaultOptionValues["searchPanelOptions"] = array();
	$g_defaultOptionValues["SelectSize"] = 1;
	$g_defaultOptionValues["searchIsRequiredForFilters"] = false;
	$g_defaultOptionValues["searchOptionsList"] = array();
	$g_defaultOptionValues["ShortName"] = "";
	$g_defaultOptionValues["shortTableName"] = "";
	$g_defaultOptionValues["showAddInPopup"] = false;
	$g_defaultOptionValues["ShowCustomExpr"] = false;
	$g_defaultOptionValues["showEditInPopup"] = false;
	$g_defaultOptionValues["ShowFileSize"] = false;
	$g_defaultOptionValues["ShowIcon"] = false;
	$g_defaultOptionValues["showSearchPanel"] = false;
	$g_defaultOptionValues["showSimpleSearchOptions"] = false;
	$g_defaultOptionValues["ShowThumbnail"] = false;
	$g_defaultOptionValues["ShowTime"] = false;
	$g_defaultOptionValues["showViewInPopup"] = false;
	$g_defaultOptionValues["showCollapsed"] = false;
	$g_defaultOptionValues["showWithNoRecords"] = false;
	$g_defaultOptionValues["SimpleAdd"] = false;
	$g_defaultOptionValues["ShowListOfThumbnails"] = false;
	$g_defaultOptionValues["sortValueType"] = SORT_BY_DISP_VALUE;
	$g_defaultOptionValues["sqlFrom"] = "";
	$g_defaultOptionValues["sqlHead"] = "";
	$g_defaultOptionValues["sqlTail"] = "";
	$g_defaultOptionValues["sqlWhereExpr"] = "";
	$g_defaultOptionValues["sqlquery"] = null;
	$g_defaultOptionValues["strField"] = "";
	$g_defaultOptionValues["strFilename"] = "";
	$g_defaultOptionValues["strName"] = "";
	$g_defaultOptionValues["strOrderBy"] = "";
	$g_defaultOptionValues["StrThumbnail"] = "";
	$g_defaultOptionValues["ThumbnailSize"] = 0;
	$g_defaultOptionValues["scrollGridBody"] = false;
	$g_defaultOptionValues["showGroupSummaryCount"] = false;
//	T
	$g_defaultOptionValues["totalsFields"] = array();
	$g_defaultOptionValues["ThumbHeight"] = 72;
	$g_defaultOptionValues["ThumbWidth"] = 72;	
	$g_defaultOptionValues["truncateText"] = false;	
//	U
	$g_defaultOptionValues["UploadFolder"] = "";
	$g_defaultOptionValues["UploadCodeExpression"] = false;
	$g_defaultOptionValues["UseCategory"] = false;
	$g_defaultOptionValues["UseRTE"] = false;
	$g_defaultOptionValues["UseTimestamp"] = false;
//	V
	$g_defaultOptionValues["validateAs"] = array();
	$g_defaultOptionValues["videoHeight"] = 200;
	$g_defaultOptionValues["videoWidth"] = 300;
	$g_defaultOptionValues["videoTitleField"] = "";
	$g_defaultOptionValues["view"] = false;
	$g_defaultOptionValues["ViewFormat"] = "";
	$g_defaultOptionValues["viewFields"] = array();
	$g_defaultOptionValues["viewMultistep"] = false;
//	W
	$g_defaultOptionValues["warnLeavingPages"] = false;
//	X
//	Y
//	Z

	
//	A
	$g_settingsType["Absolute"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["acceptFileTypes"] = SETTING_TYPE_EDIT;
	$g_settingsType["AllowToAdd"] = SETTING_TYPE_EDIT;
	$g_settingsType["autoCompleteFields"] = SETTING_TYPE_EDIT;
	$g_settingsType["autoCompleteFieldsOnEdit"] = SETTING_TYPE_EDIT;
	$g_settingsType["AutoInc"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["audioTitleField"] = SETTING_TYPE_VIEW;
	$g_settingsType["autoUpload"] = SETTING_TYPE_EDIT;
	$g_settingsType["autoUpdatable"] = SETTING_TYPE_EDIT;
//	B
	$g_settingsType["bAddPage"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bAdvancedSearch"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bEditPage"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bExportPage"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bInlineAdd"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bInlineEdit"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bIsEncrypted"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bListPage"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bPrinterPage"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bViewPage"] = SETTING_TYPE_GLOBAL;
//	C
	$g_settingsType["CompatibilityMode"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["categoryFields"] = SETTING_TYPE_EDIT;
	$g_settingsType["controlWidth"] = SETTING_TYPE_EDIT;
	$g_settingsType["CreateThumbnail"] = SETTING_TYPE_EDIT;
	$g_settingsType["CustomDisplay"] = SETTING_TYPE_EDIT;
//	D
	$g_settingsType["DateEditType"] = SETTING_TYPE_EDIT;
	$g_settingsType["DecimalDigits"] = SETTING_TYPE_VIEW;
	$g_settingsType["DeleteAssociatedFile"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["denyDuplicates"] = SETTING_TYPE_EDIT;
	$g_settingsType["DependentLookups"] = SETTING_TYPE_EDIT;
	$g_settingsType["DisplayField"] = SETTING_TYPE_EDIT;
//	E
	$g_settingsType["EditFormat"] = SETTING_TYPE_EDIT;
	$g_settingsType["EditParams"] = SETTING_TYPE_EDIT;
//	F
	$g_settingsType["fieldIsVideoUrl"] = SETTING_TYPE_VIEW;
	$g_settingsType["FieldType"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["FieldPermissions"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["Filename"] = SETTING_TYPE_VIEW;
	$g_settingsType["FormatTimeAttrs"] = SETTING_TYPE_EDIT;
	$g_settingsType["freeInput"] = SETTING_TYPE_EDIT;
	$g_settingsType["FullName"] = SETTING_TYPE_GLOBAL;
//	G
	$g_settingsType["GoodName"] = SETTING_TYPE_GLOBAL;
//	H
	$g_settingsType["HorizontalLookup"] = SETTING_TYPE_EDIT;
	$g_settingsType["hlNewWindow"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlType"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlLinkWordNameType"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlLinkWordText"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlTitleField"] = SETTING_TYPE_VIEW;
	$g_settingsType["HTML5InuptType"] = SETTING_TYPE_EDIT;
//	I
	$g_settingsType["Index"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["InitialYearFactor"] = SETTING_TYPE_EDIT;
	$g_settingsType["ImageHeight"] = SETTING_TYPE_VIEW;
	$g_settingsType["ImageWidth"] = SETTING_TYPE_VIEW;
	$g_settingsType["IsRequired"] = SETTING_TYPE_EDIT;
	$g_settingsType["isSeparate"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["UpdateLatLng"] = SETTING_TYPE_EDIT;
//	J
//	K
//	L
	$g_settingsType["Label"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["LastYearFactor"] = SETTING_TYPE_EDIT;
	$g_settingsType["LCType"] = SETTING_TYPE_EDIT;
	$g_settingsType["LinkField"] = SETTING_TYPE_EDIT;
	$g_settingsType["LinkFieldType"] = SETTING_TYPE_EDIT;
	$g_settingsType["LinkPrefix"] = SETTING_TYPE_VIEW;
	$g_settingsType["LookupConnId"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupDesc"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupOrderBy"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupTable"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupType"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupUnique"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupValues"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupWhere"] = SETTING_TYPE_EDIT;
	$g_settingsType["lowGroup"] = 1;
//	M
	$g_settingsType["mapData"] = SETTING_TYPE_VIEW;
	$g_settingsType["maxFileSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxImageHeight"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxImageWidth"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxNumberOfFiles"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxTotalFilesSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["Multiselect"] = SETTING_TYPE_EDIT;
//	N
	$g_settingsType["nCols"] = SETTING_TYPE_EDIT;
	$g_settingsType["NeedEncode"] = SETTING_TYPE_VIEW;
	$g_settingsType["NewSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["nRows"] = SETTING_TYPE_EDIT;
	$g_settingsType["nSecOptions"] = ADVSECURITY_NONE;
//	O
	$g_settingsType["ownerTable"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["OraSequenceName"] = SETTING_TYPE_GLOBAL;
//	P
//	Q
//	R
	$g_settingsType["ResizeImage"] = SETTING_TYPE_EDIT;
	$g_settingsType["RewindEnabled"] = SETTING_TYPE_VIEW;
//	S
	$g_settingsType["SelectSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["ShowCustomExpr"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowFileSize"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowIcon"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowThumbnail"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowTime"] = SETTING_TYPE_EDIT;
	$g_settingsType["SimpleAdd"] = SETTING_TYPE_EDIT;
	$g_settingsType["ShowListOfThumbnails"] = SETTING_TYPE_VIEW;
	$g_settingsType["strField"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["strFilename"] = SETTING_TYPE_VIEW;
	$g_settingsType["filterTotalFields"] = '';
	$g_settingsType["strEditMask"] = SETTING_TYPE_EDIT;
	$g_settingsType["strName"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["StrThumbnail"] = SETTING_TYPE_EDIT;	
//	T
	$g_settingsType["ThumbnailSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["ThumbHeight"] = SETTING_TYPE_VIEW;
	$g_settingsType["ThumbWidth"] = SETTING_TYPE_VIEW;
//	U
	$g_settingsType["UploadFolder"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["UploadCodeExpression"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["UseCategory"] = SETTING_TYPE_EDIT;
	$g_settingsType["UseRTE"] = SETTING_TYPE_EDIT;
	$g_settingsType["UseTimestamp"] = SETTING_TYPE_EDIT;
//	V
	$g_settingsType["validateAs"] = SETTING_TYPE_EDIT;
	$g_settingsType["videoHeight"] = SETTING_TYPE_VIEW;
	$g_settingsType["videoWidth"] = SETTING_TYPE_VIEW;
	$g_settingsType["videoTitleField"] = SETTING_TYPE_VIEW;
	$g_settingsType["ViewFormat"] = SETTING_TYPE_VIEW;
//	W
//	X
//	Y
//	Z

?>