<?php 
/**
 * That function  copies all elements from associative array to object, as object properties with same names
 * Usefull when you need to copy many properties
 *
 * @param link $obj
 * @param link $argsArr
 * @intellisense
 */
function RunnerApply (&$obj, &$argsArr)
{	
	foreach ($argsArr as $key=>$var)
		setObjectProperty($obj,$key,$argsArr[$key]);
}

/**
 * @intellisense
 */
function GetImageFromDB($gQuery, $forPDF = false, $params = array())
{
	global $cman;
	
	if(!$forPDF)
	{
		$table = postvalue("table");
		$strTableName = GetTableByShort($table);
		$settings = new ProjectSettings($strTableName);
		
		if (!checkTableName($table))
		{
			return '';
		}
		
		@ini_set("display_errors","1");
		@ini_set("display_startup_errors","1");
	
		if(!isLogged() || !CheckSecurity(@$_SESSION["_".$strTableName."_OwnerID"],"Search"))
		{ 
			HeaderRedirect("login"); 
			return;
		}
	
		$field = postvalue("field");
		if(!$settings->checkFieldPermissions($field))
			return DisplayNoImage();
	
		//	construct sql
		$keysArr = $settings->getTableKeys();
		$keys = array();
		foreach ($keysArr as $ind=>$k)
		{
			$keys[$k]=postvalue("key".($ind+1));
		}
	}
	else
	{
		$table = @$params["table"];
	
		$strTableName = GetTableByShort($table);
		
		if (!checkTableName($table))
		{
			exit(0);
		}
		
		$settings = new ProjectSettings($strTableName);
		$field = @$params["field"];
		//	construct sql
		$keysArr = $settings->getTableKeys();
		$keys = array();
		foreach ($keysArr as $ind=>$k)
		{
			$keys[$k]=@$params["key".($ind+1)];
		}
	}
	
	$connection = $cman->byTable( $strTableName );	
	
	if(!$gQuery->HasGroupBy())
	{
		// Do not select any fields except current (image) field.
		// If query has 'group by' clause then other fields are used in it and we may not simply cut 'em off.
		// Just don't do anything in that case.
		$gQuery->RemoveAllFieldsExcept($settings->getFieldIndex($field));
	}
	
	$where = KeyWhere($keys);
	
	$secOpt = $settings->getAdvancedSecurityType();
	if ($secOpt == ADVSECURITY_VIEW_OWN)
	{
		$where = whereAdd($where, SecuritySQL("Search"));
	}
	
	$sql = $gQuery->gSQLWhere($where);
	$data = $connection->query( $sql )->fetchAssoc();
	
	if($forPDF)
	{
		if( $data )
			return $data[ $field ];
	}
	else
	{
		if( !$data )
			return DisplayNoImage();
			
		if(postvalue('src') == 1)
		{
			$value = myfile_get_contents('images/icons/jpg.png');
		}
		else
			$value = $connection->stripSlashesBinary( $data[ $field ] );
			
		if(!$value)
		{
			if(postvalue("alt"))
			{
				$value = $connection->stripSlashesBinary( $data[ postvalue("alt") ] );
				if(!$value)
					return DisplayNoImage();
			}
			else
				return DisplayNoImage();
		}
		
		$itype = SupposeImageType($value);
		
		if(!$itype)
			return DisplayFile();
		
		if(!isset($pdf))
		{
			header("Content-Type: ".$itype);
			header("Cache-Control: private");
			SendContentLength(strlen_bin($value));
		}
		echoBinary($value);
		return '';
	}
}

/**
 *	Redirects to login.
 *  Displays Session expired message if the page was opened from within the project directory.
 *
 */
function redirectToLogin()
{
	
	$expired = "";
	$url = "http://";
	if( $_SERVER["HTTPS"] && $_SERVER["HTTPS"] != "off")
		$url = "https://";
	$url .= $_SERVER["HTTP_HOST"] . $_SERVER['REQUEST_URI'];

	global $logoutPerformed;
	if( !$logoutPerformed && isset($_SERVER['HTTP_REFERER']) )
	{
		if( getDirectoryFromURI( $_SERVER['HTTP_REFERER'] ) == getDirectoryFromURI( $url ) 
			&& getFilenameFromURI( $_SERVER['HTTP_REFERER'] ) != "index.htm" 
			&& $_SERVER['HTTP_REFERER'] != getDirectoryFromURI( $url ))
			$expired = "message=expired";
	}
	if( !$logoutPerformed )
		$expired = "return=true&" . $expired;
	HeaderRedirect("login", "", $expired);
	exit();
}

/**
 *	Returns server and directory path from the file-based URI
 *  Examples:
 *  http://server.com:88/Calendar/login.php?a=expired => http://server.com:88/Calendar/
 *
 *  Beware! These URLs are incorrect input:
 *  https://server.com/app1
 *  https://server.com 
 *
 */

function getDirectoryFromURI( $uri )
{
	$qMark = strpos( $uri, "?" );
	if( $qMark !== FALSE )
		$uri = substr( $uri, 0, $qMark );
	$lastSlash = strrpos( $uri, "/" );
	if( $lastSlash !== FALSE )
		return substr($uri, 0, $lastSlash )."/";
	return $uri;
}

function getFilenameFromURI( $uri )
{
	$qMark = strpos( $uri, "?" );
	if( $qMark !== FALSE )
		$uri = substr( $uri, $qMark );
	$lastSlash = strrpos( $uri, "/" );
	if( $lastSlash !== FALSE )
		return substr($uri, $lastSlash + 1 );
	return $uri;
}


/**
 * @intellisense
 */
function getLangFileName($langName)
{
	$langArr = array();
	$langArr["English"] = "English";
	return $langArr[$langName];
}

/**
 * @intellisense
 */
function GetGlobalData($name, $defValue)
{
	global $globalSettings;
	if(!array_key_exists($name, $globalSettings))
		return $defValue;
	return $globalSettings[$name];
}

/**
 * @intellisense
 */
function DisplayMap($params) 
{
	global $pageObject;
	
	$pageObject->googleMapCfg['mapsData'][$params['id']]['addressField'] = $params['addressField'] ? $params['addressField'] : "";
	$pageObject->googleMapCfg['mapsData'][$params['id']]['latField'] = $params['latField'] ? $params['latField'] : '';
	$pageObject->googleMapCfg['mapsData'][$params['id']]['lngField'] = $params['lngField'] ? $params['lngField'] : '';
	$pageObject->googleMapCfg['mapsData'][$params['id']]['weightField'] = $params['weightField'] ? $params['weightField'] : '';
	$pageObject->googleMapCfg['mapsData'][$params['id']]['clustering'] = $params['clustering'];
	$pageObject->googleMapCfg['mapsData'][$params['id']]['heatMap'] = $params['heatMap'];
	$pageObject->googleMapCfg['mapsData'][$params['id']]['showAllMarkers'] = $params['showAllMarkers'] || $params['clustering'] || $params['heatMap'];
	$pageObject->googleMapCfg['mapsData'][$params['id']]['width'] = $params['width'] ? $params['width'] : 0;
	$pageObject->googleMapCfg['mapsData'][$params['id']]['height'] = $params['height'] ? $params['height'] : 0;
	$pageObject->googleMapCfg['mapsData'][$params['id']]['type'] = 'BIG_MAP';
	$pageObject->googleMapCfg['mapsData'][$params['id']]['showCenterLink'] = $params['showCenterLink'] ? $params['showCenterLink'] : 0;
	$pageObject->googleMapCfg['mapsData'][$params['id']]['descField'] = $params['descField'] ? $params['descField'] : $pageObject->googleMapCfg['mapsData'][$params['id']]['addressField'];
	$pageObject->googleMapCfg['mapsData'][$params['id']]['descField'] = $params['description'] ? $params['description'] : $pageObject->googleMapCfg['mapsData'][$params['id']]['addressField'];
	$pageObject->googleMapCfg['mapsData'][$params['id']]['markerAsEditLink'] = $params['markerAsEditLink'];
	$pageObject->googleMapCfg['mapsData'][$params['id']]['markerIcon'] = $params['markerIcon'] ? $params['markerIcon'] : '';
	$pageObject->googleMapCfg['mapsData'][$params['id']]['markerField'] = $params['markerField'] ? $params['markerField'] : '';
	
	if (isset($params['zoom']))
		$pageObject->googleMapCfg['mapsData'][$params['id']]['zoom'] = $params['zoom'];
	
	//$pageObject->googleMapCfg['bigMapDefZoom'] = $pageObject->googleMapCfg['mapsData'][$params['id']]['zoom'];
	
	if ($pageObject->googleMapCfg['mapsData'][$params['id']]['showCenterLink'])
		$pageObject->googleMapCfg['mapsData'][$params['id']]['centerLinkText'] = $params['centerLinkText'] ? $params['centerLinkText'] : '';
	
	$pageObject->googleMapCfg['mainMapIds'][] = $params['id'];
	
	if (isset($params['APIkey']))
		$pageObject->googleMapCfg['APIcode'] = $params['APIkey'];
}

/**
 * @intellisense
 */
function checkTableName($shortTName, $type=false)
{
	if (!$shortTName)
		return false;
	
	if ("business_partner" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("item" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("credit_terms" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("shipping_terms" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("cash_receipt" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("purchase" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("cash_disbursment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("general_journal" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("report_center" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Sales_Report" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Disbursement_Reports" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Financial_Reports" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchase_Reports" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Receivable_Reports" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Taxation_Reports" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("system_user" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("fiscal_year" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("beg_bal" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("ewt" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bank" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("cost_center" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("General_Journal_Reports" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Inventory_Reports" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("accounting_period" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bank_recon" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("deposit" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("account_classification" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("chart_of_accounts" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("recon_CR" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Recon_deposit" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Recon_CV" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Recon_GV_dep" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Recon_GV_CV" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("vat_atc" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("rdo" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("2550q" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("2550m" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("dr" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("purchase_order" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("purchase_order_receiving" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("System_preference" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("ppe_list" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("discounts" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("item_group" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("location" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("meetings" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("calls" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("campaign" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("reimbursement" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("reimbursement_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Target_customers" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("quotation" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("case" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("projects" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("production" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("issue" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("issue_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("disposed" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("disposed_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("backload" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("backload_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("vehicle" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("payroll_item" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("uom" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("trip_ticket" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("accident" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("approver_list" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("area" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("attendance" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("currency" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("medical" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("position" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("service_center" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("seminar" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("service_report" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("service_report_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("tax_status" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("training" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("competitor" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("campaign_type" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("fixed_asset_return_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("fixed_assets_return" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Prospect_customers" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("trip_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("beg_baldetail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("cash_disbursement_detail2" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("case_solutions" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("cash_disbursment_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("cash_receipt_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("cash_receipt_detail2" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("general_journal_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("employee_list" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("petty_cash" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("petty_cash_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("center_replenishment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("center_replenishment_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("credit_memo" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("credit_memo_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("debit_memo" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("debit_memo_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("project_replenishment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("project_replenishment_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("business_partner_group" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Sales_Representatives" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Types_of_leaves" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("payroll_mode" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("payroll_account_type" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Types_of_Overtime" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Types_of_Loan" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("sales" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sales_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Payroll_Company_Setup" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("item_bill_of_materials" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("item_uom_conversion" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("tranasactions_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("deposit_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("purchase_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("dr_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("purchase_order_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Sales_Order_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("sales_order_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Sales_Order_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("sales_order_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Quotation_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Quotation_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("quotation_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("quotation_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Reimbursement_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("reimbursement_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Reimbursement_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("reimbursement_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Project_Replenishment_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("project_replenishment_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Project_Replenishment_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("project_replenishment_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("center_replenishment_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("center_replenishment_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchases_Order_for_checking2" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("purchase_order_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchases_Order_for_approval2" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("purchase_order_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Receiving_for_checking2" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("purchase_order_receiving_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Receiving_for_approval2" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("purchase_order_receiving_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Delivery_Receipt_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("dr_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Delivery_Receipt_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("dr_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Production_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("production_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Production_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("production_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Issue_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("supply_issue_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Issue_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("supply_issue_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Return_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("supply_return_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Return_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("supply_return_detail_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("supply_disposed_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Disposal_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("supply_disposed_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Fixed_Asset_Issue_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Fixed_Asset_Issue_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("fixed_asset_issue_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("fixed_asset_issue_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Fixed_Asset_Return_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("fixed_asset_return_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Fixed_Asset_Return_for_approval1" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("fixed_asset_return_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Petty_Cash_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("petty_cash_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Petty_Cash_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("petty_cash_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Deposit_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("deposit_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Deposit_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("deposit_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Credit_Memo_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("credit_memo_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Credit_Memo_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("credit_memo_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Debit_Memo_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("debit_memo_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("debit_memo_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("debit_memo_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchases_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("credit_purchase_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchases_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("credit_purchase_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Sales_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Sales_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("credit_sales_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("credit_sales_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Cash_Disbursement_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("check_voucher_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("check_voucher_detail2_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Cash_Disbursement_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("check_voucher_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("check_voucher_detail2_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("document" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("General_Journal_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("general_journal_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("General_Journal_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("general_journal_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("for_submissions" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("submission_details" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sales_order_detail_billing_dates" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Submission_Search" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Submission_Reminders" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("years" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("document_type" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("estimate" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("estimate_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("purchase_request" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("purchase_request_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("inv_adj_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("inventory_adj" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("inventory_tranfer" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("inventory_transfer_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Customers" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Suppliers" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Target_contacts" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Prospect_contacts" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Customer_Contacts" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Supplier_Contacts" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Statement_of_Account" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Change_Time_Schedule_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Change_Time_Schedule_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Inventory_Adjustment_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("inv_adj_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Inventory_Adjustment_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("inv_adj_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Estimate_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Estimate_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("estimate_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("estimate_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Follow_up" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("coming_meetings" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("tasks" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Project_phases" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("origin_fa" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("origin_serialized_supply" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("2301" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Prospect_customers_to_Customer" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_detail_accessories" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_detail_aluminum" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_detail_e_accessories" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_detail_glass" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("recuring_benefits" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("recuring_deductions" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("recurring_benefits_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("recurring_deductions_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("production_request" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("production_request_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("attendance_summary" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Other_Incentives" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Other_Deductions" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("department" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_cash_disbursement" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_cash_receipt" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_center_replenishment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_credit_memo" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_debit_memo" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_depreciation" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_general_journal" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_inventory_adjustment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_petty_cash" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_project_replenishment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_purchases" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_replenishment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_service_revenue" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Credit_Card_monitoring" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("sales_report_per_item_Chart" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("Leave_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("loan_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("loan_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Overtime_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchase_Request_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("purchase_request_detail_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Production_Request_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("production_request_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Inventory_Transfer_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("inventory_transfer_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Leave_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Loan_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("loan_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Overtime_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchase_Request_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Production_Request_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Inventory_Transfer_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("inventory_transfer_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("attendance_type" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Backload_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("backload_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Backload_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("backload_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Cash_Receipt_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Cash_Receipt_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("cash_receipt_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("cash_receipt_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Payroll_Entry_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Payroll_Entry_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("payroll_entry_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("payroll_entry_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Process_Payroll_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Process_Payroll_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("process_payroll_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("process_payroll_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Service_Report_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Service_Report_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("service_report_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("service_report_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Banch_Reports" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("journal_entry_production" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_deposit" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("production_request_bom" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("production_bom" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("company_info" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("return" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("return_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("fixed_asset_issue" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("fixed_asset_issue_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("production_request_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("cash_disbursment_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("cash_disbursment_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("purchase_request_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("trip_ticket_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("trip_ticket_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("trip_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("trip_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("sales_report1" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Customer_Balance_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Customer_Balance_Report_detail" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Collection_Report_per_Customer" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Aging_of_AR_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Aging_of_AR_Report_detail" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Payment_per_Supplier_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Aging_of_AP_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Aging_of_AP_Report_detail" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Inventory_Balance_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Inventory_Valuation_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Inventory_Below_Reorder_Point_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Report_per_BP" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Report_per_Item" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Report_per_Cost_Center" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Report_per_BP_detail" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Report_per_Item_detail" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Report_per_Cost_Center_detail" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Report_per_BP" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Report_per_Item" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Report_per_Cost_Center" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Report_per_BP_detail" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Report_per_Item_detail" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Report_per_Cost_Center_detail" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Transaction_Registry" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Report_per_sales_rep" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("PDC_Registry_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Available_Vehicles1" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Trip_Ticket_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Request_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Import_Costing" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Production_Costing_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Production_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Closed_Opportunities" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Cases_with_Sotutions" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Totals_Cases_Created" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Totals_Open_Cases_by_Agent" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Campaign_Contacts" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Campaign_leads" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Campaign_ROI_Analysis" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Campaign_Revenue" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Opportunity_Schedule" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_by_Account" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_by_Lead_Source" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_by_Partner" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_by_Rep" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Leave_Ledger_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Leave_Summary" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Payroll_Register" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Employee_Leave_History" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Employee_Register" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("DR_Reports" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Order_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Case_History" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("quality_control" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("quality_control_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("budget" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("budget_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("repairs_and_maintenance" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("production_form" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bluewave_employees" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bluewave_employees_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bluewave_employees_detail2" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bluewave_employees_detail3" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bluewave_employees_detail4" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bluewave_employees_detail5" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bluewave_employees_detail6" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("rfp" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("telex" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("repairs_and_maintenance_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("repair_list" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("marketing_colateral" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("email" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("accessories" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sales_order_materials_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sales_order_progress_billing_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sub_contract" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sub_contract_billing_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sub_contract_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Monthly_Pag_ibig_Summary_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Monthly_Philhealth_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Monthly_Philhealth_Summary_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Monthly_SSS_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Monthly_SSS_Report_by_Department_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Monthly_Withholding_Tax_Prooflist_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Monthly_Witholding_Tax_Collection_List_by_Department_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Overtime_Summary_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Monthly_Pag_ibig_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("sales_order" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sales_order_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("vat_relief_importation" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("vat_relief_purchases" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("vat_relief_sales" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_ppe_disposal" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("credit_memo_detail_cgs" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("marketing_request_form" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Quotation_Dashboard_Customer" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Order_Dashboard_Customer" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Delivery_Receipt_Dashboard_Customer" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Dashboard_Customer" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Backload_Dashboard_Customer" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Request_Dashboard_Supplier" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Order_Dashboard_Supplier" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Dashboard_Supplier" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Cash_Disbursement_Dashboard_Supplier" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Debit_Memo_Dashboard_Supplier" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Request_Dashboard_Item1" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Order_Dashboard_Item" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_Order_Receiving_Dashboard_Item" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Production_Dashboard_Item" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("sub_contract_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("sub_contract_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("sub_contract_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("sub_contract_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("budget_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("budget_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("budget_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("budget_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("undertime_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("undertime_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("request_for_payment_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("request_for_payment_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("repairs_and_maintenance_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("request_for_payment_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("request_for_payment_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("repairs_and_maintenance_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("repairs_and_maintenance_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("repairs_and_maintenance_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Disposal_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Active_campaign_report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("For_Billing_report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("sales_order_for_project_mngmt" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("inventory_transfer_cost_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Supplier_Balance_Report_Details" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Supplier_Balance_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Inventory_per_Cost_center_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Deposit_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("SO_for_Billing_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Billing_due_dates_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Packing_List_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Open_Trip_Ticket_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Transfer_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Production_Report_invw" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Supply_per_Department__Report_invw" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Item_Balances_Report_invw" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Open_Case_Report_s" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Order_per_Campaign_Report_marketing" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_per_campaign_Report_marketing" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Asset_per_Accountability_Report_invw" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Deposit_in_transit_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Outstanding_Checks_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Absences_Detail_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Absences_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Attendance_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("DTR_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Summary_of_Tardiness_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Followup_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Coming_Meeting_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Meeting_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Next_Action_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Calls_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("sales_per_sales_rep_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Expense_per_Project_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Tardiness_Report_Summary" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Timekeeping__Hours_Worked__Report_Summary" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Withholding_Tax_Collection_List_per_Month_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Business_Owner_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Finance_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Purchasing_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Production_Dashboard1" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Marketing_Dashboard1" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Support_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Sales_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Human_Resource_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Project_Manager_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("System_Admistration_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("purchase_report_per_item_Chart" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("System_Audit_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("sale_cost_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("currency_exchange_rate" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("cash_receipt_cost_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("production_cost_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Employee" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Quotation_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Order_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Reimbursement_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Transaction_Registry_Dashboard_SaleRep" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Loan_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Leave_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Over_Time_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Undertime_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Fieldwork_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Change_Time_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Transaction_Registry_Dashboard_Employee" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Transaction_Registry_Dashboard_Supplier" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Transaction_Registry_Dashboard_Customer" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("lyl_estimate_dr" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_dr_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_dr_detail_accessories" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_dr_detail_aluminum" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_dr_detail_e_accessories" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("lyl_estimate_dr_detail_glass" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sum_dr_qty" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("check_book_registry" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("check_book_registry_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("vat_type" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("quotation_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bal_purchase_order_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bal_sales_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("pending_items_reminders" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("petty_cash_for_liquidation" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("payment_method" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("account_type" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("phase" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("item_cost_recalculation" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("issue_components" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("issue_components_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("unbuild_assemblies" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("unbuild_assemblies_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("demand_plan" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("input_vat_allocation" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("case_type" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("qc_inspection" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("packing_list_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("packing_list_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("packing_list_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("packing_list_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("disposed_inv_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("disposed_inv" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("disposed_inv_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("disposed_inv_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("disposed_inv_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("disposed_inv_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("For_Approval_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("For_Checking_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Customer_For_Approval" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("request_for_payment_report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("credit_memo_report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("debit_memo_report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("production_request_report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Supplies_Balance_report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("approved_sales_order" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("purchase_order_for_receiving" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("packing_list" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("packing_list_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_quote" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_estimates" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_po" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_pr" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_production_request" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_qc" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_receiving" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_request_for_payment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_so" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_so_for_invoice" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_so_for_project_management" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("open_backload" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("purchase_order_receiving_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Purchase_Order_Receiving_Dashboard_Supplier1" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("avg_inv" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("open_backload_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("open_estimates_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("open_po_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("open_pr_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("open_production_request_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("open_qc_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("open_so_for_project_management_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("open_so_for_invoice_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("open_so_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("open_receiving_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("monthly_consuption_for_production_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("issue_components_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("issue_components_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("issue_components_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("issue_components_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("fieldwork_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("fieldwork_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchases_Order_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchases_Order_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Receiving_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Receiving_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("inventory_tranfer_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchase_vs_Production_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("cash_sales_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Center_Replenishment_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Center_Replenishment_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Purchase_vs_Production" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Production_Summary" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("pending_production_request" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("unbuild_assemblies_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("unbuild_assemblies_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("unbuild_assemblies_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("quality_control_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("quality_control_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("quality_control_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("raw_cake_balance" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("bal_cashreceipt" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bal_cashpayment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("opportunity" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Current_Accounts" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("bal_lyl_acc" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bal_lyl_glass" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bal_lyl_eacc" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bal_lyl_allum" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("opportunity_details" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Collection_Report_per_Sales_Rep" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Inventory_per_lot_no" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("marketing_request_form_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("marketing_request_form_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Accounts_Payable_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("201" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Cashflow_forcast" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Business_Owner_Sales_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Business_Owner_Purchase_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Business_Owner_Cash_Balance" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Business_Owner_Top_Customer_Income" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("AR_Billing_due_dates_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("AR_Aging_of_AR_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("AR_Top_5_Customer_balances" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("AR_Progress_Billing_Alert" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Account_Receivable_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Finance_Cash_Balance" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Finance_approved_request_for_payment" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Finance_Cash_flow_forcast" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Finance_Approved_Employee_Loan" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Finance_Cash_Balances" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("AP_Aging_of_Payables" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Purchasing_Item_below_reorder" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("production_per_item" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("AP_Top_5_Supplier_balances" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("Warehouse_Item_balances" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("DR_per_Item" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("Logistic_Open_Trip_Ticket" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Logistic_Available_Vehicles" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Marketing_Open_Campaign" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Marketing_Sales_per_campaign" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Support_Open_Case_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Support_Service_center" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sale_Meeting_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Next_Action_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("HR_attendance_summary" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("HR_Birthday_Alert" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("HR_holiday" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Project_Expense_per_Project" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Project_Expense_per_Project_graph" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("Petty_Cash_Balance" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Petty_Cash_Unliquidated" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Petty_Cash_Purchases_per_Item" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("Petty_Cash_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Accounting_Income_per_Center" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("open_quote_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Open_cases_per_agent" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("Marketing_sales_per_campaing_chart" == $shortTName && ($type===false || ($type!==false && $type == 3)))
		return true;
	if ("Logistics_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Accounting_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("AP_open_po_receiving" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Production_Inventory_Balance_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Production_WIP" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Warehouse_Dashboard" == $shortTName && ($type===false || ($type!==false && $type == 4)))
		return true;
	if ("Warehouse_Inventory_Balance_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("logistic_open_inventory_tranfer" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Leads" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_opportunity_Next_Action" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Sales_Active_opportunity" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("intercompany_sales_order_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("intercompany_sales_order" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("production_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("quality_control_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_accessories_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_aluminum_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_e_accessories_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_glass_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_accessories_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_aluminum_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_e_accessories_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_glass_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_accessories_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_aluminum_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_e_accessories_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_detail_glass_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_accessories_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_aluminum_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_e_accessories_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lyl_estimate_dr_detail_glass_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("EWT_Certificates_summay" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("1601e" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("1601eform" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("1601fform" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("tax_person_status" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("journal_entry_inv_transfer" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Recon_withdraw" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("lot" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("serial_no" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("request_for_quote" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("request_for_quoter_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("request_for_quote_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("request_for_quote_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("request_for_quoter_detail_for_checking" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("request_for_quoter_detail_for_approval" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("estimate_cost" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("airlines" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("airport" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("inco_terms" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("seaports" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("shipment_classification" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("shipping_lines" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bal_sales_order_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("project_cont_seal" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("project_house" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("billable_liquidation" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("billable_pettycash" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("billable_checkvoucher" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("projects_dashboard" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("employees" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("admin_rights" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("admin_members" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("admin_users" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("company" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("notifiy_quotation_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("expenses_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("petty_cash_release" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("petty_cash_replenishment" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Employee_Advance_Release" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Employee_Advance_Replenishment" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Liquidation_Release" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Liquidation_Replenishment" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("Billable_Cost_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("centerrep_pettycash" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("centerrep_empadvance" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("centerrep_liquidation" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("replenishment_pettycash" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("replenishment_liquidation" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("replenishment_empadv" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("bank_statement" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("other_charges" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("change_password" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("project_liquidation_detail2" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("balance_advance" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("project_replenishment_deposited" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("bal_center_replenishment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("Project_and_Pettycash_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Project_and_EmpAdv_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Project_and_Liquidation_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Project_and_Sales_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("Project_and_Voucher_Report" == $shortTName && ($type===false || ($type!==false && $type == 2)))
		return true;
	if ("projects_view" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("projects_lite" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("shipping_date" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("entry_detail_sales" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("entry_detail_pettycash" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("entry_detail_cash_disbursement" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("representative" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sales_sub_invoice" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("search_cash_disbursement" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("sales_business_partner" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("canvas" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("canvas_detail" == $shortTName && ($type===false || ($type!==false && $type == 1)))
		return true;
	if ("request_for_payment" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("request_for_payment_detail" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("po_netpr" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("request_for_payment_imp" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	if ("birsalesrelief" == $shortTName && ($type===false || ($type!==false && $type == 0)))
		return true;
	return false;
}

/**
 * Get password field on register page
 * @intellisense
 */
function GetPasswordField($table = "")
{
	global $cPasswordField;
	return $cPasswordField;
}

/**
 * Get user name field on register page
 * @intellisense
 */
function GetUserNameField($table = "")
{
	global $cUserNameField;
	return $cUserNameField;
}

/**
 * Get user name field on register page
 * @intellisense
 */
function GetDisplayNameField($table = "")
{
	global $cDisplayNameField;
	return $cDisplayNameField;
}

/**
 * Get user name field on register page
 * @intellisense
 */
function GetEmailField($table = "")
{
	global $cEmailField;
	return $cEmailField;
}

/**
 * @intellisense
 */
function GetTablesList($pdfMode = false)
{
	$arr = array();
	$strPerm = GetUserPermissions("business_partner");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="business_partner";
	}
	$strPerm = GetUserPermissions("item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="item";
	}
	$strPerm = GetUserPermissions("credit_terms");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_terms";
	}
	$strPerm = GetUserPermissions("shipping_terms");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="shipping_terms";
	}
	$strPerm = GetUserPermissions("cash_receipt");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_receipt";
	}
	$strPerm = GetUserPermissions("purchase");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase";
	}
	$strPerm = GetUserPermissions("cash_disbursment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_disbursment";
	}
	$strPerm = GetUserPermissions("general_journal");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="general_journal";
	}
	$strPerm = GetUserPermissions("report_center");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="report_center";
	}
	$strPerm = GetUserPermissions("Sales Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales Report";
	}
	$strPerm = GetUserPermissions("Disbursement Reports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Disbursement Reports";
	}
	$strPerm = GetUserPermissions("Financial Reports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Financial Reports";
	}
	$strPerm = GetUserPermissions("Purchase Reports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase Reports";
	}
	$strPerm = GetUserPermissions("Receivable Reports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Receivable Reports";
	}
	$strPerm = GetUserPermissions("Taxation Reports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Taxation Reports";
	}
	$strPerm = GetUserPermissions("system_user");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="system_user";
	}
	$strPerm = GetUserPermissions("fiscal_year");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fiscal_year";
	}
	$strPerm = GetUserPermissions("beg_bal");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="beg_bal";
	}
	$strPerm = GetUserPermissions("ewt");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="ewt";
	}
	$strPerm = GetUserPermissions("bank");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bank";
	}
	$strPerm = GetUserPermissions("cost_center");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cost_center";
	}
	$strPerm = GetUserPermissions("General Journal Reports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="General Journal Reports";
	}
	$strPerm = GetUserPermissions("Inventory Reports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory Reports";
	}
	$strPerm = GetUserPermissions("accounting_period");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="accounting_period";
	}
	$strPerm = GetUserPermissions("bank_recon");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bank_recon";
	}
	$strPerm = GetUserPermissions("deposit");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="deposit";
	}
	$strPerm = GetUserPermissions("account_classification");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="account_classification";
	}
	$strPerm = GetUserPermissions("chart_of_accounts");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="chart_of_accounts";
	}
	$strPerm = GetUserPermissions("recon_CR");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="recon_CR";
	}
	$strPerm = GetUserPermissions("Recon_deposit");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Recon_deposit";
	}
	$strPerm = GetUserPermissions("Recon_CV");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Recon_CV";
	}
	$strPerm = GetUserPermissions("Recon_GV_dep");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Recon_GV_dep";
	}
	$strPerm = GetUserPermissions("Recon_GV_CV");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Recon_GV_CV";
	}
	$strPerm = GetUserPermissions("vat_atc");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="vat_atc";
	}
	$strPerm = GetUserPermissions("rdo");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="rdo";
	}
	$strPerm = GetUserPermissions("2550q");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="2550q";
	}
	$strPerm = GetUserPermissions("2550m");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="2550m";
	}
	$strPerm = GetUserPermissions("dr");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="dr";
	}
	$strPerm = GetUserPermissions("purchase_order");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_order";
	}
	$strPerm = GetUserPermissions("purchase_order_receiving");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_order_receiving";
	}
	$strPerm = GetUserPermissions("System_preference");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="System_preference";
	}
	$strPerm = GetUserPermissions("ppe_list");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="ppe_list";
	}
	$strPerm = GetUserPermissions("discounts");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="discounts";
	}
	$strPerm = GetUserPermissions("item_group");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="item_group";
	}
	$strPerm = GetUserPermissions("location");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="location";
	}
	$strPerm = GetUserPermissions("meetings");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="meetings";
	}
	$strPerm = GetUserPermissions("calls");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="calls";
	}
	$strPerm = GetUserPermissions("campaign");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="campaign";
	}
	$strPerm = GetUserPermissions("reimbursement");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="reimbursement";
	}
	$strPerm = GetUserPermissions("reimbursement_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="reimbursement_detail";
	}
	$strPerm = GetUserPermissions("Target_customers");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Target_customers";
	}
	$strPerm = GetUserPermissions("quotation");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quotation";
	}
	$strPerm = GetUserPermissions("case");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="case";
	}
	$strPerm = GetUserPermissions("projects");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="projects";
	}
	$strPerm = GetUserPermissions("production");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production";
	}
	$strPerm = GetUserPermissions("issue");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="issue";
	}
	$strPerm = GetUserPermissions("issue_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="issue_detail";
	}
	$strPerm = GetUserPermissions("disposed");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="disposed";
	}
	$strPerm = GetUserPermissions("disposed_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="disposed_detail";
	}
	$strPerm = GetUserPermissions("backload");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="backload";
	}
	$strPerm = GetUserPermissions("backload_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="backload_detail";
	}
	$strPerm = GetUserPermissions("vehicle");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="vehicle";
	}
	$strPerm = GetUserPermissions("payroll_item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="payroll_item";
	}
	$strPerm = GetUserPermissions("uom");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="uom";
	}
	$strPerm = GetUserPermissions("trip_ticket");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="trip_ticket";
	}
	$strPerm = GetUserPermissions("accident");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="accident";
	}
	$strPerm = GetUserPermissions("approver_list");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="approver_list";
	}
	$strPerm = GetUserPermissions("area");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="area";
	}
	$strPerm = GetUserPermissions("attendance");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="attendance";
	}
	$strPerm = GetUserPermissions("currency");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="currency";
	}
	$strPerm = GetUserPermissions("medical");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="medical";
	}
	$strPerm = GetUserPermissions("position");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="position";
	}
	$strPerm = GetUserPermissions("service_center");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="service_center";
	}
	$strPerm = GetUserPermissions("seminar");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="seminar";
	}
	$strPerm = GetUserPermissions("service_report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="service_report";
	}
	$strPerm = GetUserPermissions("service_report_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="service_report_detail";
	}
	$strPerm = GetUserPermissions("tax_status");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="tax_status";
	}
	$strPerm = GetUserPermissions("training");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="training";
	}
	$strPerm = GetUserPermissions("competitor");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="competitor";
	}
	$strPerm = GetUserPermissions("campaign_type");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="campaign_type";
	}
	$strPerm = GetUserPermissions("fixed_asset_return_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fixed_asset_return_detail";
	}
	$strPerm = GetUserPermissions("fixed_assets_return");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fixed_assets_return";
	}
	$strPerm = GetUserPermissions("Prospect_customers");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Prospect_customers";
	}
	$strPerm = GetUserPermissions("trip_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="trip_detail";
	}
	$strPerm = GetUserPermissions("beg_baldetail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="beg_baldetail";
	}
	$strPerm = GetUserPermissions("cash_disbursement_detail2");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_disbursement_detail2";
	}
	$strPerm = GetUserPermissions("case_solutions");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="case_solutions";
	}
	$strPerm = GetUserPermissions("cash_disbursment_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_disbursment_detail";
	}
	$strPerm = GetUserPermissions("cash_receipt_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_receipt_detail";
	}
	$strPerm = GetUserPermissions("cash_receipt_detail2");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_receipt_detail2";
	}
	$strPerm = GetUserPermissions("general_journal_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="general_journal_detail";
	}
	$strPerm = GetUserPermissions("employee_list");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="employee_list";
	}
	$strPerm = GetUserPermissions("petty_cash");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="petty_cash";
	}
	$strPerm = GetUserPermissions("petty_cash_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="petty_cash_detail";
	}
	$strPerm = GetUserPermissions("center_replenishment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="center_replenishment";
	}
	$strPerm = GetUserPermissions("center_replenishment_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="center_replenishment_detail";
	}
	$strPerm = GetUserPermissions("credit_memo");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_memo";
	}
	$strPerm = GetUserPermissions("credit_memo_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_memo_detail";
	}
	$strPerm = GetUserPermissions("debit_memo");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="debit_memo";
	}
	$strPerm = GetUserPermissions("debit_memo_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="debit_memo_detail";
	}
	$strPerm = GetUserPermissions("project_replenishment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="project_replenishment";
	}
	$strPerm = GetUserPermissions("project_replenishment_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="project_replenishment_detail";
	}
	$strPerm = GetUserPermissions("business_partner_group");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="business_partner_group";
	}
	$strPerm = GetUserPermissions("Sales_Representatives");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Representatives";
	}
	$strPerm = GetUserPermissions("Types_of_leaves");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Types_of_leaves";
	}
	$strPerm = GetUserPermissions("payroll_mode");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="payroll_mode";
	}
	$strPerm = GetUserPermissions("payroll_account_type");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="payroll_account_type";
	}
	$strPerm = GetUserPermissions("Types_of_Overtime");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Types_of_Overtime";
	}
	$strPerm = GetUserPermissions("Types_of_Loan");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Types_of_Loan";
	}
	$strPerm = GetUserPermissions("sales");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales";
	}
	$strPerm = GetUserPermissions("sales_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_detail";
	}
	$strPerm = GetUserPermissions("Payroll_Company_Setup");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Payroll_Company_Setup";
	}
	$strPerm = GetUserPermissions("item_bill_of_materials");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="item_bill_of_materials";
	}
	$strPerm = GetUserPermissions("item_uom_conversion");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="item_uom_conversion";
	}
	$strPerm = GetUserPermissions("tranasactions_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="tranasactions_for_approval";
	}
	$strPerm = GetUserPermissions("deposit_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="deposit_detail";
	}
	$strPerm = GetUserPermissions("purchase_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_detail";
	}
	$strPerm = GetUserPermissions("dr_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="dr_detail";
	}
	$strPerm = GetUserPermissions("purchase_order_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_order_detail";
	}
	$strPerm = GetUserPermissions("Sales_Order_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Order_for_checking";
	}
	$strPerm = GetUserPermissions("sales_order_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_order_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Sales_Order_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Order_for_approval";
	}
	$strPerm = GetUserPermissions("sales_order_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_order_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Quotation_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Quotation_for_approval";
	}
	$strPerm = GetUserPermissions("Quotation_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Quotation_for_checking";
	}
	$strPerm = GetUserPermissions("quotation_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quotation_detail_for_checking";
	}
	$strPerm = GetUserPermissions("quotation_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quotation_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Reimbursement_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Reimbursement_for_checking";
	}
	$strPerm = GetUserPermissions("reimbursement_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="reimbursement_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Reimbursement_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Reimbursement_for_approval";
	}
	$strPerm = GetUserPermissions("reimbursement_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="reimbursement_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Project_Replenishment_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project_Replenishment_for_checking";
	}
	$strPerm = GetUserPermissions("project_replenishment_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="project_replenishment_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Project_Replenishment_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project_Replenishment_for_approval";
	}
	$strPerm = GetUserPermissions("project_replenishment_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="project_replenishment_detail_for_approval";
	}
	$strPerm = GetUserPermissions("center_replenishment_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="center_replenishment_detail_for_checking";
	}
	$strPerm = GetUserPermissions("center_replenishment_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="center_replenishment_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Purchases_Order_for_checking2");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchases_Order_for_checking2";
	}
	$strPerm = GetUserPermissions("purchase_order_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_order_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Purchases_Order_for_approval2");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchases_Order_for_approval2";
	}
	$strPerm = GetUserPermissions("purchase_order_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_order_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Receiving_for_checking2");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Receiving_for_checking2";
	}
	$strPerm = GetUserPermissions("purchase_order_receiving_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_order_receiving_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Receiving_for_approval2");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Receiving_for_approval2";
	}
	$strPerm = GetUserPermissions("purchase_order_receiving_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_order_receiving_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Delivery_Receipt_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Delivery_Receipt_for_checking";
	}
	$strPerm = GetUserPermissions("dr_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="dr_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Delivery_Receipt_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Delivery_Receipt_for_approval";
	}
	$strPerm = GetUserPermissions("dr_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="dr_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Production_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_for_checking";
	}
	$strPerm = GetUserPermissions("production_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Production_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_for_approval";
	}
	$strPerm = GetUserPermissions("production_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Issue_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Issue_for_checking";
	}
	$strPerm = GetUserPermissions("supply_issue_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="supply_issue_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Issue_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Issue_for_approval";
	}
	$strPerm = GetUserPermissions("supply_issue_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="supply_issue_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Return_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Return_for_checking";
	}
	$strPerm = GetUserPermissions("supply_return_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="supply_return_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Return_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Return_for_approval";
	}
	$strPerm = GetUserPermissions("supply_return_detail_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="supply_return_detail_approval";
	}
	$strPerm = GetUserPermissions("supply_disposed_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="supply_disposed_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Disposal_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Disposal_for_approval";
	}
	$strPerm = GetUserPermissions("supply_disposed_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="supply_disposed_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Fixed_Asset_Issue_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Fixed_Asset_Issue_for_checking";
	}
	$strPerm = GetUserPermissions("Fixed_Asset_Issue_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Fixed_Asset_Issue_for_approval";
	}
	$strPerm = GetUserPermissions("fixed_asset_issue_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fixed_asset_issue_detail_for_approval";
	}
	$strPerm = GetUserPermissions("fixed_asset_issue_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fixed_asset_issue_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Fixed_Asset_Return_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Fixed_Asset_Return_for_checking";
	}
	$strPerm = GetUserPermissions("fixed_asset_return_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fixed_asset_return_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Fixed_Asset_Return_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Fixed_Asset_Return_for_approval";
	}
	$strPerm = GetUserPermissions("fixed_asset_return_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fixed_asset_return_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Petty_Cash_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Petty_Cash_for_checking";
	}
	$strPerm = GetUserPermissions("petty_cash_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="petty_cash_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Petty_Cash_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Petty_Cash_for_approval";
	}
	$strPerm = GetUserPermissions("petty_cash_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="petty_cash_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Deposit_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Deposit_for_checking";
	}
	$strPerm = GetUserPermissions("deposit_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="deposit_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Deposit_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Deposit_for_approval";
	}
	$strPerm = GetUserPermissions("deposit_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="deposit_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Credit_Memo_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Credit_Memo_for_checking";
	}
	$strPerm = GetUserPermissions("credit_memo_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_memo_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Credit_Memo_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Credit_Memo_for_approval";
	}
	$strPerm = GetUserPermissions("credit_memo_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_memo_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Debit_Memo_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Debit_Memo_for_checking";
	}
	$strPerm = GetUserPermissions("debit_memo_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="debit_memo_detail_for_checking";
	}
	$strPerm = GetUserPermissions("debit_memo_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="debit_memo_for_approval";
	}
	$strPerm = GetUserPermissions("debit_memo_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="debit_memo_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Purchases_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchases_for_checking";
	}
	$strPerm = GetUserPermissions("credit_purchase_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_purchase_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Purchases_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchases_for_approval";
	}
	$strPerm = GetUserPermissions("credit_purchase_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_purchase_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Sales_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_for_checking";
	}
	$strPerm = GetUserPermissions("Sales_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_for_approval";
	}
	$strPerm = GetUserPermissions("credit_sales_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_sales_detail_for_checking";
	}
	$strPerm = GetUserPermissions("credit_sales_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_sales_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Cash_Disbursement_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Cash_Disbursement_for_checking";
	}
	$strPerm = GetUserPermissions("check_voucher_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="check_voucher_detail_for_checking";
	}
	$strPerm = GetUserPermissions("check_voucher_detail2_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="check_voucher_detail2_for_checking";
	}
	$strPerm = GetUserPermissions("Cash_Disbursement_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Cash_Disbursement_for_approval";
	}
	$strPerm = GetUserPermissions("check_voucher_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="check_voucher_detail_for_approval";
	}
	$strPerm = GetUserPermissions("check_voucher_detail2_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="check_voucher_detail2_for_approval";
	}
	$strPerm = GetUserPermissions("document");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="document";
	}
	$strPerm = GetUserPermissions("General_Journal_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="General_Journal_for_checking";
	}
	$strPerm = GetUserPermissions("general_journal_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="general_journal_detail_for_checking";
	}
	$strPerm = GetUserPermissions("General_Journal_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="General_Journal_for_approval";
	}
	$strPerm = GetUserPermissions("general_journal_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="general_journal_detail_for_approval";
	}
	$strPerm = GetUserPermissions("for_submissions");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="for_submissions";
	}
	$strPerm = GetUserPermissions("submission_details");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="submission_details";
	}
	$strPerm = GetUserPermissions("sales_order_detail_billing_dates");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_order_detail_billing_dates";
	}
	$strPerm = GetUserPermissions("Submission_Search");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Submission_Search";
	}
	$strPerm = GetUserPermissions("Submission_Reminders");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Submission_Reminders";
	}
	$strPerm = GetUserPermissions("years");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="years";
	}
	$strPerm = GetUserPermissions("document_type");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="document_type";
	}
	$strPerm = GetUserPermissions("estimate");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="estimate";
	}
	$strPerm = GetUserPermissions("estimate_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="estimate_detail";
	}
	$strPerm = GetUserPermissions("purchase_request");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_request";
	}
	$strPerm = GetUserPermissions("purchase_request_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_request_detail";
	}
	$strPerm = GetUserPermissions("inv_adj_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inv_adj_detail";
	}
	$strPerm = GetUserPermissions("inventory_adj");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inventory_adj";
	}
	$strPerm = GetUserPermissions("inventory_tranfer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inventory_tranfer";
	}
	$strPerm = GetUserPermissions("inventory_transfer_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inventory_transfer_detail";
	}
	$strPerm = GetUserPermissions("Customers");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Customers";
	}
	$strPerm = GetUserPermissions("Suppliers");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Suppliers";
	}
	$strPerm = GetUserPermissions("Target_contacts");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Target_contacts";
	}
	$strPerm = GetUserPermissions("Prospect_contacts");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Prospect_contacts";
	}
	$strPerm = GetUserPermissions("Customer_Contacts");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Customer_Contacts";
	}
	$strPerm = GetUserPermissions("Supplier_Contacts");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Supplier_Contacts";
	}
	$strPerm = GetUserPermissions("Statement_of_Account");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Statement_of_Account";
	}
	$strPerm = GetUserPermissions("Change_Time_Schedule_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Change_Time_Schedule_for_approval";
	}
	$strPerm = GetUserPermissions("Change_Time_Schedule_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Change_Time_Schedule_for_checking";
	}
	$strPerm = GetUserPermissions("Inventory_Adjustment_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory_Adjustment_for_checking";
	}
	$strPerm = GetUserPermissions("inv_adj_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inv_adj_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Inventory_Adjustment_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory_Adjustment_for_approval";
	}
	$strPerm = GetUserPermissions("inv_adj_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inv_adj_detail_for_approval";
	}
	$strPerm = GetUserPermissions("for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="for_checking";
	}
	$strPerm = GetUserPermissions("for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="for_approval";
	}
	$strPerm = GetUserPermissions("Estimate_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Estimate_for_approval";
	}
	$strPerm = GetUserPermissions("Estimate_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Estimate_for_checking";
	}
	$strPerm = GetUserPermissions("estimate_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="estimate_detail_for_checking";
	}
	$strPerm = GetUserPermissions("estimate_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="estimate_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Follow_up");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Follow_up";
	}
	$strPerm = GetUserPermissions("coming_meetings");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="coming_meetings";
	}
	$strPerm = GetUserPermissions("tasks");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="tasks";
	}
	$strPerm = GetUserPermissions("Project_phases");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project_phases";
	}
	$strPerm = GetUserPermissions("origin_fa");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="origin_fa";
	}
	$strPerm = GetUserPermissions("origin_serialized_supply");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="origin_serialized_supply";
	}
	$strPerm = GetUserPermissions("2301");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="2301";
	}
	$strPerm = GetUserPermissions("Prospect_customers_to_Customer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Prospect_customers_to_Customer";
	}
	$strPerm = GetUserPermissions("lyl_estimate");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_accessories");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_accessories";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_aluminum");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_aluminum";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_e_accessories");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_e_accessories";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_glass");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_glass";
	}
	$strPerm = GetUserPermissions("recuring_benefits");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="recuring_benefits";
	}
	$strPerm = GetUserPermissions("recuring_deductions");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="recuring_deductions";
	}
	$strPerm = GetUserPermissions("recurring_benefits_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="recurring_benefits_detail";
	}
	$strPerm = GetUserPermissions("recurring_deductions_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="recurring_deductions_detail";
	}
	$strPerm = GetUserPermissions("production_request");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_request";
	}
	$strPerm = GetUserPermissions("production_request_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_request_detail";
	}
	$strPerm = GetUserPermissions("attendance_summary");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="attendance_summary";
	}
	$strPerm = GetUserPermissions("Other_Incentives");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Other_Incentives";
	}
	$strPerm = GetUserPermissions("Other_Deductions");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Other_Deductions";
	}
	$strPerm = GetUserPermissions("department");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="department";
	}
	$strPerm = GetUserPermissions("journal_entry_cash_disbursement");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_cash_disbursement";
	}
	$strPerm = GetUserPermissions("journal_entry_cash_receipt");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_cash_receipt";
	}
	$strPerm = GetUserPermissions("journal_entry_center_replenishment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_center_replenishment";
	}
	$strPerm = GetUserPermissions("journal_entry_credit_memo");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_credit_memo";
	}
	$strPerm = GetUserPermissions("journal_entry_debit_memo");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_debit_memo";
	}
	$strPerm = GetUserPermissions("journal_entry_depreciation");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_depreciation";
	}
	$strPerm = GetUserPermissions("journal_entry_general_journal");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_general_journal";
	}
	$strPerm = GetUserPermissions("journal_entry_inventory_adjustment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_inventory_adjustment";
	}
	$strPerm = GetUserPermissions("journal_entry_petty_cash");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_petty_cash";
	}
	$strPerm = GetUserPermissions("journal_entry_project_replenishment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_project_replenishment";
	}
	$strPerm = GetUserPermissions("journal_entry_purchases");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_purchases";
	}
	$strPerm = GetUserPermissions("journal_entry_replenishment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_replenishment";
	}
	$strPerm = GetUserPermissions("journal_entry_service_revenue");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_service_revenue";
	}
	$strPerm = GetUserPermissions("Credit_Card_monitoring");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Credit_Card_monitoring";
	}
	$strPerm = GetUserPermissions("sales_report_per_item Chart");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_report_per_item Chart";
	}
	$strPerm = GetUserPermissions("Leave_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Leave_for_approval";
	}
	$strPerm = GetUserPermissions("loan_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="loan_for_approval";
	}
	$strPerm = GetUserPermissions("loan_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="loan_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Overtime_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Overtime_for_approval";
	}
	$strPerm = GetUserPermissions("Purchase_Request_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Request_for_approval";
	}
	$strPerm = GetUserPermissions("purchase_request_detail_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_request_detail_approval";
	}
	$strPerm = GetUserPermissions("Production_Request_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_Request_for_approval";
	}
	$strPerm = GetUserPermissions("production_request_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_request_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Inventory_Transfer_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory_Transfer_for_approval";
	}
	$strPerm = GetUserPermissions("inventory_transfer_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inventory_transfer_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Leave_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Leave_for_checking";
	}
	$strPerm = GetUserPermissions("Loan_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Loan_for_checking";
	}
	$strPerm = GetUserPermissions("loan_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="loan_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Overtime_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Overtime_for_checking";
	}
	$strPerm = GetUserPermissions("Purchase_Request_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Request_for_checking";
	}
	$strPerm = GetUserPermissions("Production_Request_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_Request_for_checking";
	}
	$strPerm = GetUserPermissions("Inventory_Transfer_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory_Transfer_for_checking";
	}
	$strPerm = GetUserPermissions("inventory_transfer_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inventory_transfer_detail_for_checking";
	}
	$strPerm = GetUserPermissions("attendance_type");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="attendance_type";
	}
	$strPerm = GetUserPermissions("Backload_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Backload_for_approval";
	}
	$strPerm = GetUserPermissions("backload_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="backload_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Backload_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Backload_for_checking";
	}
	$strPerm = GetUserPermissions("backload_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="backload_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Cash_Receipt_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Cash_Receipt_for_approval";
	}
	$strPerm = GetUserPermissions("Cash_Receipt_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Cash_Receipt_for_checking";
	}
	$strPerm = GetUserPermissions("cash_receipt_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_receipt_detail_for_checking";
	}
	$strPerm = GetUserPermissions("cash_receipt_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_receipt_detail_for_approval";
	}
	$strPerm = GetUserPermissions("Payroll_Entry_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Payroll_Entry_for_approval";
	}
	$strPerm = GetUserPermissions("Payroll_Entry_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Payroll_Entry_for_checking";
	}
	$strPerm = GetUserPermissions("payroll_entry_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="payroll_entry_detail_for_approval";
	}
	$strPerm = GetUserPermissions("payroll_entry_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="payroll_entry_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Process_Payroll_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Process_Payroll_for_approval";
	}
	$strPerm = GetUserPermissions("Process_Payroll_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Process_Payroll_for_checking";
	}
	$strPerm = GetUserPermissions("process_payroll_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="process_payroll_detail_for_approval";
	}
	$strPerm = GetUserPermissions("process_payroll_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="process_payroll_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Service_Report_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Service_Report_for_approval";
	}
	$strPerm = GetUserPermissions("Service_Report_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Service_Report_for_checking";
	}
	$strPerm = GetUserPermissions("service_report_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="service_report_detail_for_approval";
	}
	$strPerm = GetUserPermissions("service_report_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="service_report_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Banch Reports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Banch Reports";
	}
	$strPerm = GetUserPermissions("journal_entry_production");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_production";
	}
	$strPerm = GetUserPermissions("journal_entry_deposit");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_deposit";
	}
	$strPerm = GetUserPermissions("production_request_bom");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_request_bom";
	}
	$strPerm = GetUserPermissions("production_bom");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_bom";
	}
	$strPerm = GetUserPermissions("company_info");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="company_info";
	}
	$strPerm = GetUserPermissions("return");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="return";
	}
	$strPerm = GetUserPermissions("return_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="return_detail";
	}
	$strPerm = GetUserPermissions("fixed_asset_issue");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fixed_asset_issue";
	}
	$strPerm = GetUserPermissions("fixed_asset_issue_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fixed_asset_issue_detail";
	}
	$strPerm = GetUserPermissions("production_request_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_request_detail_for_checking";
	}
	$strPerm = GetUserPermissions("cash_disbursment_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_disbursment_detail_for_approval";
	}
	$strPerm = GetUserPermissions("cash_disbursment_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_disbursment_detail_for_checking";
	}
	$strPerm = GetUserPermissions("purchase_request_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_request_detail_for_checking";
	}
	$strPerm = GetUserPermissions("trip_ticket_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="trip_ticket_for_approval";
	}
	$strPerm = GetUserPermissions("trip_ticket_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="trip_ticket_for_checking";
	}
	$strPerm = GetUserPermissions("trip_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="trip_detail_for_approval";
	}
	$strPerm = GetUserPermissions("trip_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="trip_detail_for_checking";
	}
	$strPerm = GetUserPermissions("sales_report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_report";
	}
	$strPerm = GetUserPermissions("Customer_Balance_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Customer_Balance_Report";
	}
	$strPerm = GetUserPermissions("Customer_Balance_Report_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Customer_Balance_Report_detail";
	}
	$strPerm = GetUserPermissions("Collection_Report_per_Customer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Collection_Report_per_Customer";
	}
	$strPerm = GetUserPermissions("Aging_of_AR_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Aging_of_AR_Report";
	}
	$strPerm = GetUserPermissions("Aging_of_AR_Report_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Aging_of_AR_Report_detail";
	}
	$strPerm = GetUserPermissions("Payment_per_Supplier_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Payment_per_Supplier_Report";
	}
	$strPerm = GetUserPermissions("Aging of_AP_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Aging of_AP_Report";
	}
	$strPerm = GetUserPermissions("Aging of_AP_Report_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Aging of_AP_Report_detail";
	}
	$strPerm = GetUserPermissions("Inventory_Balance_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory_Balance_Report";
	}
	$strPerm = GetUserPermissions("Inventory_Valuation_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory_Valuation_Report";
	}
	$strPerm = GetUserPermissions("Inventory_Below_Reorder_Point_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory_Below_Reorder_Point_Report";
	}
	$strPerm = GetUserPermissions("Sales_Report_per_BP");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Report_per_BP";
	}
	$strPerm = GetUserPermissions("Sales_Report_per_Item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Report_per_Item";
	}
	$strPerm = GetUserPermissions("Sales_Report_per_Cost_Center");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Report_per_Cost_Center";
	}
	$strPerm = GetUserPermissions("Sales_Report_per_BP_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Report_per_BP_detail";
	}
	$strPerm = GetUserPermissions("Sales_Report_per_Item_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Report_per_Item_detail";
	}
	$strPerm = GetUserPermissions("Sales_Report_per_Cost_Center_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Report_per_Cost_Center_detail";
	}
	$strPerm = GetUserPermissions("Purchase_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Report";
	}
	$strPerm = GetUserPermissions("Purchase_Report_per_BP");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Report_per_BP";
	}
	$strPerm = GetUserPermissions("Purchase_Report_per_Item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Report_per_Item";
	}
	$strPerm = GetUserPermissions("Purchase_Report_per_Cost_Center");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Report_per_Cost_Center";
	}
	$strPerm = GetUserPermissions("Purchase_Report_per_BP_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Report_per_BP_detail";
	}
	$strPerm = GetUserPermissions("Purchase_Report_per_Item_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Report_per_Item_detail";
	}
	$strPerm = GetUserPermissions("Purchase_Report_per_Cost_Center_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Report_per_Cost_Center_detail";
	}
	$strPerm = GetUserPermissions("Transaction Registry");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Transaction Registry";
	}
	$strPerm = GetUserPermissions("Sales_Report_per_sales_rep");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Report_per_sales_rep";
	}
	$strPerm = GetUserPermissions("PDC_Registry_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="PDC_Registry_Report";
	}
	$strPerm = GetUserPermissions("Available_Vehicles");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Available_Vehicles";
	}
	$strPerm = GetUserPermissions("Trip_Ticket_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Trip_Ticket_Report";
	}
	$strPerm = GetUserPermissions("Purchase_Request_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Request_Report";
	}
	$strPerm = GetUserPermissions("Purchase_Import_Costing");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Import_Costing";
	}
	$strPerm = GetUserPermissions("Production_Costing_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_Costing_Report";
	}
	$strPerm = GetUserPermissions("Production_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_Report";
	}
	$strPerm = GetUserPermissions("Closed_Opportunities");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Closed_Opportunities";
	}
	$strPerm = GetUserPermissions("Cases_with_Sotutions");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Cases_with_Sotutions";
	}
	$strPerm = GetUserPermissions("Totals_Cases_Created");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Totals_Cases_Created";
	}
	$strPerm = GetUserPermissions("Totals_Open_Cases_by_Agent");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Totals_Open_Cases_by_Agent";
	}
	$strPerm = GetUserPermissions("Campaign_Contacts");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Campaign_Contacts";
	}
	$strPerm = GetUserPermissions("Campaign_leads");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Campaign_leads";
	}
	$strPerm = GetUserPermissions("Campaign_ROI_Analysis");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Campaign_ROI_Analysis";
	}
	$strPerm = GetUserPermissions("Campaign_Revenue");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Campaign_Revenue";
	}
	$strPerm = GetUserPermissions("Opportunity_Schedule");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Opportunity_Schedule";
	}
	$strPerm = GetUserPermissions("Sales_by_Account");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_by_Account";
	}
	$strPerm = GetUserPermissions("Sales_by_Lead_Source");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_by_Lead_Source";
	}
	$strPerm = GetUserPermissions("Sales_by_Partner");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_by_Partner";
	}
	$strPerm = GetUserPermissions("Sales_by_Rep");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_by_Rep";
	}
	$strPerm = GetUserPermissions("Leave_Ledger_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Leave_Ledger_Report";
	}
	$strPerm = GetUserPermissions("Leave_Summary");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Leave_Summary";
	}
	$strPerm = GetUserPermissions("Payroll_Register");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Payroll_Register";
	}
	$strPerm = GetUserPermissions("Employee_Leave_History");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Employee_Leave_History";
	}
	$strPerm = GetUserPermissions("Employee_Register");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Employee_Register";
	}
	$strPerm = GetUserPermissions("DR_Reports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="DR_Reports";
	}
	$strPerm = GetUserPermissions("Sales_Order_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Order_Report";
	}
	$strPerm = GetUserPermissions("Case_History");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Case_History";
	}
	$strPerm = GetUserPermissions("quality_control");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quality_control";
	}
	$strPerm = GetUserPermissions("quality_control_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quality_control_detail";
	}
	$strPerm = GetUserPermissions("budget");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="budget";
	}
	$strPerm = GetUserPermissions("budget_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="budget_detail";
	}
	$strPerm = GetUserPermissions("repairs_and_maintenance");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="repairs_and_maintenance";
	}
	$strPerm = GetUserPermissions("production_form");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_form";
	}
	$strPerm = GetUserPermissions("bluewave_employees");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bluewave_employees";
	}
	$strPerm = GetUserPermissions("bluewave_employees_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bluewave_employees_detail";
	}
	$strPerm = GetUserPermissions("bluewave_employees_detail2");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bluewave_employees_detail2";
	}
	$strPerm = GetUserPermissions("bluewave_employees_detail3");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bluewave_employees_detail3";
	}
	$strPerm = GetUserPermissions("bluewave_employees_detail4");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bluewave_employees_detail4";
	}
	$strPerm = GetUserPermissions("bluewave_employees_detail5");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bluewave_employees_detail5";
	}
	$strPerm = GetUserPermissions("bluewave_employees_detail6");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bluewave_employees_detail6";
	}
	$strPerm = GetUserPermissions("rfp");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="rfp";
	}
	$strPerm = GetUserPermissions("telex");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="telex";
	}
	$strPerm = GetUserPermissions("repairs_and_maintenance_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="repairs_and_maintenance_detail";
	}
	$strPerm = GetUserPermissions("repair_list");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="repair_list";
	}
	$strPerm = GetUserPermissions("marketing_colateral");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="marketing_colateral";
	}
	$strPerm = GetUserPermissions("email");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="email";
	}
	$strPerm = GetUserPermissions("accessories");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="accessories";
	}
	$strPerm = GetUserPermissions("sales_order_materials_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_order_materials_detail";
	}
	$strPerm = GetUserPermissions("sales_order_progress_billing_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_order_progress_billing_detail";
	}
	$strPerm = GetUserPermissions("sub_contract");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sub_contract";
	}
	$strPerm = GetUserPermissions("sub_contract_billing_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sub_contract_billing_detail";
	}
	$strPerm = GetUserPermissions("sub_contract_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sub_contract_detail";
	}
	$strPerm = GetUserPermissions("Monthly_Pag-ibig_Summary_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Monthly_Pag-ibig_Summary_Report";
	}
	$strPerm = GetUserPermissions("Monthly_Philhealth_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Monthly_Philhealth_Report";
	}
	$strPerm = GetUserPermissions("Monthly_Philhealth_Summary_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Monthly_Philhealth_Summary_Report";
	}
	$strPerm = GetUserPermissions("Monthly_SSS_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Monthly_SSS_Report";
	}
	$strPerm = GetUserPermissions("Monthly_SSS_Report_by_Department_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Monthly_SSS_Report_by_Department_Report";
	}
	$strPerm = GetUserPermissions("Monthly_Withholding_Tax_Prooflist_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Monthly_Withholding_Tax_Prooflist_Report";
	}
	$strPerm = GetUserPermissions("Monthly_Witholding_Tax_Collection_List_by_Department_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Monthly_Witholding_Tax_Collection_List_by_Department_Report";
	}
	$strPerm = GetUserPermissions("Overtime_Summary_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Overtime_Summary_Report";
	}
	$strPerm = GetUserPermissions("Monthly_Pag-ibig_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Monthly_Pag-ibig_Report";
	}
	$strPerm = GetUserPermissions("sales_order");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_order";
	}
	$strPerm = GetUserPermissions("sales_order_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_order_detail";
	}
	$strPerm = GetUserPermissions("vat_relief_importation");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="vat_relief_importation";
	}
	$strPerm = GetUserPermissions("vat_relief_purchases");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="vat_relief_purchases";
	}
	$strPerm = GetUserPermissions("vat_relief_sales");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="vat_relief_sales";
	}
	$strPerm = GetUserPermissions("journal_ppe_disposal");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_ppe_disposal";
	}
	$strPerm = GetUserPermissions("credit_memo_detail_cgs");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_memo_detail_cgs";
	}
	$strPerm = GetUserPermissions("marketing_request_form");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="marketing_request_form";
	}
	$strPerm = GetUserPermissions("Quotation_Dashboard_Customer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Quotation_Dashboard_Customer";
	}
	$strPerm = GetUserPermissions("Sales_Order_Dashboard_Customer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Order_Dashboard_Customer";
	}
	$strPerm = GetUserPermissions("Delivery_Receipt_Dashboard_Customer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Delivery_Receipt_Dashboard_Customer";
	}
	$strPerm = GetUserPermissions("Sales_Dashboard_Customer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Dashboard_Customer";
	}
	$strPerm = GetUserPermissions("Backload_Dashboard_Customer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Backload_Dashboard_Customer";
	}
	$strPerm = GetUserPermissions("Purchase_Request_Dashboard_Supplier");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Request_Dashboard_Supplier";
	}
	$strPerm = GetUserPermissions("Purchase_Order_Dashboard_Supplier");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Order_Dashboard_Supplier";
	}
	$strPerm = GetUserPermissions("Purchase_Dashboard_Supplier");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Dashboard_Supplier";
	}
	$strPerm = GetUserPermissions("Cash_Disbursement_Dashboard_Supplier");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Cash_Disbursement_Dashboard_Supplier";
	}
	$strPerm = GetUserPermissions("Debit_Memo_Dashboard_Supplier");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Debit_Memo_Dashboard_Supplier";
	}
	$strPerm = GetUserPermissions("Purchase_Request_Dashboard_Item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Request_Dashboard_Item";
	}
	$strPerm = GetUserPermissions("Purchase_Order_Dashboard_Item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Order_Dashboard_Item";
	}
	$strPerm = GetUserPermissions("Purchase_Order_Receiving_Dashboard_Item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Order_Receiving_Dashboard_Item";
	}
	$strPerm = GetUserPermissions("Production_Dashboard_Item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_Dashboard_Item";
	}
	$strPerm = GetUserPermissions("sub_contract_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sub_contract_for_approval";
	}
	$strPerm = GetUserPermissions("sub_contract_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sub_contract_for_checking";
	}
	$strPerm = GetUserPermissions("sub_contract_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sub_contract_detail_for_approval";
	}
	$strPerm = GetUserPermissions("sub_contract_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sub_contract_detail_for_checking";
	}
	$strPerm = GetUserPermissions("budget_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="budget_for_approval";
	}
	$strPerm = GetUserPermissions("budget_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="budget_for_checking";
	}
	$strPerm = GetUserPermissions("budget_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="budget_detail_for_checking";
	}
	$strPerm = GetUserPermissions("budget_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="budget_detail_for_approval";
	}
	$strPerm = GetUserPermissions("undertime_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="undertime_for_approval";
	}
	$strPerm = GetUserPermissions("undertime_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="undertime_for_checking";
	}
	$strPerm = GetUserPermissions("request_for_payment_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_payment_for_approval";
	}
	$strPerm = GetUserPermissions("request_for_payment_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_payment_for_checking";
	}
	$strPerm = GetUserPermissions("repairs_and_maintenance_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="repairs_and_maintenance_detail_for_approval";
	}
	$strPerm = GetUserPermissions("request_for_payment_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_payment_detail_for_approval";
	}
	$strPerm = GetUserPermissions("request_for_payment_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_payment_detail_for_checking";
	}
	$strPerm = GetUserPermissions("repairs_and_maintenance_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="repairs_and_maintenance_for_approval";
	}
	$strPerm = GetUserPermissions("repairs_and_maintenance_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="repairs_and_maintenance_for_checking";
	}
	$strPerm = GetUserPermissions("repairs_and_maintenance_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="repairs_and_maintenance_detail_for_checking";
	}
	$strPerm = GetUserPermissions("Disposal_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Disposal_for_checking";
	}
	$strPerm = GetUserPermissions("Active_campaign_report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Active_campaign_report";
	}
	$strPerm = GetUserPermissions("For_Billing_report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="For_Billing_report";
	}
	$strPerm = GetUserPermissions("sales_order_for_project_mngmt");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_order_for_project_mngmt";
	}
	$strPerm = GetUserPermissions("inventory_transfer_cost_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inventory_transfer_cost_detail";
	}
	$strPerm = GetUserPermissions("Supplier_Balance_Report_Details");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Supplier_Balance_Report_Details";
	}
	$strPerm = GetUserPermissions("Supplier_Balance_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Supplier_Balance_Report";
	}
	$strPerm = GetUserPermissions("Inventory_per_Cost_center_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory_per_Cost_center_Report";
	}
	$strPerm = GetUserPermissions("Deposit_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Deposit_Report";
	}
	$strPerm = GetUserPermissions("SO_for_Billing_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="SO_for_Billing_Report";
	}
	$strPerm = GetUserPermissions("Billing_due_dates_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Billing_due_dates_Report";
	}
	$strPerm = GetUserPermissions("Packing_List_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Packing_List_Report";
	}
	$strPerm = GetUserPermissions("Open_Trip_Ticket_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Open_Trip_Ticket_Report";
	}
	$strPerm = GetUserPermissions("Transfer_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Transfer_Report";
	}
	$strPerm = GetUserPermissions("Production_Report_invw");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_Report_invw";
	}
	$strPerm = GetUserPermissions("Supply_per_Department__Report_invw");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Supply_per_Department__Report_invw";
	}
	$strPerm = GetUserPermissions("Item_Balances_Report_invw");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Item_Balances_Report_invw";
	}
	$strPerm = GetUserPermissions("Open_Case_Report_s");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Open_Case_Report_s";
	}
	$strPerm = GetUserPermissions("Sales_Order_per_Campaign_Report_marketing");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Order_per_Campaign_Report_marketing";
	}
	$strPerm = GetUserPermissions("Sales_per_campaign_Report_marketing");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_per_campaign_Report_marketing";
	}
	$strPerm = GetUserPermissions("Asset_per_Accountability_Report_invw");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Asset_per_Accountability_Report_invw";
	}
	$strPerm = GetUserPermissions("Deposit_in_transit_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Deposit_in_transit_Report";
	}
	$strPerm = GetUserPermissions("Outstanding_Checks_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Outstanding_Checks_Report";
	}
	$strPerm = GetUserPermissions("Absences_Detail_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Absences_Detail_Report";
	}
	$strPerm = GetUserPermissions("Absences_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Absences_Report";
	}
	$strPerm = GetUserPermissions("Attendance_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Attendance_Report";
	}
	$strPerm = GetUserPermissions("DTR_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="DTR_Report";
	}
	$strPerm = GetUserPermissions("Summary_of_Tardiness_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Summary_of_Tardiness_Report";
	}
	$strPerm = GetUserPermissions("Followup_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Followup_Report";
	}
	$strPerm = GetUserPermissions("Coming_Meeting_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Coming_Meeting_Report";
	}
	$strPerm = GetUserPermissions("Meeting_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Meeting_Report";
	}
	$strPerm = GetUserPermissions("Next_Action_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Next_Action_Report";
	}
	$strPerm = GetUserPermissions("Calls_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Calls_Report";
	}
	$strPerm = GetUserPermissions("sales_per_sales_rep_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_per_sales_rep_Report";
	}
	$strPerm = GetUserPermissions("Expense_per_Project_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Expense_per_Project_Report";
	}
	$strPerm = GetUserPermissions("Tardiness_Report_Summary");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Tardiness_Report_Summary";
	}
	$strPerm = GetUserPermissions("Timekeeping_(Hours_Worked)_Report_Summary");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Timekeeping_(Hours_Worked)_Report_Summary";
	}
	$strPerm = GetUserPermissions("Withholding_Tax_Collection_List_per_Month_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Withholding_Tax_Collection_List_per_Month_Report";
	}
	$strPerm = GetUserPermissions("Business_Owner_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Business_Owner_Dashboard";
	}
	$strPerm = GetUserPermissions("Finance_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Finance_Dashboard";
	}
	$strPerm = GetUserPermissions("Purchasing_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchasing_Dashboard";
	}
	$strPerm = GetUserPermissions("Production_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_Dashboard";
	}
	$strPerm = GetUserPermissions("Marketing_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Marketing_Dashboard";
	}
	$strPerm = GetUserPermissions("Support_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Support_Dashboard";
	}
	$strPerm = GetUserPermissions("Sales_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Dashboard";
	}
	$strPerm = GetUserPermissions("Human_Resource_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Human_Resource_Dashboard";
	}
	$strPerm = GetUserPermissions("Project_Manager_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project_Manager_Dashboard";
	}
	$strPerm = GetUserPermissions("System_Admistration_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="System_Admistration_Dashboard";
	}
	$strPerm = GetUserPermissions("purchase_report_per_item Chart");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_report_per_item Chart";
	}
	$strPerm = GetUserPermissions("System_Audit_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="System_Audit_Report";
	}
	$strPerm = GetUserPermissions("sale_cost_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sale_cost_detail";
	}
	$strPerm = GetUserPermissions("currency_exchange_rate");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="currency_exchange_rate";
	}
	$strPerm = GetUserPermissions("cash_receipt_cost_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_receipt_cost_detail";
	}
	$strPerm = GetUserPermissions("production_cost_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_cost_detail";
	}
	$strPerm = GetUserPermissions("Employee");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Employee";
	}
	$strPerm = GetUserPermissions("Quotation_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Quotation_Dashboard";
	}
	$strPerm = GetUserPermissions("Sales_Order_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Order_Dashboard";
	}
	$strPerm = GetUserPermissions("Reimbursement_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Reimbursement_Dashboard";
	}
	$strPerm = GetUserPermissions("Transaction_Registry_Dashboard_SaleRep");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Transaction_Registry_Dashboard_SaleRep";
	}
	$strPerm = GetUserPermissions("Loan_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Loan_Dashboard";
	}
	$strPerm = GetUserPermissions("Leave_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Leave_Dashboard";
	}
	$strPerm = GetUserPermissions("Over_Time_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Over_Time_Dashboard";
	}
	$strPerm = GetUserPermissions("Undertime_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Undertime_Dashboard";
	}
	$strPerm = GetUserPermissions("Fieldwork_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Fieldwork_Dashboard";
	}
	$strPerm = GetUserPermissions("Change_Time_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Change_Time_Dashboard";
	}
	$strPerm = GetUserPermissions("Transaction_Registry_Dashboard_Employee");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Transaction_Registry_Dashboard_Employee";
	}
	$strPerm = GetUserPermissions("Transaction_Registry_Dashboard_Supplier");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Transaction_Registry_Dashboard_Supplier";
	}
	$strPerm = GetUserPermissions("Transaction_Registry_Dashboard_Customer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Transaction_Registry_Dashboard_Customer";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_accessories");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_accessories";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_aluminum");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_aluminum";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_e_accessories");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_e_accessories";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_glass");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_glass";
	}
	$strPerm = GetUserPermissions("sum_dr_qty");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sum_dr_qty";
	}
	$strPerm = GetUserPermissions("check_book_registry");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="check_book_registry";
	}
	$strPerm = GetUserPermissions("check_book_registry_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="check_book_registry_detail";
	}
	$strPerm = GetUserPermissions("vat_type");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="vat_type";
	}
	$strPerm = GetUserPermissions("quotation_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quotation_detail";
	}
	$strPerm = GetUserPermissions("bal_purchase_order_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_purchase_order_detail";
	}
	$strPerm = GetUserPermissions("bal_sales_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_sales_detail";
	}
	$strPerm = GetUserPermissions("pending_items_reminders");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="pending_items_reminders";
	}
	$strPerm = GetUserPermissions("petty_cash_for_liquidation");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="petty_cash_for_liquidation";
	}
	$strPerm = GetUserPermissions("payment_method");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="payment_method";
	}
	$strPerm = GetUserPermissions("account_type");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="account_type";
	}
	$strPerm = GetUserPermissions("phase");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="phase";
	}
	$strPerm = GetUserPermissions("item_cost_recalculation");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="item_cost_recalculation";
	}
	$strPerm = GetUserPermissions("issue_components");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="issue_components";
	}
	$strPerm = GetUserPermissions("issue_components_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="issue_components_detail";
	}
	$strPerm = GetUserPermissions("unbuild_assemblies");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="unbuild_assemblies";
	}
	$strPerm = GetUserPermissions("unbuild_assemblies_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="unbuild_assemblies_detail";
	}
	$strPerm = GetUserPermissions("demand_plan");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="demand_plan";
	}
	$strPerm = GetUserPermissions("input_vat_allocation");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="input_vat_allocation";
	}
	$strPerm = GetUserPermissions("case_type");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="case_type";
	}
	$strPerm = GetUserPermissions("qc_inspection");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="qc_inspection";
	}
	$strPerm = GetUserPermissions("packing_list_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="packing_list_for_checking";
	}
	$strPerm = GetUserPermissions("packing_list_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="packing_list_for_approval";
	}
	$strPerm = GetUserPermissions("packing_list_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="packing_list_detail_for_checking";
	}
	$strPerm = GetUserPermissions("packing_list_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="packing_list_detail_for_approval";
	}
	$strPerm = GetUserPermissions("disposed_inv_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="disposed_inv_detail";
	}
	$strPerm = GetUserPermissions("disposed_inv");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="disposed_inv";
	}
	$strPerm = GetUserPermissions("disposed_inv_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="disposed_inv_for_checking";
	}
	$strPerm = GetUserPermissions("disposed_inv_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="disposed_inv_for_approval";
	}
	$strPerm = GetUserPermissions("disposed_inv_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="disposed_inv_detail_for_checking";
	}
	$strPerm = GetUserPermissions("disposed_inv_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="disposed_inv_detail_for_approval";
	}
	$strPerm = GetUserPermissions("For_Approval_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="For_Approval_Dashboard";
	}
	$strPerm = GetUserPermissions("For_Checking_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="For_Checking_Dashboard";
	}
	$strPerm = GetUserPermissions("Customer_For_Approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Customer_For_Approval";
	}
	$strPerm = GetUserPermissions("request_for_payment_report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_payment_report";
	}
	$strPerm = GetUserPermissions("credit_memo_report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="credit_memo_report";
	}
	$strPerm = GetUserPermissions("debit_memo_report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="debit_memo_report";
	}
	$strPerm = GetUserPermissions("production_request_report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_request_report";
	}
	$strPerm = GetUserPermissions("Supplies_Balance_report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Supplies_Balance_report";
	}
	$strPerm = GetUserPermissions("approved_sales_order");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="approved_sales_order";
	}
	$strPerm = GetUserPermissions("purchase_order_for_receiving");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_order_for_receiving";
	}
	$strPerm = GetUserPermissions("Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Dashboard";
	}
	$strPerm = GetUserPermissions("packing_list");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="packing_list";
	}
	$strPerm = GetUserPermissions("packing_list_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="packing_list_detail";
	}
	$strPerm = GetUserPermissions("open_quote");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_quote";
	}
	$strPerm = GetUserPermissions("open_estimates");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_estimates";
	}
	$strPerm = GetUserPermissions("open_po");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_po";
	}
	$strPerm = GetUserPermissions("open_pr");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_pr";
	}
	$strPerm = GetUserPermissions("open_production_request");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_production_request";
	}
	$strPerm = GetUserPermissions("open_qc");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_qc";
	}
	$strPerm = GetUserPermissions("open_receiving");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_receiving";
	}
	$strPerm = GetUserPermissions("open_request_for_payment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_request_for_payment";
	}
	$strPerm = GetUserPermissions("open_so");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_so";
	}
	$strPerm = GetUserPermissions("open_so_for_invoice");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_so_for_invoice";
	}
	$strPerm = GetUserPermissions("open_so_for_project_management");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_so_for_project_management";
	}
	$strPerm = GetUserPermissions("open_backload");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_backload";
	}
	$strPerm = GetUserPermissions("purchase_order_receiving_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="purchase_order_receiving_detail";
	}
	$strPerm = GetUserPermissions("Purchase_Order_Receiving_Dashboard_Supplier");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_Order_Receiving_Dashboard_Supplier";
	}
	$strPerm = GetUserPermissions("avg_inv");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="avg_inv";
	}
	$strPerm = GetUserPermissions("open_backload_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_backload_Report";
	}
	$strPerm = GetUserPermissions("open_estimates_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_estimates_Report";
	}
	$strPerm = GetUserPermissions("open_po_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_po_Report";
	}
	$strPerm = GetUserPermissions("open_pr_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_pr_Report";
	}
	$strPerm = GetUserPermissions("open_production_request_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_production_request_Report";
	}
	$strPerm = GetUserPermissions("open_qc_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_qc_Report";
	}
	$strPerm = GetUserPermissions("open_so_for_project_management_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_so_for_project_management_Report";
	}
	$strPerm = GetUserPermissions("open_so_for_invoice_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_so_for_invoice_Report";
	}
	$strPerm = GetUserPermissions("open_so_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_so_Report";
	}
	$strPerm = GetUserPermissions("open_receiving_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_receiving_Report";
	}
	$strPerm = GetUserPermissions("monthly_consuption_for_production_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="monthly_consuption_for_production_Report";
	}
	$strPerm = GetUserPermissions("issue_components_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="issue_components_for_approval";
	}
	$strPerm = GetUserPermissions("issue_components_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="issue_components_for_checking";
	}
	$strPerm = GetUserPermissions("issue_components_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="issue_components_detail_for_checking";
	}
	$strPerm = GetUserPermissions("issue_components_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="issue_components_detail_for_approval";
	}
	$strPerm = GetUserPermissions("fieldwork_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fieldwork_for_approval";
	}
	$strPerm = GetUserPermissions("fieldwork_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="fieldwork_for_checking";
	}
	$strPerm = GetUserPermissions("Purchases_Order_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchases_Order_for_checking";
	}
	$strPerm = GetUserPermissions("Purchases_Order_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchases_Order_for_approval";
	}
	$strPerm = GetUserPermissions("Receiving_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Receiving_for_checking";
	}
	$strPerm = GetUserPermissions("Receiving_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Receiving_for_approval";
	}
	$strPerm = GetUserPermissions("inventory_tranfer_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inventory_tranfer_Report";
	}
	$strPerm = GetUserPermissions("Purchase_vs_Production_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_vs_Production_Report";
	}
	$strPerm = GetUserPermissions("cash_sales_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="cash_sales_Report";
	}
	$strPerm = GetUserPermissions("Center_Replenishment_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Center_Replenishment_for_approval";
	}
	$strPerm = GetUserPermissions("Center_Replenishment_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Center_Replenishment_for_checking";
	}
	$strPerm = GetUserPermissions("Purchase_vs_Production");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchase_vs_Production";
	}
	$strPerm = GetUserPermissions("Production_Summary");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_Summary";
	}
	$strPerm = GetUserPermissions("pending_production_request");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="pending_production_request";
	}
	$strPerm = GetUserPermissions("unbuild_assemblies_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="unbuild_assemblies_for_checking";
	}
	$strPerm = GetUserPermissions("unbuild_assemblies_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="unbuild_assemblies_for_approval";
	}
	$strPerm = GetUserPermissions("unbuild_assemblies_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="unbuild_assemblies_detail_for_checking";
	}
	$strPerm = GetUserPermissions("quality_control_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quality_control_for_approval";
	}
	$strPerm = GetUserPermissions("quality_control_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quality_control_for_checking";
	}
	$strPerm = GetUserPermissions("quality_control_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quality_control_detail_for_approval";
	}
	$strPerm = GetUserPermissions("raw_cake_balance");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="raw_cake_balance";
	}
	$strPerm = GetUserPermissions("bal_cashreceipt");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_cashreceipt";
	}
	$strPerm = GetUserPermissions("bal_cashpayment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_cashpayment";
	}
	$strPerm = GetUserPermissions("opportunity");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="opportunity";
	}
	$strPerm = GetUserPermissions("Current_Accounts");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Current_Accounts";
	}
	$strPerm = GetUserPermissions("bal_lyl_acc");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_lyl_acc";
	}
	$strPerm = GetUserPermissions("bal_lyl_glass");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_lyl_glass";
	}
	$strPerm = GetUserPermissions("bal_lyl_eacc");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_lyl_eacc";
	}
	$strPerm = GetUserPermissions("bal_lyl_allum");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_lyl_allum";
	}
	$strPerm = GetUserPermissions("opportunity_details");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="opportunity_details";
	}
	$strPerm = GetUserPermissions("Collection_Report_per_Sales_Rep");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Collection_Report_per_Sales_Rep";
	}
	$strPerm = GetUserPermissions("Inventory_per_lot_no");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Inventory_per_lot_no";
	}
	$strPerm = GetUserPermissions("marketing_request_form_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="marketing_request_form_for_checking";
	}
	$strPerm = GetUserPermissions("marketing_request_form_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="marketing_request_form_for_approval";
	}
	$strPerm = GetUserPermissions("Accounts_Payable_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Accounts_Payable_Dashboard";
	}
	$strPerm = GetUserPermissions("201");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="201";
	}
	$strPerm = GetUserPermissions("Cashflow_forcast");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Cashflow_forcast";
	}
	$strPerm = GetUserPermissions("Business_Owner_Sales_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Business_Owner_Sales_Report";
	}
	$strPerm = GetUserPermissions("Business_Owner_Purchase_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Business_Owner_Purchase_Report";
	}
	$strPerm = GetUserPermissions("Business_Owner_Cash_Balance");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Business_Owner_Cash_Balance";
	}
	$strPerm = GetUserPermissions("Business_Owner_Top_Customer_Income");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Business_Owner_Top_Customer_Income";
	}
	$strPerm = GetUserPermissions("AR_Billing_due_dates_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="AR_Billing_due_dates_Report";
	}
	$strPerm = GetUserPermissions("AR_Aging_of_AR_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="AR_Aging_of_AR_Report";
	}
	$strPerm = GetUserPermissions("AR_Top_5_Customer_balances");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="AR_Top_5_Customer_balances";
	}
	$strPerm = GetUserPermissions("AR_Progress_Billing_Alert");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="AR_Progress_Billing_Alert";
	}
	$strPerm = GetUserPermissions("Account_Receivable_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Account_Receivable_Dashboard";
	}
	$strPerm = GetUserPermissions("Finance_Cash_Balance");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Finance_Cash_Balance";
	}
	$strPerm = GetUserPermissions("Finance_approved_request_for_payment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Finance_approved_request_for_payment";
	}
	$strPerm = GetUserPermissions("Finance_Cash_flow_forcast");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Finance_Cash_flow_forcast";
	}
	$strPerm = GetUserPermissions("Finance_Approved_Employee_Loan");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Finance_Approved_Employee_Loan";
	}
	$strPerm = GetUserPermissions("Finance_Cash_Balances");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Finance_Cash_Balances";
	}
	$strPerm = GetUserPermissions("AP_Aging_of_Payables");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="AP_Aging_of_Payables";
	}
	$strPerm = GetUserPermissions("Purchasing_Item_below_reorder");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Purchasing_Item_below_reorder";
	}
	$strPerm = GetUserPermissions("production_per_item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_per_item";
	}
	$strPerm = GetUserPermissions("AP_Top_5_Supplier_balances");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="AP_Top_5_Supplier_balances";
	}
	$strPerm = GetUserPermissions("Warehouse_Item_balances");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Warehouse_Item_balances";
	}
	$strPerm = GetUserPermissions("DR_per_Item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="DR_per_Item";
	}
	$strPerm = GetUserPermissions("Logistic_Open_Trip_Ticket");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Logistic_Open_Trip_Ticket";
	}
	$strPerm = GetUserPermissions("Logistic_Available_Vehicles");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Logistic_Available_Vehicles";
	}
	$strPerm = GetUserPermissions("Marketing_Open_Campaign");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Marketing_Open_Campaign";
	}
	$strPerm = GetUserPermissions("Marketing_Sales_per_campaign");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Marketing_Sales_per_campaign";
	}
	$strPerm = GetUserPermissions("Support_Open_Case_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Support_Open_Case_Report";
	}
	$strPerm = GetUserPermissions("Support_Service_center");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Support_Service_center";
	}
	$strPerm = GetUserPermissions("Sale_Meeting_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sale_Meeting_Report";
	}
	$strPerm = GetUserPermissions("Sales_Next_Action_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Next_Action_Report";
	}
	$strPerm = GetUserPermissions("HR_attendance_summary");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="HR_attendance_summary";
	}
	$strPerm = GetUserPermissions("HR_Birthday_Alert");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="HR_Birthday_Alert";
	}
	$strPerm = GetUserPermissions("HR_holiday");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="HR_holiday";
	}
	$strPerm = GetUserPermissions("Project_Expense_per_Project");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project_Expense_per_Project";
	}
	$strPerm = GetUserPermissions("Project_Expense_per_Project_graph");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project_Expense_per_Project_graph";
	}
	$strPerm = GetUserPermissions("Petty_Cash_Balance");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Petty_Cash_Balance";
	}
	$strPerm = GetUserPermissions("Petty_Cash_Unliquidated");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Petty_Cash_Unliquidated";
	}
	$strPerm = GetUserPermissions("Petty_Cash_Purchases_per_Item");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Petty_Cash_Purchases_per_Item";
	}
	$strPerm = GetUserPermissions("Petty_Cash_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Petty_Cash_Dashboard";
	}
	$strPerm = GetUserPermissions("Accounting_Income_per_Center");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Accounting_Income_per_Center";
	}
	$strPerm = GetUserPermissions("open_quote_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="open_quote_Report";
	}
	$strPerm = GetUserPermissions("Open_cases_per_agent");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Open_cases_per_agent";
	}
	$strPerm = GetUserPermissions("Marketing_sales_per_campaing_chart");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Marketing_sales_per_campaing_chart";
	}
	$strPerm = GetUserPermissions("Logistics_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Logistics_Dashboard";
	}
	$strPerm = GetUserPermissions("Accounting_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Accounting_Dashboard";
	}
	$strPerm = GetUserPermissions("AP_open_po_receiving");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="AP_open_po_receiving";
	}
	$strPerm = GetUserPermissions("Production_Inventory_Balance_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_Inventory_Balance_Report";
	}
	$strPerm = GetUserPermissions("Production_WIP");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Production_WIP";
	}
	$strPerm = GetUserPermissions("Warehouse_Dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Warehouse_Dashboard";
	}
	$strPerm = GetUserPermissions("Warehouse_Inventory_Balance_Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Warehouse_Inventory_Balance_Report";
	}
	$strPerm = GetUserPermissions("logistic_open_inventory_tranfer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="logistic_open_inventory_tranfer";
	}
	$strPerm = GetUserPermissions("Sales_Leads");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Leads";
	}
	$strPerm = GetUserPermissions("Sales_opportunity_Next_Action");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_opportunity_Next_Action";
	}
	$strPerm = GetUserPermissions("Sales_Active_opportunity");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Sales_Active_opportunity";
	}
	$strPerm = GetUserPermissions("intercompany_sales_order_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="intercompany_sales_order_detail";
	}
	$strPerm = GetUserPermissions("intercompany_sales_order");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="intercompany_sales_order";
	}
	$strPerm = GetUserPermissions("production_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="production_detail";
	}
	$strPerm = GetUserPermissions("quality_control_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="quality_control_detail_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_accessories_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_accessories_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_aluminum_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_aluminum_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_e_accessories_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_e_accessories_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_glass_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_glass_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_accessories_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_accessories_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_aluminum_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_aluminum_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_e_accessories_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_e_accessories_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_glass_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_glass_for_approval";
	}
	$strPerm = GetUserPermissions("lyl_estimate_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_accessories_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_accessories_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_aluminum_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_aluminum_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_e_accessories_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_e_accessories_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_detail_glass_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_detail_glass_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_accessories_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_accessories_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_aluminum_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_aluminum_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_e_accessories_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_e_accessories_for_checking";
	}
	$strPerm = GetUserPermissions("lyl_estimate_dr_detail_glass_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lyl_estimate_dr_detail_glass_for_checking";
	}
	$strPerm = GetUserPermissions("EWT_Certificates_summay");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="EWT_Certificates_summay";
	}
	$strPerm = GetUserPermissions("1601e");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="1601e";
	}
	$strPerm = GetUserPermissions("1601eform");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="1601eform";
	}
	$strPerm = GetUserPermissions("1601fform");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="1601fform";
	}
	$strPerm = GetUserPermissions("tax_person_status");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="tax_person_status";
	}
	$strPerm = GetUserPermissions("journal_entry_inv_transfer");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="journal_entry_inv_transfer";
	}
	$strPerm = GetUserPermissions("Recon_withdraw");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Recon_withdraw";
	}
	$strPerm = GetUserPermissions("lot");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="lot";
	}
	$strPerm = GetUserPermissions("serial_no");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="serial_no";
	}
	$strPerm = GetUserPermissions("request_for_quote");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_quote";
	}
	$strPerm = GetUserPermissions("request_for_quoter_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_quoter_detail";
	}
	$strPerm = GetUserPermissions("request_for_quote_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_quote_for_checking";
	}
	$strPerm = GetUserPermissions("request_for_quote_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_quote_for_approval";
	}
	$strPerm = GetUserPermissions("request_for_quoter_detail_for_checking");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_quoter_detail_for_checking";
	}
	$strPerm = GetUserPermissions("request_for_quoter_detail_for_approval");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_quoter_detail_for_approval";
	}
	$strPerm = GetUserPermissions("estimate_cost");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="estimate_cost";
	}
	$strPerm = GetUserPermissions("airlines");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="airlines";
	}
	$strPerm = GetUserPermissions("airport");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="airport";
	}
	$strPerm = GetUserPermissions("inco_terms");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="inco_terms";
	}
	$strPerm = GetUserPermissions("seaports");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="seaports";
	}
	$strPerm = GetUserPermissions("shipment_classification");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="shipment_classification";
	}
	$strPerm = GetUserPermissions("shipping_lines");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="shipping_lines";
	}
	$strPerm = GetUserPermissions("bal_sales_order_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_sales_order_detail";
	}
	$strPerm = GetUserPermissions("project_cont_seal");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="project_cont_seal";
	}
	$strPerm = GetUserPermissions("project_house");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="project_house";
	}
	$strPerm = GetUserPermissions("billable_liquidation");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="billable_liquidation";
	}
	$strPerm = GetUserPermissions("billable_pettycash");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="billable_pettycash";
	}
	$strPerm = GetUserPermissions("billable_checkvoucher");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="billable_checkvoucher";
	}
	$strPerm = GetUserPermissions("projects_dashboard");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="projects_dashboard";
	}
	$strPerm = GetUserPermissions("employees");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="employees";
	}
	$strPerm = GetUserPermissions("admin_rights");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="admin_rights";
	}
	$strPerm = GetUserPermissions("admin_members");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="admin_members";
	}
	$strPerm = GetUserPermissions("admin_users");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="admin_users";
	}
	$strPerm = GetUserPermissions("company");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="company";
	}
	$strPerm = GetUserPermissions("notifiy quotation Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="notifiy quotation Report";
	}
	$strPerm = GetUserPermissions("expenses Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="expenses Report";
	}
	$strPerm = GetUserPermissions("petty cash release");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="petty cash release";
	}
	$strPerm = GetUserPermissions("petty cash replenishment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="petty cash replenishment";
	}
	$strPerm = GetUserPermissions("Employee Advance Release");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Employee Advance Release";
	}
	$strPerm = GetUserPermissions("Employee Advance Replenishment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Employee Advance Replenishment";
	}
	$strPerm = GetUserPermissions("Liquidation Release");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Liquidation Release";
	}
	$strPerm = GetUserPermissions("Liquidation Replenishment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Liquidation Replenishment";
	}
	$strPerm = GetUserPermissions("Billable Cost Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Billable Cost Report";
	}
	$strPerm = GetUserPermissions("centerrep_pettycash");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="centerrep_pettycash";
	}
	$strPerm = GetUserPermissions("centerrep_empadvance");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="centerrep_empadvance";
	}
	$strPerm = GetUserPermissions("centerrep_liquidation");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="centerrep_liquidation";
	}
	$strPerm = GetUserPermissions("replenishment_pettycash");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="replenishment_pettycash";
	}
	$strPerm = GetUserPermissions("replenishment_liquidation");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="replenishment_liquidation";
	}
	$strPerm = GetUserPermissions("replenishment_empadv");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="replenishment_empadv";
	}
	$strPerm = GetUserPermissions("bank_statement");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bank_statement";
	}
	$strPerm = GetUserPermissions("other_charges");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="other_charges";
	}
	$strPerm = GetUserPermissions("change_password");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="change_password";
	}
	$strPerm = GetUserPermissions("project_liquidation_detail2");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="project_liquidation_detail2";
	}
	$strPerm = GetUserPermissions("balance_advance");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="balance_advance";
	}
	$strPerm = GetUserPermissions("project_replenishment_deposited");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="project_replenishment_deposited";
	}
	$strPerm = GetUserPermissions("bal_center_replenishment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="bal_center_replenishment";
	}
	$strPerm = GetUserPermissions("Project and Pettycash Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project and Pettycash Report";
	}
	$strPerm = GetUserPermissions("Project and EmpAdv Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project and EmpAdv Report";
	}
	$strPerm = GetUserPermissions("Project and Liquidation Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project and Liquidation Report";
	}
	$strPerm = GetUserPermissions("Project and Sales Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project and Sales Report";
	}
	$strPerm = GetUserPermissions("Project and Voucher Report");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="Project and Voucher Report";
	}
	$strPerm = GetUserPermissions("projects_view");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="projects_view";
	}
	$strPerm = GetUserPermissions("projects_lite");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="projects_lite";
	}
	$strPerm = GetUserPermissions("shipping_date");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="shipping_date";
	}
	$strPerm = GetUserPermissions("entry_detail_sales");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="entry_detail_sales";
	}
	$strPerm = GetUserPermissions("entry_detail_pettycash");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="entry_detail_pettycash";
	}
	$strPerm = GetUserPermissions("entry_detail_cash_disbursement");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="entry_detail_cash_disbursement";
	}
	$strPerm = GetUserPermissions("representative");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="representative";
	}
	$strPerm = GetUserPermissions("sales_sub_invoice");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_sub_invoice";
	}
	$strPerm = GetUserPermissions("search_cash_disbursement");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="search_cash_disbursement";
	}
	$strPerm = GetUserPermissions("sales_business_partner");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="sales_business_partner";
	}
	$strPerm = GetUserPermissions("canvas");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="canvas";
	}
	$strPerm = GetUserPermissions("canvas_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="canvas_detail";
	}
	$strPerm = GetUserPermissions("request_for_payment");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_payment";
	}
	$strPerm = GetUserPermissions("request_for_payment_detail");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_payment_detail";
	}
	$strPerm = GetUserPermissions("po_netpr");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="po_netpr";
	}
	$strPerm = GetUserPermissions("request_for_payment_imp");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="request_for_payment_imp";
	}
	$strPerm = GetUserPermissions("birsalesrelief");
	if(strpos($strPerm, "P")!==false || ($pdfMode && strpos($strPerm, "S")!==false))
	{
		$arr[]="birsalesrelief";
	}
	return $arr;
}

/**
 * DEPRECATED! Use RunnerPage::_getFieldSQLDecrypt instead
 * Return the full database field original name
 * 
 * @param string	$field
 * @param string	$table The datasource table name
 * @param boolean	$addAs OPTIONAL
 * 
 * @return String
 * @intellisense
 * @deprecated
 */
function GetFullFieldName($field, $table = "", $addAs = true, $connection = null)
{
	global $strTableName, $cman;
	
	if( $table == "" )
		$table = $strTableName;
	
	if( !$connection )
		$connection = $cman->byTable( $table );
	$pSet = new ProjectSettings($table);
	
	$fname = RunnerPage::_getFieldSQL($field, $connection, $pSet);
	
	if($pSet->hasEncryptedFields() && !isEncryptionByPHPEnabled())
	{
		$cipherer = new RunnerCipherer($table);
		return $cipherer->GetFieldName($fname, $field)
			.($cipherer->isFieldEncrypted($field) && $addAs ? " as ".$connection->addFieldWrappers($field) : "");
	}
	return $fname;
}

/**
 * returns Chart type
 * @intellisense
 */
function GetChartType($shorttable)
{
	if($shorttable=="sales_report_per_item_Chart")
		return "2DPie";
	if($shorttable=="purchase_report_per_item_Chart")
		return "2DPie";
	if($shorttable=="Business_Owner_Top_Customer_Income")
		return "2DPie";
	if($shorttable=="AR_Top_5_Customer_balances")
		return "2DColumn";
	if($shorttable=="Finance_Cash_Balances")
		return "2DColumn";
	if($shorttable=="production_per_item")
		return "2DColumn";
	if($shorttable=="AP_Top_5_Supplier_balances")
		return "2DColumn";
	if($shorttable=="Warehouse_Item_balances")
		return "2DBar";
	if($shorttable=="DR_per_Item")
		return "2DColumn";
	if($shorttable=="Project_Expense_per_Project_graph")
		return "2DColumn";
	if($shorttable=="Petty_Cash_Purchases_per_Item")
		return "2DPie";
	if($shorttable=="Accounting_Income_per_Center")
		return "2DColumn";
	if($shorttable=="Open_cases_per_agent")
		return "2DPie";
	if($shorttable=="Marketing_sales_per_campaing_chart")
		return "2DPie";
	return "";
}

////////////////////////////////////////////////////////////////////////////////
// data output functions
////////////////////////////////////////////////////////////////////////////////

/**
 * @intellisense
 */
function GetShorteningForLargeText($strValue, $cNumberOfChars)
{
	$ret = runner_substr($strValue, 0, $cNumberOfChars );
		
	return runner_htmlspecialchars($ret);
}

/**
 * add prefix to the URL
 * @intellisense
 */
function AddLinkPrefix($pSet, $field, $link)
{
	if(strpos($link,"://")===false && substr($link,0,7)!="mailto:")
		return $pSet->getLinkPrefix($field).$link;
	return $link;
}

/**
 * @intellisense
 */
function GetTotalsForTime($value)
{
	$time=parsenumbers($value);
	while(count($time)<3)
		$time[]=0;
	return $time;
}

/**
 * return Totals string
 * @intellisense
 */
function GetTotals($field, $value, $stype, $iNumberOfRows, $sFormat, $ptype, $pSet)
{
//	global $strTableName;
//	$pSet = new ProjectSettings($strTableName, $ptype);
	$days = 0;
	if($stype == "AVERAGE")
	{
		if($iNumberOfRows)
		{	
			if($sFormat == FORMAT_TIME)
			{
				if($value)
				{
					$value = round($value/$iNumberOfRows,0);
					$s = $value % 60;
					$value -= $s;
					$value /= 60;
					$m = $value % 60;
					$value -= $m;
					$value /= 60;
					$h = $value % 24;
					$value -= $h;
					$value /= 24;
					$d = $value;
					
					$value = ($d!=0 ? $d.'d ' : ''). mysprintf("%02d:%02d:%02d",array($h,$m,$s));
				}
			}
			else $value = round($value/$iNumberOfRows,2);
		}
		else
			return "";
	}
	if($stype == "TOTAL")
	{
		if($sFormat == FORMAT_TIME)
		{
			if($value)
			{
				$s = $value % 60;
				$value -= $s;
				$value /= 60;
				$m = $value % 60;
				$value -= $m;
				$value /= 60;
				$h = $value % 24;
				$value -= $h;
				$value /= 24;
				$d = $value;
				$value = ($d!=0 ? $d.'d ' : ''). mysprintf("%02d:%02d:%02d",array($h,$m,$s));
			}
		}
	}
	$sValue = "";
	$data = array($field => $value);
	if($sFormat == FORMAT_CURRENCY)
	 	$sValue = str_format_currency($value);
	else if($sFormat == FORMAT_PERCENT)
		$sValue = str_format_number($value*100)."%"; 
	else if($sFormat == FORMAT_NUMBER)
 		$sValue = str_format_number($value, $pSet->isDecimalDigits($field));
	else if($sFormat == FORMAT_CUSTOM && $stype!="COUNT")
	{
		include_once getabspath('classes/controls/ViewControlsContainer.php');
		$viewControls = new ViewControlsContainer($pSet, $ptype);			
		$sValue = $viewControls->showDBValue($field, $data);
	}
	else 
 		$sValue = $value;
	
	if($stype == "COUNT") 
		return $value;
	if($stype == "TOTAL") 
		return $sValue;
	if($stype == "AVERAGE") 
		return $sValue;
	return "";
}


/**
 * @intellisense
 */
function DisplayNoImage()
{
	$path = getabspath("images/no_image.gif");
	header("Content-Type: image/gif");
	printfile($path);
}

/**
 * @intellisense
 */
function DisplayFile()
{
	$path = getabspath("images/file.gif");
	header("Content-Type: image/gif");
	printfile($path);
}

////////////////////////////////////////////////////////////////////////////////
// miscellaneous functions
////////////////////////////////////////////////////////////////////////////////

/**
 * analog of strrpos function
 * @intellisense
 */
function my_strrpos($haystack, $needle)
{
	$index = strpos(strrev($haystack), strrev($needle));
	if($index === false)
		return false;
	$index = strlen($haystack) - strlen($needle) - $index;
	return $index;
}

/**
 * prepare string for JavaScript. Replace ' with \' and linebreaks with \r\n
 * @intellisense
 */
function jsreplace($str)
{
	$ret= str_replace(array("\\","'","\r","\n"),array("\\\\","\\'","\\r","\\n"),$str);
	return my_str_ireplace("</script>","</scr'+'ipt>",$ret);
}

/**
 * @intellisense
 */
function LogInfo($SQL)
{/*
	global $dSQL,$dDebug;
	$dSQL=$SQL;
	if($dDebug)
	{
		echo $dSQL;
		echo "<br>";
	}*/
}

/**
 * check if file extension is image extension
 * @intellisense
 */
function CheckImageExtension($filename)
{
	if(strlen($filename)<4)
		return false;
	$ext=strtoupper(substr($filename,strlen($filename)-4));
	if($ext==".GIF" || $ext==".JPG" || $ext=="JPEG" || $ext==".PNG" || $ext==".BMP")
		return $ext;
	return false;
} 

/**
 * @intellisense
 */
function html_special_decode($str)
{
	$ret=$str;
	$ret=str_replace("&gt;",">",$ret);
	$ret=str_replace("&lt;","<",$ret);
	$ret=str_replace("&quot;","\"",$ret);
	$ret=str_replace("&#039;","'",$ret);
	$ret=str_replace("&#39;","'",$ret);
	$ret=str_replace("&amp;","&",$ret);
	return $ret;
}

////////////////////////////////////////////////////////////////////////////////
// database and SQL related functions
////////////////////////////////////////////////////////////////////////////////

/**
 * add clause to WHERE or HAVING expression
 * @intellisense
 */
function whereAdd($where,$clause)
{
	if(!strlen($clause))
		return $where;
	if(!strlen($where))
		return $clause;
	return "(".$where.") and (".$clause.")";
}

/**
 * combine SQL logincal clauses from array using AND (default) or OR
 * @intellisense
 */
function combineSQLCriteria($arrElements, $and = true)
{
	$ret="";
	$union = $and ? " AND " : " OR ";
	foreach($arrElements as $e)
	{
		if(strlen($e))
		{
			if(!strlen($ret))
			{
				$ret = "(".$e.")";
			}
			else
			{
				$ret .= $union."(".$e.")";
			}
		}
	}
	return $ret;
}

/**
 * add WHERE clause to SQL string
 * @intellisense
 */
function AddWhere($sql,$where)
{
	if(!strlen($where))
		return $sql;
	$sql=str_replace(array("\r\n","\n","\t")," ",$sql);
	$tsql = strtolower($sql);
	$n = my_strrpos($tsql," where ");
	$n1 = my_strrpos($tsql," group by ");
	$n2 = my_strrpos($tsql," order by ");
	if($n1===false)
		$n1=strlen($tsql);
	if($n2===false)
		$n2=strlen($tsql);
	if ($n1>$n2)
		$n1=$n2;
	if($n===false)
		return substr($sql,0,$n1)." where ".$where.substr($sql,$n1);
	else
		return substr($sql,0,$n+strlen(" where "))."(".substr($sql,$n+strlen(" where "),$n1-$n-strlen(" where ")).") and (".$where.")".substr($sql,$n1);
}

/**
 *  DEPRECATED. Use RunnerPage::keysSQLExpression instead
 *	Construct WHERE clause with key values
 * 
 * @param &Array	$keys
 * @param String	$table Teh data source table name OPTIONAL
 * 
 * @return String
 * @intellisense
 * @deprecated
 */
function KeyWhere(&$keys, $table = "")
{
	global $strTableName, $cman;
	
	if( !$table )
		$table = $strTableName;
	$strWhere="";
	
	$pSet = new ProjectSettings($table);
	$cipherer = new RunnerCipherer($table);
	$connection = $cman->byTable( $table );
	
	$keyFields = $pSet->getTableKeys();
	foreach($keyFields as $kf)
	{
		if( strlen($strWhere) )
			$strWhere.= " and ";
			
		$value = $cipherer->MakeDBValue($kf, $keys[ $kf ], "", true);
		
		if( $connection->dbType == nDATABASE_Oracle )
			$valueisnull = $value === "null" || $value == "''";
		else
			$valueisnull = $value === "null";
		
		if( $valueisnull )
			$strWhere.= RunnerPage::_getFieldSQL($kf, $connection, $pSet)." is null";
		else
			$strWhere.= RunnerPage::_getFieldSQLDecrypt($kf, $connection, $pSet, $cipherer)."=".$cipherer->MakeDBValue($kf, $keys[ $kf ], "", true);
	}
	return $strWhere;
}

/**
 * Get count of rows from the query
 * @param String strSQL
 * @param Connection connection
 * @return Number
 * @intellisense
 */
function GetRowCount($strSQL, $connection)
{
	$strSQL = str_replace(array("\r\n","\n","\t")," ",$strSQL);
	$tstr = strtoupper($strSQL);
	$ind1 = strpos($tstr, "SELECT ");
	$ind2 = my_strrpos($tstr," FROM ");
	$ind3 = my_strrpos($tstr," GROUP BY ");
	if($ind3 === false)
	{
		$ind3 = strpos($tstr," ORDER BY ");
		if($ind3 === false)
			$ind3 = strlen($strSQL);
	}
	
	$countstr = substr($strSQL, 0, $ind1 + 6)." count(*) ".substr($strSQL, $ind2 + 1, $ind3 -$ind2);
	$countdata = $connection->query( $countstr )->fetchNumeric();
	return $countdata[0];
}

/**
 * 	add MSSQL Server TOP clause
 * @intellisense
 */
function AddTop($strSQL, $n)
{
	$tstr = strtoupper($strSQL);
	$ind1 = strpos($tstr,"SELECT");
	return substr($strSQL,0,$ind1+6)." top ".$n." ".substr($strSQL,$ind1+6);
}
/**
 * 	add DB2 Server TOP clause
 * @intellisense
 */
function AddTopDB2($strSQL, $n)
{	
	return $strSQL." fetch first ".$n." rows only";
}
/**
 * @intellisense
 */
function AddTopIfx($strSQL,$n)
{
	return substr($strSQL,0,7)."limit ".$n." ".substr($strSQL,7);
}
/**
 * 	add Oracle ROWNUMBER checking
 * @intellisense
 */
function AddRowNumber($strSQL, $n)
{
	return "select * from (".$strSQL.") where rownum<".($n+1);
}

/**
 * Apply a limit to an SQL-query
 * @param String sql
 * @param Number N
 * @param Number dbType 
 * @return String
 */
function applyDBrecordLimit($sql, $N, $dbType) 
{
	if( !strlen($dbType) )
		return $sql;
		
	if( $dbType == nDATABASE_MySQL || $dbType == nDATABASE_PostgreSQL || $dbType == nDATABASE_SQLite3 )
		return $sql." LIMIT ".$N;
	
	if( $dbType == nDATABASE_Oracle )
		return AddRowNumber($sql, $N);
	
	if( $dbType == nDATABASE_MSSQLServer || $dbType == nDATABASE_Access )	
		return AddTop($sql, $N);
	
	if( $dbType == nDATABASE_Informix )
		return AddTopIfx($sql, $N);
	
	if( $dbType == nDATABASE_DB2 )
		return AddTopDB2($sql, $N);

	return $sql;
}

/**
 *  test database type if values need to be quoted
 * @intellisense
 */
function NeedQuotesNumeric($type)
{
    if($type == 203 || $type == 8 || $type == 129 || $type == 130 || 
		$type == 7 || $type == 133 || $type == 134 || $type == 135 ||
		$type == 201 || $type == 205 || $type == 200 || $type == 202 || $type==72 || $type==13)
		return true;
	else
		return false;
}

//	using ADO DataTypeEnum constants
//	the full list available at:
//	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/ado270/htm/mdcstdatatypeenum.asp

/**
 * @intellisense
 */
function IsNumberType($type)
{
	if($type==20 || $type==14 || $type==5 || $type==10 || $type==6 
	|| $type==3 || $type==131 || $type==4 || $type==2 || $type==16
	|| $type==21 || $type==19 || $type==18 || $type==17 || $type==139
	|| $type==11)
		return true;
	return false;
}

/**
 * @intellisense
 */
function IsFloatType($type)
{
	if($type==14 || $type==5 || $type==131 || $type==4 || $type==6)
		return true;
	return false;
}


/**
 * @intellisense
 */
function NeedQuotes($type)
{
	return !IsNumberType($type);
}

/**
 * @intellisense
 */
function IsBinaryType($type)
{
	if($type==128 || $type==205 || $type==204)
		return true;
	return false;
}

/**
 * date and datetime
 * @intellisense
 */
function IsDateFieldType($type)
{
	if($type==7 || $type==133 || $type==135)
		return true;
	return false;
}

/**
 * only datetime
 * @intellisense
 */
function IsDateTimeFieldType($type)
{
	if($type==135)
		return true;
	return false;
}

/**
 * @intellisense
 */
function IsTimeType($type)
{
	if($type==134)
		return true;
	return false;
}

/**
 * @intellisense
 */
function IsCharType($type)	
{
	if(IsTextType($type) || $type==8 || $type==129 || $type==200 || $type==202 ||  $type==130)
		return true;
	return false;
}

/**
 * @intellisense
 */
function IsTextType($type)
{
	if($type==201 || $type==203)
		return true;
	return false;
}

/**
 * @intellisense
 */
function IsGuid($type)
{
	if($type==72)
		return true;
	return false;
}

function IsBigInt($type)
{
	if($type==20)
		return true;
	return false;
}

////////////////////////////////////////////////////////////////////////////////
// security functions
////////////////////////////////////////////////////////////////////////////////
/**
 * @param String userID
 * @intellisense
 */
function ReadUserPermissions($userID = "")
{
	global $gPermissionsRead, $gPermissionsRefreshTime, $caseInsensitiveUsername, $cman;
	
	if (!strlen($userID))
		$userID = $_SESSION["UserID"];
		
	$needreload = false;
	if( !isset( $_SESSION["UserRights"] ) )
		$needreload = true;
	elseif( !isset( $_SESSION["UserRights"][ $userID ] ) )
		$needreload = true;
		
	if(!$needreload && ($gPermissionsRead || time()-@$_SESSION["LastReadRights"]<=$gPermissionsRefreshTime))
		return;
		
	$groups = array();
	$bIsAdmin = false;
	if($userID != "Guest")
	{
		$grConnection = $cman->getForUserGroups();
		
		if(!$caseInsensitiveUsername)
		{
			$sql = "select ".$grConnection->addFieldWrappers( "GroupID" )
				.", ".$grConnection->addFieldWrappers( "UserName" )
				." from ". $grConnection->addTableWrappers( "8boxerpugmembers" )
				." where ".$grConnection->addFieldWrappers( "UserName" )."=".$grConnection->prepareString($userID);
		}
		else
		{
			$sql = "select ".$grConnection->addFieldWrappers( "GroupID" )
				.", ".$grConnection->addFieldWrappers( "UserName" )
				." from ". $grConnection->addTableWrappers( "8boxerpugmembers" )
				." where ".$grConnection->addFieldWrappers( "UserName" )."=".strtoupper( $grConnection->prepareString($userID) );
		}

		$qResult = $grConnection->query( $sql );
		while( $data = $qResult->fetchNumeric() )
		{
			if ($caseInsensitiveUsername || strcmp($data[1],$userID) == 0)
				$groups[] = $data[0];
		}
		
		if( !count($groups) )
			$groups[] = -2;
	}
	else
		$groups[] = -3;
	
	$rConnection = $cman->getForUserGroups();
	
	$groupstr = "";
	foreach($groups as $g)
	{
		if($groupstr != "")
			$groupstr.= ",";
		$groupstr.= $g;
		if($g == -1)
			$bIsAdmin = true;
	}
	$rights = array();
	
	$sql = "select ". $rConnection->addFieldWrappers( "TableName" )
		.", ". $rConnection->addFieldWrappers( "AccessMask" )
		." from ". $rConnection->addTableWrappers( "8boxerpugrights" )
		." where ". $rConnection->addFieldWrappers( "GroupID" ) ." in (".$groupstr.")";
	
	$qResult = $rConnection->query( $sql );	
	while( $data = $qResult->fetchNumeric() )
	{
		if(!array_key_exists($data[0], $rights))
		{
			$rights[ $data[0] ] = $data[1];
			continue;
		}
		for($i = 0; $i < strlen($data[1]); $i++)
		{
			if( strpos($rights[ $data[0] ], substr($data[1], $i, 1)) === false )
				$rights[ $data[0] ].= substr($data[1], $i, 1);
		}
	} 
	
	if(!array_key_exists("UserRights", $_SESSION))
		$_SESSION["UserRights"] = array();
		
	if($bIsAdmin)
		$rights[".IsAdmin"] = true;
	$rights[".Groups"] = $groups;
	$_SESSION["UserRights"][ $userID ] = &$rights;
	$_SESSION["LastReadRights"] = time();
	
	$gPermissionsRead = true;
}

/**
 * @intellisense
 */
function guestHasPermissions() 
{	
	ReadUserPermissions("Guest");
	if(!count($_SESSION["UserRights"]["Guest"]))
		return false;
	if(array_key_exists("business_partner",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_terms",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("shipping_terms",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_receipt",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_disbursment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("general_journal",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("report_center",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Disbursement Reports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Financial Reports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase Reports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Receivable Reports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Taxation Reports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("system_user",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fiscal_year",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("beg_bal",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("ewt",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bank",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cost_center",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("General Journal Reports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory Reports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("accounting_period",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bank_recon",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("deposit",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("account_classification",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("chart_of_accounts",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("recon_CR",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Recon_deposit",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Recon_CV",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Recon_GV_dep",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Recon_GV_CV",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("vat_atc",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("rdo",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("2550q",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("2550m",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("dr",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_order",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_order_receiving",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("System_preference",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("ppe_list",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("discounts",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("item_group",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("location",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("meetings",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("calls",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("campaign",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("reimbursement",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("reimbursement_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Target_customers",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quotation",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("case",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("projects",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("issue",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("issue_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("disposed",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("disposed_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("backload",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("backload_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("vehicle",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("payroll_item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("uom",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("trip_ticket",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("accident",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("approver_list",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("area",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("attendance",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("currency",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("medical",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("position",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("service_center",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("seminar",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("service_report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("service_report_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("tax_status",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("training",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("competitor",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("campaign_type",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fixed_asset_return_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fixed_assets_return",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Prospect_customers",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("trip_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("beg_baldetail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_disbursement_detail2",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("case_solutions",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_disbursment_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_receipt_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_receipt_detail2",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("general_journal_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("employee_list",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("petty_cash",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("petty_cash_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("center_replenishment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("center_replenishment_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_memo",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_memo_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("debit_memo",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("debit_memo_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("project_replenishment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("project_replenishment_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("business_partner_group",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Representatives",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Types_of_leaves",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("payroll_mode",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("payroll_account_type",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Types_of_Overtime",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Types_of_Loan",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Payroll_Company_Setup",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("item_bill_of_materials",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("item_uom_conversion",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("tranasactions_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("deposit_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("dr_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_order_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Order_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_order_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Order_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_order_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Quotation_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Quotation_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quotation_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quotation_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Reimbursement_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("reimbursement_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Reimbursement_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("reimbursement_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project_Replenishment_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("project_replenishment_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project_Replenishment_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("project_replenishment_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("center_replenishment_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("center_replenishment_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchases_Order_for_checking2",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_order_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchases_Order_for_approval2",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_order_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Receiving_for_checking2",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_order_receiving_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Receiving_for_approval2",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_order_receiving_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Delivery_Receipt_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("dr_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Delivery_Receipt_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("dr_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Issue_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("supply_issue_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Issue_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("supply_issue_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Return_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("supply_return_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Return_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("supply_return_detail_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("supply_disposed_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Disposal_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("supply_disposed_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Fixed_Asset_Issue_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Fixed_Asset_Issue_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fixed_asset_issue_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fixed_asset_issue_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Fixed_Asset_Return_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fixed_asset_return_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Fixed_Asset_Return_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fixed_asset_return_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Petty_Cash_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("petty_cash_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Petty_Cash_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("petty_cash_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Deposit_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("deposit_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Deposit_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("deposit_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Credit_Memo_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_memo_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Credit_Memo_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_memo_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Debit_Memo_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("debit_memo_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("debit_memo_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("debit_memo_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchases_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_purchase_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchases_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_purchase_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_sales_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_sales_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Cash_Disbursement_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("check_voucher_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("check_voucher_detail2_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Cash_Disbursement_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("check_voucher_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("check_voucher_detail2_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("document",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("General_Journal_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("general_journal_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("General_Journal_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("general_journal_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("for_submissions",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("submission_details",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_order_detail_billing_dates",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Submission_Search",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Submission_Reminders",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("years",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("document_type",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("estimate",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("estimate_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_request",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_request_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inv_adj_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inventory_adj",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inventory_tranfer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inventory_transfer_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Customers",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Suppliers",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Target_contacts",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Prospect_contacts",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Customer_Contacts",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Supplier_Contacts",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Statement_of_Account",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Change_Time_Schedule_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Change_Time_Schedule_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory_Adjustment_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inv_adj_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory_Adjustment_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inv_adj_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Estimate_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Estimate_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("estimate_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("estimate_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Follow_up",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("coming_meetings",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("tasks",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project_phases",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("origin_fa",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("origin_serialized_supply",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("2301",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Prospect_customers_to_Customer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_accessories",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_aluminum",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_e_accessories",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_glass",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("recuring_benefits",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("recuring_deductions",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("recurring_benefits_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("recurring_deductions_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_request",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_request_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("attendance_summary",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Other_Incentives",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Other_Deductions",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("department",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_cash_disbursement",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_cash_receipt",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_center_replenishment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_credit_memo",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_debit_memo",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_depreciation",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_general_journal",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_inventory_adjustment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_petty_cash",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_project_replenishment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_purchases",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_replenishment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_service_revenue",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Credit_Card_monitoring",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_report_per_item Chart",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Leave_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("loan_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("loan_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Overtime_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Request_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_request_detail_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_Request_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_request_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory_Transfer_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inventory_transfer_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Leave_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Loan_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("loan_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Overtime_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Request_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_Request_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory_Transfer_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inventory_transfer_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("attendance_type",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Backload_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("backload_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Backload_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("backload_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Cash_Receipt_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Cash_Receipt_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_receipt_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_receipt_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Payroll_Entry_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Payroll_Entry_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("payroll_entry_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("payroll_entry_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Process_Payroll_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Process_Payroll_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("process_payroll_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("process_payroll_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Service_Report_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Service_Report_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("service_report_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("service_report_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Banch Reports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_production",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_deposit",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_request_bom",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_bom",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("company_info",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("return",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("return_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fixed_asset_issue",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fixed_asset_issue_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_request_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_disbursment_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_disbursment_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_request_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("trip_ticket_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("trip_ticket_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("trip_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("trip_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Customer_Balance_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Customer_Balance_Report_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Collection_Report_per_Customer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Aging_of_AR_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Aging_of_AR_Report_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Payment_per_Supplier_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Aging of_AP_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Aging of_AP_Report_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory_Balance_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory_Valuation_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory_Below_Reorder_Point_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Report_per_BP",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Report_per_Item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Report_per_Cost_Center",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Report_per_BP_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Report_per_Item_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Report_per_Cost_Center_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Report_per_BP",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Report_per_Item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Report_per_Cost_Center",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Report_per_BP_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Report_per_Item_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Report_per_Cost_Center_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Transaction Registry",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Report_per_sales_rep",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("PDC_Registry_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Available_Vehicles",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Trip_Ticket_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Request_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Import_Costing",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_Costing_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Closed_Opportunities",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Cases_with_Sotutions",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Totals_Cases_Created",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Totals_Open_Cases_by_Agent",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Campaign_Contacts",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Campaign_leads",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Campaign_ROI_Analysis",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Campaign_Revenue",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Opportunity_Schedule",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_by_Account",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_by_Lead_Source",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_by_Partner",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_by_Rep",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Leave_Ledger_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Leave_Summary",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Payroll_Register",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Employee_Leave_History",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Employee_Register",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("DR_Reports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Order_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Case_History",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quality_control",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quality_control_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("budget",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("budget_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("repairs_and_maintenance",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_form",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bluewave_employees",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bluewave_employees_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bluewave_employees_detail2",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bluewave_employees_detail3",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bluewave_employees_detail4",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bluewave_employees_detail5",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bluewave_employees_detail6",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("rfp",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("telex",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("repairs_and_maintenance_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("repair_list",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("marketing_colateral",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("email",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("accessories",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_order_materials_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_order_progress_billing_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sub_contract",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sub_contract_billing_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sub_contract_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Monthly_Pag-ibig_Summary_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Monthly_Philhealth_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Monthly_Philhealth_Summary_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Monthly_SSS_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Monthly_SSS_Report_by_Department_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Monthly_Withholding_Tax_Prooflist_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Monthly_Witholding_Tax_Collection_List_by_Department_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Overtime_Summary_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Monthly_Pag-ibig_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_order",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_order_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("vat_relief_importation",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("vat_relief_purchases",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("vat_relief_sales",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_ppe_disposal",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_memo_detail_cgs",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("marketing_request_form",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Quotation_Dashboard_Customer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Order_Dashboard_Customer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Delivery_Receipt_Dashboard_Customer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Dashboard_Customer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Backload_Dashboard_Customer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Request_Dashboard_Supplier",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Order_Dashboard_Supplier",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Dashboard_Supplier",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Cash_Disbursement_Dashboard_Supplier",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Debit_Memo_Dashboard_Supplier",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Request_Dashboard_Item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Order_Dashboard_Item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Order_Receiving_Dashboard_Item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_Dashboard_Item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sub_contract_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sub_contract_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sub_contract_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sub_contract_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("budget_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("budget_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("budget_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("budget_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("undertime_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("undertime_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_payment_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_payment_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("repairs_and_maintenance_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_payment_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_payment_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("repairs_and_maintenance_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("repairs_and_maintenance_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("repairs_and_maintenance_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Disposal_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Active_campaign_report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("For_Billing_report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_order_for_project_mngmt",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inventory_transfer_cost_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Supplier_Balance_Report_Details",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Supplier_Balance_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory_per_Cost_center_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Deposit_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("SO_for_Billing_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Billing_due_dates_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Packing_List_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Open_Trip_Ticket_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Transfer_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_Report_invw",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Supply_per_Department__Report_invw",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Item_Balances_Report_invw",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Open_Case_Report_s",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Order_per_Campaign_Report_marketing",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_per_campaign_Report_marketing",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Asset_per_Accountability_Report_invw",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Deposit_in_transit_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Outstanding_Checks_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Absences_Detail_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Absences_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Attendance_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("DTR_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Summary_of_Tardiness_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Followup_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Coming_Meeting_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Meeting_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Next_Action_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Calls_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_per_sales_rep_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Expense_per_Project_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Tardiness_Report_Summary",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Timekeeping_(Hours_Worked)_Report_Summary",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Withholding_Tax_Collection_List_per_Month_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Business_Owner_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Finance_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchasing_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Marketing_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Support_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Human_Resource_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project_Manager_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("System_Admistration_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_report_per_item Chart",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("System_Audit_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sale_cost_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("currency_exchange_rate",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_receipt_cost_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_cost_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Employee",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Quotation_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Order_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Reimbursement_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Transaction_Registry_Dashboard_SaleRep",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Loan_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Leave_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Over_Time_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Undertime_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Fieldwork_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Change_Time_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Transaction_Registry_Dashboard_Employee",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Transaction_Registry_Dashboard_Supplier",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Transaction_Registry_Dashboard_Customer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_accessories",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_aluminum",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_e_accessories",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_glass",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sum_dr_qty",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("check_book_registry",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("check_book_registry_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("vat_type",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quotation_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_purchase_order_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_sales_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("pending_items_reminders",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("petty_cash_for_liquidation",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("payment_method",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("account_type",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("phase",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("item_cost_recalculation",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("issue_components",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("issue_components_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("unbuild_assemblies",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("unbuild_assemblies_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("demand_plan",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("input_vat_allocation",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("case_type",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("qc_inspection",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("packing_list_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("packing_list_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("packing_list_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("packing_list_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("disposed_inv_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("disposed_inv",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("disposed_inv_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("disposed_inv_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("disposed_inv_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("disposed_inv_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("For_Approval_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("For_Checking_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Customer_For_Approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_payment_report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("credit_memo_report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("debit_memo_report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_request_report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Supplies_Balance_report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("approved_sales_order",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_order_for_receiving",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("packing_list",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("packing_list_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_quote",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_estimates",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_po",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_pr",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_production_request",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_qc",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_receiving",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_request_for_payment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_so",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_so_for_invoice",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_so_for_project_management",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_backload",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("purchase_order_receiving_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_Order_Receiving_Dashboard_Supplier",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("avg_inv",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_backload_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_estimates_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_po_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_pr_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_production_request_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_qc_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_so_for_project_management_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_so_for_invoice_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_so_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_receiving_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("monthly_consuption_for_production_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("issue_components_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("issue_components_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("issue_components_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("issue_components_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fieldwork_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("fieldwork_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchases_Order_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchases_Order_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Receiving_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Receiving_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inventory_tranfer_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_vs_Production_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("cash_sales_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Center_Replenishment_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Center_Replenishment_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchase_vs_Production",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_Summary",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("pending_production_request",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("unbuild_assemblies_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("unbuild_assemblies_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("unbuild_assemblies_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quality_control_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quality_control_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quality_control_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("raw_cake_balance",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_cashreceipt",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_cashpayment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("opportunity",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Current_Accounts",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_lyl_acc",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_lyl_glass",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_lyl_eacc",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_lyl_allum",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("opportunity_details",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Collection_Report_per_Sales_Rep",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Inventory_per_lot_no",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("marketing_request_form_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("marketing_request_form_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Accounts_Payable_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("201",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Cashflow_forcast",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Business_Owner_Sales_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Business_Owner_Purchase_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Business_Owner_Cash_Balance",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Business_Owner_Top_Customer_Income",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("AR_Billing_due_dates_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("AR_Aging_of_AR_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("AR_Top_5_Customer_balances",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("AR_Progress_Billing_Alert",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Account_Receivable_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Finance_Cash_Balance",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Finance_approved_request_for_payment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Finance_Cash_flow_forcast",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Finance_Approved_Employee_Loan",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Finance_Cash_Balances",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("AP_Aging_of_Payables",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Purchasing_Item_below_reorder",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_per_item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("AP_Top_5_Supplier_balances",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Warehouse_Item_balances",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("DR_per_Item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Logistic_Open_Trip_Ticket",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Logistic_Available_Vehicles",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Marketing_Open_Campaign",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Marketing_Sales_per_campaign",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Support_Open_Case_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Support_Service_center",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sale_Meeting_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Next_Action_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("HR_attendance_summary",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("HR_Birthday_Alert",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("HR_holiday",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project_Expense_per_Project",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project_Expense_per_Project_graph",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Petty_Cash_Balance",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Petty_Cash_Unliquidated",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Petty_Cash_Purchases_per_Item",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Petty_Cash_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Accounting_Income_per_Center",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("open_quote_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Open_cases_per_agent",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Marketing_sales_per_campaing_chart",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Logistics_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Accounting_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("AP_open_po_receiving",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_Inventory_Balance_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Production_WIP",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Warehouse_Dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Warehouse_Inventory_Balance_Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("logistic_open_inventory_tranfer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Leads",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_opportunity_Next_Action",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Sales_Active_opportunity",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("intercompany_sales_order_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("intercompany_sales_order",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("production_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("quality_control_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_accessories_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_aluminum_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_e_accessories_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_glass_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_accessories_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_aluminum_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_e_accessories_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_glass_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_accessories_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_aluminum_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_e_accessories_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_detail_glass_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_accessories_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_aluminum_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_e_accessories_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lyl_estimate_dr_detail_glass_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("EWT_Certificates_summay",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("1601e",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("1601eform",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("1601fform",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("tax_person_status",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("journal_entry_inv_transfer",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Recon_withdraw",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("lot",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("serial_no",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_quote",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_quoter_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_quote_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_quote_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_quoter_detail_for_checking",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_quoter_detail_for_approval",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("estimate_cost",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("airlines",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("airport",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("inco_terms",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("seaports",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("shipment_classification",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("shipping_lines",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_sales_order_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("project_cont_seal",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("project_house",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("billable_liquidation",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("billable_pettycash",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("billable_checkvoucher",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("projects_dashboard",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("employees",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("admin_rights",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("admin_members",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("admin_users",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("company",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("notifiy quotation Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("expenses Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("petty cash release",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("petty cash replenishment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Employee Advance Release",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Employee Advance Replenishment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Liquidation Release",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Liquidation Replenishment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Billable Cost Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("centerrep_pettycash",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("centerrep_empadvance",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("centerrep_liquidation",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("replenishment_pettycash",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("replenishment_liquidation",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("replenishment_empadv",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bank_statement",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("other_charges",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("change_password",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("project_liquidation_detail2",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("balance_advance",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("project_replenishment_deposited",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("bal_center_replenishment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project and Pettycash Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project and EmpAdv Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project and Liquidation Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project and Sales Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("Project and Voucher Report",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("projects_view",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("projects_lite",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("shipping_date",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("entry_detail_sales",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("entry_detail_pettycash",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("entry_detail_cash_disbursement",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("representative",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_sub_invoice",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("search_cash_disbursement",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("sales_business_partner",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("canvas",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("canvas_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_payment",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_payment_detail",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("po_netpr",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("request_for_payment_imp",$_SESSION["UserRights"]["Guest"]))
		return true;
	if(array_key_exists("birsalesrelief",$_SESSION["UserRights"]["Guest"]))
		return true;
	return false;
}

/**
 * @intellisense
 */
function GetUserPermissionsDynamic($table="")
{
	if( !isLogged() ) 
		return "";
	global $strTableName,$gPermissionsRefreshTime,$gPermissionsRead;
	if(!$table)
		$table=$strTableName;
		
	ReadUserPermissions();
	if(IsAdmin())
	{
		if($table=="admin_rights")
			return "ADESPIM";
		if($table=="admin_members")
			return "ADESPIM";
		if($table=="admin_users")
			return "ADESPIM";
	}

	return @$_SESSION["UserRights"][$_SESSION["UserID"]][$table];
}

// end of the bCreateLoginPage block


/**
 * Check if the current user is admin user
 * @return Boolean
 * @intellisense
 */
function IsAdmin()
{
	global $gPermissionsRefreshTime, $gPermissionsRead;
	ReadUserPermissions();
	return array_key_exists(".IsAdmin", @$_SESSION["UserRights"][ $_SESSION["UserID"] ]);		
}

/**
 * Returns user permissions for a particular table
 *	A - Add
 *	D - Delete
 *	E - Edit
 *	S - List/View/Search
 *	P - Print/Expor
 * @param String table
 * @return String
 * @intellisense
 */
function GetUserPermissions($table="")
{
	global $strTableName, $globalEvents;
	if(!$table)
		$table = $strTableName;
	$permissions = "";
	if( !IsLogged() )
		return "";
		$permissions =  GetUserPermissionsDynamic($table);
	
	if($globalEvents->exists("GetTablePermissions", $table))
	{
		$permissions = $globalEvents->GetTablePermissions($permissions, $table);
	}
	return $permissions;
}

/**
 * @intellisense
 */
function isLogged()
{
	if (@$_SESSION["UserID"])
		return true;
	return false;
}


/**
 * Set session variables and permissions after login via Facebook
 * @intellisense
 */
function AfterFBLogIn($pUsername, $pPassword, &$pageObject = null)
{
	global $cman, $cUserNameFieldType, $cPasswordFieldType, $cUserNameField, $cDisplayNameField, $globalEvents;
	
	$logged = false;
	$strUsername = (string)$pUsername;
	$sUsername = $strUsername;
	$connection = $cman->getForLogin();
	
	if(NeedQuotes($cUserNameFieldType))
		$strUsername = $connection->prepareString($strUsername);
	else
		$strUsername = (0+$strUsername);
	
	$strSQL = "select * from ".$connection->addTableWrappers("system_user")
		." where ".$connection->addFieldWrappers($cUserNameField)."=".$strUsername."";

 	$data = $connection->query( $strSQL )->fetchAssoc();
	if( count($data) )
	{
		$logged = true;
		$pDisplayUsername = $data[ $cDisplayNameField ] != '' ? $data[ $cDisplayNameField ] : $sUsername;
		DoLogin(false, $pUsername, $pDisplayUsername, "", ACCESS_LEVEL_USER, $pPassword, $pageObject);
		SetAuthSessionData($pUsername, $data, true, $pPassword, $pageObject);
	}
}

/**
 * SetAuthSessionData
 * Add to session auth data and permissions
 * @param {string} user identifier
 * @param {string} user display name
 * @param {object} fetched row from DB with user data
 * @param {object} page object
 * @intellisense
 */
function SetAuthSessionData($pUsername, &$data, $fromFacebook, $password, &$pageObject = null)
{
	global $globalEvents;
	$_SESSION["GroupID"] = $data["role"];


		$_SESSION["OwnerID"] = $data["cost_center"];
	$_SESSION["_cash_receipt_OwnerID"] = $data["cost_center"];
		$_SESSION["_purchase_OwnerID"] = $data["cost_center"];
		$_SESSION["_cash_disbursment_OwnerID"] = $data["cost_center"];
		$_SESSION["_Sales Report_OwnerID"] = $data["cost_center"];
		$_SESSION["_Disbursement Reports_OwnerID"] = $data["service_center"];
		$_SESSION["_Financial Reports_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase Reports_OwnerID"] = $data["service_center"];
		$_SESSION["_Receivable Reports_OwnerID"] = $data["service_center"];
		$_SESSION["_Taxation Reports_OwnerID"] = $data["cost_center"];
		$_SESSION["_General Journal Reports_OwnerID"] = $data["service_center"];
		$_SESSION["_Inventory Reports_OwnerID"] = $data["service_center"];
		$_SESSION["_deposit_OwnerID"] = $data["cost_center"];
		$_SESSION["_chart_of_accounts_OwnerID"] = $data["cost_center"];
		$_SESSION["_dr_OwnerID"] = $data["cost_center"];
		$_SESSION["_purchase_order_OwnerID"] = $data["cost_center"];
		$_SESSION["_purchase_order_receiving_OwnerID"] = $data["cost_center"];
		$_SESSION["_ppe_list_OwnerID"] = $data["cost_center"];
		$_SESSION["_meetings_OwnerID"] = $data["employee_name"];
		$_SESSION["_calls_OwnerID"] = $data["employee_name"];
		$_SESSION["_reimbursement_OwnerID"] = $data["employee_name"];
		$_SESSION["_quotation_OwnerID"] = $data["employee_name"];
		$_SESSION["_case_OwnerID"] = $data["service_center"];
		$_SESSION["_production_OwnerID"] = $data["cost_center"];
		$_SESSION["_issue_OwnerID"] = $data["cost_center"];
		$_SESSION["_disposed_OwnerID"] = $data["cost_center"];
		$_SESSION["_backload_OwnerID"] = $data["cost_center"];
		$_SESSION["_service_report_OwnerID"] = $data["service_center"];
		$_SESSION["_service_report_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_fixed_assets_return_OwnerID"] = $data["cost_center"];
		$_SESSION["_center_replenishment_OwnerID"] = $data["cost_center"];
		$_SESSION["_credit_memo_OwnerID"] = $data["cost_center"];
		$_SESSION["_debit_memo_OwnerID"] = $data["cost_center"];
		$_SESSION["_project_replenishment_OwnerID"] = $data["employee_name"];
		$_SESSION["_sales_OwnerID"] = $data["cost_center"];
		$_SESSION["_estimate_OwnerID"] = $data["cost_center"];
		$_SESSION["_purchase_request_OwnerID"] = $data["cost_center"];
		$_SESSION["_inventory_adj_OwnerID"] = $data["cost_center"];
		$_SESSION["_inventory_tranfer_OwnerID"] = $data["cost_center"];
		$_SESSION["_Customers_OwnerID"] = $data["cost_center"];
		$_SESSION["_Suppliers_OwnerID"] = $data["cost_center"];
		$_SESSION["_for_checking_OwnerID"] = $data["id"];
		$_SESSION["_for_approval_OwnerID"] = $data["id"];
		$_SESSION["_2301_OwnerID"] = $data["cost_center"];
		$_SESSION["_production_request_OwnerID"] = $data["cost_center"];
		$_SESSION["_department_OwnerID"] = $data["department"];
		$_SESSION["_sales_report_per_item Chart_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase_Request_for_approval_OwnerID"] = $data["cost_center"];
		$_SESSION["_Backload_for_approval_OwnerID"] = $data["cost_center"];
		$_SESSION["_Service_Report_for_approval_OwnerID"] = $data["role"];
		$_SESSION["_Service_Report_for_checking_OwnerID"] = $data["service_center"];
		$_SESSION["_service_report_detail_for_approval_OwnerID"] = $data["service_center"];
		$_SESSION["_service_report_detail_for_checking_OwnerID"] = $data["service_center"];
		$_SESSION["_Banch Reports_OwnerID"] = $data["service_center"];
		$_SESSION["_return_OwnerID"] = $data["cost_center"];
		$_SESSION["_fixed_asset_issue_OwnerID"] = $data["cost_center"];
		$_SESSION["_sales_report_OwnerID"] = $data["service_center"];
		$_SESSION["_Customer_Balance_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Customer_Balance_Report_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_Collection_Report_per_Customer_OwnerID"] = $data["service_center"];
		$_SESSION["_Aging_of_AR_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Aging_of_AR_Report_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_Payment_per_Supplier_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Aging of_AP_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Aging of_AP_Report_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_Inventory_Balance_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Inventory_Valuation_Report_OwnerID"] = $data["cost_center"];
		$_SESSION["_Inventory_Below_Reorder_Point_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Sales_Report_per_BP_OwnerID"] = $data["service_center"];
		$_SESSION["_Sales_Report_per_Item_OwnerID"] = $data["service_center"];
		$_SESSION["_Sales_Report_per_Cost_Center_OwnerID"] = $data["service_center"];
		$_SESSION["_Sales_Report_per_BP_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_Sales_Report_per_Item_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_Sales_Report_per_Cost_Center_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase_Report_per_BP_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase_Report_per_Item_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase_Report_per_Cost_Center_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase_Report_per_BP_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase_Report_per_Item_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase_Report_per_Cost_Center_detail_OwnerID"] = $data["service_center"];
		$_SESSION["_Transaction Registry_OwnerID"] = $data["cost_center"];
		$_SESSION["_Sales_Report_per_sales_rep_OwnerID"] = $data["service_center"];
		$_SESSION["_PDC_Registry_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Trip_Ticket_Report_OwnerID"] = $data["cost_center"];
		$_SESSION["_Purchase_Request_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Purchase_Import_Costing_OwnerID"] = $data["cost_center"];
		$_SESSION["_Production_Costing_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Production_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Closed_Opportunities_OwnerID"] = $data["cost_center"];
		$_SESSION["_Sales_by_Account_OwnerID"] = $data["cost_center"];
		$_SESSION["_Sales_by_Lead_Source_OwnerID"] = $data["cost_center"];
		$_SESSION["_Sales_by_Partner_OwnerID"] = $data["cost_center"];
		$_SESSION["_Sales_by_Rep_OwnerID"] = $data["cost_center"];
		$_SESSION["_Leave_Ledger_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_DR_Reports_OwnerID"] = $data["service_center"];
		$_SESSION["_Sales_Order_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_email_OwnerID"] = $data["email_address"];
		$_SESSION["_sub_contract_OwnerID"] = $data["cost_center"];
		$_SESSION["_Monthly_Pag-ibig_Summary_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Monthly_Philhealth_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Monthly_Philhealth_Summary_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Monthly_SSS_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Monthly_SSS_Report_by_Department_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Monthly_Withholding_Tax_Prooflist_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Monthly_Witholding_Tax_Collection_List_by_Department_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Overtime_Summary_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Monthly_Pag-ibig_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_sales_order_OwnerID"] = $data["cost_center"];
		$_SESSION["_Production_Dashboard_Item_OwnerID"] = $data["cost_center"];
		$_SESSION["_Active_campaign_report_OwnerID"] = $data["service_center"];
		$_SESSION["_For_Billing_report_OwnerID"] = $data["service_center"];
		$_SESSION["_sales_order_for_project_mngmt_OwnerID"] = $data["cost_center"];
		$_SESSION["_Supplier_Balance_Report_Details_OwnerID"] = $data["service_center"];
		$_SESSION["_Supplier_Balance_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Inventory_per_Cost_center_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Deposit_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_SO_for_Billing_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Billing_due_dates_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Packing_List_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Open_Trip_Ticket_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Production_Report_invw_OwnerID"] = $data["service_center"];
		$_SESSION["_Item_Balances_Report_invw_OwnerID"] = $data["service_center"];
		$_SESSION["_Open_Case_Report_s_OwnerID"] = $data["service_center"];
		$_SESSION["_Sales_Order_per_Campaign_Report_marketing_OwnerID"] = $data["service_center"];
		$_SESSION["_Sales_per_campaign_Report_marketing_OwnerID"] = $data["service_center"];
		$_SESSION["_Asset_per_Accountability_Report_invw_OwnerID"] = $data["service_center"];
		$_SESSION["_Deposit_in_transit_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Outstanding_Checks_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Absences_Detail_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Absences_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Attendance_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_DTR_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Summary_of_Tardiness_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Followup_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Coming_Meeting_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Meeting_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Calls_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_sales_per_sales_rep_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Expense_per_Project_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Tardiness_Report_Summary_OwnerID"] = $data["service_center"];
		$_SESSION["_Timekeeping_(Hours_Worked)_Report_Summary_OwnerID"] = $data["service_center"];
		$_SESSION["_Withholding_Tax_Collection_List_per_Month_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_purchase_report_per_item Chart_OwnerID"] = $data["service_center"];
		$_SESSION["_System_Audit_Report_OwnerID"] = $data["cost_center"];
		$_SESSION["_unbuild_assemblies_OwnerID"] = $data["cost_center"];
		$_SESSION["_disposed_inv_OwnerID"] = $data["cost_center"];
		$_SESSION["_For_Approval_Dashboard_OwnerID"] = $data["id"];
		$_SESSION["_For_Checking_Dashboard_OwnerID"] = $data["id"];
		$_SESSION["_raw_cake_balance_OwnerID"] = $data["service_center"];
		$_SESSION["_Collection_Report_per_Sales_Rep_OwnerID"] = $data["service_center"];
		$_SESSION["_marketing_request_form_for_checking_OwnerID"] = $data["cost_center"];
		$_SESSION["_marketing_request_form_for_approval_OwnerID"] = $data["cost_center"];
		$_SESSION["_201_OwnerID"] = $data["cost_center"];
		$_SESSION["_Business_Owner_Purchase_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_AR_Billing_due_dates_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_AR_Aging_of_AR_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Logistic_Open_Trip_Ticket_OwnerID"] = $data["service_center"];
		$_SESSION["_Marketing_Sales_per_campaign_OwnerID"] = $data["service_center"];
		$_SESSION["_Support_Open_Case_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Sale_Meeting_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Production_Inventory_Balance_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_Warehouse_Inventory_Balance_Report_OwnerID"] = $data["service_center"];
		$_SESSION["_intercompany_sales_order_OwnerID"] = $data["cost_center"];
		$_SESSION["_Employee Advance Release_OwnerID"] = $data["employee_name"];
		$_SESSION["_Employee Advance Replenishment_OwnerID"] = $data["employee_name"];
		$_SESSION["_change_password_OwnerID"] = $data["Username"];
		$_SESSION["_project_replenishment_deposited_OwnerID"] = $data["employee_name"];
		$_SESSION["_sales_business_partner_OwnerID"] = $data["cost_center"];
		$_SESSION["_canvas_OwnerID"] = $data["cost_center"];
	if($globalEvents->exists("AfterSuccessfulLogin"))
	{
		$globalEvents->AfterSuccessfulLogin($pUsername != "Guest" ? $pUsername : "", $password, $data, $pageObject);
	}	
}

/**
 * @intellisense
 */
function DoLogin($callAfterLoginEvent = false, $userID = "Guest", $userName = "", $groupID = "<Guest>", $accessLevel = ACCESS_LEVEL_GUEST, $password = "", &$pageObject = null)
{
	global $globalEvents;
	
	if($userID == "Guest" && $userName == "")
		$userName = "Guest";
	
	$_SESSION["UserID"] = $userID;
	$_SESSION["UserName"] = runner_htmlspecialchars( $userName );
	$_SESSION["GroupID"] = $groupID;
	$_SESSION["AccessLevel"] = $accessLevel;
	$auditObj = GetAuditObject();
	if($auditObj)
	{
		$auditObj->LogLogin($userID);
		$auditObj->LoginSuccessful();
	}
	if($callAfterLoginEvent && $globalEvents->exists("AfterSuccessfulLogin"))
	{
		$dummy = array();
		$globalEvents->AfterSuccessfulLogin($userID != "Guest" ? $userID : "", $password, $dummy, $pageObject);
	}
}

/**
 * @intellisense
 */
function CheckSecurity($strValue, $strAction, $table = "")
{

	global $cAdvSecurityMethod, $strTableName;
	if( $table == "" )
		$table = $strTableName;
	$pSet = new ProjectSettings($table);
	
	if($_SESSION["AccessLevel"]==ACCESS_LEVEL_ADMIN)
		return true;

	$strPerm = GetUserPermissions();
	if( strpos($strPerm, "M") === false )
	{
		if($table=="cash_receipt")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="purchase")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="cash_disbursment")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Disbursement Reports")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Financial Reports")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase Reports")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Receivable Reports")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Taxation Reports")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="General Journal Reports")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Inventory Reports")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="deposit")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="chart_of_accounts")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="dr")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="purchase_order")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="purchase_order_receiving")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="ppe_list")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="meetings")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="calls")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="reimbursement")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="quotation")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="case")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="production")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="issue")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="disposed")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="backload")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="service_report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="service_report_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="fixed_assets_return")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="center_replenishment")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="credit_memo")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="debit_memo")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="project_replenishment")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="sales")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Order_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Order_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Quotation_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Quotation_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Reimbursement_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Reimbursement_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Project_Replenishment_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Project_Replenishment_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchases_Order_for_checking2")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchases_Order_for_approval2")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Receiving_for_checking2")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Receiving_for_approval2")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Delivery_Receipt_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Delivery_Receipt_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Production_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Production_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Issue_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Issue_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Disposal_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Fixed_Asset_Issue_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Fixed_Asset_Issue_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Fixed_Asset_Return_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Fixed_Asset_Return_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Petty_Cash_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Petty_Cash_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Deposit_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Deposit_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Credit_Memo_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Credit_Memo_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Debit_Memo_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="debit_memo_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchases_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchases_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Cash_Disbursement_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Cash_Disbursement_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="General_Journal_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="General_Journal_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="estimate")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="inventory_adj")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="inventory_tranfer")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Customers")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Suppliers")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Change_Time_Schedule_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Change_Time_Schedule_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Estimate_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Estimate_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="2301")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="production_request")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="department")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="sales_report_per_item Chart")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Leave_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="loan_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Overtime_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Request_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Production_Request_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Inventory_Transfer_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Leave_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Loan_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Overtime_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Request_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Production_Request_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Inventory_Transfer_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Backload_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Backload_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Cash_Receipt_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Cash_Receipt_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Payroll_Entry_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Payroll_Entry_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Process_Payroll_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Process_Payroll_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Service_Report_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Service_Report_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="service_report_detail_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="service_report_detail_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Banch Reports")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="return")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="fixed_asset_issue")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="sales_report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Customer_Balance_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Customer_Balance_Report_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Collection_Report_per_Customer")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Aging_of_AR_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Aging_of_AR_Report_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Payment_per_Supplier_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Aging of_AP_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Aging of_AP_Report_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Inventory_Balance_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Inventory_Valuation_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Inventory_Below_Reorder_Point_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Report_per_BP")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Report_per_Item")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Report_per_Cost_Center")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Report_per_BP_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Report_per_Item_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Report_per_Cost_Center_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Report_per_BP")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Report_per_Item")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Report_per_Cost_Center")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Report_per_BP_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Report_per_Item_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Report_per_Cost_Center_detail")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Transaction Registry")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Report_per_sales_rep")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="PDC_Registry_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Trip_Ticket_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Request_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchase_Import_Costing")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Production_Costing_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Production_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Closed_Opportunities")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_by_Account")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_by_Lead_Source")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_by_Partner")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_by_Rep")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Leave_Ledger_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="DR_Reports")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Order_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="email")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="sub_contract")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Monthly_Pag-ibig_Summary_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Monthly_Philhealth_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Monthly_Philhealth_Summary_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Monthly_SSS_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Monthly_SSS_Report_by_Department_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Monthly_Withholding_Tax_Prooflist_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Monthly_Witholding_Tax_Collection_List_by_Department_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Overtime_Summary_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Monthly_Pag-ibig_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="sales_order")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Production_Dashboard_Item")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Disposal_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Active_campaign_report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="For_Billing_report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="sales_order_for_project_mngmt")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Supplier_Balance_Report_Details")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Supplier_Balance_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Inventory_per_Cost_center_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Deposit_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="SO_for_Billing_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Billing_due_dates_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Packing_List_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Open_Trip_Ticket_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Production_Report_invw")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Item_Balances_Report_invw")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Open_Case_Report_s")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_Order_per_Campaign_Report_marketing")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sales_per_campaign_Report_marketing")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Asset_per_Accountability_Report_invw")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Deposit_in_transit_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Outstanding_Checks_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Absences_Detail_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Absences_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Attendance_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="DTR_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Summary_of_Tardiness_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Followup_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Coming_Meeting_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Meeting_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Calls_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="sales_per_sales_rep_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Expense_per_Project_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Tardiness_Report_Summary")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Timekeeping_(Hours_Worked)_Report_Summary")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Withholding_Tax_Collection_List_per_Month_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="purchase_report_per_item Chart")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="System_Audit_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="unbuild_assemblies")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="disposed_inv")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="For_Approval_Dashboard")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="For_Checking_Dashboard")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchases_Order_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Purchases_Order_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Receiving_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Receiving_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Center_Replenishment_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Center_Replenishment_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="raw_cake_balance")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Collection_Report_per_Sales_Rep")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="marketing_request_form_for_checking")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="marketing_request_form_for_approval")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="201")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Business_Owner_Purchase_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="AR_Aging_of_AR_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Logistic_Open_Trip_Ticket")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Marketing_Sales_per_campaign")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Support_Open_Case_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Sale_Meeting_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Production_Inventory_Balance_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="Warehouse_Inventory_Balance_Report")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="intercompany_sales_order")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
		if($table=="change_password")
		{
			
				if(!($pSet->getCaseSensitiveUsername((string)$_SESSION["_".$table."_OwnerID"])===$pSet->getCaseSensitiveUsername((string)$strValue)))
				return false;
		}
	}
	//	 check user group permissions
	$localAction = strtolower($strAction);
	if($localAction=="add" && !(strpos($strPerm, "A")===false) ||
	   $localAction=="edit" && !(strpos($strPerm, "E")===false) ||
	   $localAction=="delete" && !(strpos($strPerm, "D")===false) ||
	   $localAction=="search" && !(strpos($strPerm, "S")===false) ||
	   $localAction=="import" && !(strpos($strPerm, "I")===false) ||
	   $localAction=="export" && !(strpos($strPerm, "P")===false) )
		return true;
	else
		return false;
	return true;
}

/**
 * @intellisense
 */
function CheckTablePermissions($strTableName, $permission)
{
	if( strpos(GetUserPermissions($strTableName), $permission) === false )
		return false;
	
	return true;
}

function pagetypeToPermissions($pageType)
{
	global $_pagetypeToPermissions_dict;
	
	if(!$_pagetypeToPermissions_dict)
	{
		$_pagetypeToPermissions_dict = array();
		$_pagetypeToPermissions_dict["list"] = "S";
		$_pagetypeToPermissions_dict["search"] = "S";
		$_pagetypeToPermissions_dict["view"] = "S";
		$_pagetypeToPermissions_dict["add"] = "A";
		$_pagetypeToPermissions_dict["edit"] = "E";
		$_pagetypeToPermissions_dict["print"] = "P";
		$_pagetypeToPermissions_dict["export"] = "P";
		$_pagetypeToPermissions_dict["import"] = "I";
	}
	
	return $_pagetypeToPermissions_dict[$pageType];
}

/**
 * Add security WHERE clause to SELECT SQL command
 * @intellisense
 */
function SecuritySQL($strAction, $table="", $strPerm="")
{
	global $cAdvSecurityMethod,$strTableName;
	
	if (!strlen($table))	
		$table = $strTableName;
	
	$pSet = new ProjectSettings($table);
	
   	$ownerid=@$_SESSION["_".$table."_OwnerID"];
	$ret="";
	if(@$_SESSION["AccessLevel"]==ACCESS_LEVEL_ADMIN)
		return "";
		
	$ret="";
	if(!strlen($strPerm))
		$strPerm = GetUserPermissions($table);

	if(strpos($strPerm,"M")===false)
	{
		if($table=="cash_receipt")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="purchase")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="cash_disbursment")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Disbursement Reports")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Financial Reports")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase Reports")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Receivable Reports")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Taxation Reports")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="General Journal Reports")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Inventory Reports")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="deposit")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="chart_of_accounts")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="dr")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="purchase_order")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="purchase_order_receiving")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="ppe_list")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="meetings")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="calls")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="reimbursement")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="quotation")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="case")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="production")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="issue")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="disposed")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="backload")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="service_report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="service_report_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="fixed_assets_return")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="center_replenishment")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="credit_memo")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="debit_memo")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="project_replenishment")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="sales")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Order_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Order_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Quotation_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Quotation_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Reimbursement_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Reimbursement_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Project_Replenishment_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Project_Replenishment_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchases_Order_for_checking2")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchases_Order_for_approval2")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Receiving_for_checking2")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Receiving_for_approval2")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Delivery_Receipt_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Delivery_Receipt_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Production_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Production_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Issue_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Issue_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Disposal_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Fixed_Asset_Issue_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Fixed_Asset_Issue_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Fixed_Asset_Return_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Fixed_Asset_Return_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Petty_Cash_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Petty_Cash_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Deposit_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Deposit_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Credit_Memo_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Credit_Memo_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Debit_Memo_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="debit_memo_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchases_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchases_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Cash_Disbursement_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Cash_Disbursement_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="General_Journal_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="General_Journal_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="estimate")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="inventory_adj")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="inventory_tranfer")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Customers")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Suppliers")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Change_Time_Schedule_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Change_Time_Schedule_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Estimate_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Estimate_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="2301")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="production_request")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="department")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="sales_report_per_item Chart")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Leave_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="loan_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Overtime_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Request_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Production_Request_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Inventory_Transfer_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Leave_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Loan_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Overtime_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Request_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Production_Request_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Inventory_Transfer_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Backload_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Backload_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Cash_Receipt_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Cash_Receipt_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Payroll_Entry_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Payroll_Entry_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Process_Payroll_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Process_Payroll_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Service_Report_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Service_Report_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="service_report_detail_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="service_report_detail_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Banch Reports")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="return")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="fixed_asset_issue")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="sales_report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Customer_Balance_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Customer_Balance_Report_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Collection_Report_per_Customer")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Aging_of_AR_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Aging_of_AR_Report_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Payment_per_Supplier_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Aging of_AP_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Aging of_AP_Report_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Inventory_Balance_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Inventory_Valuation_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Inventory_Below_Reorder_Point_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Report_per_BP")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Report_per_Item")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Report_per_Cost_Center")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Report_per_BP_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Report_per_Item_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Report_per_Cost_Center_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Report_per_BP")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Report_per_Item")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Report_per_Cost_Center")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Report_per_BP_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Report_per_Item_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Report_per_Cost_Center_detail")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Transaction Registry")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Report_per_sales_rep")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="PDC_Registry_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Trip_Ticket_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Request_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchase_Import_Costing")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Production_Costing_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Production_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Closed_Opportunities")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_by_Account")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_by_Lead_Source")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_by_Partner")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_by_Rep")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Leave_Ledger_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="DR_Reports")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Order_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="email")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="sub_contract")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Monthly_Pag-ibig_Summary_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Monthly_Philhealth_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Monthly_Philhealth_Summary_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Monthly_SSS_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Monthly_SSS_Report_by_Department_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Monthly_Withholding_Tax_Prooflist_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Monthly_Witholding_Tax_Collection_List_by_Department_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Overtime_Summary_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Monthly_Pag-ibig_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="sales_order")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Production_Dashboard_Item")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Disposal_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Active_campaign_report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="For_Billing_report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="sales_order_for_project_mngmt")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Supplier_Balance_Report_Details")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Supplier_Balance_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Inventory_per_Cost_center_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Deposit_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="SO_for_Billing_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Billing_due_dates_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Packing_List_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Open_Trip_Ticket_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Production_Report_invw")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Item_Balances_Report_invw")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Open_Case_Report_s")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_Order_per_Campaign_Report_marketing")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sales_per_campaign_Report_marketing")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Asset_per_Accountability_Report_invw")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Deposit_in_transit_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Outstanding_Checks_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Absences_Detail_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Absences_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Attendance_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="DTR_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Summary_of_Tardiness_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Followup_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Coming_Meeting_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Meeting_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Calls_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="sales_per_sales_rep_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Expense_per_Project_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Tardiness_Report_Summary")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Timekeeping_(Hours_Worked)_Report_Summary")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Withholding_Tax_Collection_List_per_Month_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="purchase_report_per_item Chart")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="System_Audit_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="unbuild_assemblies")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="disposed_inv")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="For_Approval_Dashboard")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="For_Checking_Dashboard")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchases_Order_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Purchases_Order_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Receiving_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Receiving_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Center_Replenishment_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Center_Replenishment_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="raw_cake_balance")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Collection_Report_per_Sales_Rep")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="marketing_request_form_for_checking")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="marketing_request_form_for_approval")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="201")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Business_Owner_Purchase_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="AR_Aging_of_AR_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Logistic_Open_Trip_Ticket")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Marketing_Sales_per_campaign")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Support_Open_Case_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Sale_Meeting_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Production_Inventory_Balance_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="Warehouse_Inventory_Balance_Report")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="intercompany_sales_order")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
		if($table=="change_password")
		{
				$ret = GetFullFieldName($pSet->getTableOwnerID(), $table, false)."=".make_db_value($pSet->getTableOwnerID(), $ownerid, "", "", $table);
		}
	}
	
	if($strAction=="Edit" && !(strpos($strPerm, "E")===false) ||
	   $strAction=="Delete" && !(strpos($strPerm, "D")===false) ||
	   $strAction=="Search" && !(strpos($strPerm, "S")===false) ||
	   $strAction=="Export" && !(strpos($strPerm, "P")===false) )
		return $ret;
	else
		return "1=0";
	return "";
}

////////////////////////////////////////////////////////////////////////////////
// editing functions
////////////////////////////////////////////////////////////////////////////////
/**
 * @intellisense
 */
function make_db_value($field,$value,$controltype="",$postfilename="",$table="")
{	
	$ret = prepare_for_db($field, $value, $controltype, $postfilename, $table);
	
	if($ret === false)
		return $ret;
		
	return add_db_quotes($field, $ret, $table);
}

/**
 * @param String field
 * @param Mixed value
 * @param String table		The datasource table name
 * @param String type
 * @intellisense
 */
function add_db_quotes($field, $value, $table = "", $type = null)
{
	global $strTableName, $locale_info, $cman;
	
	if( $table == "" )
		$table=$strTableName;
	
	$pSet = new ProjectSettings($table);
	$connection = $cman->byTable( $table );
	
	if( $type == null )
		$type = $pSet->getFieldType($field);
		
	if( IsBinaryType($type) )
		return $connection->addSlashesBinary( $value );
	
	if( ($value === "" || $value === FALSE || is_null($value)) && !IsCharType($type) )
		return "null";
	
	if(NeedQuotes($type))
	{
		if( !IsDateFieldType($type) )
			$value = $connection->prepareString($value);
		else
		{
			$y = "(\d\d\d\d)";
			$m = "(0?[1-9]|1[0-2])";
			$d = "(0?[1-9]|[1-2][0-9]|3[0-1])";
			$delim = "(-|".preg_quote($locale_info["LOCALE_SDATE"], "/").")";
			$reg = "/".$d.$delim.$m.$delim.$y."|".$m.$delim.$d.$delim.$y."|".$y.$delim.$m.$delim.$d."/";
			if( !preg_match($reg, $value, $matches) )
				return "null";

			$value = $connection->addDateQuotes( $value );
		}
	}
	else
	{
		$strvalue = (string)$value;
		if(is_numeric($strvalue))
			$value = str_replace(",",".",$strvalue);
		else
			$value=0;
		
		if( $connection->dbType == nDATABASE_PostgreSQL )
		{
			// if boolean type field, add quotes
			if ($type == 11)
			{
				$value = strtolower($value);
				if (!strlen($value) || $value == 0 || $value == "0" || $value == "false" || $value == "f" || $value == "n" || $value == "no" || $value == "off")
					$value = "f";
				else
					$value = "t";
				$value = $connection->prepareString($value);
			}
		}
	}
	return $value;
}

/**
 * @param String field
 * @param Mixed value
 * @param String controltype
 * @param String postfilename
 * @param String table			The datasource table name
 * @intellisense
 */
function prepare_for_db($field, $value, $controltype = "", $postfilename = "", $table = "")
{
	global $strTableName, $cman;
	
	if($table == "")
		$table = $strTableName;
		
	$pSet = new ProjectSettings($table);
	$connection = $cman->byTable( $table );
	
	$filename = "";
	$type = $pSet->getFieldType($field);
	if((!$controltype || $controltype == "multiselect") && !IsTimeType($type))
	{
		if(is_array($value))
			$value = combinevalues($value);
		if(($value === "" || $value === FALSE) && !IsCharType($type))
			return "";
		if(IsGuid($type))
		{
			if(!IsGuidString($value))
				return "";
		}
		
		if( IsFloatType($type) )
			return makeFloat($value);
		
		if( IsNumberType($type) && !is_int($value) )
		{
			$value = trim($value);
			if( !is_numeric( str_replace(",", ".", $value) ) )
				$value = "";
		}
		return $value;
	}	
	else if($controltype == "time" || IsTimeType($type))
	{
		if(!strlen($value))
			return "";
			
		$time = localtime2db($value);
		if( $connection->dbType == nDATABASE_PostgreSQL )
		{
			$timeArr = explode(":",$time);
			if($timeArr[0]>24 || $timeArr[1]>59 || $timeArr[2]>59)
				return "";
		}
		
		if( IsDateFieldType($type) )
			$time = "2000-01-01 ".$time;
		
		return $time;
	}
	else if(substr($controltype, 0, 4) == "date")
	{
		$dformat = substr($controltype, 4);
		if($dformat == EDIT_DATE_SIMPLE || $dformat == EDIT_DATE_SIMPLE_INLINE || $dformat == EDIT_DATE_SIMPLE_DP)
		{
			$time = localdatetime2db($value);
			if($time == "null")
				return "";
			return $time;
		}
		else if($dformat == EDIT_DATE_DD || $dformat == EDIT_DATE_DD_INLINE || $dformat == EDIT_DATE_DD_DP)
		{
			$a = explode("-",$value);
			if(count($a) < 3)
				return "";
			else
			{
				$y = $a[0];
				$m = $a[1];
				$d = $a[2];
			}
			if($y < 100)
			{
				if($y < 70)
					$y += 2000;
				else
					$y += 1900;
			}
			return mysprintf("%04d-%02d-%02d",array($y,$m,$d));
		}
		else
			return "";
	}
	else if(substr($controltype, 0, 8) == "checkbox")
	{
		if($value == "on")
			$ret = 1;
		else if($value == "none")
			return "";
		else 
			$ret = 0;
		return $ret;
	}
	else
		return false;
}

/**
 * 	delete uploaded files when deleting the record
 * @intellisense
 */
function DeleteUploadedFiles($pSet, $deleted_values)
{
	foreach($deleted_values as $field => $value)
	{
		if(($pSet->getEditFormat($field) == EDIT_FORMAT_FILE || $pSet->getPageTypeByFieldEditFormat($field, EDIT_FORMAT_FILE) != "") 
			&& $pSet->isDeleteAssociatedFile($field))
		{
			if(!strlen($value))
				return;
				
			$filesArray = my_json_decode($value);
			if(!is_array($filesArray) || count($filesArray) == 0)
			{
				$filesArray = array(array("name" => $pSet->getUploadFolder($field).$value));
				if($pSet->getCreateThumbnail($field))
					$filesArray[0]["thumbnail"] = $pSet->getUploadFolder($field).$pSet->getStrThumbnail($field).$value;
			}
		
			foreach($filesArray as $delFile)
			{
				$filename = $delFile["name"];
				$isAbs = $pSet->isAbsolute($field) || isAbsolutePath($filename);
				if(!$isAbs)
					$filename = getabspath($filename);
				runner_delete_file($filename);
				if($delFile["thumbnail"] != "")
				{
					$filename = $delFile["thumbnail"];
					if(!$isAbs)
						$filename = getabspath($filename);
					runner_delete_file($filename);
				}
			}
		}
	}
}

/**
 * 	combine checked values from multi-select list box
 * @intellisense
 */
function combinevalues($arr)
{
	$ret="";
	foreach($arr as $item)
	{
		$val = $item;
		if(strlen($ret))
			$ret.=",";
		if(strpos($val,",")===false && strpos($val,'"')===false)
			$ret.=$val;
		else
		{
			$val=str_replace('"','""',$val);
			$ret.='"'.$val.'"';
		}
	}
	return $ret;
}

/**
 * 	split values for multi-select list box
 * @intellisense
 */
function splitvalues($str)
{
	$arr=array();
	if($str=="")
	{
		$arr[] = "";
		return $arr;
	}
	$start=0;
	$i=0;
	$inquot=false;
	while($i<=strlen($str))
	{
		if($i<strlen($str) && substr($str,$i,1)=='"')
			$inquot=!$inquot;
		else if($i==strlen($str) || !$inquot && substr($str,$i,1)==',')
		{
			$val=substr($str,$start,$i-$start);
			$start=$i+1;
			if(strlen($val) && substr($val,0,1)=='"')
			{
				$val=substr($val,1,strlen($val)-2);
				$val=str_replace('""','"',$val);
			}
			
			if( $val !== FALSE )
			$arr[]=$val;
		}
		$i++;
	}
	return $arr;
}

/**
 * 
 */
function GetLookupFieldsIndexes($pSet, $field)
{
	$lookupTable = $pSet->getLookupTable($field);
	$lookupType = $pSet->getLookupType($field);
	$displayFieldName = $pSet->getDisplayField($field);
	$linkFieldName = $pSet->getLinkField($field);
	$linkAndDisplaySame = $linkFieldName == $displayFieldName;
	if($lookupType == LT_QUERY)
	{
		$lookupPSet = new ProjectSettings($lookupTable);
		$linkFieldIndex = $lookupPSet->getFieldIndex($linkFieldName) - 1;
		if($linkAndDisplaySame)
			$displayFieldIndex = $linkFieldIndex;
		else
		{
			if($pSet->getCustomDisplay($field))
				$displayFieldIndex = $lookupPSet->getCustomExpressionIndex($pSet->_table, $field);
			else
				$displayFieldIndex = $lookupPSet->getFieldIndex($displayFieldName) - 1;
		}
	}
	else 
	{
		$linkFieldIndex = 0;
		$displayFieldIndex = $linkAndDisplaySame ? 0 : 1;
	}
	return array("linkFieldIndex" => $linkFieldIndex, "displayFieldIndex" => $displayFieldIndex);
}

//////////////////////////////////////////////////////////////////////////////
/**
 * Get locale, am, pm for field edit as time
 * @param integer $convention - 24 or 12 hours format for timePicker
 * @param boolean $useTimePicker -  use timePicker or not
 * @return array
 * @intellisense
 */
function getLacaleAmPmForTimePicker($convention, $useTimePicker = false)
{
	$am = '';
	$pm = '';
	global $locale_info;
	if($useTimePicker)
	{
		$locale_convention = $locale_info["LOCALE_ITIME"] ? 24 : 12;
		if($convention == $locale_convention)
		{
			$am = $locale_info["LOCALE_S1159"];
			$pm = $locale_info["LOCALE_S2359"];
			$locale = $locale_info["LOCALE_STIMEFORMAT"];
		}
		else
		{
			if($convention == 24)
			{
				$am = '';
				$pm = '';
				$locale = "H:mm:ss";
			}
			else
			{
				$am = 'am';
				$pm = 'pm';
				$locale = "h:mm:ss tt";
			}
		}
	}
	else
		$locale = $locale_info["LOCALE_STIMEFORMAT"];
		
	return array('am'=>$am,'pm'=>$pm,'locale'=>$locale);	
}

/**
 * Get value for field edit as time and get dpTime settings
 * @param integer $convention - 24 or 12 hours format for timePicker
 * @param string $type - type of field
 * @param string $value - value of field
 * @param boolean $useTimePicker -  use timePicker or not
 * @return array
 * @intellisense
 */
function getValForTimePicker($type,$value,$locale)
{
	$val = "";
	$dbtime = array();
	if(IsDateFieldType($type))
	{
		$dbtime = db2time($value);
		if(count($dbtime))
			$val = format_datetime_custom($dbtime, $locale);
	}
	else 
	{
		$arr = parsenumbers($value);
		if(count($arr))
		{
			while(count($arr)<3)
				$arr[] = 0;
			$dbtime = array(0, 0, 0, $arr[0], $arr[1], $arr[2]);
			$val = format_datetime_custom($dbtime, $locale);
		}
	}
	
	return array('val'=>$val,'dbTime'=>$dbtime);
}


/**
 * @intellisense
 */
function my_stripos($str,$needle, $offest)
{
	if (strlen($needle)==0 || strlen($str)==0)
		return false;
	return strpos(strtolower($str),strtolower($needle), $offest);
} 

/**
 * @intellisense
 */
function my_str_ireplace($search, $replace,$str)
{
	$pos=my_stripos($str,$search,0);
	if($pos===false)
		return $str;
	return substr($str,0,$pos).$replace.substr($str,$pos+strlen($search));
} 


/**
 * @intellisense
 */
function in_assoc_array($name, $arr)
{
	foreach ($arr as $key => $value) 
	{
		if ($key==$name)
			return true;
	}
	return false;
}

/**
 * @intellisense
 */
function xmlencode($str)
{
	$str = str_replace("&","&amp;",$str);
	$str = str_replace("<","&lt;",$str);
	$str = str_replace(">","&gt;",$str);
	$str = str_replace("'","&apos;",$str);
	return escapeEntities($str);
}

/**
 * @intellisense
 */
function print_inline_array(&$arr,$printkey=false)
{
	if(!$printkey)
	{
		foreach ( $arr as $key=>$val )
			echo str_replace(array("&","<","\\","\r","\n"),array("&amp;","&lt;","\\\\","\\r","\\n"),str_replace(array("\\","\r","\n"),array("\\\\","\\r","\\n"),$val))."\\n";
	}
	else
	{
		foreach( $arr as $key=>$val )
			echo str_replace(array("&","<","\\","\r","\n"),array("&amp;","&lt;","\\\\","\\r","\\n"),str_replace(array("\\","\r","\n"),array("\\\\","\\r","\\n"),$key))."\\n";
	}
		
}

/**
 * @intellisense
 */
function checkpassword($pwd)
{
	$len = strlen($pwd);
	if($len < 8)
		return false;
	$cUnique = array();
	$cLower = $cUpper = $cDigit = 0;
	for($i=0; $i<$len; $i++)
	{
		$c = substr($pwd, $i, 1);
		if($c >= 'a' && $c <= 'z')
			$cLower++;
		else if($c >= 'A' && $c<= 'Z')
			$cUpper++;
		else
			$cDigit++;
			
		$cUnique[$c] = 1;
	}
	if(count($cUnique)<4)
		return false;
	if($cDigit<2)
		return false;
	return true;
}

/**
 * @intellisense
 */
function GetChartXML($chartname)
{
	$strTableName = GetTableByShort($chartname);	
	$settings = new ProjectSettings($strTableName);
	return $settings->getChartXml();
}

function isSecureProtocol() {
  return
    (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off')
    || $_SERVER['SERVER_PORT'] == 443;
}

/**
 * Returns site address without path
 * Examples:
 * http://server.com:88
 * https://server.com
 *
 * @intellisense
 */
function GetSiteUrl()
{
	$proto = "http://";
	if( $_SERVER["HTTPS"] && $_SERVER["HTTPS"] != "off")
		$proto = "https://";
	return $proto . $_SERVER["HTTP_HOST"];
}

/**
 * Returns site address with path
 * Examples:
 * http://server.com:88/Calendar
 * https://server.com
 *
 * @intellisense
 */
function GetFullSiteUrl()
{
	return getDirectoryFromURI( GetSiteUrl() . $_SERVER['REQUEST_URI'] );
}


/**
 * @intellisense
 */
function GetAuditObject($table="")
{
	
	$linkAudit = false;
	if(!$table)
	{
		$linkAudit = true;
	}
	else
	{
		$settings = new ProjectSettings($table);
		$linkAudit = $settings->auditEnabled();
	}
	if ($linkAudit)
	{	
		require_once(getabspath("include/audit.php"));
		return new AuditTrailTable();
	}
	else
	{
		return NULL;
	}
}

/**
 * @intellisense
 */
function GetLockingObject($table="")
{
	return NULL;

	if(!$table)
	{
		global $strTableName;
		$table = $strTableName;
	}
	$settings = new ProjectSettings($table);
	if ($settings->lockingEnabled())
	{	
		require_once(getabspath("include/locking.php"));
		return new oLocking();
	}
	else
	{
		return NULL;
	}
}

/**
 * @intellisense
 */
function isEnableSection508()
{
	return GetGlobalData("isSection508",false);
}

/**
 * @intellisense
 */
function isEncryptionEnabled()
{
	return GetGlobalData("isUseEncryption", ENCRYPTION_NONE) == ENCRYPTION_PHP 
		|| GetGlobalData("isUseEncryption", ENCRYPTION_NONE) == ENCRYPTION_DB;
}

/**
 * @intellisense
 */
function isEncryptionByPHPEnabled()
{
	return GetGlobalData("isUseEncryption", ENCRYPTION_NONE) == ENCRYPTION_PHP;
}

/**
 * Returns validation type which defined in js validation object.
 * Use this function, because runner constants has another names of validation functions
 *
 * @param string $name
 * @return string
  * @intellisense
 */
function getJsValidatorName($name) 
{	
	switch ($name) 
	{
		case "Number":
			return "IsNumeric";
			break;
		case "Password":
			return "IsPassword";
			break;
		case "Email":
			return "IsEmail";
			break;
		case "Currency":
			return "IsMoney";
			break;
		case "US ZIP Code":
			return "IsZipCode";
			break;
		case "US Phone Number":
			return "IsPhoneNumber";
			break;
		case "US State":
			return "IsState";
			break;
		case "US SSN":
			return "IsSSN";
			break;
		case "Credit Card":
			return "IsCC";
			break;
		case "Time":
			return "IsTime";
			break;
		case "Regular expression":
			return "RegExp";
			break;						
		default:
			return $name;
			break;
	}
}

/**
 * @intellisense
 */
function SetLangVars($xt, $prefix, $pageName = "", $extraparams = "")
{
	$xt->assign("lang_label", true);
	
	if( @$_REQUEST["language"] )
		$_SESSION["language"] = @$_REQUEST["language"];

	$currentLang = mlang_getcurrentlang();
		
	$var = GoodFieldName($currentLang)."_langattrs";
	$xt->assign($var, "selected");
	
	$xt->assign("EnglishLANGLINK", "English" != $currentLang);
	
	if( isEnableSection508() )
		$xt->assign_section("lang_label", "<label for=\"languageSelector\">","</label>");
		
	if( $extraparams )
		$extraparams = $extraparams."&";
		
	$dataAttr = 'data-params="'.$extraparams.'" data-prefix="'.$prefix.'"';	
	$xt->assign("langselector_attrs", "id=\"languageSelector\" ".$dataAttr);
	
	$xt->assign("languages_block", true);
}

/**
 * @intellisense
 */
function GetTableCaption($table)
{
	global $tableCaptions;
	return @$tableCaptions[mlang_getcurrentlang()][$table];
}

/**
 * @intellisense
 */
function GetFieldByLabel($table, $label) 
{
	global $field_labels, $strTableName;
	if (!$table)
	{
		$table = $strTableName;
	}
	
	if(!array_key_exists($table,$field_labels))
		return "";
	$currLang = mlang_getcurrentlang();
	if(!array_key_exists($currLang,$field_labels[$table]))
		return "";	
	$lables = $field_labels[$table][mlang_getcurrentlang()];
	foreach ($lables as $key=>$val)
	{
		if ($val == $label)
		{
			return $key;
		}
	}
	return '';
}

/**
 * @intellisense
 */
function GetFieldLabel($table,$field)
{
	global $field_labels;
	if(!array_key_exists($table,$field_labels))
		return "";
	return @$field_labels[$table][mlang_getcurrentlang()][$field];
}


/**
 * @intellisense
 */
function GetFieldToolTip($table, $field)
{
	global $fieldToolTips;
	if(!array_key_exists($table, $fieldToolTips))
		return "";
	return @$fieldToolTips[$table][mlang_getcurrentlang()][$field];
}

/**
 * @intellisense
 */
function GetCustomLabel($custom)
{
	global $custom_labels;
	return @$custom_labels[mlang_getcurrentlang()][$custom];
}

/**
 * @intellisense
 */
function mlang_getcurrentlang()
{
	global $_currentLanguage, $mlang_messages,$mlang_defaultlang;
	if(@$_REQUEST["language"])
		$_SESSION["language"]=@$_REQUEST["language"];
	if(@$_SESSION["language"])
		return $_SESSION["language"];
	return $mlang_defaultlang;
}

function isRTL()
{
	global $mlang_charsets;
	$cp = strtolower($mlang_charsets[mlang_getcurrentlang()]);
	return ($cp == 'windows-1256' || $cp == 'windows-1255');
}


/**
 * @intellisense
 */
function mlang_getlanglist()
{
	global $mlang_messages,$mlang_defaultlang;
	return array_keys($mlang_messages);
}

/**
 * @intellisense
 */
function displayDetailsOn($table,$page)
{
	global $detailsTablesData;
	if(!isset($detailsTablesData[$table]) || !is_array($detailsTablesData[$table]))
		return false;
	if($page == PAGE_EDIT)
		$key="previewOnEdit";
	elseif($page == PAGE_ADD)
		$key="previewOnAdd";
	elseif($page == PAGE_VIEW)
		$key="previewOnView";
	else
		$key="previewOnList";
	for($i=0;$i<count($detailsTablesData[$table]);$i++)
	{
		if($detailsTablesData[$table][$i][$key])
			return true;
	}
	return false;
}

/**
 * @intellisense
 */
function showDetailTable($params)
{
	global $strTableName;
	$oldTableName = $strTableName;
	$strTableName = $params["table"];
	// show page
	if($params["dpObject"]->isDispGrid())
		$params["dpObject"]->showPage();	
	$strTableName = $oldTableName;
}


/**
 * update record on Edit page
 * @intellisense
 */
function DoUpdateRecordSQL( $pageObject )
{
	$table = $pageObject->pSet->getOriginalTableName();
	$strWhereClause = $pageObject->getWhereClause( true );
	$evalues = $pageObject->getNewRecordData();
	$blobfields = $pageObject->getBlobFields();

	if(!count($evalues))
		return true;
		
	$strSQL = "update ".$pageObject->connection->addTableWrappers($table)." set ";
	$blobs = PrepareBlobs($evalues, $blobfields, $pageObject);
	//	construct SQL string
	foreach($evalues as $ekey=>$value)
	{
		if(in_array($ekey,$blobfields))			
			$strValue = $value;
		else
		{
			if( is_null( $pageObject->cipherer ) )
				$strValue = add_db_quotes( $ekey, $value );
			else 
				$strValue = $pageObject->cipherer->AddDBQuotes( $ekey, $value );
		}
		$strSQL .= $pageObject->getTableField($ekey)."=".$strValue.", ";
	}
	$strSQL = substr( $strSQL, 0, strlen($strSQL) - 2 );
	if($strWhereClause === "")
	{
		$strWhereClause = " (1=1) ";
	}
	$strSQL.=" where ".$strWhereClause;
	
	if(SecuritySQL("Edit", $pageObject->tName))	
		$strSQL .= " and (".SecuritySQL("Edit", $pageObject->tName).")";

	if( !ExecuteUpdate($pageObject, $strSQL, $blobs) )
		return false;

	return true;
}

/**
 * insert record on Add & Register pages
 * @intellisense
 */
function DoInsertRecordSQL($table, &$avalues, &$blobfields, &$pageObject)
{
	//	make SQL string
	$strSQL = "insert into ".$pageObject->connection->addTableWrappers($table)." ";
	
	$strFields = "(";
	$strValues = "(";
	$blobs = PrepareBlobs($avalues, $blobfields, $pageObject);
	
	foreach($avalues as $akey => $value)
	{
		$strFields.= $pageObject->getTableField($akey).", ";
		
		if( in_array($akey, $blobfields) )			
			$strValues.= $value.", ";
		else
		{
			if( is_null($pageObject->cipherer) )
				$strValues.= add_db_quotes($akey, $value).", ";
			else 
				$strValues.= $pageObject->cipherer->AddDBQuotes($akey, $value).", ";
		}
	}
	
	if( substr($strFields, -2) == ", " )
		$strFields = substr($strFields, 0, strlen($strFields) - 2);
	
	if( substr($strValues, -2) == ", " )
		$strValues = substr($strValues, 0, strlen($strValues) - 2);
	
	$strSQL.= $strFields.") values ".$strValues.")";
	
	if( !ExecuteUpdate($pageObject, $strSQL, $blobs) )
		return false;
		
	$pageObject->ProcessFiles();

	return true;
}

/**
 * insert record on Add page 
 * @param RunnerPage &pageObject
 * @intellisense
 */
function DoInsertRecordSQLOnAdd( &$pageObject )
{
	$table = $pageObject->pSet->getOriginalTableName();
	$avalues = $pageObject->getNewRecordData();
	$blobfields = $pageObject->getBlobFields();
	
	//	make SQL string
	$strSQL = "insert into ".$pageObject->connection->addTableWrappers($table)." ";
	
	$strFields="(";
	$strValues="(";
	$blobs = PrepareBlobs( $avalues, $blobfields, $pageObject );
	
	foreach($avalues as $akey => $value)
	{
		$strFields.= $pageObject->getTableField($akey).", ";
		
		if( in_array($akey, $blobfields) )			
			$strValues.= $value.", ";
		else
		{
			if( is_null( $pageObject->cipherer ) )
				$strValues.= add_db_quotes($akey, $value).", ";
			else 
				$strValues.= $pageObject->cipherer->AddDBQuotes($akey, $value).", ";
		}
	}
	
	if( substr($strFields, -2) == ", ")
		$strFields = substr($strFields, 0, strlen($strFields) - 2);
		
	if( substr($strValues, -2) == ", ")
		$strValues = substr($strValues, 0, strlen($strValues) - 2);
	
	$strSQL.= $strFields.") values ".$strValues.")";
	
	if( !ExecuteUpdate($pageObject, $strSQL, $blobs) )
		return false;

	return true;
}


function &getEventObject($table)
{
	global $tableEvents;
	$ret = null;
	if(!array_key_exists($table,$tableEvents))
		return $ret;
	return $tableEvents[$table];
}

/**
 * @intellisense
 */
function tableEventExists($event,$table)
{
	global $tableEvents;
	if(!array_key_exists($table,$tableEvents))
		return false;
	return $tableEvents[$table]->exists($event);
}

/**
 * @intellisense
 */
function add_nocache_headers()
{
	header("Cache-Control: no-cache, no-store, max-age=0, must-revalidate");
	header("Pragma: no-cache");
	header("Expires: Fri, 01 Jan 1990 00:00:00 GMT");
}

/**
 * @intellisense
 */
function IsGuidString(&$str)
{
//	{3F2504E0-4F89-11D3-9A0C-0305E82C3301} 
	if(strlen($str)==36 && substr($str,0,1)!="{" && substr($str,-1)!="}")
		$str="{".$str."}";
	elseif(strlen($str)==37 && substr($str,0,1)=="{" && substr($str,-1)!="}")
		$str=$str."}";
	elseif(strlen($str)==37 && substr($str,0,1)!="{" && substr($str,-1)=="}")
		$str="{".$str;
		
	if(strlen($str)!=38)
		return false;
	for($i=0;$i<38;$i++)
	{
		$c = substr($str,$i,1);
		if($i==0)
		{
			if($c!='{')
				return false;
		}
		elseif($i==37 )
		{
			if($c!='}')
				return false;
		}
		elseif($i==9 || $i==14 || $i==19 || $i==24)
		{
			if($c!='-')
				return false;
		}
		else
		{
			if(($c<'0' || $c>'9') && ($c<'a' || $c>'f') && ($c<'A' || $c>'F'))
				return false;
		}
	}
	return true;
}

/**
 * @intellisense
 */
function IsStoredProcedure($strSQL)
{
	if(strlen($strSQL)>6)
	{
		$c=strtolower(substr($strSQL,6,1));
		if(strtolower(substr($strSQL,0,6))=="select" && ($c<'0' || $c>'9') && ($c<'a' || $c>'z') && $c!='_')
			return false;
		else
			return true;
	}
	else
		return true;
}

/**
 * Checks whether an browser for mobile devices
 * Returns false - Browser stationary or it was not possible to determine
 * 1-4 - the browser is running on your mobile device
 * @return Mixed
 * @intellisense
 */
function MobileDetected() 
{
	$user_agent = strtolower($_SERVER['HTTP_USER_AGENT']);
	$accept = strtolower($_SERVER['HTTP_ACCEPT']);
 
	if( strpos($accept,'text/vnd.wap.wml')!==false || strpos($accept,'application/vnd.wap.xhtml+xml')!==false ) 
		return 1; // Mobile browser detected by HTTP-headers
 
	if( isset($_SERVER['HTTP_X_WAP_PROFILE']) || isset($_SERVER['HTTP_PROFILE']) ) 
		return 2; // Mobile browser detected by server settings


	if(preg_match('/(ipad|android|symbianos|opera mini|ipod|blackberry|'.
		'palm os|palm|hiptop|avantgo|plucker|xiino|blazer|elaine|iris|3g_t|'.
		'windows ce|opera mobi|windows ce; smartphone;|windows ce; iemobile|'.
		'mini 9.5|vx1000|lge |m800|e860|u940|ux840|compal|'.
		'wireless| mobi|ahong|lg380|lgku|lgu900|lg210|lg47|lg920|lg840|'.
		'lg370|sam-r|mg50|s55|g83|t66|vx400|mk99|d615|d763|el370|sl900|'.
		'mp500|samu3|samu4|vx10|xda_|samu5|samu6|samu7|samu9|a615|b832|'.
		'm881|s920|n210|s700|c-810|_h797|mob-x|sk16d|848b|mowser|s580|'.
		'r800|471x|v120|rim8|c500foma:|160x|x160|480x|x640|t503|w839|'.
		'i250|sprint|w398samr810|m5252|c7100|mt126|x225|s5330|s820|'.
		'htil-g1|fly v71|s302|-x113|novarra|k610i|-three|8325rc|8352rc|'.
		'sanyo|vx54|c888|nx250|n120|mtk |c5588|s710|t880|c5005|i;458x|'.
		'p404i|s210|c5100|teleca|s940|c500|s590|foma|samsu|vx8|vx9|a1000|'.
		'_mms|myx|a700|gu1100|bc831|e300|ems100|me701|me702m-three|sd588|'.
		's800|8325rc|ac831|mw200|brew |d88|htc\/|htc_touch|355x|m50|km100|'.
		'd736|p-9521|telco|sl74|ktouch|m4u\/|me702|8325rc|kddi|phone|lg |'.
		'sonyericsson|samsung|240x|x320vx10|nokia|sony cmd|motorola|'.
		'up.browser|up.link|mmp|symbian|smartphone|midp|wap|vodafone|o2|'.
		'pocket|kindle|silk|hpwos|mobile|psp|treo)/', $user_agent)) 
	{
		return 3; // Mobile browser detected by User Agent signature
	}

	if(in_array(substr($user_agent,0,4),
		Array("1207", "3gso", "4thp", "501i", "502i", "503i", "504i", "505i", "506i",
			  "6310", "6590", "770s", "802s", "a wa", "abac", "acer", "acoo", "acs-",
			  "aiko", "airn", "alav", "alca", "alco", "amoi", "anex", "anny", "anyw",
			  "aptu", "arch", "argo", "aste", "asus", "attw", "au-m", "audi", "aur ",
			  "aus ", "avan", "beck", "bell", "benq", "bilb", "bird", "blac", "blaz",
			  "brew", "brvw", "bumb", "bw-n", "bw-u", "c55/", "capi", "ccwa", "cdm-",
			  "cell", "chtm", "cldc", "cmd-", "cond", "craw", "dait", "dall", "dang",
			  "dbte", "dc-s", "devi", "dica", "dmob", "doco", "dopo", "ds-d", "ds12",
			  "el49", "elai", "eml2", "emul", "eric", "erk0", "esl8", "ez40", "ez60",
			  "ez70", "ezos", "ezwa", "ezze", "fake", "fetc", "fly-", "fly_", "g-mo",
			  "g1 u", "g560", "gene", "gf-5", "go.w", "good", "grad", "grun", "haie",
			  "hcit", "hd-m", "hd-p", "hd-t", "hei-", "hiba", "hipt", "hita", "hp i",
			  "hpip", "hs-c", "htc ", "htc-", "htc_", "htca", "htcg", "htcp", "htcs",
			  "htct", "http", "huaw", "hutc", "i-20", "i-go", "i-ma", "i230", "iac",
			  "iac-", "iac/", "ibro", "idea", "ig01", "ikom", "im1k", "inno", "ipaq",
			  "iris", "jata", "java", "jbro", "jemu", "jigs", "kddi", "keji", "kgt",
			  "kgt/", "klon", "kpt ", "kwc-", "kyoc", "kyok", "leno", "lexi", "lg g",
			  "lg-a", "lg-b", "lg-c", "lg-d", "lg-f", "lg-g", "lg-k", "lg-l", "lg-m",
			  "lg-o", "lg-p", "lg-s", "lg-t", "lg-u", "lg-w", "lg/k", "lg/l", "lg/u",
			  "lg50", "lg54", "lge-", "lge/", "libw", "lynx", "m-cr", "m1-w", "m3ga",
			  "m50/", "mate", "maui", "maxo", "mc01", "mc21", "mcca", "medi", "merc",
			  "meri", "midp", "mio8", "mioa", "mits", "mmef", "mo01", "mo02", "mobi",
			  "mode", "modo", "mot ", "mot-", "moto", "motv", "mozz", "mt50", "mtp1",
			  "mtv ", "mwbp", "mywa", "n100", "n101", "n102", "n202", "n203", "n300",
			  "n302", "n500", "n502", "n505", "n700", "n701", "n710", "nec-", "nem-",
			  "neon", "netf", "newg", "newt", "nok6", "noki", "nzph", "o2 x", "o2-x",
			  "o2im", "opti", "opwv", "oran", "owg1", "p800", "palm", "pana", "pand",
			  "pant", "pdxg", "pg-1", "pg-2", "pg-3", "pg-6", "pg-8", "pg-c", "pg13",
			  "phil", "pire", "play", "pluc", "pn-2", "pock", "port", "pose", "prox",
			  "psio", "pt-g", "qa-a", "qc-2", "qc-3", "qc-5", "qc-7", "qc07", "qc12",
			  "qc21", "qc32", "qc60", "qci-", "qtek", "qwap", "r380", "r600", "raks",
			  "rim9", "rove", "rozo", "s55/", "sage", "sama", "samm", "sams", "sany",
			  "sava", "sc01", "sch-", "scoo", "scp-", "sdk/", "se47", "sec-", "sec0",
			  "sec1", "semc", "send", "seri", "sgh-", "shar", "sie-", "siem", "sk-0",
			  "sl45", "slid", "smal", "smar", "smb3", "smit", "smt5", "soft", "sony",
			  "sp01", "sph-", "spv ", "spv-", "sy01", "symb", "t-mo", "t218", "t250",
			  "t600", "t610", "t618", "tagt", "talk", "tcl-", "tdg-", "teli", "telm",
			  "tim-", "topl", "tosh", "treo", "ts70", "tsm-", "tsm3", "tsm5", "tx-9",
			  "up.b", "upg1", "upsi", "utst", "v400", "v750", "veri", "virg", "vite",
			  "vk-v", "vk40", "vk50", "vk52", "vk53", "vm40", "voda", "vulc", "vx52",
			  "vx53", "vx60", "vx61", "vx70", "vx80", "vx81", "vx83", "vx85", "vx98",
			  "w3c ", "w3c-", "wap-", "wapa", "wapi", "wapj", "wapm", "wapp", "wapr",
			  "waps", "wapt", "wapu", "wapv", "wapy", "webc", "whit", "wig ", "winc",
			  "winw", "wmlb", "wonu", "x700", "xda-", "xda2", "xdag", "yas-", "your",
			  "zeto", "zte-"))) 
	{
		return 4; // Mobile browser detected by User Agent signature
	}

	return false; // Mobile browser not found
} 

/**
 * Check if user is using IE 8
 * @intellisense
 */
function isIE8()
{
	$matches = "";
	preg_match('/MSIE (.*?);/', $_SERVER['HTTP_USER_AGENT'], $matches);
	return (count($matches)>1 && $matches[1]<=8);
}

/**
 * Check if the client application's part is run on a mobile device
 * @intellisense
 */
function isMobile()
{
	return false;
}

/**
 * Check if the client application's part is run on a mobile device
 * @return Boolean
 */ 
function detectMobileDevice()
{
	return false;
}

/**
 * GetPageLayout
 * Return reference to layout object by table name, page type and section (or tab) name
 * @param {string} tableName				the short table name (may be empty)
 * @param {string} pageType					the page's type
 * @param {string} suffixName (optional) 	It could contain a section/tab name or a pages layout's suffix (eg 'success')
 * @return {reference} 						a reference to the layout object
 */
function & GetPageLayout($tableName, $pageType, $suffixName = '')
{
	global $page_layouts, $bUseMobileStyleOnly;
	$layoutName = ($tableName != '' ? $tableName.'_' : '').$pageType.($suffixName != '' ? '_'.$suffixName : '');
	$layout = $page_layouts[$layoutName]; 
	if($layout)
	{
		if($bUseMobileStyleOnly && isMobile())
		{
			$layout->style = $layout->styleMobile;
		}
		else if(postvalue("pdf"))
		{
			$layout->style = $layout->pdfStyle();
		}
	}
	return $layout;
}

/**
 * Check if the mobile template is set
 * @param String templateFileName
 * @return Boolean
 */
function isPageLayoutMobile( $templateFileName )
{
	return false;
}

/**
 * @intellisense
 */
function extractStyle($str)
{
	$pos = my_stripos($str,'style="',0);
	$quot = '"';
	if($pos === false)
	{
		$pos = my_stripos($str,'style=\'',0);
		$quot = '\'';
	}
	if($pos === false)
		return;
	$pos1 = strpos($str,$quot, $pos+7);
	if($pos1 === false)
		return "";
	return substr($str, $pos+7, $pos1-$pos-7);
}

/**
 * @intellisense
 */
function injectStyle($str, $style)
{
	$pos = my_stripos($str,'style="',0);
	$quot = '"';
	if($pos === false)
	{
		$pos = my_stripos($str,'style=\'',0);
		$quot = '\'';
	}
	if($pos === false)
		return $str.' style="'.$style.'"';
	return substr($str,0, $pos+7).$style.";".substr($str, $pos+7);
}

/**
 * @intellisense
 */
function isSingleSign()
{
	if( GetGlobalData("ADSingleSign",0) && $_SERVER["REMOTE_USER"] )
		return false;
		
	return true;
}

/**
 * @intellisense
 */
function generatePassword($length)
{
	$password="";
	for($i=0;$i<$length;$i++)
	{
		$j = rand(0,35);
		if($j<26)
			$password.= chr(ord('a')+$j);
		else
			$password.= chr(ord('0')-26+$j);
	}
	return $password;
}

/**
 * @intellisense
 */
function securityCheckFileName($fileName)
{
	$maliciousStrings = array("../", "..\\");
	for($i = 0; $i < count($maliciousStrings); $i++)
	{
		while(strpos($fileName, $maliciousStrings[$i]) !== FALSE)
		{
			$fileName = str_replace($maliciousStrings, "", $fileName);
		}
	}
	return $fileName;
}

/**
 * @intellisense
 */
function getOptionsForMultiUpload($pSet, $field)
{
	if($pSet->isAbsolute($field))
		$uploadDir = $pSet->getUploadFolder($field);
	else
		$uploadDir = getabspath($pSet->getUploadFolder($field));
	$options = array(
		"max_file_size" => $pSet->getMaxFileSize($field),
		"max_totalFile_size" => $pSet->getMaxTotalFilesSize($field),
		"max_number_of_files" => $pSet->getMaxNumberOfFiles($field),
		"max_width" => $pSet->getMaxImageWidth($field),
		"max_height" => $pSet->getMaxImageHeight($field));
	if($pSet->getResizeOnUpload($field))
	{
		$options["resizeOnUpload"] = true;
		$options["max_width"] = $pSet->getNewImageSize($field);
		$options["max_height"] = $options["max_width"];
	}
	if($pSet->getCreateThumbnail($field))
	{
		$options['image_versions'] = array(
                'thumbnail' => array(
                    'max_width' => $pSet->getThumbnailSize($field),
                    'max_height' => $pSet->getThumbnailSize($field),
					'thumbnailPrefix' => $pSet->getStrThumbnail($field)
                ));
	}
    return $options;
}

/**
 * @intellisense
 */
function getContentTypeByExtension($ext)
{
	if( substr($ext,0,1) != "." )
		$ext = ".".$ext;
	
		if($ext==".asf")
		$ctype = "video/x-ms-asf";
	elseif($ext==".avi")
		$ctype = "video/avi";
	elseif($ext==".doc")
		$ctype = "application/msword";
	elseif($ext==".zip")
		$ctype = "application/zip";
	elseif($ext==".xls")
		$ctype = "application/vnd.ms-excel";
	elseif($ext==".png")
		$ctype = "image/png";
	elseif($ext==".gif")
		$ctype = "image/gif";
	elseif($ext==".jpg" || $ext=="jpeg")
		$ctype = "image/jpeg";
	elseif($ext==".wav")
		$ctype = "audio/wav";
	elseif($ext==".mp3")
		$ctype = "audio/mpeg";
	elseif($ext==".mpg" || $ext=="mpeg")
		$ctype = "video/mpeg";
	elseif($ext==".rtf")
		$ctype = "application/rtf";
	elseif($ext==".htm" || $ext=="html")
		$ctype = "text/html";
	elseif($ext==".asp")
		$ctype = "text/asp";
	elseif($ext == ".flv")
		$ctype = "video/flv";
	elseif($ext == ".mp4")
		$ctype = "video/mp4";
	elseif($ext == ".webm")
		$ctype = "video/webm";
	else
		$ctype = "application/octet-stream";	
		
	return $ctype;
}

/**
 * @intellisense
 */
function getLatLngByAddr($addr) 
{
	switch( getMapProvider() ){
		case GOOGLE_MAPS:	$url = 'http://maps.googleapis.com/maps/api/geocode/json?address='.rawurlencode($addr).'&sensor=false';
				$result = my_json_decode(myurl_get_contents($url));
				if($result['status'] == 'OK') 
				{
					return $result['results'][0]['geometry']['location'];
				}
				break;
		case OPEN_STREET_MAPS: $url = 'http://nominatim.openstreetmap.org/search/'.rawurlencode($addr).'?format=json&addressdetails=1&limit=1';
				$result = my_json_decode(myurl_get_contents($url));
				if($result) 
				{
					$lat = $result[0]['lat'];
					if( !$lat )
						$lat = 0;
					$lng = $result[0]['lon'];
					if( !$lng )
						$lng = 0;
					return array("lat"=>$lat,"lng"=>$lng);
				}
				break;
		case BING_MAPS: 
				if( !GetGlobalData("apiGoogleMapsCode","") )
					return false;
				$url = 'http://dev.virtualearth.net/REST/v1/Locations?query='.rawurlencode( $addr ).'&output=json&key='.GetGlobalData("apiGoogleMapsCode","");
				$result = my_json_decode(myurl_get_contents($url));
				if($result) 
				{
					$lat = $result["resourceSets"][0]["resources"][0]["geocodePoints"][0]["coordinates"][0];
					if( !$lat )
						$lat = 0;
					$lng = $result["resourceSets"][0]["resources"][0]["geocodePoints"][0]["coordinates"][1];
					if( !$lng )
						$lng = 0;
					return array("lat"=>$lat,"lng"=>$lng);
				}
				break;
	}
	return false;	
} 

/**
 * @intellisense
 */
function isLoggedAsGuest()
{
	if($_SESSION["UserID"] == "Guest" && $_SESSION["AccessLevel"] == ACCESS_LEVEL_GUEST)
		return true;
		
	return false;
}

/**
 * Check if the "Login as Guest" option is turned on 
 * @return Boolean
 */
function isGuestLoginAvailable()
{
	// if guest have any permissions
	if(guestHasPermissions())
		return true;		
	return false;
}

/**
 * @intellisense
 */
function func_Override($page)
{
	global $globalSettings;
	if(!isset($globalSettings["override"][$page]))
		return otNone;
	return $globalSettings["override"][$page];
}


/**
 * printMFHandlerHeaders
 * Print HTTP headers for mfhandler.php
 * @intellisense
 */
function printMFHandlerHeaders()
{
	header('Content-Disposition: inline; filename="files.json"');
	header('X-Content-Type-Options: nosniff');
	header('Access-Control-Allow-Origin: *');
	header('Access-Control-Allow-Methods: OPTIONS, HEAD, GET, POST');
	header('Access-Control-Allow-Headers: X-File-Name, X-File-Type, X-File-Size');
}

/**
 * @intellisense
 */
function GetFieldType($field,$table="")
{
	global $pageObject;
	if($table != "" || !isset($pageObject))
	{
		if($table == "")
		{
			global $strTableName;
			$table = $strTableName;
		}
		$newSet = new ProjectSettings($table);
		return $newSet->getFieldType($field);
	}
	else
		return $pageObject->pSet->getFieldType($field);
}

/**
 * @intellisense
 */
function Label($field,$table="")
{
	global $pageObject;
	if($table != "" || !isset($pageObject))
	{
		if($table == "")
		{
			global $strTableName;
			$table = $strTableName;
		}
		$newSet = new ProjectSettings($table);
		$result = $newSet->label($field); 
	}
	else
		$result = $pageObject->pSet->label($field);
	return $result != "" ? $result : $field;
}

/**
 * @intellisense
 */
function getIconByFileType($fileType, $sourceFileName)
{
	switch($fileType)
	{
		case "text/html":
			$fileName = "html.png";
			break;
		case "text/asp":
			$fileName = "code.png";
			break;
		case "application/msword":
		case "application/vnd.openxmlformats-officedocument.wordprocessingml.document":
			$fileName = "doc.png";
			break;
		case "application/vnd.ms-excel":
		case "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet":
			$fileName = "xls.png";
			break;
		case "application/rtf":
			$fileName = "rtf.png";
			break;
		case "image/png":
		case "image/x-png":
			$fileName = "png.png";
			break;
		case "image/gif":
			$fileName = "gif.png";
			break;
		case "image/jpeg":
		case "image/pjpeg":
			$fileName = "jpg.png";
			break;
		case "audio/wav":
			$fileName = "wma.png";
			break;
		case "audio/mp3":
		case "audio/mpeg3":
		case "audio/mpeg":
			$fileName = "mp2.png";
			break;
		case "video/mpeg":
			$fileName = "mpeg.png";
			break;
		case "video/flv":
			$fileName = "flv.png";
			break;
		case "video/mp4":
			$fileName = "mp4.png";
			break;
		case "video/x-ms-asf":
			$fileName = "asf.png";
			break;
		case "video/webm":
		case "video/x-webm":
		case "video/avi":
			$fileName = "mpg.png";
			break;
		case "application/zip":
		case "application/x-zip-compressed":
			$fileName = "zip.png";
			break;
		default:
			$fileName = "text.png";
			$dotPosition = strrpos($sourceFileName, '.');
			if($dotPosition !== false && $dotPosition < strlen($sourceFileName) - 1)
			{
				$ext = substr($sourceFileName, $dotPosition + 1);
				$icons = array();
	           	$icons['7z'] = '7z';
	        		$icons['asf'] = 'asf';
	        		$icons['asp'] = 'code';
	        		$icons['avi'] = 'mpg';
	        		$icons['chm'] = 'chm';
	        		$icons['doc'] = 'doc';
	        		$icons['docx'] = 'doc';
	        		$icons['flv'] = 'flv';
	        		$icons['gz'] = 'gz';
	        		$icons['html'] = 'html';
	        		$icons['mdb'] = 'mdb';
	        		$icons['mdbx'] = 'mdb';
	        		$icons['mp3'] = 'mp2';
	        		$icons['mp4'] = 'mp4';
	        		$icons['mpeg'] = 'mpeg';
	        		$icons['mpg'] = 'mpg';
	        		$icons['mov'] = 'mov';
	        		$icons['pdf'] = 'pdf';
	        		$icons['php'] = 'code';
	        		$icons['pps'] = 'pps';
	        		$icons['ppt'] = 'powerpoint';
	        		$icons['psd'] = 'psd';
	        		$icons['rar'] = 'rar';
	        		$icons['rtf'] = 'rtf';
	        		$icons['swf'] = 'swf';
	        		$icons['tif'] = 'tif';
	        		$icons['ttf'] = 'ttf';
	        		$icons['txt'] = 'txt';
	        		$icons['wav'] = 'wav';
	        		$icons['webm'] = 'mpg';
	        		$icons['wma'] = 'wma';
	        		$icons['wmv'] = 'emv';
	        		$icons['xls'] = 'xls';
	        		$icons['xlsx'] = 'xls';
	        		$icons['zip'] = 'zip';
				if(isset($icons[$ext]))
					$fileName = $icons[$ext].".png";
			}
	}
	return $fileName;
}

function isImageType($type)
{
	switch($type)
	{
		case "image/png":
		case "image/x-png":
		case "image/gif":
		case "image/jpeg":
		case "image/pjpeg":
			return true;
	}
	return false;
}

function initArray(&$array, $key)
{
	if(!isset($array[$key]))
	{
		$array[$key] = array();
	}
}
		
/**
 * GetKeysArray
 * Form aray of primary keys and their values for audit
 * @param {array} $arr array of inserting values
 * @param {bool} $searchId - find last inserted id or not
 * @return {array} array of keys and their values
 */
function GetKeysArray($arr, $pageObject, $searchId = false)
{
	$keyfields = $pageObject->pSet->GetTableKeys();
	$aKeys = array();
    if(count($keyfields))
    {
    	foreach ($keyfields as $kfield)
		{
    		if(array_key_exists($kfield, $arr))
    			$aKeys[$kfield] = $arr[$kfield];
	    }
		if(count($aKeys) == 0 && $searchId)
	    {
	    	$lastId = $pageObject->connection->getInsertedId();
	    	if($lastId > 0)
				$aKeys[$keyfields[0]] = $lastId;
	    }
    }
    return $aKeys;
}

function GetBaseScriptsForPage($isDisplayLoading, $additionalScripts = "", $customText = "")
{
	$result = "";
	$result .= "<script type=\"text/javascript\" src=\"".GetRootPathForResources("include/loadfirst.js")."\"></script>";
	$result .= $additionalScripts;
	$result .= "<script type=\"text/javascript\" src=\"".GetRootPathForResources("include/lang/".getLangFileName(mlang_getcurrentlang()).".js")."\"></script>";
	$result .= "<script type=\"text/javascript\">var getCurrentLang=function(){return '".mlang_getcurrentlang()."';}</script>";

	if( getMapProvider() == BING_MAPS )
		$result .= "<script type=\"text/javascript\" src=\"http://ecn.dev.virtualearth.net/mapcontrol/mapcontrol.ashx?v=7.0&mkt=".getBingMapsLang()."\"></script>";
	
	
	if($isDisplayLoading)
	{
		$result .= "<script type=\"text/javascript\">Runner.runLoading('".$customText."');</script>";
	}
	
	return $result;
}		

/**
 * @param Mixed data
 * @param Bolean plainJSON (optional)	The flag indicating that the JSON's html special chars mustn't be convarted
 * @return String
 */
function printJSON($data, $returnPlainJSON = false)
{
	$rJSON = my_json_encode( $data );
		
	return $returnPlainJSON ? $rJSON : runner_htmlspecialchars( $rJSON );
}

/**
 * Get the value of an interval limit's custom PHP expression
 * @param String table
 * @param String field
 * @param Number idx
 * @param Boolean isLowerBound
 * @return Mixed
 */
function getIntervalLimitsExprs($table, $field, $idx, $isLowerBound)
{


}

function import_error_handler($errno, $errstr, $errfile, $errline) 
{
	/*global $error_happened;

	$error_happened=1;*/
}
function PrepareForExcel($ret)
{
	//$ret = htmlspecialchars($str); commented for bug #6823
	if (substr($ret,0,1)== "=") 
		$ret = "&#61;".substr($ret,1);
	return $ret;

}

function countTotals(&$totals, $totalsFields, $data)
{
	for($i = 0; $i < count($totalsFields); $i ++) 
	{
		if($totalsFields[$i]['totalsType'] == 'COUNT') 
			$totals[$totalsFields[$i]['fName']]["value"] += ($data[$totalsFields[$i]['fName']]!= "");
		else if($totalsFields[$i]['viewFormat'] == "Time") 
		{
			$time = GetTotalsForTime($data[$totalsFields[$i]['fName']]);
			$totals[$totalsFields[$i]['fName']]["value"] += $time[2]+$time[1]*60 + $time[0]*3600;
		} 
		else 
			$totals[$totalsFields[$i]['fName']]["value"] += ($data[$totalsFields[$i]['fName']]+ 0);
		
		if($totalsFields[$i]['totalsType'] == 'AVERAGE')
		{
			if(!is_null($data[$totalsFields[$i]['fName']]) && $data[$totalsFields[$i]['fName']]!=="")
				$totals[$totalsFields[$i]['fName']]['numRows']++;
		}
	}
}

function XMLNameEncode($strValue)
{	
	$search=array(" ","#","'","/","\\","(",")",",","[");
	$ret=str_replace($search,"",$strValue);
	$search=array("]","+","\"","-","_","|","}","{","=");
	$ret=str_replace($search,"",$ret);
	return $ret;
}

/**
 * Get the file extention string
 * @param String fileName
 * @return String
 */
function getFileExtension($fileName)
{
	$pos = strrpos($fileName, ".");
	if( $pos === FALSE ) 
		return "";
		
	return substr($fileName, $pos + 1);
}

/**
 * Get the default db connection object
 * @return Connection
 */
function getDefaultConnection()
{
	global $cman;
	return $cman->getDefault();
}

function isIOS()
{
	return stripos($_SERVER['HTTP_USER_AGENT'],"iPod") !== false 
		|| stripos($_SERVER['HTTP_USER_AGENT'],"iPad") !== false 
		|| stripos($_SERVER['HTTP_USER_AGENT'],"iPhone") !== false;
}


/* Get map provider google = 0, openStreetMap = 1*/
function getMapProvider(){
	return GetGlobalData("mapProvider", true);
}

function getBingMapsLang()
{
	$arrBimgMapLang = array();
	$arrBimgMapLang["Czech"] = "cs-CZ";
	$arrBimgMapLang["Danish"] = "da-DK";
	$arrBimgMapLang["Dutch"] = "nl-NL";
	$arrBimgMapLang["English"] = "en-US";
	$arrBimgMapLang["French"] = "fr-FR";
	$arrBimgMapLang["German"] = "de-DE";
	$arrBimgMapLang["Italian"] = "it-IT";
	$arrBimgMapLang["Japanese"] = "ja-JP";
	$arrBimgMapLang["Norwegian"] = "nb-NO";
	$arrBimgMapLang["Polish"] = "pl-PL";
	$arrBimgMapLang["Portugal"] = "pt-PT";
	$arrBimgMapLang["Portuguese"] = "pt-BR";
	$arrBimgMapLang["Russian"] = "ru-RU";
	$arrBimgMapLang["Spanish"] = "es-ES";
	$arrBimgMapLang["Swedish"] = "sw-SE";
	$arrBimgMapLang["Chinese"] = "zh-TW";
	$arrBimgMapLang["Hongkong"] = "zh-HK";
	if( array_key_exists( mlang_getcurrentlang(), $arrBimgMapLang ) )
		return $arrBimgMapLang[mlang_getcurrentlang()];

	return	$arrBimgMapLang["English"];
}

function getDefaultLanguage()
{
	if( strlen($_SESSION["language"]) == 0 && $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) 
	{
		$arrWizardLang = array();
		$arrWizardLang[] = "English";
		$arrLang = array();
		$arrLang["af"] = "Afrikaans";
		$arrLang["ar"] = "Arabic";
		$arrLang["bs"] = "Bosnian"; //?
		$arrLang["bg"] = "Bulgarian";
		$arrLang["ca"] = "Catalan";
		$arrLang["zh"] = "Chinese";// 1
		$arrLang["hr"] = "Croatian";
		$arrLang["cs"] = "Czech";
		$arrLang["da"] = "Danish";
		$arrLang["nl"] = "Dutch";
		$arrLang["en"] = "English";
		$arrLang["fa"] = "Farsi"; //?
		$arrLang["fr"] = "French";
		$arrLang["ka"] = "Georgian";
		$arrLang["de"] = "German";
		$arrLang["el"] = "Greek";
		$arrLang["he"] = "Hebrew";//?
		$arrLang["hk"] = "Hongkong";// 1
		$arrLang["hu"] = "Hungarian";
		$arrLang["id"] = "Indonesian";//?
		$arrLang["it"] = "Italian";
		$arrLang["ja"] = "Japanese";
		$arrLang["ms"] = "Malay";
		$arrLang["no"] = "Norwegian";
		$arrLang["fl"] = "Phillipines";//?
		$arrLang["pl"] = "Polish";
		$arrLang["pt"] = "Portugal"; // 2
		$arrLang["br"] = "Portuguese"; // 2
		$arrLang["ro"] = "Romanian";
		$arrLang["ru"] = "Russian";
		$arrLang["sk"] = "Slovak";
		$arrLang["es"] = "Spanish";
		$arrLang["sv"] = "Swedish";
		$arrLang["tw"] = "Taiwan";//??
		$arrLang["th"] = "Thai";
		$arrLang["tr"] = "Turkish";
		$arrLang["ur"] = "Urdu";
		$arrLang["cy"] = "Welsh";
		
		$http_lang = strtolower($_SERVER['HTTP_ACCEPT_LANGUAGE']); //return string ex.:fr-FR,fr;q=0.8,en-US;q=0.6,en;q=0.4 
		$http_lang = str_replace(";",",",$http_lang);
		$http_lang = str_replace("-",",",$http_lang);
		
		$langcode = array();
		$langcode = explode(",", $http_lang);
		
		foreach($langcode as $lang)
		{
			if( in_array($arrLang[$lang], $arrWizardLang) )
				return $arrLang[$lang];
		}
	}
	return "English";
}

function xt_showchart($params)
{
	global $useFlashChartLibrary;	
	
	$width = 700;
	$height = 530;
	
	$chartPreview = "";
	if( $params["chartPreview"] )
		$chartPreview = "&chartPreview=true";
	
	if( isset($params["custom1"]) )
		$width = $params["custom1"];
	
	if( isset($params["custom2"]) )
		$height = $params["custom2"];

	if( $params["dashResize"] )
	{
		if( $params["dashWidth"] && $params["dashHeight"] )
		{
			$width = $params["dashWidth"];
			$height = $params["dashHeight"];
		} 
		elseif( $params["dashWidth"] )
		{
			$height = round( $height * $params["dashWidth"] / $width );
			$width = $params["dashWidth"];
		}
		elseif( $params["dashHeight"] )
		{
			$width = round( $width * $params["dashHeight"] / $height );
			$height = $params["dashHeight"];		
		}
		
		// adjust the chart size to fit it in the dash cell
		$width*= 0.95; 
		$height*= 0.95; 
	}		
	elseif( $params["resize"] )
	{
		$maxWidth = 400;
		$maxHeight = 280;
		$r = $maxWidth / $maxHeight;
		$r2 = $width / $height;
		if (($width > $maxWidth) || ($height > $maxHeight))
		{
			if ($r2 >= $r)
			{ // width
				$height = round( $height * $maxWidth / $width );
				$width = $maxWidth;
			}
			else
			{
				$width = round ( $width * $maxHeight / $height );
				$height = $maxHeight;
			}
		}
	}

	$showDetails = isset( $params["showDetails"] ) ? $params["showDetails"] :  true;

	$settings = new ProjectSettings(GetTableByShort($params["chartname"]));
	$refresh = $settings->getChartRefreshTime();
	if( !$useFlashChartLibrary && ( $params["ctype"]=="2DPie" || $params["ctype"]=="2DDoughnut" || $params["ctype"]=="Funnel" 
		|| $params["ctype"]=="OHLC" || $params["ctype"]=="Candle" ) )
	{
		$refresh = 0;
	}
	
	$chartParams = array();
	$chartParams['forceFlash'] = $useFlashChartLibrary;
	$chartParams['width'] = $width;
	$chartParams['height'] = $height;
	$chartParams['showDetails'] = $showDetails;
	$chartParams['chartName'] = $params["chartname"];
	$chartParams['containerId'] = $params["chartname"].$params["id"];
	$chartParams['chartType'] = $params["ctype"];
	$chartParams['refreshTime'] = $refresh;	
	$chartParams['xmlFile'] = GetTableLink("dchartdata") . '?chartname=' . $params["chartname"] . $chartPreview . 
		'&ctype=' . $params["ctype"] .
		'&showDetails=' . $showDetails;
	
	if( isset( $params["dash"] ) && $params["dash"] ) 
	{
		$chartParams['xmlFile'] .= '&dashChart=' . $params["dash"];
		$chartParams['dashChart'] = !!$params["dash"];
	}
	
	if( isset( $params["dashTName"] ) && $params["dashTName"] ) 
	{
		$chartParams['dashTName'] = $params["dashTName"];
		$chartParams['dashElementName'] = $params["dashElementName"];
		$chartParams['pageId'] = $params["id"];
		
		$chartParams['xmlFile'] .= '&dashTName=' . $params["dashTName"];
		$chartParams['xmlFile'] .= '&dashElName=' . $params["dashElementName"];
		$chartParams['xmlFile'] .= '&pageId=' . $params["id"];
		$dashSet = new ProjectSettings( $params["dashTName"] );
		$dashElement = $dashSet->getDashboardElementData( $params["dashElementName"] );
		if( $dashElement )
		{
			if( $dashElement["reload"] )
			{
				$chartParams['refreshTime'] = $dashElement["reload"];	
			}
		}
	}
	

	echo '<div id="' . $chartParams['containerId'] . '" style="width:' . $width . 'px;height:' . $height . 'px"></div>';
	if( !$params["singlePage"] )
	{
		$chartParams["webRootPath"] = GetWebRootPath();
		
		echo '<script type="text/javascript" language="javascript" src="' . GetRootPathForResources( 'libs/js/AnyChart.js' ) . '"></script>';
		echo '<script type="text/javascript" language="javascript" src="' . GetRootPathForResources( 'libs/js/AnyChartHTML5.js' ) . '"></script>';
		
		echo '<script type="text/javascript" language="javascript"> 
		(function(){ 
			var params = ' . my_json_encode( $chartParams ) . ';
			if ( Runner && Runner.Charts ) {
				Runner.Charts.createChart( params );
			} else {
				$(document).ready( function() { 
					Runner.Charts.createChart( params ); 
				} );
			}
		})();		
		</script>';
	}
}


?>